      PROGRAM gendata

*  This program is used for generation input data file 'tapein' in HABP.
* Program allows generate input data for simple geometry configurations, such as
* bodies of revolution (spherical nose, truncated cones, cylinders, etc. - 
* Ellipse Generation method) with flat surfaces (Element Data Input method).
* This file serves as a sample for building more complex geometry configurations.
* Note: common coordinate system OXYZ is used, where axis OX directed from nose 
*       to aft (positive X), OY directed up, and OZ to side.
* 
*
*    ***  Program was written by Igor Polyakov  ***
*                      Ukraine.
*  Input:
*
* name - Title
* r - blunting radius
* xo - X coordinate of leading point  (usually 0.0)
* ndxzat - number of points on spherical blunting
* ficone - half an angle of first cone (degrees)
* nk - number of cones and cylinders
* xk - X coordinates of base points (xk(0) may be any value because it computed
*       in program as transition point between sphere and first cone)
* yk - Y coordinates of base points (yk(0) may be any value because it computed
*       in program as transition point between sphere and first cone)
* ndx - numbers of segments in each cone and cylinder
* nr - number of rows on flat surface 
*      (number of sections in longitudinal direction)
* nc - number of columns on flat surface 
*      (number of sections in lateral direction), must be odd number
* X, Y, Z - arrays of x, y, z coordinates of flat surface (trapezium) vertexes
* MACH - Mach number
* ALPHA - angle of attack (degrees)

* NOTE: Input order of flat surface vertixes see below in commentaries 
* to SUBROUTINE Trapez

      DIMENSION  xk(10), yk(10), ndx(10)
      DIMENSION  IPG(20)
      DIMENSION  IMPACT(3), ISHAD(3), IPANL(10)
      CHARACTER *80 name
      CHARACTER *4 Sect
      INTEGER Case
      REAL  MACH, MAC

      OPEN( 1, FILE = 'input')
      OPEN( 2, FILE = 'tapein')
      READ( 1, '( a // 2f7.0, i7, f7.0, i7, 2(// 10f7.0) // 10i7 )' )
     +            name, r, xo, ndxzat, ficone, nk, xk, yk, ndx

      THETO = 0.0
      THETL = 180.0
      NNi = 18
      DELZ = 0.0
      DELY = 0.0
      Case = 0
      IType = 5
      Sect = ' Sph'

      WRITE( 2, "('********** Executive Flag Card ***MAIN***(//2I7)*****
     +*************************' / '  IEROR INMONT' )" )
      WRITE( 2, '(2i7)' ) 0, 0

      WRITE( 2, "('********* System Control Card ***MAIN***(//20I1,39X,A
     +4)***********************' / '  IPG(1-20) :  1-GEOM 2-AERO 3-GRAPH
     + 4-AUXILI              Case')" )
      WRITE( 2, '( 2i1, t60, 4i1 )' )  1, 2, 0, 0, 0, 1

      WRITE( 2, "('********** Geometry Control Card ***GEOM***(7I7//15A4
     +)*************************' / '   IOUT   IREW PRINTS  IQUAD  I3MAX
     +   NEW  NPMAX')" )
      WRITE( 2, '( 7i7 )' )  8, 0, 0, 0, 0, 0, 0

      WRITE( 2, "( ' CONFIG' )" )
      WRITE( 2, '( a )' )  name

********************** Blunting Geometry Generation ***********************

      LAST = 0
      IGEOM = 2
      CALL PanIdCard( 'Spher  ', LAST, IGEOM )

      WRITE( 2, "( '*** Ellipse Generation Control Card ***ELLIP***(//12
     +A4,11X,2I1,t65,I2,1A4,I2)***' / 'Title [12a4]                     
     +                     DisconIpr CsSectIType' / '                   
     +                                        10     Fslg 4' / '      (
     +3F7.0,I7,4F7.0,I7,t65,I2,1a4,i2 )' / '      X  THETO  THETL    NNi
     +      A      B   DELZ   DELY   LAST CsSectIType' )" )

      pi = ATAN(1.0) * 4.0
      gtr = pi / 180.0

      fisopr = 90. - ficone
      xc = xo + r * ( 1 - COS( fisopr*gtr ) )
      yc = r * SIN( fisopr*gtr )
      xk(1) = xc
      yk(1) = yc
      dx = (xc-xo) / ndxzat

      DO  x = xo, xc+dx/100, dx
         y = r * SIN ( ( ACOS ( 1.0 - (x-xo)/r ) ) )
         IF( ABS( x-xc ) .LE. dx/100 )  THEN
            LAST = 4
         ELSE
            LAST = 0
         ENDIF
         WRITE( 2, '( f7.3, t8,2F7.1,I7, f7.3,t36, f7.3,t43,2f7.3,I7,
     +          t65,I2,1a4,i2 )' ) -x, THETO, THETL, nni, y, y, DELZ,
     +          DELY, LAST, Case, Sect, ITYPE
      ENDDO

******************* Body of Revolution Geometry Generation *************

      LAST = 0
      IGEOM = 2
      CALL PanIdCard( 'Body   ', LAST, IGEOM )

      WRITE( 2, "( '*** Ellipse Generation Control Card ***ELLIP***(//12
     +A4,11X,2I1,t65,I2,1A4,I2)***' / 'Title [12a4]                     
     +                     DisconIpr CsSectIType' / '                   
     +                                        10     Fslg 4' / '      (
     +3F7.0,I7,4F7.0,I7,t65,I2,1a4,i2 )' / '      X  THETO  THETL    NNi
     +      A      B   DELZ   DELY   LAST CsSectIType' )" )

* Sphere to cone transition point
      WRITE( 2, '(  f7.3,t8,2F7.1,I7, f7.3,t36, f7.3,t43,2f7.3,I7,
     +       t65,I2,1a4,i2 )' ) -xc, THETO, THETL, nni, yc, yc, DELZ,
     +       DELY, LAST, Case, Sect, ITYPE

      DO  i = 1, nk
         tanti = ( yk(i+1) - yk(i) ) / ( xk(i+1) - xk(i) )
         dxki = ( xk(i+1) - xk(i) ) / ndx(i)
         DO x = xk(i)+dxki, xk(i+1)+dxki/100, dxki
            y = yk(i) + ( x - xk(i) ) * tanti
            IF( ABS( x-xk(nk+1) ) .LE. dxki/100 )  THEN
               LAST = 4
            ELSE
               LAST = 0
            ENDIF
            WRITE( 2, '( g12.6,t8,2F7.1,I7,g12.6,t36,g12.6,t43,2f7.3,I7,
     +             t65,I2,1a4,i2 )' ) -x, THETO, THETL, nni, y, y, DELZ,
     +             DELY, LAST, Case, Sect, ITYPE
         ENDDO
      ENDDO

********************* Flat Surface Geometry Generation **********************

      LAST = 1
      IGEOM = 1
      CALL PanIdCard( 'Stb.   ', LAST, IGEOM )
      WRITE( 2, "( '********** Element Read Control Card ***IELE***(//4I
     +7)***************************' / '  I3MAX     IN   IREW     I3' /
     +                                 '      1      0      0      0' /
     +'*** Element Data Input Card ***IELE***(//(3F10.0,I2,3F10.0,I2,t65
     +,I2,1A4,I2,4X,I4))**' / '         x         y         z STAT     x
     +x        yy        zzSTATSectIType Seq' )" )

      CALL Trapez

*
      WRITE(2,"('*********************** START OF INPUT TO AERO PROGRAM 
     +**********************' / '                           ******** Tit
     +le ********' / 'TITLE(15)' / '
     +       ' / '                      ****** Program Flags **********'
     +/'IPG(1) - up to 20 flags: 1 - flow fields; 2 - shielding; ' /
     + '                         3 - inviscid  pressure; 5 - auxiliary p
     +rograms; ')")
* / '                          4 - viscous forces; 6 - flow lines.')")

      DO i = 1,20
         IPG(i) = 0
      ENDDO

      IPG(1) = 3
      IPG(2) = 5

      WRITE( 2, '( 10i7 )' )  IPG

      READ( 1, '( / 2f7.0 )' )  MACH, ALPHA
      WRITE( 2, "( '                       ****** Flight conditions ****
     +*' / '   MACH    ALT  PSTAG  TSTAG   IGAS    NAB' / 4f7.1, 2i7 )")
     +      MACH, 0., 0., 0., 0, 1

      PI = ATAN(1.) * 4
      SREF = PI * yk(nk+1) ** 2
      MAC = xk(nk+1)
      SPAN = 2 * yk(nk+1)

      WRITE( 2, "( '                      ******** Reference sizes *****
     +****' / '   SREF    MAC   SPAN    XCG    YCG    ZCG' / f7.2,
     + 5f7.3)" )  SREF, MAC, SPAN, 0.0, 0.0, 0.0

      WRITE(2,"( '                    ******** Alpha-Beta Data ********'
     + / '                (Number of such lines must be equal to NAB)' /
     +  '  ALPHA   BETA    ROL CDELTA     QI     RI     PI' / 7f7.2 )")
     +     ALPHA, 0., 0., 0., 0., 0., 0.

      NCOMP = 3
      WRITE(2,"( '********************* INVISCID PRESSURE CALCULATION **
     +************************' / '************************* Title and M
     +ain Flags *********************** PRES **' / '  NCOMP IFSAVE  TITL
     +E' / 2i7 )" )  NCOMP, 0

      IMPACT(1) = 2
      ISHAD(1)  = 2
      IMPACT(2) = 5
      ISHAD(2)  = 4
      IMPACT(3) = 4
      ISHAD(3)  = 1

      DO i = 1, 10
         IPANL(i) = 0
      ENDDO

      DO i = 1, NCOMP
         IPANL(1) = i
         CALL  CompOrg( IPANL, IMPACT(i), ISHAD(i) )
      ENDDO

      WRITE(2,"('*******************************************************
     +***********************' / '************ OPTION :  A U X I L I A R
     + Y      P R O G R A M S ****************' / '*********************
     +*********************************************************' /
     + 'Array of suboptions IPG(20)' / '  1  0  0  0  0  0  0  0  0  0  
     +0  0  0  0  0  0  0  0  0  0' / '*********************************
     +*********************************************' / '           SUBOP
     +TION :  SUMMATION OF COMPONENTS' /                    '***********
     +******************************************************************
     +*' / '   LAST   ISUM  ISAVE IPRINT IPUNCH   IRUN' / '      1      
     +1      1      0      0      0' / '   ICOMP(20)' / '  1  2  3  0  0
     +  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0' )" )

      STOP
      END


      SUBROUTINE  PanIdCard( Panel_Id, LAST, IGEOM )
      CHARACTER *7 Panel_Id
      WRITE( 2, "( '********** Panel Identification Card ***GEOM***(//A7
     +,2I7//10I7//6I7)**********' / 'PANEL(i)  LAST IORN(i)')" )
      WRITE( 2, '( a7, 2i7 )' )  Panel_Id, LAST, 0
      WRITE( 2, "('  IGEOM [10i7]   Method Flags: 1-IELE; 2-ELLIP; 3-CUB
     +IC; 4-AIRCFT' )" )
      WRITE( 2, '( i7 )' )  IGEOM
      WRITE( 2, "( 'SYMFCT(i) IFA(i) NADJ1 NADJ2  NADJ3  NADJ4' )" )
      WRITE( 2, '( 6i7 )' )  0, 0, 0, 0, 0, 0
      END


      SUBROUTINE Trapez
*
*   This program is used for generation of flat surface points coordinates.
* Flat surface is presented as trapezium. Coordinates of generated points
* correspond to Element Geometry Data Input in HABP Mark-4.
*  Input order of points: counter-clockwise at the sight from external normal.
* Points are numbered in following order: lines 1-2 and 3-4 must be side edges
* of trapezium, and lines 2-3 and 4-1 - parallel edges. Order of increasing of
* elements in bands: from points 3 and 4 to points 2 and 1.
*  If flat surface is triangle then point 3 must be same as point 4.
*  Program perform test of parallelism edges 2-3 and 4-1. This test may be
* successfull only when edges 2-3 and 4-1 lies in plane, which is parallel to
* one of three coordinate planes.

*
      DIMENSION  x(4), y(4), z(4)
      INTEGER stat1, stat2
      REAL L41, L32, L12
      CHARACTER *4 Sect
      INTEGER Case

      Case = 0
      IType = 3
      Sect = 'Stab'
      ISEQ = 301

      READ( 1, '( / 2i7 // (4f7.0) )' )
     +            nr, nc, x, z, y
*
      IF( IAND( nc, 1 ) .NE. 1 )  THEN
         WRITE( 2, "('WARNING: ',
     +   'Number of sections in lateral direction NC must be ODD' )" )
         STOP 'Program Failure: Number of sections in lateral direction 
     +NC must be ODD'
      ENDIF
*
* Set triangle flag
      itriangl = 0
      IF( (x(3).EQ.x(4)) .AND. (y(3).EQ.y(4)) .AND. (z(3).EQ.z(4)) )
     +   itriangl = 1
*
* Length of segments 4-1 and 3-2:
*
      L41 = SQRT( ( x(4) - x(1) )**2 + ( y(4) - y(1) )**2 +
     +            ( z(4) - z(1) )**2 )
      L32 = SQRT( ( x(3) - x(2) )**2 + ( y(3) - y(2) )**2 +
     +            ( z(3) - z(2) )**2 )
*
* Direct cosines of segments 4-1 and 3-2:
*
      t41x = ( x(1) - x(4) ) / L41
      t41y = ( y(1) - y(4) ) / L41
      t41z = ( z(1) - z(4) ) / L41
      t32x = ( x(2) - x(3) ) / L32
      t32y = ( y(2) - y(3) ) / L32
      t32z = ( z(2) - z(3) ) / L32
*
* If this is not triangle, then perform test of parallelism
      IF( itriangl .EQ. 0 .AND. ( t41x .NE. t32x .OR. t41y .NE. t32y
     +                            .OR. t41z .NE. t32z ) )  THEN
         WRITE( 2, "('Program Failure: ',
     +      'Base edges of trapezium are not parallel !!!')" )
         STOP 'Program Failure: Base edges of trapezium are not parallel
     + !!!'
      ENDIF

      DO i = 1, nr+1
         x1 = x(4) + L41*t41x*(i-1)/nr
         y1 = y(4) + L41*t41y*(i-1)/nr
         z1 = z(4) + L41*t41z*(i-1)/nr
         x2 = x(3) + L32*t32x*(i-1)/nr
         y2 = y(3) + L32*t32y*(i-1)/nr
         z2 = z(3) + L32*t32z*(i-1)/nr
         L12 = SQRT( ( x2 - x1 )**2 + ( y2 - y1 )**2 + ( z2 - z1 )**2 )
         IF( L12 .GT. 1.0e-06 ) THEN
            t12x = ( x2 - x1 ) / L12
            t12y = ( y2 - y1 ) / L12
            t12z = ( z2 - z1 ) / L12
         ELSE
            t12x = 1.
            t12y = 1.
            t12z = 1.
         ENDIF
         DO j = 1, (nc+1)/2
            stat1 = 0
            stat2 = 0
            xx =  x1  +  L12 * t12x * ( 2*j - 2 ) / nc
            xxx = x1  +  L12 * t12x * ( 2*j - 1 ) / nc
            yy =  y1  +  L12 * t12y * ( 2*j - 2 ) / nc
            yyy = y1  +  L12 * t12y * ( 2*j - 1 ) / nc
            zz =  z1  +  L12 * t12z * ( 2*j - 2 ) / nc
            zzz = z1  +  L12 * t12z * ( 2*j - 1 ) / nc
            IF( j .EQ. 1 )  stat1 = 1
            IF( i .EQ. 1 .AND. j .EQ. 1 )  stat1 = 2
            IF( i .EQ. nr+1 .AND. j .EQ. (nc+1)/2 )  stat2 = 3
            WRITE(2,'( 3f10.3, i2, 3f10.3, i2, t65, I2, 1A4, I2,4X,I4)')
     +             -xx, yy, zz, stat1, -xxx, yyy, zzz, stat2,
     +             CASE, SECT, ITYPE, ISEQ
            ISEQ = ISEQ + 1
         ENDDO
      ENDDO

      END


      SUBROUTINE  CompOrg( IPANL, IMPACT, ISHAD )
      DIMENSION  IPANL(10)

      WRITE( 2, "( '                   ****** Component Organization ***
     +*****' / 'IPANL(10) - numbers of all Panels on storage unit 4',
     +         ' of this component' / 10i7 )" )  IPANL

      WRITE( 2,"('    IPM pressure computation method flag' / '      1'/
     + '    INT interference computation method flag' / '      0' /
     + '  ISHEF shielding elements flag' / '      0' )" )

      WRITE( 2, "('                 ****** Pressure computation method *
     +**PRES***' / ' IMPACT  ISHAD IPRINT   IPIN  ISAVE' / 5i7 )" )
     +       IMPACT, ISHAD, 0, 0, 0

      WRITE( 2, "('                    PDATA' /
     +      '    (1)    (2)    (3)    (4)    (5)    (6)' /
     +      '   1.84    1.0    1.0    1.0    0.0    3.0' )" )

      END
