!+
!!!      OVERLAY(LWB,0,0)
      PROGRAM W12SC3
!      (OUTPUT=128,TAPE5=128,TAPE6=OUTPUT,                 USSAERO
!     X TAPE7=1024,TAPE8=1024,TAPE9=1024,TAPE10=2048,TAPE11=1024,
!     X TAPE12=1024,TAPE13=1024,TAPE14=1024,TAPE20=1024,TAPE21=1024,
!     X TAPE22=1024,TAPE23=1024,TAPE24=1024,TAPE25=1024,TAPE26=1024,
!     X TAPE27=1024,TAPE28=1024,TAPE29=1024,TAPE30=1024,TAPE31=1024,
!     1 TAPE15=1024,TAPE16=1024,TAPE17=1024,TAPE32=1024,TAPE33=1024,
!     2 TAPE34=1024,TAPE35=1024,TAPE36=1024,TAPE37=1024,TAPE38=1024,
!     3 TAPE39=1024)
!
!     *******************************************VERSION B00************
!
!
!      THE GRUMMAN W12/SC3 PROGRAM
!
!     CODE DEVELOPED FOR NASA LRC SUPERSONIC MANEUVER WING STUDY
!
!
!      INCLUDES  1. MIXED ANALYSIS/DESIGN AS DESCRIBED IN
!                   AIAA PAPER 79-0233, BY ALEX CENKO
!
!                2. BRUCE ROSEN'S VERSION OF THE CARLSON CORRECTION
!
!                3. SC3 PANEL OPTION
!
!      THE PROGRAM IS BASED ON THE GRUMMAN W12 PROGRAM
!      WHICH USES THE NASA USSAERO CODE BY FRANK WOODWARD AS A BASE
!
!
!
!     PROGRAM USSAERO COMPUTES THE SUBSONIC AND SUPERSONIC POTENTIAL
!     FLOW AERODYNAMIC CHARACTERISTICS OF CANARD-WING-BODY-TAIL
!     CONFIGURATIONS. THE BODY IS REPRESENTED BY SOURCE PANELS AND THE
!     CANARD, WING, AND TAIL ARE REPRESENTED BY LINEARLY VARYING VORTEX
!     PANELS. THE THEORY IS DESCRIBED IN NASA CR-2228 (PART I) AND THE
!     COMPUTER PROGRAM IS DESCRIBED IN NASA CR-2228 (PART II).
!
!     THIS PROGRAM WAS PREPARED FOR NASA LANGLEY RESEARCH CENTER UNDER
!     CONTRACT NAS1-10408 BY AEROPHYSICS RESEARCH CORPORATION, BELLEVUE,
!     WASHINGTON.
!
!     THE INVESTIGATION WAS CONDUCTED BY MR. FRANK A. WOODWARD OF
!     ANALYTICAL METHODS, INCORPORATED, BELLEVUE, WASHINGTON. AREA CODE
!     206-454-6119 OR AREA CODE 206-454-6115.
!
!     ANY ERRORS OR PROBLEMS ENCOUNTERED IN USING THE PROGRAM SHOULD BE
!     DIRECTED TO MR. CHARLES H. FOX, JR. AT NASA LANGLEY. AREA CODE 804
!     -827-3711.PROGRAM USERS ARE REQUESTED TO SUBMIT THEIR NAME AND
!     MAILING ADDRESS TO MR. FOX SO THAT THEY MAY BE NOTIFIED OF PROGRAM
!     CORRECTIONS AND IMPROVEMENTS.
!
!
!     A CARD DECK AND DOCUMENTATION FOR THE PROGRAM ARE AVAILABLE FROM
!     COSMIC, UNIVERSITY OF GEORGIA, ATHENS, GEORGIA, 30601.
!
!     THIS PROGRAM IS WRITTEN IN CDC FORTRAN IV, VERSION 2.3, TO RUN ON
!     CDC 6600 SERIES COMPUTERS WITH THE SCOPE 3.0 OPERATING SYSTEM AND
!     LIBRARY TAPE.
!
!
!     ***********************FEBRUARY 1973*******VERSION A00************
!
!     VERSION A00 IS THE BASE VERSION OF USSAERO DOCUMENTED IN NASA
!     CR-2228.
!
!     ******************************************************************
!
!
!     ******************SEPTEMBER 1973*****VERSION A01******************
!
!     VERSION A01 IS THE INITIAL RELEASE OF PROGRAM USSAERO TO COSMIC.
!
!
!     NUMEROUS CORRECTIONS A01 HAVE BEEN MADE TO THE PROGRAM. THESE
!     CORRECTIONS HAVE ESSENTIALLY NO IMPACT ON THE DOCUMENTAION IN
!     NASA CR-2228 WITH THE OBVIOUS EXCEPTION OF THE PROGRAM LISTING
!     OF PART II.
!
!     ..................................................................
!
!     REVISIONS A01 HAVE BEEN MADE TO THE PROGRAM WHICH ARE NOT
!     DOCUMENTED IN NASA CR-2228.
!
!     ADDITIONAL ARGUMENTS HAVE BEEN ADDED TO THE CALLING SEQUENCES FOR
!     SUBROUTINES VORPAN AND PRESS.
!
!     FOR THE SUBSONIC CASE (PLANAR BOUNDARY CONDITION OPTION ONLY) AND
!     FOR THE SUPERSONIC CASE, THE PRESSURE COEFFICIENT ON THE WING,
!     CANARD (HORIZONTAL TAIL), AND FIN (VERTICAL TAIL) IS COMPUTED ON
!     THE BASIS OF EXACT LINEARIZED THEORY BUT IS NOT PERMITTED TO
!     EXCEED VACUUM PRESSURE COEFFICIENT.
!
!
!     FOR A WING OR CANARD (HORIZONTAL TAIL), AN ADDITIONAL COLUMN OF
!     VORTEX PANELS HAS BEEN INCLUDED WHICH EXTENDS FROM THE FUSELAGE
!     CENTER LINE TO THE INBOARD EDGE OF THE WING OR CANARD (HORIZONTAL
!     TAIL).
!
!     A CONVERGENCE CRITERIA TEST HAS BEEN INCORPORATED INTO SUBROUTINE
!     ITRATE TO CONTROL THE ITERATIVE SOLUTION PROCEDURE. THE MAXIMUM
!     NUMBER OF ITERATIONS AND THE CONVERGENCE CRITERIA TEST PARAMETER
!     HAVE BEEN ADDED TO THE PROGRAM INPUT. EXPERIENCE HAS INDICATED
!     THE NEED TO FURTHER MODIFY THE SOLUTION PROCEDURE TO AN
!     ACCELERATED ITERATIVE SOLUTION PROCEDURE, HOWEVER NO DATE HAS BEEN
!     ESTABLISHED FOR ACCOMPLISHING THIS IMPROVEMENT.
!
!     A CALL TO FTNBIN HAS BEEN ADDED TO USSAERO. FTNBIN IS A SYSTEM
!     ROUTINE WHICH BLOCKS THE I/O OF ALL BINARY FILES. THIS
!     SUBSTANTIALLY IMPROVES THE EFFICIENCY OF USSAERO.
!
!     ..................................................................
!
!     VERSION A01 CONTAINS THE FOLLOWING LIMITATIONS OR KNOWN PROBLEMS.
!
!     THE PROGRAM IS INTENDED TO HANDLE 600 BODY PANELS AND 600 WING
!     PANELS,HOWEVER THE INPUT SPECIFIES CUTTING PLANES RATHER THAN
!     PANELS WHICH MAY LIMIT THE INPUT TO DEFING LESS THAN 600 PANELS.
!
!     THE PROGRAM WILL NOT HANDLE NACELLES.
!
!     FOR THE PLANAR BOUNDARY CONDITION OPTION, THE WING, CANARD
!     (HORIZONTAL TAIL), AND FIN (VERTICAL TAIL) CANNOT CONTAIN DIHEDRAL
!     (ROLL OUT) OR INCORRECT PRESSURES MAY BE CALCULATED. AS PART OF
!     THIS PROBLEM, A CENTER LINE FIN (VERTICAL TAIL) MAY HAVE INCORRECT
!     PRESSURES CALCULATED ON IT.
!
!     THE TRAILING VORTEX SHEET FROM THE WING LEAVES THE TRAILING EDGE
!     IN A PLANE PARALLEL TO THE X AXIS.
!
!     THE NONPLANAR BOUNDARY CONDITION OPTION IS NOT AVAILABLE IN
!     SUPERSONIC FLOW.
!
!     WING PANELS WITH SWEPT FORWARD EDGES MAY GIVE INCORRECT RESULTS IN
!     SUPERSONIC FLOW.
!
!
!     ******************************************************************
!
!
!     ************************DECEMBER 1974*******VERSION B00***********
!
!     VERSION B00 IS THE INITIAL PLOT VERSION OF PROGRAM USSAERO.
!
!     THE PLOTTING MODIFICATIONS TO THE PROGRAM WERE MADE BY OCTAVIO
!     WINTER OF THE COMPUTER SCIENCES CORPORATION LANGLEY OFFICE.
!
!     REVISIONS B00 HAVE BEEN MADE TO THE PROGRAM WHICH ARE NOT
!     DOCUMENTED IN NASA CR-2228.
!
!     THE PLOTTING PROGRAM OF NASA TMX-2074 HAS BEEN MERGED WITH PROGRAM
!     USSAERO AND SEVERAL ADDITIONAL PLOT OPTIONS HAVE BEEN ADDED.
!
!     ONE CAN OBTAIN GEOMETRY PANELING PLOTS, SINGULARITY PANELING
!     PLOTS, AND PRESSURE DISTRIBUTION PLOTS.
!
!     THE CARD INPUT HAS BEEN REVISED TO PERMIT PLOTTING.
!
!     SUBROUTINE ITRATE HAS BEEN COMPLETELY REVISED. PROFESSOR F. J.
!     MARSHALL OF PURDUE UNIVERSITY EXPANDED THE CAPABILITIES OF ITRATE
!     UNDER A GRANT FORM NASA LANGLEY. VERSION A01 OF ITRATE EMPLOYED A
!     BLOCKED JACOBI ITERATION. VERSION B00 OF ITRATE GIVES THE USER THE
!     CHOICE OF EMPLOYING A BLOCKED JACOBI ITERATION, A BLOCKED GAUSS-
!     SEIDEL ITERATION, A BLOCKED SUCCESSIVE OVERRELAXATION ITERATION,
!     OR A BLOCKED CONTROLLED SUCCESSIVE OVERRELAXATION ITERATION. WHILE
!     THE LATTER METHOD APPEARS TO YIELD THE MOST EFFICIENT SOLUTION
!     PROCEDURE, THE UNDERLYING MATHEMATICAL THEORY HAS NOT YET BEEN
!     EXPLORED RIGOROUSLY. THEREFORE THE DEFAULT METHOD USED IN THE
!     PROGRAM IS BLOCKED GAUSS-SEIDEL.
!
!     SUBROUTINE ZPARAM HAS BEEN ADDED TO THE PROGRAM. A NUMBER OF CALLS
!     TO ZPARAM HAVE ALSO BEEN ADDED THROUGHOUT THE PROGRAM. ZPARAM
!     ACCESSES THE LANGLEY OPERATING SYSTEM TO DETERMINE CPU TIME,
!     OPERATING SYSTEM CALLS BY EACH FILE, AND COST. THESE STATISTICS
!     ARE WRITTEN ON THE OUTPUT FILE FOR SUBSEQUENT ANALYSIS.
!
!
!     NUMEROUS CORRECTIONS B00 HAVE ALSO BEEN MADE TO THE PROGRAM.
!
!     THE FOLLOWING PROBLEMS IN VERSION A01 HAVE BEEN FIXED IN VERSION
!     B00.
!
!     CORRECTIONS HAVE BEEN MADE TO PERMIT DIHEDRAL AND ROLL OUT FOR
!     CANARDS AND FINS AND TO CORRECTLY HANDLE A CENTERLINE FIN.
!
!     CORRECTIONS HAVE BEEN MADE TO HANDLE WINGS WITH SWEPT FORWARD
!     EDGES IN SUPERSONIC FLOW.
!
!
!     ******************************************************************
!
!
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,    &
!!!     & TAPE9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15,TAPE16,TAPE17
      INCLUDE 'files.cmn'
      COMMON /BLANK/DUM(30)
      COMMON /BLANK2/ DUM2(25)
      COMMON /LWB/ LWB
      COMMON /PARAM/ NBODY,NWING,NTAIL,LBC,THK,MACH,ALPHA,REF(7)
      COMMON /POINT/ ARRAY(6000)
      COMMON /SCRAT/ BLOCK(7500)
!!!      COMMON /HEAD/ TITLE(28)
      INCLUDE 'head.cmn'
      COMMON /ONE/ NAN2
      COMMON /SEG/ XS(261)
      COMMON /BTHET/ TB(600)
      COMMON /NEWCOM/ K0,K1,K2,K3,K4,K5,K6,K(79)
      COMMON /MATCOM/ MATIN
      COMMON /BODCOM/DODCOM(21)
      COMMON /VELCOM/N(5),EM,L(54),DUMMY(30)
      COMMON /COMPS/ OMPS(11)
      COMMON /COMPV/ OMPV(10)
      COMMON /COEF/ OEF(400)
      COMMON /TRAN/ RAN(15)
      COMMON /FORM/ ORM(66)
      COMMON /ITERAT/ ITMAX,CCTEST,DCTEST,ITMETH,ALF1,ALF2
      COMMON /GRAPH/ PLOT
      COMMON /PRESS/ LUG(21)
      COMMON /SCALE/ CSC(8)
      COMMON /CONPLT/ KONPLT
      COMMON /PTYPE/ ORT,VU3,STE,PER
      COMMON /PI/ PI,RADEG,DEGRAD
      COMMON /EPS/ EPSPAN,EPSWP1,EPSWP2,EPSTP,EPSVC,EPSSP1,EPSSP2,EPSLV,&
     &             EPSSV,EPSVV,EPSWV,EPSVP,EPSIT1,EPSIT2,EPSFM,EPSSB1,  &
     &             EPSSB2
      COMMON /CLINE/CLUMSY(60)
!*
      COMMON /TAPES/ ITAPES(20)
!!!      COMMON /OPTION/ NOPT(10),NXSTNS,XSTN(20),XAPEX,YAPEX,XLAMDA
      INCLUDE 'option.cmn'
      COMMON /DES/ CLBAR, CMBAR
      COMMON /ROWS/ NWCOL, NWRPC(20)
      COMMON /PSAVE/ IFREE(600), CTFREE, CNFREE
      COMMON /NEWCAM/ CAMMID(600)
      COMMON /DESIGN/ CAMBER(600)
      COMMON /CYCLE/ NEWAIC, IPREP, NDRAG1, NDRAG2, IDIR
      COMMON /STAR/XSTAR(3600),NSEGW
      COMMON /SHELL/ NPTSB,NSEGWT,NPTSWT,NSEGN,NPTSN,NSEGS,NPTSS,       &
     &               NSEGNS,NPTSNS,NNAC,NSHELL,NQ,IPART,IIDUM
!*
      COMMON /POLAR/NCASE,CDBP(10),CLBP(10),CMBP(10),CDWP(10),CLWP(10), &
     &CMWP(10),ALPHAP(10),MACHP(10),NSPAN,XSPAN(20)
!
!!!      DIMENSION CARD(14)
      CHARACTER(LEN=80):: card
!
!!!      EQUIVALENCE (BLOCK(1),CARD(1))
!
      REAL MACH
      INTEGER PLOT   !!!,TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8
!!!      INTEGER TAPE9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15,TAPE16
!!!      INTEGER TAPE17
      INTEGER:: FFILE,TAPE20,TAPE21,TAPE22,TAPE23,TAPE24,TAPE25
      INTEGER TAPE26,TAPE27,TAPE28,TAPE29,TAPE30
!*
      NAMELIST/AEROIN/NOPT,NXSTNS,XSTN,XAPEX,YAPEX,XLAMDA
!*
!
!     INITIALIZE CONSTANTS IN COMMON
!
!
!     NOTE THAT COMMON /FILES/ CONTAINS THE FORTRAN INPUT/OUTPUT UNIT
!     NUMBERS USED IN THE PROGRAM. TAPE1, TAPE2, TAPE4, AND TAPE15 ARE
!     NOT USED AT PRESENT. THE PROGRAM READS INPUT FROM TAPE5 AND WRITES
!     OUTPUT ON TAPE6. THE TEMPORARY DISK FILES ARE TAPE3, TAPE7, TAPE8,
!     TAPE9, TAPE10, TAPE11, TAPE12, TAPE13, AND TAPE14.
!
!
!     NOTE THAT COMMON /EPS/ CONTAINS EPSILON TOLERANCES WHICH ARE USED
!     BY VARIOUS ROUTINES THROUGHOUT THE PROGRAM. THESE EPSILONS WERE
!     CHOSEN FOR THE CDC6600.
!
      INTEGER:: errCode
      CHARACTER(LEN=80):: fileName

       CHARACTER(LEN=*),PARAMETER:: HEADER_LINE_1 = &
         "00000000011111111112222222222333333333344444444445555555555666666666677777777778"
       CHARACTER(LEN=*),PARAMETER:: HEADER_LINE_2 = &
         "12345678901234567890123456789012345678901234567890123456789012345678901234567890"
!----------------------------------------------------------------------------
  WRITE(*,*) 'w12sc3'
  DO
    WRITE(*,*) 'Enter the name of the input file:'
    READ(*,'(A)') fileName
    IF (LEN_TRIM(fileName)==0) STOP
    OPEN(UNIT=TAPE5,FILE=fileName,STATUS='OLD',IOSTAT=errCode,ACTION='READ')
    IF (errCode==0) EXIT
    OPEN(UNIT=TAPE5,FILE=TRIM(fileName)//'.inp', &
      STATUS='OLD',IOSTAT=errCode,ACTION='READ')
    IF (errCode==0) EXIT
    WRITE(*,*) 'Unable to open this file. Try again.'
  END DO
  OPEN(UNIT=TAPE6,FILE='w12sc3.out',STATUS='REPLACE',ACTION='WRITE')
  OPEN(UNIT=TAPE7,FILE='w12sc3tape7.out', &
    FORM='UNFORMATTED', STATUS='REPLACE',ACTION='READWRITE')
  OPEN(UNIT=TAPE9,FILE='w12sc3tape9.out', &
    FORM='UNFORMATTED', STATUS='REPLACE',ACTION='READWRITE')
  OPEN(UNIT=TAPE10,FILE='w12sc3tape10.out', &
    FORM='UNFORMATTED', STATUS='REPLACE',ACTION='READWRITE')
  OPEN(UNIT=TAPE11,FILE='w12sc3tape11.out', &
    FORM='UNFORMATTED', STATUS='REPLACE',ACTION='READWRITE')
  OPEN(UNIT=32,FILE='w12sc3tape32.out', &
    FORM='UNFORMATTED', STATUS='REPLACE',ACTION='READWRITE')
  OPEN(UNIT=33,FILE='w12sc3tape33.out', &
    FORM='UNFORMATTED', STATUS='REPLACE',ACTION='READWRITE')

      NCASE=0
      KPLOT=0
      camber(:)=0.0
      em=-1.0
!
!     INITIALIZE PLOTTING PROCESSOR
!
      CALL PLOTS(IBUF,NLOC,LDEV)
!
      WRITE(TAPE6,*) "   GRUMMAN W12/SC3 PROGRAM                    SEPT 1980"
      WRITE(TAPE6,*) " "
      WRITE(TAPE6,*) "LIST OF INPUT CARDS"
      WRITE(TAPE6,*) HEADER_LINE_1
      WRITE(TAPE6,*) HEADER_LINE_2

!
!     LIST INPUT CARDS
!
   10 CONTINUE
      READ(TAPE5,'(A)',IOSTAT=errCode) card
      IF (errCode < 0) GO TO 30   ! RLC
      WRITE (TAPE6,*) card
      GO TO 10

   30 CONTINUE
      WRITE(TAPE6,*) " "
      WRITE(TAPE6,*) " "
      WRITE(TAPE6,*) HEADER_LINE_1
      WRITE(TAPE6,*) HEADER_LINE_2

      REWIND(UNIT=TAPE5)
   40 CONTINUE
      WRITE(TAPE6,90)
      PLOT=0
!
!     INPUT CONFIGURATION GEOMETRY AND COMPUTE PANELS
!
!!!      CALL OVERLAY (LWB,1,0)
      CALL Geom()
      NEWAIC = 0
      IF (K2.EQ.0) GO TO 50
      READ (TAPE10) ARRAY
      WRITE (TAPE7) ARRAY
      REWIND TAPE7
      REWIND TAPE10
   50 CONTINUE
      REWIND TAPE3
      REWIND TAPE8
      REWIND TAPE13
      PLOT=0
      IPLOT=PLOT
      IF (PLOT) 60,70,60
   60 CONTINUE
      KONPLT=1
!!!      CALL OVERLAY (LWB,4,0)
      CALL ControlPlot()
!
!     INPUT MACH NUMBER AND COMPUTE AERODYNAMIC MATRIX
!
   70 CONTINUE   ! keep coming back here after calculations are complete
!*
!
!     READ PROGRAM OPTIONS
!
!
!     PROGRAM TERMINATES IF
!
!     NAMELIST NOT ENCOUNTERED ( END OF FILE )
!
!     OR
!
!     NOPT(1) ) 0
!
!
      READ(TAPE5,AEROIN,IOSTAT=errCode)
      IF (errCode < 0) GO TO 85
!!!      IF(EOF(TAPE5).NE.0) GO TO 85
      IF(NOPT(1).LT.0) GO TO 85
      IF(NOPT(4).EQ.2) GO TO 320
      IF(NOPT(1).EQ.1.AND.NOPT(2).EQ.0.AND.NOPT(6).EQ.0.AND.K6.EQ.0)    &
     & GO TO 320
      NOPT(4)=2
      WRITE(TAPE6,300)
      WRITE(TAPE6,310)
  300 FORMAT(1H1,//)
  310 FORMAT('  DUE TO CHOICE OF NOPT(1), NOPT(2), NOPT(6), AND/OR K6,'//  &
     & ' W12SC3 HAS SET NOPT(4) = 2,',                             &
     & ' AND WILL USE WOODWARD I TYPE VORTEX SINGULARITIES.')
  320 CONTINUE
      WRITE(TAPE6,130)
      WRITE(TAPE6,AEROIN)
      WRITE(TAPE6,130)
      XLAMDA=XLAMDA*DEGRAD
      NSPAN=0
      IF(NOPT(8).NE.0.AND.KPLOT.EQ.0) CALL PLOTS(0,0,0)
      IF(NOPT(8).NE.0) KPLOT=1
      IF(NXSTNS.EQ.0) GO TO 210
      NSPAN=NXSTNS
      DO 200 I=1,NSPAN
  200 XSPAN(I)=XSTN(I)
  210 CONTINUE
!**
!**
!!!      CALL OVERLAY (LWB,2,0)
      CALL VelCmp()
!
!     A NEGATIVE MACH NUMBER IS USED TO TERMINATE MACH NUMBER AND ANGLE
!     OF ATTACK CASES FOR A GIVEN GEOMETRY
!
      IF (MACH.LT.0.) GO TO 40
!*
!!!      DO 75 I = 1, NWING
!!!   75 IFREE(I) = 1
      ifree(1:nwing)=1
!
!     CALCULATE CAMBER FOR MINIMUM DRAG
!
      CALL WINGS(1)
      IF (NPTSNS.NE.0) CALL WINGS(3)
!!!      CALL OVERLAY(LWB,5,0)
      CALL Olay50()
!*
      WRITE (TAPE6,100)
!
!     SOLVE RESULTING MATRIX EQUATIONS AND
!     COMPUTE PRESSURES, FORCES, AND MOMENTS
!
!!!      CALL OVERLAY (LWB,3,0)
      CALL Solve()
!*
!*
   86 CONTINUE
      IF (NOPT(8).EQ.0) GO TO 76
      CALL SMPLOT
   76 CONTINUE
      IF (IPLOT) 80,70,80
   80 CONTINUE
      KONPLT=1
!!!      CALL OVERLAY (LWB,4,0)
      CALL ControlPlot()
      GO TO 70
!
   85 CONTINUE
      IF (KPLOT.NE.0) CALL PLOTE
      STOP
!
   90 FORMAT (1H1,20X,25HBEGIN A NEW CONFIGURATION)
  100 FORMAT (1H1,20X,16HBEGIN A NEW CASE)
  110 FORMAT(1H1,10X,23HGRUMMAN W12/SC3 PROGRAM ,35X,                   &
     & 9HSEPT 1980   //)
  120 FORMAT (10X,80H000000000111111111122222222223333333333444444444455&
     &55555555666666666677777777778/10X,80H12345678901234567890123456789&
     &012345678901234567890123456789012345678901234567890//)
  130 FORMAT (//)
  140 FORMAT (/25X,'LIST OF INPUT CARDS'//)

  170 FORMAT(10I5)
!*
      END Program W12SC3   ! ================================================

      BLOCK DATA
!!!      COMMON /OPTION/ NOPT(10),NXSTNS,XSTN(20),XAPEX,YAPEX,XLAMDA
      INCLUDE 'option.cmn'
      COMMON /PTYPE/ORT,VU3,STE,PER
      COMMON /PI/PI,RADEG,DEGRAD
!!!      COMMON /LWB/LWB
      COMMON /EPS/EPSPAN,EPSWP1,EPSWP2,EPSTP,EPSVC,EPSSP1,EPSSP2,        &
     & EPSLV,EPSSV,EPSVV,EPSWV,EPSVP,EPSIT1,EPSIT2,EPSFM,EPSSB1,        &
     & EPSSB2
!!!      COMMON /DESIGN/CAMBER(600)
      COMMON /TAPES/ITAPES(20)
!!!      COMMON /VELCOM/N(5),EM,L(54),DUMMY(20)
      COMMON /ONE/NAN2
!!!      COMMON /FILES/TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,     &
!!!     & TAPE9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15,TAPE16,TAPE17
      INCLUDE 'files.cmn'

!!!      INTEGER TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,          &
!!!     & TAPE9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15,TAPE16,TAPE17
!
      DATA PI/3.1415926535898/,RADEG/57.29577951/,DEGRAD/.0174532925/

!!! can't do this anymore ....
!!!      DATA ORT/3HORT/,VU3/3HVU3/,STE/3HSTE/,PER/3HPER/
!!!
!!!      DATA LWB/3HLWB/
!!! EM/-1.0/
      DATA NAN2/24/
!!!      DATA CAMBER/600*0.0/
      DATA ITAPES/20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,      &
     &            36,37,38,39/
      DATA NOPT/1,9*0/,NXSTNS/0/,XSTN/20*0.0/,XAPEX/0.0/,YAPEX/0.0/,    &
     & XLAMDA/57.0/
      DATA EPSPAN/1.E-6/,EPSWP1/1.E-6/,EPSWP2/1.E-3/,EPSTP/1.E-6/,      &
     & EPSVC/1.E-3/,EPSSP1/1.E-5/,EPSSP2/1.E-10/,EPSLV/1.E-6/,          &
     & EPSSV/1.E-6/,EPSVV/1.E-6/,EPSWV/1.E-6/,EPSVP/1.E-6/,             &
     & EPSIT1/1.E-3/,EPSIT2/1.E3/,EPSFM/1.E-6/,EPSSB1/1.E-5/,           &
     & EPSSB2/1.E5/
      DATA TAPE1/0/,TAPE2/0/,TAPE3/8/,TAPE4/0/,TAPE5/5/,TAPE6/6/,       &
     & TAPE7/7/,TAPE8/8/,TAPE9/9/,TAPE10/10/,TAPE11/11/,TAPE12/12/,     &
     & TAPE13/13/,TAPE14/14/,TAPE15/15/,TAPE16/16/,TAPE17/17/
!
      END

       SUBROUTINE SMPLOT
!***********************************************************************
!*                                                                     *
!*        THIS ROUTINE IS FOR THE W12SC3 CDC BATCH SYSTEM.             *
!*        THIS PROGRAM IS CAPABLE OF PLOTTING PRESSURE                 *
!*        COEFFICIENTS IN THE SPANWISE + CHORDWISE DIRECTION           *
!*        FOR ISOLATED WINGS AND BODIES AS WELL AS WING/BODY           *
!*        COMBINATIONS. IT CAN ALSO PLOT FORCE + MOMENT                *
!*        COEFFICIENTS.                                                *
!*                                                                     *
!*        THE OPTIONS ARE DEFINED AS FOLLOWS%                          *
!*                                                                     *
!*                WING-CHORD  WING-SPAN  WING-BODY FORCES AND          *
!*        NOPT(8) BODY-RADIAL BODY-AXIAL SPANWISE   MOMENTS            *
!*        ******* *********** ********** ********* **********          *
!*                                                                     *
!*             0       NO         NO         NO        NO              *
!*             1      YES         NO         NO        NO              *
!*            10       NO        YES         NO        NO              *
!*            11      YES        YES         NO        NO              *
!*           100       NO         NO        YES        NO              *
!*           101      YES         NO        YES        NO              *
!*           110       NO        YES        YES        NO              *
!*           111      YES        YES        YES        NO              *
!*          1000       NO         NO         NO       YES              *
!*          1001      YES         NO         NO       YES              *
!*          1010       NO        YES         NO       YES              *
!*          1011      YES        YES         NO       YES              *
!*          1100       NO         NO        YES       YES              *
!*          1101      YES         NO        YES       YES              *
!*          1110       NO        YES        YES       YES              *
!*          1111      YES        YES        YES       YES              *
!*                                                                     *
!*                          CODE WRITTEN BY JOE LAIOSA  10/80          *
!*                          CODE REVISED BY BRUCE ROSEN  8/81          *
!*                                                                     *
!*                                                                     *
!***********************************************************************
!
!
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAP&
!!!     &E9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15,TAPE16,TAPE17
      INCLUDE 'files.cmn'
      COMMON /PARAM/ NBODY,NWING,NTAIL,LBC,THK,MACH,ALPHA,REFA,REFB,REFC&
     &,REFD,REFL,REFX,REFZ
      COMMON /BLANK/ J0,J1,J2,J3,J4,J5,J6,NWAF,NWAFOR,NFUS,NRADX(4),NFOR&
     &X(4),NP,NPODOR,NF,NFINOR,NCAN,NCANOR,J2TEST,NW,NC
      COMMON /VELCOM/ NPOINT,NPART,IMAX,JMAX,NMAX,EM,PRINT,NWTHK,NWBLOK,&
     &NWROW(20),NBBLOK,NBROW(60)
      COMMON /NEWCOM/K0,K1,K2,K3,K4,K5,K6,KWAF,KWAFOR,KFUS,KRADX(4),     &
     &              KFORX(4),KF(6),KFINOR(6),KAN(6),KANOR(6),KRAD,MAX,  &
     &              KOL,NCPT,LOCPT(20),XCPT(20)
      COMMON /SEG/ NSEG,NROW(20),NCOL(20),COSS(20),SINS(20),BT(20),DUM(6&
     &0),BL(20)
      COMMON /MATCOM/ MATIN
!!!      COMMON /OPTION/ NOPT(10)
      INCLUDE 'option.cmn'
      COMMON /POLAR/NCASE,CDBP(10),CLBP(10),CMBP(10),CDWP(10),CLWP(10), &
     &CMWP(10),ALPHAP(10),MACHP(10),NSPAN,XSPAN(20)
       COMMON /PI/ PI,RADEG,DEGRAD
!!!      COMMON /HEAD/TITLE1(14),TITLE2(14)
      INCLUDE 'head.cmn'
      COMMON /JSCALE/XSCALE,YSCALE,TSCALE
      COMMON /SHELL/ NPTSB,NSEGWT,NPTSWT,NSEGN,NPTSN,NSEGS,NPTSS,       &
     &               NSEGNS,NPTSNS,NNAC,NSHELL,NQ,IPART,iidum
!
      DIMENSION CP(62),XP(32),YP(62),ZP(32),XQ(32),YQ(32),ZQ(32),CPSP(62&
     &),THETA(32),BUFFER(512),R(22),JSP(32),XQA(32),YSP(32),JM(32),A(30)&
     &,YS(62),CPSAVE(32),IS(32),CL(12),CM(12),CD(12),B(30),ALP(10),     &
     &ZSP(32),JKAN(6),JKANOR(6)
      REAL MACH
      LOGICAL LBC,SUB,SUPLE,SUPTE,THK
!!!      INTEGER TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAPE9,TAPE&
!!!     &10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15,FFILE,TAPE16,TAPE17
      INTEGER::COMPT
!
      REWIND TAPE15
      REWIND TAPE16
      REWIND TAPE17
      CALL WINGS(1)
      C=3.5
      D=3.5
      CPM1=.3
      CPM2=-.4
      YSCALE=10.
      XSCALE=5.
      ASCALE=.5
      TSCALE=1./36.
      TOL=1.E-6
      KWP=0
      NCP=0
      NWD1=0
      NWD2=0
      NCD1=0
      NCD2=0
      IF(NWING.EQ.0) GO TO 6
      IF(J1.EQ.0) GO TO 8
      NWD1=NWAF-1
      NWD2=IABS(NWAFOR)-1
      IF(KWAF.NE.0) NWD1=KWAF-1
      IF(KWAFOR.NE.0) NWD2=KWAFOR-1
      KWP=NWD1*NWD2
    8 CONTINUE
      IF(J5.EQ.0) GO TO 7
      NCD1=NCAN
      NCD2=IABS(NCANOR)-1
      IF(K5.EQ.0) GO TO 7
      NS1=0
      NS2=0
      DO 9 I=1,NCAN
      JKAN(I)=NCD1
      JKANOR(I)=NCD2
      IF(IABS(KAN(I)).NE.0) JKAN(I)=IABS(KAN(I))-1
      IF(KANOR(I).NE.0) JKANOR(I)=KANOR(I)-1
      NS1=NS1+JKAN(I)
      NS2=NS2+JKANOR(I)
    9 END DO
      NCD1=NS1
      NCD2=NS2
      NCP=NCD1*NCD2
    7 CONTINUE
      NWP=NCP+KWP
    6 CONTINUE
      CALL PLOT(3.,0.,-3)
      IF((NOPT(8)+1)/2*2.NE.NOPT(8)+1) GO TO 410
      IF(NBODY.EQ.0) GO TO 410
      READ (TAPE15) NP,NPASS,COMPT
!*******     BODY PLOTTING SECTION  ***********
      DO 321 IF=1,KFUS
      NMAXB=NFORX(IF)-1
      IF(KFORX(IF).NE.0) NMAXB=KFORX(IF)-1
      MMAXB=NRADX(IF)-1
      IF(IABS(KRADX(IF)).NE.0) MMAXB=IABS(KRADX(IF))-1
      NB=NMAXB*MMAXB
      DO 25 J=1,NMAXB
      IE=0
      DO 26 I=1,MMAXB
      READ(TAPE15) XP(I),YP(I),ZP(I),CP(I),XQ(I),YQ(I),ZQ(I)
      IF(CP(I).GT.CPM1)CP(I)=CPM1
      IF(CP(I).LT.CPM2)CP(I)=CPM2
      IPASS=0
      IF(CP(I).EQ.CPM1.OR.CP(I).EQ.CPM2) IPASS=1
      IF(IPASS.EQ.0) GO TO 160
      IE=IE+1
      A(IE)=(PI/2.+ATAN2(ZP(I),YP(I)))*TSCALE*RADEG
      B(IE)=CP(I)*(-YSCALE)
  160 CONTINUE
   26 END DO
      CP(MMAXB+1)=0.0
      CP(MMAXB+2)=-1./YSCALE
      THETA(MMAXB+1)=0.0
      THETA(MMAXB+2)=1./TSCALE
      DO 10 K=1,MMAXB
      THETA(K)= PI/2.+ATAN2(ZP(K),YP(K))
   10 THETA(K)=THETA(K)*RADEG
      CALL AXIS(1.,1.,2HCP,2,7.,90.,0.3,-0.1)
      CALL AXIS(1.5,4.,5HTHETA,-5,5.,0.,0.,36.)
      CALL SYMBOL(1.,8.8,.15,34H BODY RADIAL PRESSURE DISTRIBUTION,0.,  &
     &34)
!     CALL JGRID(0.,0.,0.2,-0.1,5.,7.,6,8)
      CALL W12HED(NMAXB,MMAXB,NB,XP(1),1)
      CALL LORGN(1.5,4.)
      CALL LINE (THETA,CP,MMAXB,1,1,3)
      IF(IE.EQ.0) GO TO 161
      DO 190 IP=1,IE
      XX=A(IP)
      YY=B(IP)
      CALL SYMBOL(XX,YY,.1,0,0.,-1)
  190 END DO
      CALL SYMBOL(C,D,.1,0,0.,-1)
      CALL SYMBOL(3.75,3.45,.1,9HOFF SCALE,0.,9)
  161 CONTINUE
      CALL LORGN (-1.5,-4.)
      CALL PLOT(8.5,0.,-3)
   25 END DO
   30 CONTINUE
  321 END DO
      REWIND TAPE15
  410 CONTINUE
      IF((NOPT(8)/10+1)/2*2.NE.NOPT(8)/10+1) GO TO 50
      IF(NBODY.EQ.0) GO TO 50
!**** BODY AXIAL PLOTS *******
      DO 322 IZ=1,KFUS
      NMAXB=NFORX(IZ)-1
      IF(KFORX(IZ).NE.0) NMAXB=KFORX(IZ)-1
      MMAXB=NRADX(IZ)-1
      IF(IABS(KRADX(IZ)).NE.0) MMAXB=IABS(KRADX(IZ))-1
      NB=NMAXB*MMAXB
      DO 320 I=1,MMAXB
      REWIND TAPE15
      READ(TAPE15)ND1,ND2,ND3
      IE=0
      DO 305 J=1,NMAXB
      DO 310 L=1,MMAXB
      READ(TAPE15)XP(L),YP(L),ZP(L),CP(L),XQ(L),YQ(L),ZQ(L)
  310 END DO
      CPSP(J)=CP(I)
      XQA(J)=XQ(I)
      IF(CPSP(J).GT.CPM1)CPSP(J)=CPM1
      IF(CPSP(J).LT.CPM2)CPSP(J)=CPM2
      IPASS=0
      IF(CPSP(J).EQ.CPM1.OR.CPSP(J).EQ.CPM2) IPASS=1
      IF(IPASS.EQ.0) GO TO 163
      IE=IE+1
      A(IE)=XQA(J)*XSCALE
      B(IE)=CPSP(J)*(-YSCALE)
  163 CONTINUE
      THET=(PI/2. + ATAN2(ZP(I),YP(I)))*RADEG
  305 END DO
      CPSP(NMAXB+1)=0.0
      CPSP(NMAXB+2)=-1./YSCALE
      XQA(NMAXB+1)=0.0
      XQA(NMAXB+2)=1./XSCALE
      CALL AXIS(1.,1.,2HCP,2,7.,90.,0.3,-0.1)
      CALL AXIS(1.5,4.,3HX/L,-3,5.,0.,0.,0.2)
      CALL SYMBOL(1.,8.8,.15,33H BODY AXIAL PRESSURE DISTRIBUTION,0.,   &
     &33)
!     CALL JGRID(0.,0.,0.2,-0.1,5.,7.,6,8)
      CALL W12HED(NMAXB,MMAXB,NB,THET,3)
      CALL LORGN (1.5,4.)
      CALL LINE (XQA,CPSP,NMAXB,1,1,3)
      IF(IE.EQ.0) GO TO 164
      DO 191 IP=1,IE
      XX=A(IP)
      YY=B(IP)
      CALL SYMBOL(XX,YY,.1,0,0.,-1)
  191 END DO
      CALL SYMBOL(C,D,.1,0,0.,-1)
      CALL SYMBOL(3.75,3.45,.1,9HOFF SCALE,0.,9)
  164 CONTINUE
      CALL LORGN (-1.5,-4.)
      CALL PLOT(8.5,0.,-3)
  320 END DO
  322 END DO
   50 CONTINUE
      REWIND TAPE15
      IF((NOPT(8)+1)/2*2.NE.NOPT(8)+1) GO TO 102
      IF(NWING.EQ.0) GO TO 102
!***********    WING PLOTTING SECTION   ************
!************    WING CHORDWISE PLOTS    **********
      READ(TAPE16) ND1,ND2,ND3
      READ(TAPE17) ND1,ND2,ND3
      IPT=0
      ICAN=1
      NPLUS=NWD1
      DO 101 IN=1,NSEG
      NMAXW=NCOL(IN)
      MMAXW=NROW(IN)
      IF(LOCPT(IN).NE.0) MMAXW=MMAXW+1
      DO 100 J=1,NMAXW
      IPT=IPT+1
      CALL AXIS(1.5,4.,3HX/C,-3,5.,0.,0.,0.2)
      CALL AXIS(1.,1.,2HCP,2,7.,90.,0.3,-0.1)
      CALL SYMBOL(1.,8.8,.15,37H WING CHORDWISE PRESSURE DISTRIBUTION,  &
     &0.,37)
!     CALL JGRID(0.,0.,.2,-0.1,5.,7.,6,8)
      CALL LORGN(1.5,4.)
      IE=0
      DO 27 I=1,MMAXW
      READ(TAPE16)  XP(I),YP(I),XQ(I),CP(I),ZP(I)
      IF(CP(I).GT.CPM1) CP(I)=CPM1
      IF(CP(I).LT.CPM2) CP(I)=CPM2
      IPASS=0
      IF(CP(I).EQ.CPM1.OR.CP(I).EQ.CPM2) IPASS=1
      IF(IPASS.EQ.0) GO TO 165
      IE=IE+1
      A(IE)=XQ(I)*XSCALE
      B(IE)=CP(I)*(-YSCALE)
  165 CONTINUE
   27 END DO
      CP(MMAXW+1)=0.0
      CP(MMAXW+2)=-1./YSCALE
      XQ(MMAXW+1)=0.0
      XQ(MMAXW+2)=1./XSCALE
   60 CALL LINE(XQ,CP,MMAXW,1,1,3)
      IDONE=0
      IF(IE.EQ.0) GO TO 162
      DO 192 IP=1,IE
      XX=A(IP)
      YY=B(IP)
      CALL SYMBOL(XX,YY,.1,0,0.,-1)
  192 END DO
      CALL SYMBOL(C,D,.1,0,0.,-1)
      CALL SYMBOL(3.75,3.45,.1,9HOFF SCALE,0.,9)
      IDONE=1
  162 CONTINUE
      IE=0
      DO 28 I=1,MMAXW
      READ(TAPE17)  XP(I),YP(I),XQ(I),CP(I),ZP(I)
      IF(CP(I).GT.CPM1) CP(I)=CPM1
      IF(CP(I).LT.CPM2) CP(I)=CPM2
      IPASS=0
      IF(CP(I).EQ.CPM1.OR.CP(I).EQ.CPM2) IPASS=1
      IF(IPASS.EQ.0) GO TO 202
      IE=IE+1
      A(IE)=XQ(I)*XSCALE
      B(IE)=CP(I)*(-YSCALE)
  202 CONTINUE
   28 END DO
      CP(MMAXW+1)=0.0
      CP(MMAXW+2)=-1./YSCALE
      XQ(MMAXW+1)=0.0
      XQ(MMAXW+2)=1./XSCALE
   70 CALL LINE (XQ,CP,MMAXW,1,1,4)
      IF(IE.EQ.0) GO TO 203
      DO 199 IP=1,IE
      XX=A(IP)
      YY=B(IP)
      CALL SYMBOL(XX,YY,.1,0,0.,-1)
  199 END DO
      IF(IDONE.EQ.1) GO TO 203
      CALL SYMBOL(C,D,.1,0,0.,-1)
      CALL SYMBOL(3.75,3.45,.1,9HOFF SCALE,0.,9)
  203 CONTINUE
      CALL LORGN(-1.5,-4.)
      NPP=NWD1
      MPP=NWD2
      NWT=KWP
      IF(IPT.LE.NWD1) CALL SYMBOL(3.25,8.15,.1,4HWING,0.,4)
      IF(NCAN.EQ.0) GO TO 106
      IF(IPT.LE.NWD1) GO TO 106
      IF((IPT-NPLUS).LE.JKAN(ICAN)) GO TO 108
      NPLUS=NPLUS+JKAN(ICAN)
      ICAN=ICAN+1
  108 CONTINUE
      TN=ICAN
      MPP=JKANOR(ICAN)
      NPP=JKAN(ICAN)
      NWT=MPP*NPP
      CALL SYMBOL(3.0,8.15,.1,10HCANARD NO.,0.,10)
      CALL NUMBER(4.1,8.15,.1,TN,0.,-1)
  106 CONTINUE
      CALL W12HED(NPP,MPP,NWT,YP(1),2)
      CALL PLOT(8.5,0.,-3)
  100 END DO
  101 END DO
  102 CONTINUE
      REWIND TAPE16
      REWIND TAPE17
      IF((NOPT(8)/10+1)/2*2.NE.NOPT(8)/10+1) GO TO 200
      IF(NWING.EQ.0) GO TO 200
      IF(NSPAN.EQ.0) GO TO 200
!***** WING SPANWISE PLOTS/CROSS PLOT *****
      DO 355 L=1,NSPAN
      JJ=0
      JPLACE=0
      REWIND TAPE16
      REWIND TAPE17
      READ(TAPE16) ND1,ND2,ND3
      READ(TAPE17) ND1,ND2,ND3
      IPT=0
      DO 340 IN=1,NSEG
      JS=0
      NMAXW=NCOL(IN)
      MMAXW=NROW(IN)
      IF(LOCPT(IN).NE.0) MMAXW=MMAXW+1
      DO 340 I=1,NMAXW
      IF(IN.GT.NSEGWT) GO TO 340
      JS=0
      DO 331 II=1,MMAXW
      READ(TAPE16) XP(II),YP(II),XQ(II),CP(II),ZP(II)
  331 END DO
      ID=MMAXW/2
      DIFF=ABS(XP(ID)-XP(ID-1))*2.0
      DO 335 J=1,MMAXW
      IPT=IPT+1
      DELN=ABS(XP(J)-XSPAN(L))
      IF(J.EQ.1)DELO=DELN
      IF(J.EQ.1) GO TO 330
      IF(DELN.GT.DELO) GO TO 330
      DELO=DELN
      JS=J
  330 CONTINUE
  335 END DO
      IF(JS.EQ.0) JS=1
      IF(DELO.GT.DIFF) GO TO 44
      JJ=JJ+1
      JSP(JJ)=JS
      IS(JJ)=I
      CPSP(JJ)=CP(JS)
      YSP(JJ)=YP(JS)
   44 CONTINUE
  340 CONTINUE
      IF(JJ.EQ.0)GO TO 355
      CALL AXIS(1.,1.,2HCP,2,7.,90.,0.3,-0.1)
      CALL AXIS(1.5,4.,4H2Y/B,-4,5.,0.,0.,.2)
      CALL SYMBOL(1.,8.8,.15,36H WING SPANWISE PRESSURE DISTRIBUTION,   &
     &0.,36)
      MW=NWD2+NCD2
      NWW=NWD1+NCD1
      NWT=KWP
!     CALL JGRID(0.,0.,0.2,-0.1,5.,7.,6,8)
      CALL W12HED(NWW,MW,NWT,XSPAN(L),1)
      CALL LORGN (1.5,4.)
      IE=0
      DO 550 J=1,JJ
      YS(J)=YSP(J)
      YSP(J)=YSP(J)/REFB
      IF(CPSP(J).GT.CPM1) CPSP(J)=CPM1
      IF(CPSP(J).LT.CPM2) CPSP(J)=CPM2
      IPASS=0
      IF(CPSP(J).EQ.CPM1.OR.CPSP(J).EQ.CPM2) IPASS=1
      IF(IPASS.EQ.0) GO TO 166
      IE=IE+1
      A(IE)=YSP(J)*XSCALE
      B(IE)=CPSP(J)*(-YSCALE)
  166 CONTINUE
  500 FORMAT(1H1)
  501 FORMAT(5X,23HWING SPANWISE PRESSURES,2X,23HAT AXIAL POSITION OF X=&
     &,F10.5,//)
  502 FORMAT(5X,13HUPPER SURFACE,/)
  503 FORMAT(26X,1HY,18X,4H2Y/B,17X,2HCP,/)
  504 FORMAT(10X,3F20.5)
  550 END DO
      CPSP(JJ+1)=0.0
      CPSP(JJ+2)=-1./YSCALE
      YSP(JJ+1)=0.0
      YSP(JJ+2)=1./XSCALE
      CALL LINE (YSP,CPSP,JJ,1,1,3)
      IDONE=0
      IF(IE.EQ.0) GO TO 141
      DO 140 IP=1,IE
      XX=A(IP)
      YY=B(IP)
      CALL SYMBOL(XX,YY,.1,0,0.,-1)
  140 END DO
      CALL SYMBOL(C,D,.1,0,0.,-1)
      CALL SYMBOL(3.75,3.45,.1,9HOFF SCALE,0.,9)
  141 CONTINUE
      IB=1
      DO 62 IN=1,NSEG
      NMAXW=NCOL(IN)
      MMAXW=NROW(IN)
      IF(LOCPT(IN).NE.0) MMAXW=MMAXW+1
      DO 62 JK=1,NMAXW
      IF(IN.GT.NSEGWT) GO TO 62
      DO 63 JB=1,MMAXW
      READ(TAPE17) XP(JBB),YP(JB),XQ(JB),CP(JB),ZP(JB)   ! subscript on ZP was H
   63 END DO
      IF(IB.GT.JJ) GO TO 47
      IF(JK.NE.IS(IB)) GO TO 47
      CPSP(IB)=CP(JSP(IB))
      IB=IB+1
   47 CONTINUE
   62 CONTINUE
      IE=0
      DO 551 J=1,JJ
      IF(CPSP(J).GT.CPM1) CPSP(J)=CPM1
      IF(CPSP(J).LT.CPM2) CPSP(J)=CPM2
      IPASS=0
      IF(CPSP(J).EQ.CPM1.OR.CPSP(J).EQ.CPM2) IPASS=1
      IF(IPASS.EQ.0) GO TO 167
      IE=IE+1
      A(IE)=YSP(J)*XSCALE
      B(IE)=CPSP(J)*(-YSCALE)
  167 CONTINUE
  505 FORMAT(5X,13HLOWER SURFACE,/)
  551 END DO
      CPSP(JJ+1)=0.0
      CPSP(JJ+2)=-1./YSCALE
      CALL LINE(YSP,CPSP,JJ,1,1,4)
      IF(IE.EQ.0) GO TO 168
      DO 193 IP=1,IE
      XX=A(IP)
      YY=B(IP)
      CALL SYMBOL(XX,YY,.1,0,0.,-1)
  193 END DO
      IF(IDONE.EQ.1) GO TO 168
      CALL SYMBOL(C,D,.1,0,0.,-1)
      CALL SYMBOL(3.75,3.45,.1,9HOFF SCALE,0.,9)
  168 CONTINUE
      CALL LORGN(-1.5,-4.)
      CALL PLOT(8.5,0.,-3)
  355 END DO
  200 CONTINUE
      REWIND TAPE16
      REWIND TAPE17
      IF((NOPT(8)/100+1)/2*2.NE.NOPT(8)/100+1) GO TO 201
      IF(NBODY.EQ.0.OR.NWING.EQ.0) GO TO 201
      IF(NSPAN.EQ.0) GO TO 201
!***** WING/BODY SPANWISE PLOTS/CROSS PLOT *****
      DO 611 L=1,NSPAN
      JJ=0
      REWIND TAPE15
      REWIND TAPE16
      REWIND TAPE17
      READ(TAPE15) ND1,ND2,ND3
      READ(TAPE16) ND1,ND2,ND3
      READ(TAPE17) ND1,ND2,ND3
      DO 701 IF=1,KFUS
      NMAXB=NFORX(IF)-1
      IF(KFORX(IF).NE.0) NMAXB=KFORX(IF)-1
      MMAXB=NRADX(IF)-1
      IF(IABS(KRADX(IF)).NE.0) MMAXB=IABS(KRADX(IF))-1
      DO 701 IB=1,NMAXB
      DO 600 JB=1,MMAXB
      READ(TAPE15) XP(JB),YP(JB),ZP(JB),CP(JB),XQ(JB),YQ(JB),ZQ(JB)
  600 END DO
      DELN=ABS(XP(1)-XSPAN(L))
      IF(IB.EQ.1) DELO=DELN
      IF(IB.EQ.1) GO TO 620
      IF(DELO.LT.DELN) GO TO 620
      MB=MMAXB
      DELO=DELN
      DO 700 K=1,MMAXB
      CPSP(K)=CP(K)
      YSP(K)=YP(K)
      ZSP(K)=ZP(K)
  700 END DO
  620 CONTINUE
  701 CONTINUE
      DO 606 IN=1,NSEG
      NMAXW=NCOL(IN)
      MMAXW=NROW(IN)
      IF(LOCPT(IN).NE.0) MMAXW=MMAXW+1
      DO 606 IK=1,NMAXW
      IF(IN.GT.NSEGWT) GO TO 606
      JS=0
      DO 602 II=1,MMAXW
      READ(TAPE17) XP(II),YP(II),XQ(II),CP(II),ZP(II)
  602 END DO
      ID=MMAXW/2
      DIFF=ABS(XP(ID)-XP(ID-1))*2.0
      DO 604 J=1,MMAXW
      DELN=ABS(XP(J)-XSPAN(L))
      IF(J.EQ.1)DELO=DELN
      IF(J.EQ.1) GO TO 603
      IF(DELN.GT.DELO) GO TO 603
      MW=MMAXW
      NWW=NMAXW
      DELO=DELN
      JS=J
  603 CONTINUE
  604 END DO
      IF(JS.EQ.0) JS=1
      IF(DELO.GT.DIFF) GO TO 605
      JJ=JJ+1
      JSP(JJ)=JS
      IS(JJ)=IK
      CPSAVE(JJ)=CP(JS)
      YS(JJ)=YP(JS)
      IF(IK.EQ.1.AND.IN.EQ.1) ZROOT=ZP(1)
  605 CONTINUE
  606 CONTINUE
      IF(JJ.EQ.0) GO TO 611
      JSAVE=0
      DO 650 I=1,MB
      IF(ZSP(I).LT.ZROOT) GO TO 651
      JSAVE=JSAVE+1
      CP(JSAVE)=CPSP(I)
      YP(JSAVE)=YSP(I)
  651 CONTINUE
  650 END DO
      CALL AXIS(1.,1.,2HCP,2,7.,90.,0.3,-0.1)
      CALL AXIS(1.5,4.,4H2Y/B,-4,5.,0.,0.,.2)
      CALL SYMBOL(1.,8.8,.15,28H WING/BODY SPANWISE PRESSURE,0.,28)
!     CALL JGRID(0.,0.,0.2,-0.1,5.,7.,6,8)
      NPT=NBODY+NWP
      CALL W12HED(NBODY,NWP,NPT,XSPAN(L),-1)
      CALL LORGN (1.5,4.)
      JME=MB-JSAVE
      N=JJ+JME
      IE=0
      ISE=0
      DO 613 K=1,N
      IF(K.LE.JME) GO TO 614
      ISE=ISE+1
      YSP(K)=YS(ISE)
      CPSP(K)=CPSAVE(ISE)
  614 CONTINUE
      YQ(K)=YSP(K)
      YSP(K)=YSP(K)/REFB
      IF(CPSP(K).GT.CPM1) CPSP(K)=CPM1
      IF(CPSP(K).LT.CPM2) CPSP(K)=CPM2
      IPASS=0
      IF(CPSP(K).EQ.CPM1.OR.CPSP(K).EQ.CPM2) IPASS=1
      IF(IPASS.EQ.0) GO TO 169
      IE=IE+1
      A(IE)=YSP(K)*XSCALE
      B(IE)=CPSP(K)*(-YSCALE)
  169 CONTINUE
  506 FORMAT(5X,'WING/BODY SPANWISE PRESSURES',                         &
     & ' AT AXIAL POSITION OF X=',F10.5,/)
  613 END DO
      IDONE=0
      IF(IE.EQ.0) GO TO 170
      DO 194 IP=1,IE
      XX=A(IP)
      YY=B(IP)
      CALL SYMBOL(XX,YY,.1,0,0.,-1)
  194 END DO
      CALL SYMBOL(C,D,.1,0,0.,-1)
      CALL SYMBOL(3.75,3.45,.1,9HOFF SCALE,0.,9)
  170 CONTINUE
      CPSP(N+1)=0.0
      CPSP(N+2)=-1./YSCALE
      YSP(N+1)=0.0
      YSP(N+2)=1./XSCALE
      CALL LINE (YSP,CPSP,N,1,1,4)
      DO 615 J=1,JSAVE
      K=JSAVE-J+1
      CPSP(J)=CP(K)
      YSP(J)=YP(K)
  615 END DO
      IB=1
      DO 609 IN=1,NSEG
      NMAXW=NCOL(IN)
      MMAXW=NROW(IN)
      IF(LOCPT(IN).NE.0) MMAXW=MMAXW+1
      DO 609 JK=1,NMAXW
      IF(IN.GT.NSEGWT) GO TO 609
      DO 619 JB=1,MMAXW
      READ(TAPE16) XP(JB),YP(JB),XQ(JB),CP(JB),ZP(JB)
  619 END DO
      IF(IB.GT.JJ) GO TO 608
      IF(JK.NE.IS(IB)) GO TO 608
      CPSAVE(IB)=CP(JSP(IB))
      YS(IB)=YP(JSP(IB))
      IB=IB+1
  608 CONTINUE
  609 CONTINUE
      NN=MB-JSAVE
      M=JSAVE+JJ
      DO 801 I=1,JJ
      CPSP(JSAVE+I)=CPSAVE(I)
      YSP(JSAVE+I)=YS(I)
  801 END DO
      IE=0
      DO 640 J=1,M
      YS(J)=YSP(J)
      YSP(J)=YSP(J)/REFB
      IF(CPSP(J).GT.CPM1) CPSP(J)=CPM1
      IF(CPSP(J).LT.CPM2) CPSP(J)=CPM2
      IPASS=0
      IF(CPSP(J).EQ.CPM1.OR.CPSP(J).EQ.CPM2) IPASS=1
      IF(IPASS.EQ.0) GO TO 171
      IE=IE+1
      A(IE)=YSP(J)*XSCALE
      B(IE)=CPSP(J)*(-YSCALE)
  171 CONTINUE
  640 END DO
      CPSP(M+1)=0.0
      CPSP(M+2)=-1./YSCALE
      YSP(M+1)=0.0
      YSP(M+2)=1./XSCALE
      CALL LINE(YSP,CPSP,M,1,1,3)
      IF(IE.EQ.0) GO TO 172
      DO 195 IP=1,IE
      XX=A(IP)
      YY=B(IP)
      CALL SYMBOL(XX,YY,.1,0,0.,-1)
  195 END DO
      IF(IDONE.EQ.1) GO TO 172
      CALL SYMBOL(C,D,.1,0,0.,-1)
      CALL SYMBOL(3.75,3.45,.1,9HOFF SCALE,0.,9)
  172 CONTINUE
      CALL LORGN(-1.5,-4.)
      CALL PLOT(8.5,0.,-3)
  611 END DO
  201 CONTINUE
      REWIND TAPE15
      REWIND TAPE16
      REWIND TAPE17
      IF((NOPT(8)/1000+1)/2*2.NE.NOPT(8)/1000+1) GO TO 300
!*********   FORCE + MOMENT PLOTS *******
      ICL=0
      ICM=0
      ICD=0
      BEG1=-0.1
      BEG3=0.1
      BEG2=.0
      DEL1=.1
      DEL2=.01
      DEL3=-.1
      JSIGN=4
      IF(NBODY.EQ.0) GO TO 78
      NMAXB=0
      MMAXB=0
      DO 79 I=1,NFUS
      NT2=NFORX(I)-1
      IF(KFORX(I).NE.0) NT2=KFORX(I)-1
      NT1=NRADX(I)-1
      IF(IABS(KRADX(I)).NE.0) NT1=IABS(KRADX(I))-1
      MMAXB=MMAXB+NT1
      NMAXB=NMAXB+NT2
   79 END DO
      IF(NWING.EQ.0) NPT=NBODY
   78 CONTINUE
      DO 77 I=1,NCASE
      ALP(I)=ALPHAP(I)
   77 END DO
      ALP(NCASE+1)=0.0
      ALP(NCASE+2)=1./ASCALE
      IF (NWING.NE.0.AND.NBODY.NE.0) JSIGN=-4
      IF (NWING.NE.0.AND.NBODY.NE.0) GO TO 401
      IF(NWING.NE.0) GO TO 401
      DEL1=.002
      DEL2=.002
      BEG1=-0.002
      BEG2=.0
      BEG3=.01
      DEL3=-.01
      DO 402 I=1,NCASE
      CD(I)=CDBP(I)
      CL(I)=CLBP(I)
      CM(I)=CMBP(I)
      IF(CL(I).GT..01.OR.CL(I).LT.-.002) ICL=1
      IF(CD(I).GT..012.OR.CD(I).LT.TOL) ICD=1
      IF(CM(I).GT..02.OR.CM(I).LT.-.02) ICM=1
      IF(CD(I).GT..012) CD(I)=.012
      IF(CL(I).GT..01) CL(I)=.01
      IF(CM(I).GT..02) CM(I)=.02
      IF(CD(I).LT.TOL) CD(I)=.0
      IF(CL(I).LT.-.002) CL(I)=-.002
      IF(CM(I).LT.-.02) CM(I)=-.02
  402 END DO
      NMAXX=NMAXB
      MMAX=MMAXB
      GO TO 405
  401 CONTINUE
      DO 403 I=1,NCASE
      CL(I)=CLWP(I)
      CM(I)=CMWP(I)
      CD(I)=CDWP(I)
      IF(CL(I).GT..5.OR.CL(I).LT.-.1) ICL=1
      IF(CD(I).GT..06.OR.CD(I).LT.TOL) ICD=1
      IF(CM(I).GT..2.OR.CM(I).LT.-.2) ICM=1
      IF(CD(I).GT..06) CD(I)=.06
      IF(CL(I).GT..5) CL(I)=.5
      IF(CM(I).GT..2) CM(I)=.2
      IF(CD(I).LT.TOL) CD(I)=.0
      IF(CL(I).LT.-.1) CL(I)=-.1
      IF(CM(I).LT.-.2) CM(I)=-.2
  403 END DO
  405 CONTINUE
      CL(NCASE+1)=0.0
      CM(NCASE+1)=0.0
      CD(NCASE+1)=0.0
      CL(NCASE+2)=DEL1
      CD(NCASE+2)=DEL2
      CM(NCASE+2)=DEL3
      IF(NWING.EQ.0) GO TO 250
      IF(NBODY.EQ.0) GO TO 230
      NMAXX=NBODY
      MMAX=NWP
      NPT=MMAX + NMAXX
      GO TO 250
  230 CONTINUE
      IF(NCAN.EQ.0) GO TO 231
      NMAXX=NCD2+NCD1
      MMAX=NWD2+NWD1
      JSIGN=-4
      GO TO 232
  231 CONTINUE
      NMAXX=NWD1
      MMAX=NWD2
  232 CONTINUE
      NPT=NWP
  250 CONTINUE
      CLM1=BEG1
      CLM2=DEL1*5.
      CMM1=BEG3*2.
      CMM2=DEL3*2.
      CDM1=BEG2
      CDM2=DEL2*5.
      CALL AXIS(1.,2.,2HCD,-2,6.,0.,BEG2,DEL2)
      CALL AXIS(1.,1.,2HCL,2,6.,90.,BEG1,DEL1)
      CALL SYMBOL(1.,8.8,.15,11H DRAG POLAR,0.,11)
!     CALL JGRID(0.,0.,0.2,-0.1,5.,7.,6,8)
      CALL W12HED(NMAXX,MMAX,NPT,0.0,JSIGN)
      CALL LORGN (1.,2.)
      CALL LINE (CD,CL,NCASE,1,-1,4)
      J=0
      IF(ICL.EQ.0.AND.ICD.EQ.0) GO TO 173
      DO 174 JC=1,NCASE
      IF(CD(JC).NE.CDM2.AND.CD(JC).NE..0.AND.CL(JC).NE.CLM2.AND.CL(JC)  &
     & .NE.CLM1) GO TO 174
  180 J=J+1
      A(J)=CD(JC)/DEL2
      B(J)=CL(JC)/DEL1
  174 END DO
      IF(J.EQ.0) GO TO 173
      DO 196 IP=1,J
      XX=A(IP)
      YY=B(IP)
      CALL SYMBOL(XX,YY,.1,0,0.,-1)
  196 END DO
      CALL SYMBOL(C,D,.1,0,0.,-1)
      CALL SYMBOL(3.75,3.45,.1,9HOFF SCALE,0.,9)
  173 CONTINUE
      CALL LORGN (-1.,-2.)
      CALL PLOT(8.5,0.,-3)
      CALL AXIS(2.,1.,2HCL,2,6.,90.,BEG1,DEL1)
      CALL AXIS(1.,2.,3HALP,-3,6.,0.,-2.,2.)
      CALL SYMBOL(1.,8.8,.15,17H LIFT CURVE SLOPE,0.,17)
!     CALL JGRID(0.,0.,0.2,-0.1,5.,7.,6,8)
      CALL W12HED(NMAXX,MMAX,NPT,0.0,JSIGN)
      CALL LORGN (2.,2.)
      CALL LINE (ALP,CL,NCASE,1,-1,4)
      J=0
      IF(ICL.EQ.0) GO TO 175
      DO 176 JC=1,NCASE
      IF(CL(JC).NE.CLM2.AND.CL(JC).NE.CLM1) GO TO 176
  181 J=J+1
      A(J)=ALPHAP(JC)*ASCALE
      B(J)=CL(JC)/DEL1
  176 END DO
      IF(J.EQ.0) GO TO 175
      DO 197 IP=1,J
      XX=A(IP)
      YY=B(IP)
      CALL SYMBOL(XX,YY,.1,0,0.,-1)
  197 END DO
      CALL SYMBOL(C,D,.1,0,0.,-1)
      CALL SYMBOL(3.75,3.45,.1,9HOFF SCALE,0.,9)
  175 CONTINUE
      CALL LORGN (-2.,-2.)
      CALL PLOT(8.5,0.,-3)
      CALL AXIS(4.,1.,2HCL,2,6.,90.,BEG1,DEL1)
      CALL AXIS(3.,2.,2HCM,-2,2.,0.,BEG3,DEL3)
      CALL SYMBOL(1.,8.8,.15,16H PITCHING MOMENT,0.,16)
!     CALL JGRID(0.,0.,0.2,-0.1,5.,7.,6,8)
      CALL W12HED(NMAXX,MMAX,NPT,0.0,JSIGN)
      CALL LORGN (4.,2.)
      CALL LINE (CM,CL,NCASE,1,-1,4)
      J=0
      IF(ICL.EQ.0.AND.ICM.EQ.0) GO TO 177
      DO 178 JC=1,NCASE
      IF(CM(JC).NE.CMM1.AND.CM(JC).NE.CMM2.AND.CL(JC).NE.CLM2.AND.CL(JC)&
     & .NE.CLM1) GO TO 178
  185 J=J+1
      A(J)=CM(JC)/DEL3
      B(J)=CL(JC)/DEL1
  178 END DO
      IF(J.EQ.0) GO TO 177
      DO 198 IP=1,J
      XX=A(IP)
      YY=B(IP)
      CALL SYMBOL(XX,YY,.1,0,0.,-1)
  198 END DO
      CALL SYMBOL(C,D,.1,0,0.,-1)
      CALL SYMBOL(3.75,3.45,.1,9HOFF SCALE,0.,9)
  177 CONTINUE
      CALL LORGN (-4.,-2.)
      CALL PLOT(12.0,0.,-3)
  300 CONTINUE
      RETURN
      END Subroutine SmPlot

      SUBROUTINE W12HED(NMAX,MMAX,NP,SPOT,NPRNT)
!!!      COMMON /HEAD/TITLE1(14),TITLE2(14)
      INCLUDE 'head.cmn'
      COMMON /NEWCOM/K0,K1,K2,K3,K4,K5,K6,KWAF,KWAFOR,KFUS,KRADX(4),     &
     &              KFORX(4),KF(6),KFINOR(6),KAN(6),KANOR(6),KRAD,MAX,  &
     &              KOL,NCPT
      COMMON /SEG/ NSEG,NROW(20),NCOL(20),COSS(20),SINS(20),BT(20),DUM(6&
     &0),BL(20)
      COMMON /PARAM/ NBODY,NWING,NTAIL,LBC,THK,MACH,ALPHA,REFA
!!!      COMMON /OPTION/ NOPT(10)
      INCLUDE 'option.cmn'
      DIMENSION R(20)   !!! ,TITLEN(8)
      CHARACTER(LEN=80):: titlen
!     INTEGER TITLE1,TITLE2

      TNMAX=NMAX
      TMMAX=MMAX
      TNP=NP
!!!      ENCODE(80,10,TITLEN) TITLE1     !!! gotta deal with this
!!!   10 FORMAT(13A6,A2)
      titlen=title1

      CALL JOBNAME(JOB)
      CALL SYMBOL(1.5,0.7,.10,JOB,0.,7)
      CALL SYMBOL(1.,9.4,.15," GRUMMAN'S W12SC3 AERODYNAMIC CODE",0.,34)
      CALL SYMBOL(1.15,9.1,.15,TITLEN,0.,40)
      CALL SYMBOL(4.5,.7,.10,10HPANEL NO.=,0.,10)
      CALL SYMBOL(6.6,.7,.10,1H=,0.,1)
      CALL NUMBER(5.65,.7,.1,TNMAX,0.,-1)
      CALL NUMBER(6.25,.7,.1,TMMAX,0.,-1)
      CALL NUMBER(7.,.7,.1,TNP,0.,-1)
      IF(NOPT(4).EQ.0) CALL SYMBOL(1.5,8.15,.1,'WII OPTION',0.,10)
      IF(NOPT(4).EQ.2) CALL SYMBOL(1.5,8.15,.1,'WI OPTION',0.,9)
      IF(NPRNT.LT.0)  CALL SYMBOL(6.0,.75,.1,3,0.,-1)
      IF(NPRNT.GT.0)  CALL SYMBOL(6.0,.75,.1,4,0.,-1)
      IF(IABS(NPRNT).EQ.4)  CALL SYMBOL(3.5,8.5,.15,'M=',0.,2)
      IF(IABS(NPRNT).EQ.4)  CALL NUMBER(3.9,8.5,.15,MACH,0.,2)
      IF(IABS(NPRNT).EQ.4) GO TO 200
      IF(IABS(NPRNT).EQ.3)CALL SYMBOL(4.,8.5,.15,'THETA=',0.,6)
      IF(IABS(NPRNT).EQ.2) CALL SYMBOL(4.5,8.5,.15,'Y=',0.,2)
      IF(IABS(NPRNT).EQ.1)CALL SYMBOL(4.5,8.5,.15,'X=',0.,2)
      CALL SYMBOL(1.2,8.5,.15,'M=',0.,2)
      CALL SYMBOL(2.5,8.5,.15,'ALP=',0.,4)
      CALL NUMBER(1.75,8.5,.15,MACH,0.,2)
      CALL NUMBER(3.2,8.5,.15,ALPHA,0.,2)
      CALL NUMBER(5.0,8.5,.15,SPOT,0.,2)
      IF(NBODY.EQ.NP) GO TO 200
      CALL SYMBOL(5.0,8.3,.1,3,0.,-1)
      CALL SYMBOL(5.0,8.1,.1,4,0.,-1)
      CALL SYMBOL(5.3,8.25,.1,'UPPER SURFACE',0.,13)
      CALL SYMBOL(5.3,8.05,.1,'LOWER SURFACE',0.,13)
  200 CONTINUE
      RETURN
      END Subroutine W12Hed 

      SUBROUTINE PLOTE
      CALL PLOT(0,0,999)
      RETURN
      END

      SUBROUTINE JOBNAME(JOB)
      RETURN
      END
      SUBROUTINE PLOT(X,Y,IPEN)
      RETURN
      END
      SUBROUTINE PLOTS(IBUF,NLOC,LDEV)
      RETURN
      END
      SUBROUTINE SYMBOL(X,Y,H,IBCD,ANG,NCHAR)
      RETURN
      END
      SUBROUTINE AXIS(X,Y,IBCD,NCHAR,XL,ANG,FIRST,DELTAV)
      RETURN
      END
      SUBROUTINE NUMBER(X,Y,H,DUM,ANG,NCHAR)
      RETURN
      END
      SUBROUTINE LORGN(X,Y)
      RETURN
      END
      SUBROUTINE CALPLT(X,Y,IPEN)
      CALL PLOT(X,Y,IPEN)
      RETURN
      END

      SUBROUTINE NOTATE(XPAGE,YPAGE,HEIGHT,IBCD,ANGLE,NCHAR)
      DIMENSION IBCD(1)
      CALL SYMBOL(XPAGE,YPAGE,HEIGHT,IBCD,ANGLE,NCHAR)
      RETURN
      END
      SUBROUTINE PSEUDO (N)
      RETURN
      END
      SUBROUTINE NFRAME
      CALL PLOT(0.,0.,-3)
      RETURN
      END
      SUBROUTINE LINE (A,B,I,J,K,L,M)
      RETURN
      END
      SUBROUTINE XFER( INPUT, IOUT, IROWS, ICOLS, DUMMY )
      DIMENSION DUMMY(ICOLS)
      REWIND IOUT
      DO 10 I = 1, IROWS
      CALL RNRW( - INPUT, DUMMY, ICOLS)
      CALL RNRW(   IOUT, DUMMY, ICOLS)
   10 END DO
      REWIND IOUT
      RETURN
      END
      SUBROUTINE TRPOSE(INPUT,BUFFER,STORE,NCOLS,NROWS,OUTPUT,SCRTCH,   &
     &ISIZE)
      INTEGER OUTPUT,SCRTCH
      DIMENSION BUFFER(1),STORE(1)
!
!  ******************************************************************
!  *                                                                *
!  *  FORMS THE TRANSPOSE OF A MATRIX, USING OUT-OF-CORE            *
!  *  STORAGE WHERE NECESSARY                                       *
!  *                                                                *
!  ******************************************************************
!
!***** INPUT  = INPUT UNIT CONTAINING ORIGINAL MATRIX
!***** BUFFER = BUFFER TO HOLD PART OF MATRIX (SIZE=ISIZE)
!***** STORE  = BUFFER TO HOLD ROW OF TRANSPOSE (SIZE=NROWS)
!***** NCOLS  = NUMBER OF COLUMNS IN ORIGINAL MATRIX
!***** NROWS  = NUMBER OF ROWS IN ORIGINAL MATRIX
!***** OUTPUT = OUTPUT UNIT CONTAINING TRANSPOSED MATRIX
!***** SCRTCH = SCRATCH UNIT
!***** ISIZE  = SIZE OF BUFFER
!
      REWIND INPUT
      NSIZE = NROWS * NCOLS
      IF( NSIZE .GT. ISIZE ) GO TO 30
!
!     TRANSPOSE OF ORIGINAL MATRIX IS FORMED IN CORE.
!
      DO 10 I = 1,NROWS
      I1 = I
      I2 = I1 + ( NCOLS - 1 ) * NROWS
      READ(INPUT) ( BUFFER(J) , J = I1,I2,NROWS )
   10 END DO
      REWIND OUTPUT
      J2 = 0
      DO 20 I = 1,NCOLS
      J1 = J2 + 1
      J2 = I * NROWS
      WRITE(OUTPUT) ( BUFFER(J) , J = J1,J2 )
   20 END DO
      REWIND OUTPUT
      RETURN
   30 CONTINUE
!
!     TRANSPOSE OF ORIGINAL MATRIX IS FORMED USING OUT OF CORE STORAGE.
!
      REWIND OUTPUT
      REWIND SCRTCH
!
!     DETERMINE MAXIMUM NUMBER OF ROWS,(MAXROW), WHICH WILL FIT WITHIN
!     CORE STORAGE,(ISIZE).(ICYCLE) GIVES TOTAL NUMBER OF ROW PARTITIONS
!     THAT ARE REQUIRED.  THERE ARE (NUM) PARTITIONS CONTAINING (MAXROW)
!     ROWS AND ONE PARTITION CONTAINING (LAST) ROWS.
!
      MAXROW = ISIZE / NCOLS
      NUM = NROWS / MAXROW
      LAST = NROWS - NUM * MAXROW
      ICYCLE = NUM
      IF( LAST .NE. 0 ) ICYCLE = NUM + 1
!
!     OUT OF CORE STORAGE FOR EACH OF THE (ICYCLE) PARTITIONS SWITCHES
!     BETWEEN UNITS (OUTPUT) AND (SCRTCH). IF (ICYCLE) IS AN ODD NUMBER,
!     START WITH (OUTPUT). IF (ICYCLE) IS AN EVEN NUMBER, START WITH
!     (SCRTCH). THIS WILL INSURE THAT THE COMPLETE TRANSPOSE IS WRITTEN
!     ONTO UNIT (OUTPUT) DURING THE LAST PARTITION.
!
      ICT = ICYCLE / 2
      ITEST = ICYCLE - 2 * ICT
      INEW = OUTPUT
      IOLD = SCRTCH
      IF( ITEST .NE. 0 ) GO TO 40
      INEW = SCRTCH
      IOLD = OUTPUT
   40 CONTINUE
!
!     THE COUNTER (ISUMBF) GIVES THE TOTAL NUMBER OF PARTITION ROWS
!     CURRENTLY IN THE BUFFER.  (ISUMTP) GIVES THE TOTAL NUMBER OF ROWS
!     WHICH HAVE BEEN WRITTEN TO OUT OF CORE STORAGE.
!
      ISUMBF = 0
      ISUMTP = 0
      IROWS = MAXROW
      DO 100 I = 1,ICYCLE
      IF( I .EQ. ICYCLE .AND. LAST .NE. 0 ) IROWS = LAST
      ISUMBF = IROWS
!
!     FORM TRANSPOSE OF NEXT PARTITION AND STORE IN (BUFFER) BY ROWS.
!
      DO 50 J = 1,IROWS
      I1 = J
      I2 = I1 + ( NCOLS - 1 ) * IROWS
      READ(INPUT) ( BUFFER(K) , K = I1,I2,IROWS )
   50 END DO
!
!     READ FROM UNIT (IOLD) THE INTERMEDIATE TRANSPOSED MATRIX
!     BY ROWS, CONTAINING (ISUMTP) ELEMENTS, AND STORE IN
!     ARRAY (STORE). ADD NEW ELEMENTS FOR THE CURRENT PARTITION FROM
!     ARRAY (BUFFER), AND WRITE THIS NEW ROW WITH (IWRITE) ELEMENTS
!     ONTO NEW UNIT, (INEW).
      IWRITE = ISUMTP + ISUMBF
   99 CONTINUE
      K1 = 0
      K2 = 0
      DO 90 J = 1,NCOLS
      IF( I.EQ. 1 ) GO TO 70
      READ(IOLD) ( STORE(K), K=1,ISUMTP)
   70 K1 = K2 + 1
      K2 = K2 + ISUMBF
      J1 = ISUMTP
      DO 80 K = K1,K2
      J1 = J1 + 1
      STORE(J1) = BUFFER(K)
   80 END DO
      WRITE(INEW) ( STORE(K) , K=1,IWRITE )
   90 END DO
!
!     UPDATE ROW COUNTER, (ISUMTP) , FOR OUT OF CORE STORAGE.
!     SWITCH UNITS FOR NEXT PARTITION.
!
      ISUMTP = IWRITE
      ISAVE = IOLD
      IOLD = INEW
      INEW = ISAVE
      REWIND INEW
      REWIND IOLD
  100 END DO
      REWIND OUTPUT
      RETURN
      END

      SUBROUTINE TRIXY(INA,INB,IOC,NROW,MM,NCOL,A,B,C,LENGTH,IRET)
!
!
!  ******************************************************************
!  *                                                                *
!  *  COLUMN-SORT MATRIX MULTIPLIER...... MULTIPLIES RECTANGULAR    *
!  *  MATRIX A (STORED IN COLUMN SORT ON INA) BY MATRIX B           *
!  *  (STORED IN COLUMN SORT ON INB) AND PUTS RESULTING MATRIX      *
!  *  C ONTO IOC IN COLUMN SORT.  OPERATIONS ARE DONE BY STORING    *
!  *  P COLUMNS OF A AND B IN CORE AT SAME TIME.  REQUIRES ONLY     *
!  *  P REWINDS OF INA.  (A IS NROW BY MM, B IS MM BY NCOL).        *
!  *                                                                *
!  ******************************************************************
!
!***** INA    = INPUT UNIT CONTAINING MATRIX A
!***** INB    = INPUT UNIT CONTAINING MATRIX B
!***** IOC    = OUTPUT UNIT CONTAINING MATRIX C
!***** NROW   = NUMBER OF ROWS IN A AND C
!***** MM     = NUMBER OF COLUMNS IN A = NUMBER OF ROWS IN B
!***** NCOL   = NUMBER OF COLUMNS IN B AND C
!***** A      = BUFFER TO HOLD A PART OF A
!***** B      = BUFFER TO HOLD A PART OF B
!***** C      = BUFFER TO HOLD A PART OF C
!***** LENGTH = SIZE OF EACH BUFFER
!***** IRET   = 0 OUT-OF-CORE STORAGE NEEDED
!*****        = 1 ALL MATRICES CAN BE FIT IN CORE
!
      DIMENSION A(LENGTH), B(LENGTH), C(LENGTH)
      REWIND INA
   99 IRET=0
!
! COMPUTE P AS THE LARGEST NO. OF WHOLE
! COLUMNS THAT CAN FIT IN CORE STORAGE. IF ALL
! COLUMNS OF A CAN FIT DO MULTIPLICATION DIRECTLY.
!
      IP1= LENGTH/NROW
      IP2= LENGTH/MM
      IP = MIN0(IP1,IP2)
      IF(IP1.LT.MM) GO TO 10
!
! READ IN ALL OF A
!
      DO 1 I= 1,MM
      II= (I-1)*NROW + 1
      CALL RNRW(-INA,A(II),NROW)
    1 END DO
!
! READ A COLUMN OF B AND DOT
! WITH ROW OF A TO FORM ENTRY
! IN COL OF C.  WRITE OUT C
! WHEN COL IS FILLED.
!
      DO 2 J= 1,NCOL
      CALL RNRW(-INB,B(1),MM)
      DO 3 I= 1,NROW
      SUM = 0.0
    3 C(I)= SOSCAP(A,B,SUM,MM,NROW,1,I,1)
    2 CALL RNRW(IOC,C(1),NROW)
      IRET= 1
      REWIND INA
      RETURN
!
! IF NO COLUMNS WILL FIT, FLAG  ERROR
!
   10 IF(IP1.GE.1 .AND. IP2.GE.1) GO TO 20
      IRET=-1
      RETURN
!
! ONLY P COLUMNS WILL FIT AT A TIME
!
   20 CONTINUE
      NROFA = 0
      NROFB = 0
!
! DO IP2 COLUMNS OF B REMAIN &
! IF SO, READ THEM INTO CORE.
! OTHERWISE, READ WHAT IS LEFT.
!
   21 CONTINUE
      ILEFT= NCOL - NROFB
      IB = MIN0(IP,ILEFT)
      DO 22 I= 1,IB
      II= (I-1)*MM + 1
   22 CALL RNRW(-INB,B(II),MM)
!
! ZERO OUT C TO HOLD
! NEXT PORTION OF A*B
!
      NRIB= NROW*IB
      DO 23 I= 1,NRIB
   23 C(I)= 0.
!
! DO IP1 COLUMNS OF A REMAIN&
! IF SO, READ THEM INTO CORE.
! OTHERWISE, READ WHAT IS LEFT.
!
   31 CONTINUE
      ILEFT= MM - NROFA
      IA = MIN0(IP,ILEFT)
      DO 32 I= 1,IA
      II= (I-1)*NROW + 1
   32 CALL RNRW(-INA,A(II),NROW)
!
! ACCUMULATE A(I,K)*B(K,J)
! INTO C FOR I= ALL ROWS,
! J= IB COLUMNS, AND K= IA
!
      DO 41 J= 1,IB
      DO 41 I= 1,NROW
      IJ= I + (J-1)*NROW
      DO 42 K= 1,IA
      KJ= (J-1)*MM + NROFA + K
      IK= I + (K-1)*NROW
   42 C(IJ)= A(IK)*B(KJ) + C(IJ)
   41 CONTINUE
      NROFA= NROFA + IA
!
! IF ANY COLUMNS OF A ARE LEFT,
! GO TO BACK AND GET THEM.
! OTHERWISE REWIND INA, WRITE C,
! AND GO BACK FOR NEXT COLUMNS
! OF B, IF ANY.
!
      IF(NROFA.LT.MM) GO TO 31
      REWIND INA
      NROFA= 0
      DO 24 I= 1,IB
      II= (I-1)*NROW + 1
   24 CALL RNRW(IOC,C(II),NROW)
      NROFB= NROFB + IB
      IF(NROFB.LT.NCOL) GO TO 21
      RETURN
      END Subroutine TriXY   ! ----------------------------------------------

      FUNCTION SOSCAP(X,Y,S,N,IX,IY,ISX,ISY)
      DIMENSION X(*),Y(*)

      IF( N ) 120 , 120 , 100
  100 JX = ISX
      JY = ISY
      DO 110 J = 1,N
      S = S + DBLE( X (JX) ) * DBLE( Y (JY) )
      JX = JX + IX
  110 JY = JY + IY

  120 SOSCAP = S
      RETURN
      END Function Soscap   ! ----------------------------------------------

      SUBROUTINE RNRW(IU,A,MID)
      DIMENSION A(MID)
!
!     READ AND WRITE FROM/TO DISK STORAGE
!
!     IF IU .GT. 0 , WRITE OPTION
!     IF IU .LT. 0 , READ OPTION
!
      IF( IU .LT. 0 ) GO TO 10
      WRITE(IU) A
      RETURN
   10 IB = IABS( IU )
      READ(IB) A
      RETURN
      END Subroutine Rnrw   ! --------------------------------------------

      SUBROUTINE FUTSOL (ND,MD,KD,NI,MM,NO,NAT,NW,LTAPE,RHSTAP)
!
!   ***   THIS IS THE REAL VERSION OF FUTSOL
!
!
!
!           ****    ***/  *******  *****  *       ***/          *
!           *   *  *  /*     *     *      *      *  /*        ******
!     ****  ****   * / *     *     ***    *      * / *  ****     *  *
!           *      */  *     *     *      *      */  *           ***
!           *      /***      *     *      *****  /***            *  *
!
!             D I R E C T   M A T R I X   S O L U T I O N
!
!     WRITTEN BY J. L. HESS * PROGRAMMED BY C. D. SCHOOR
!*** ***LTAPE IS THE TAPE THE L(I,J) MATRIX WILL BE PUT ON
!*** ***RHSTAP IS THE TAPE THAT THE RIGHT HAND SIDES ARE INPUT ON
      INTEGER RHSTAP
!
      DIMENSION  A(3700)
!
      LOGICAL JPASS1
      LOGICAL  LASTRS
      LOGICAL  LAST
!
    1 CONTINUE
      NATAPE = NAT
      REWIND NATAPE
      REWIND NW
      N = ND
      KORE = KD
!*** *  RHSTAP = 0 IF THERE ARE NO RHS TO BE PROCESSED THIS RUN
      IF(RHSTAP .NE. 0 )GO TO 5
      MRHS = 0
      GO TO 6
    5 REWIND RHSTAP
      READ(RHSTAP)MRHS
    6 M = KORE/N -1
      MMAX = MIN0(MRHS,M)
      NPM = N + MMAX
      IF (MAX0(3 * NPM, M * N) .GT. KORE) RETURN
      M = 0
      NPM = N
      MT = MM
      REWIND MT
      NIN = NI
      REWIND NIN
      NOUT = NO
      REWIND NOUT
      NN = N
      NEL = NPM
!
! - - CALCULATE THE MAXIMUM NO. OF ROWS, @K@
!
!*** ***CALCULATE THE NUMBER OF COLUMNS TO BRING OFF OF THE RHS TAPE
      MTOTAL = 0
      M = MMAX
      IF(M .EQ. 0)GO TO 295
!*** ***MTOTALIS THE TOTAL NUMBER OF RHS COLUMNS ALREADY BROUGHT IN
  109 MTOTAL = MTOTAL + M
      LASTRS = MTOTAL.GE. MRHS
      MTOTAL = MTOTAL - M
      IF (LASTRS)M = MRHS - MTOTAL
      MP1 = M + 1
      MM1 = M - 1
      MTOTAL = MTOTAL + M
!*** ***BRING IN M COLUMNS OF RHS
      KINIT = KORE - (M*N)
      IINIT = KINIT
      NBEG = KINIT + 1
      NEND = KINIT+ N
      DO 110 J =1,M
      READ (RHSTAP)( A(I),I=NBEG,NEND)
      NBEG = NEND + 1
  110 NEND = NEND + N
!*** ***BRING IN L(I,J) MATRIX AND APPLY IT TO RHS
      NBEG = 1 + KINIT
      NEND = 1 + (M-1) * N + KINIT
      KSUM = 0
      NLCNT = 0
!*** ***DO TRIANGULAR SECTION OF LMATRIX
  111 READ (LTAPE)K
      NLCNT = NLCNT + 1
!*** ***KSUM IS THE TOTAL NUMBER OF L ROWS THAT WILL
!*** ***BE READ AFTER THIS TRIANGULAR SECTION IS FINISHED
      KSUM = KSUM + K
      KM1 = K - 1
!***   ***NOTE THAT KM1 CAN@T BE 0 SINCE K CAN@T BE 1 AND STILL HAVE SOM
!***   ***ON THE LTAPE
      DO 114  I = 1,KM1
      NBEG = NBEG + 1
      NEND = NEND + 1
!*** ***READ 1 ROW OF L(I,J) FROM LTAPE---K-1 TIMES---EACH TIME
!*** ***STARTING WITH L(1)
      READ (LTAPE) ( A(NN),NN=1,I)
      JCNT = -1
!*** ***REDUCE THE RHS BY GOING ACROSS A SOLUTION ROW (WHICH
!*** ***ARE NOT IN CONSECUTIVE ORDER, BUT A(1), A(N+1), A(2N+1) ETC.)
      DO 113 NPP = NBEG,NEND,N
      JCNT = JCNT + 1
      SUM = 0.0
      NROW = KINIT + ( JCNT * N )
      DO 112 NN=1,I
      NROW = NROW + 1
  112 SUM = SUM + ( A(NN)*A(NROW))
  113 A(NPP) = A(NPP) - SUM
  114 END DO
      IF (KSUM .EQ. N)GO TO 116
!*** ***KSUM = N IF YOU HAVE READ ENTIRE LMATRIX AND
!*** ***THERE IS NO CONSTANT SECTION LEFT
      NTBEG = NBEG
      NTEND = NEND
      KSUMP1 = KSUM + 1
!*** ***READ REST OF LROWS 1 ROW AT A TIME FOR CONSTANT SECTION
      DO 115 I=KSUMP1,N
      NTBEG = NTBEG + 1
      NTEND = NTEND + 1
      READ (LTAPE)( A(NN),NN=1,K)
      JCNT = -1
!*** ***PARTIALLY  REDUCE A RHS ACROSS A RHS ROW BY APPLYING K NUMBER
!*** ***OF L(I,J) S
      DO 124 NPP = NTBEG,NTEND,N
      JCNT = JCNT + 1
      SUM = 0.0
      NROW = KINIT + ( JCNT * N )
      DO 123 NN = 1,K
      NROW = NROW + 1
  123 SUM = SUM + ( A(NN) * A(NROW) )
  124 A(NPP) = A(NPP) - SUM
  115 END DO
      NBEG = NBEG + 1
      NEND = NEND + 1
!*** ***KINIT IS HOW FAR DOWN A COLUMN OF RHS TO START MULTIPLYING BY
!*** ***L(I,J) AT EACH PASS THROUGH
      KINIT = KINIT + K
!*** ***IF KSUMP1 = N THERE ARE NO MORE L(I,J)@S LEFT
      IF(KSUMP1 .LT. N)GO TO 111
!*** ***WRITE OUT ALL BUT LAST K ROWS OF RHS IN ROW ORDER ON NATAPE
  116 B = 4*M + 3
      C = -2 * KORE
      K = (  -B  +  SQRT( B**2  -4.0*C )  )/2.0
      IF(K .GT. ND) K = ND
      KF = K
      KM1 = K - 1
      KLEFT = N - KF + IINIT
      INITP1 = IINIT + 1
      NEND = (M-1)*N + IINIT
      DO 117 NPP = INITP1,KLEFT
      NEND = NEND + 1
  117 WRITE(NATAPE) ( A(J),J=NPP,NEND,N)
      REWIND NATAPE
!*** ***JPASS1 IS TRUE ON 1ST PASS THRU BACK SOLUTION
      JPASS1 = .TRUE.
!
!***   ***PUT REMAINING RHS IN CONTIGUOUS LOCATIONS BY COLUMNS
!***  FROM KORE - (M * KF) + 1  TO  KORE
!
      NNEW = KORE - KF + 1
!
!***   ***IF M = 1, THE ELTS OF THE 1 RHS COLUMN ARE ALREADY IN CONTIGUO
!***   ***LOCATIONS
!
      IF (M.EQ.1)  GO TO 1118
      DO 118 I = 1,MM1
      NOLD = KORE - (I*N) + 1
      DO 118 J = 1,KF
      NNEW = NNEW - 1
      NOLD = NOLD - 1
      A(NNEW) = A(NOLD)
  118 CONTINUE
 1118 CONTINUE
!
!***   ***NOW NNEW = KORE - (M*KF) + 1
!***   ***NOW NOLD = KORE - (M - 1) * N + 1 - KF
!  **   SKIP 1ST PART OF TRAPEZOIDAL MATRIX ON LTAPE
!  **   READ IN LAST K ROWS OF TRAPEZOIDAL MATRIX AND
!*** ***ATTATCH RHS TO IT SO THAT EVERYTHING IS IN CONSECUTIVE ORDER
      NREMAN = ND - K
      IF( NREMAN .EQ. 0 )GO TO 126
      DO 122 I=1,NREMAN
  122 READ(LTAPE) IDUMMY
  126 NEND = 0
      KCNT = K
      NNEW = NNEW - 1
!***   ***NOTE THAT K = KF WHICH IS ALREADY KNOWN IN CORE
      DO 121 JCNT = 1,K
      NBEG = NEND + 1
      KCNT = KCNT - 1
      NEND = NBEG + KCNT
      READ(LTAPE)IDUMMY,(A(J),J=NBEG,NEND)
      NNEW = NNEW + 1
      KEND = (MM1 * KF) + NNEW
      DO 121 NPP=NNEW,KEND,KF
      NEND = NEND + 1
  121 A(NEND) = A(NPP)
      REWIND LTAPE
!
!  **  SKIP OVER L MATRIX ON LTAPE TO GET TO TRAPEZOIDAL MATRIX
!
      KRED = 0
      DO 128 I=1,NLCNT
      READ(LTAPE)KREAD
      KRED = KRED + KREAD
      KREAD = KREAD + ( N - KRED - 1)
      DO 128 LREAD = 1,KREAD
  128 READ(LTAPE)
!
! - - THERE, NOW WE CAN START THE BACK-SOLUTION
! * * NOTE..THE FIRST AVAILABLE LOCATION FOR THE SOLUTIONS IS A(N1)
!
!
!***   ***NL IS THE LAST SUBSCRIPT + 1 OF THE TRAPEZOIDAL A MATRIX THAT
!***   ***CORE
!
      NL = NEND + 1
      NREM = N
      NPM = N + M
      NEL = NPM
      MP1 = M + 1
      LAST = K .EQ. N
      NPASS = 0
!
! - - SOLVE FOR THE ANSWERS CORRESPONDING TO @K@ ROWS
!
  119 KM1 = K - 1
      KP1 = K + 1
      NS = NL - MP1
      NPASS = NPASS + 1
      DO 130 MN = 1, M
      NF = NS + MN
      A(NF) = A(NF) / A(NS)
      NT = NS
      IF (KM1 .EQ. 0) GO  TO 130
      DO 125 IB = 1, KM1
      NF = NF - IB - M
      NT = NT - MP1 - IB
      SUM = 0.0
      NP = NF
      N2 = MP1 + IB
      DO 120 IO = 1, IB
      NN = NT + IO
      NP = NP + N2 - IO
  120 SUM = SUM + A(NN) * A(NP)
  125 A(NF) = (A(NF) - SUM) / A(NT)
  130 END DO
!
! - - MOVE THE SOLUTIONS TO CONTIGUOUS LOCATIONS STARTING AT A(N1)
!
      N1 = KORE + 1
      DO 140 NN = 1, K
      DO 135 MN = 1, M
      NL = NL - 1
      N1 = N1 - 1
  135 A(N1) = A(NL)
  140 NL = NL - NN
!
! - - WRITE THE SOLUTIONS ON TAPE
!
      WRITE (NIN) K
      NS = N1 - 1
      DO 145 MN = 1, M
      NT = NS + MN
  145 WRITE ( NIN ) (A(IO), IO = NT, KORE, M)
!
! - - TEST IF THIS IS THE LAST PASS
!
      IF (LAST) GO TO 200
!
! - - WE MUST NOW MODIFY THE TRIANGULAR MATRIX TO REFLECT THE EFFECT OF
!     THE SOLUTIONS OBTAINED SO FAR  (EQ 21)
! * * NOTE..LOCATIONS A(1) TO A(N1-1) ARE NOW FREE TO USE
!
! - - CALCULATE THE NEXT VALUES OF @NEL@ AND @NREM@
!
      NELOLD = NEL
      KOLD = K
      NEL = NEL - K
      NREM = NREM - K
!
      NROW = NREM - K + 1
      IF (K .LT. NREM) GO TO 150
      LAST = .TRUE.
      NROW = 1
      K = NREM
  150 NS = 1
      NT = NELOLD + 1
!
! - - READ IN THE ROWS TO BE MODIFIED
!
      DO 190 IB = 1, NREM
      NT = NT - 1
      IF (IB .LE. NROW) GO TO 160
      NS = NS + NN
      NT = NT + NN
  160 IF (.NOT. JPASS1)GO TO 161
      NBEG = NT - M + 1
!*** ***READ RHS FROM NATAPE
      READ (NATAPE) ( A(IO),IO = NBEG,NT)
      NT = NT - M
      READ(LTAPE)NN,(A(IO),IO=NS,NT)
      NT = NT + M
      NN = NN + M
      GO TO 163
  161 READ(MT)NN,(A(IO),IO=NS,NT)
  163 NP = N1 - 1
      NF = NT - M - KM1
      NN = NN - KOLD
      DO 170 MN = 1, M
      N2 = NF
      NA = NP + MN
      NB = NA
      SUM = 0.0
      DO 165 IO = 1, KOLD
      SUM = SUM + A(N2) * A(NA)
      N2 = N2 + 1
  165 NA = NA + M
      N2 = N2 + MN - 1
  170 A(N2) = A(N2) - SUM
!
! - - WRITE THE MODIFIED ROW ON TAPE OR CONDENSE THE ROW
!
      NL = NT - M + 1
      IF (IB .GE. NROW) GO TO 175
      NF = NL - KP1
      WRITE (NOUT) NN, (A(IO), IO = NS, NF), (A(IO), IO = NL, NT)
      GO TO 190
  175 NF = NL - KOLD
      DO 180 MN = NL, NT
      A(NF) = A(MN)
  180 NF = NF + 1
  190 END DO
      IF( .NOT. JPASS1 ) GO TO 195
      JPASS1= .FALSE.
      REWIND NATAPE
  195 REWIND MT
      REWIND NOUT
!
! - - SWITCH THE TAPES
!
      NT = MT
      MT = NOUT
      NOUT = NT
!
! - - LOOP BACK THRU THE SOLUTION
!
      NL = NF
      GO TO 119
!
! - - START TO WRAP IT UP
!
  200 REWIND NIN
      N2 = N
!
! * * NOTE.. AT THIS POINT ALL LOCATIONS A(1) THRU A(KORE) ARE FREE
!
      DO 220 IB = 1, NPASS
      READ (NIN) K
      N1 = N2 - K + 1
      NS = N1
      NT = N2
!
! - - READ IN THE SOLUTIONS
!
      DO 210 IO = 1, M
      READ (NIN) (A(NN), NN = NS, NT)
      NT = NT + N
  210 NS = NS + N
  220 N2 = N1 - 1
!
!  ---  REWIND ALL INPUT TAPES
      REWIND NIN
      REWIND MT
      REWIND NOUT
! - - WRITE THE SOLUTIONS ON TAPE
!
      NT = 0
      DO 230 IO = 1, M
      NS = NT + 1
      NT = NT + N
  230 WRITE (NW) (A(NN), NN = NS, NT)
!
      IF(LASTRS) GO TO 295
!*** ***IF THERE ARE MORE RHS TO BE GOTTEN FROM RHS TAPE, SWITCH TAPES
!*** ***BACK SO THAT  MT WILL CONAIN THE TRAPEZOIDAL MATRIX
!*** ***NATAPE WILL HAVE NOTHING USEFUL ON IT.
      NTEMP = NATAPE
      NATAPE = MT
      MT = NTEMP
      REWIND NATAPE
      REWIND LTAPE
      GO TO 109
  295 CONTINUE
!
!   ***   REWIND ALL FILES EXCEPT THE OUTPUT FILE   NW
!
      REWIND NI
      REWIND MM
      REWIND NO
      REWIND NAT
      REWIND RHSTAP
      MD = MTOTAL
 9999 CONTINUE
      RETURN
      END
      SUBROUTINE QUAS (ND,MD,KD,NI,MM,NO,NAT,NW,LTAPE,RHSTAP)
!   ***   REAL   SOLVIT     QUASI INVERSE
!
!           ****    ***/  *******  *****  *       ***/          *
!           *   *  *  /*     *     *      *      *  /*        ******
!     ****  ****   * / *     *     ***    *      * / *  ****     *  *
!           *      */  *     *     *      *      */  *           ***
!           *      /***      *     *      *****  /***            *  *
!
!             D I R E C T   M A T R I X   S O L U T I O N
!
!*** ***LTAPE IS THE TAPE THE L(I,J) MATRIX WILL BE PUT ON
!*** ***RHSTAP IS THE TAPE THAT THE RIGHT HAND SIDES ARE INPUT ON
      DIMENSION  A(3700)
      INTEGER RHSTAP
!
!
      LOGICAL JPASS1
      LOGICAL  LASTRS
      LOGICAL  LAST
!
    1 CONTINUE
      NATAPE = NAT
      REWIND NATAPE
      REWIND NW
      N = ND
      KORE = KD
!*** *  RHSTAP = 0 IF THERE ARE NO RHS TO BE PROCESSED THIS RUN
      IF(RHSTAP .NE. 0 )GO TO 5
      MRHS = 0
      GO TO 6
    5 REWIND RHSTAP
      READ(RHSTAP)MRHS
    6 M = KORE / N  -  1
      MMAX = MIN0(MRHS,M)
      NPM = N + MMAX
      IF (MAX0(3 * NPM, M * N) .GT. KORE) RETURN
      M = 0
      NPM = N
      MT = MM
      REWIND MT
      NIN = NI
      REWIND NIN
      NOUT = NO
      REWIND NOUT
      MP1 = M + 1
      NN = N
      NEL = NPM
!
! - - CALCULATE THE MAXIMUM NO. OF ROWS, @K@
!
   10 K = (KORE - NEL) / NEL
!
! - - TEST TO SEE IF THE REST OF THE MATRIX WILL FIT IN CORE
!
      LAST = K .GE. NN
      IF( .NOT. LAST ) GO TO 30
      K = NN
      B = 3 + MMAX*2
      C = 2 * (1 + MMAX - KORE )
      KTEMP = ( -B + SQRT(B**2  - 4.0* C )  )  / 2.0
      IF(KTEMP .GE. K)GO TO 30
!
!***   *  WE MUST REDUCE THE FINAL K
!
      K = KTEMP
      LAST = .FALSE.
!
! - - READ @K@ ROWS OF THE AUGMENTED @A@ MATRIX
!
   30 NT = 0
      DO 40 IB = 1, K
      NS = NT + 1
      NT = NT + NEL
   40 READ (NIN) (A(IO), IO = NS, NT)
!
! - - CHECK TO SEE IF WE WERE UNLUCKY ENOUGH TO END UP WITH ONLY ONE ROW
!
      IF (K .EQ. 1) GO TO 90
!
! - - @K@ IS GREATER THAN @1@ SO WE CAN START THE TRIANGULARIZATION
!
      NELP1 = NEL + 1
      NS = - NEL
      NELP2 = NELP1 + 1
!
! - - FORM THE @TRAPEZOIDAL@ ARRAY  (8)
!
      DO 50 IB = 2, K
      NP = NELP2 - IB
      NS = NS + NELP1
      NT = NS
      DO 50 IO = IB, K
      NT = NT + NEL
      MN = NT
      NB = NS
      A(NT) = A(NT) / A(NS)
      DO 50 NF = 2, NP
      MN = MN + 1
      NB = NB + 1
   50 A(MN) = A(MN) - A(NT) * A(NB)
!*** ***WRITE PART OF THE LMATRIX ON LTAPE   (TRIANGULAR PART)
      WRITE (LTAPE)K
      LBEG =NELP1
      KM1 = K - 1
      DO 55 IB = 1,KM1
      LEND = LBEG + IB- 1
      WRITE (LTAPE) ( A(L),L=LBEG,LEND)
   55 LBEG = LBEG + NN
!
! - - WRITE THE @TRAPEZOIDAL@ MATRIX ON TAPE
!
      NT = 0
      NP = NEL
      NS = - NEL
      DO 60 IO = 1, K
      NS = NS + NELP1
      NT = NT + NEL
      WRITE (MT) NP, (A(IB), IB = NS, NT)
   60 NP = NP - 1
      IF (LAST) GO TO 90
      NP = NP - M
      NS = KORE - NEL + 1
!
! - - READ ANOTHER ROW
!
      DO 80 IO = 1, NP
      READ (NIN) (A(IB), IB = NS, KORE)
!
! - - MODIFY THIS ROW BY THE @TRAPEZOIDAL@ ARRAY
!
      NT = 1
      MN = NS
      DO 70 IB = 1, K
      NB = NT
      NF = MN + 1
      A(MN) = A(MN) / A(NT)
      DO 65 NN = NF, KORE
      NB = NB + 1
   65 A(NN) = A(NN) - A(MN) * A(NB)
      MN = NF
   70 NT = NT + NELP1
!
! - - WRITE THE MODIFIED ROW ON TAPE
!
!*** ***WRITE REST OF LMATRIX ON LTAPE
      MNM1 = MN - 1
      WRITE(LTAPE)(A(J),J=NS,MNM1)
   80 WRITE (NOUT)     (A(NT), NT = MN, KORE)
      REWIND NOUT
      REWIND NIN
!
! - - SWITCH THE TAPES
!
      NT = NIN
      NIN = NOUT
      NOUT = NT
!
! - - RE-CALCULATE ROW LENGTH AND LOOP BACK
!
      NEL = NEL - K
      NN = NEL - M
      GO TO 10
!
! - - REWIND ALL TAPES
!
   90 REWIND NIN
      REWIND NOUT
!
  105 N1 = KORE - K * M + 1
      REWIND LTAPE
      REWIND MT
!*** ***CALCULATE THE NUMBER OF COLUMNS TO BRING OFF OF THE RHS TAPE
      MTOTAL = 0
      M = MMAX
      IF(M .EQ. 0)GO TO 295
!*** ***MTOTALIS THE TOTAL NUMBER OF RHS COLUMNS ALREADY BROUGHT IN
  109 MTOTAL = MTOTAL + M
      LASTRS = MTOTAL.GE. MRHS
      MTOTAL = MTOTAL - M
      IF (LASTRS)M = MRHS - MTOTAL
      MTOTAL = MTOTAL + M
!*** ***BRING IN M COLUMNS OF RHS
      KINIT = KORE - (M*N)
      IINIT = KINIT
      NBEG = KINIT + 1
      NEND = KINIT+ N
      DO 110 J =1,M
      READ (RHSTAP)( A(I),I=NBEG,NEND)
      NBEG = NEND + 1
  110 NEND = NEND + N
!*** ***BRING IN L(I,J) MATRIX AND APPLY IT TO RHS
      NBEG = 1 + KINIT
      NEND = 1 + (M-1) * N + KINIT
      KSUM = 0
!*** ***DO TRIANGULAR SECTION OF LMATRIX
      NLCNT = 0
  111 READ (LTAPE)K
      NLCNT = NLCNT + 1
!*** ***KSUM IS THE TOTAL NUMBER OF L ROWS THAT WILL
!*** ***BE READ AFTER THIS TRIANGULAR SECTION IS FINISHED
      KSUM = KSUM + K
      KM1 = K - 1
!***   ***NOTE THAT KM1 CAN@T BE 0 SINCE K CAN@T BE 1 AND STILL HAVE SOM
!***   ***ON THE LTAPE
      DO 114  I = 1,KM1
      NBEG = NBEG + 1
      NEND = NEND + 1
!*** ***READ 1 ROW OF L(I,J) FROM LTAPE---K-1 TIMES---EACH TIME
!*** ***STARTING WITH L(1)
 1114 CONTINUE
      READ (LTAPE) ( A(NN),NN=1,I)
      JCNT = -1
!*** ***REDUCE THE RHS BY GOING ACROSS A SOLUTION ROW (WHICH
!*** ***ARE NOT IN CONSECUTIVE ORDER, BUT A(1), A(N+1), A(2N+1) ETC.)
      DO 113 NPP = NBEG,NEND,N
      JCNT = JCNT + 1
      SUM = 0.0
      NROW = KINIT + ( JCNT * N )
      DO 112 NN=1,I
      NROW = NROW + 1
  112 SUM = SUM + ( A(NN)*A(NROW))
  113 A(NPP) = A(NPP) - SUM
  114 END DO
      IF (KSUM .EQ. N)GO TO 116
!*** ***KSUM = N IF YOU HAVE READ ENTIRE LMATRIX AND
!*** ***THERE IS NO CONSTANT SECTION LEFT
      NTBEG = NBEG
      NTEND = NEND
      KSUMP1 = KSUM + 1
!*** ***READ REST OF LROWS 1 ROW AT A TIME FOR CONSTANT SECTION
      DO 115 I=KSUMP1,N
      NTBEG = NTBEG + 1
      NTEND = NTEND + 1
 1115 CONTINUE
      READ (LTAPE)( A(NN),NN=1,K)
      JCNT = -1
!*** ***PARTIALLY  REDUCE A RHS ACROSS A RHS ROW BY APPLYING K NUMBER
!*** ***OF L(I,J) S
      DO 124 NPP = NTBEG,NTEND,N
      JCNT = JCNT + 1
      SUM = 0.0
      NROW = KINIT + ( JCNT * N )
      DO 123 NN = 1,K
      NROW = NROW + 1
  123 SUM = SUM + ( A(NN) * A(NROW) )
  124 A(NPP) = A(NPP) - SUM
  115 END DO
      NBEG = NBEG + 1
      NEND = NEND + 1
!*** ***KINIT IS HOW FAR DOWN A COLUMN OF RHS TO START MULTIPLYING BY
!*** ***L(I,J) AT EACH PASS THROUGH
      KINIT = KINIT + K
!*** ***IF KSUMP1 = N THERE ARE NO MORE L(I,J)@S LEFT
      IF(KSUMP1 .LT. N)GO TO 111
!*** ***WRITE OUT ALL BUT LAST K ROWS OF RHS IN ROW ORDER ON NATAPE
  116 B = 4*M + 3
      C = -2 * KORE
      K = (  -B  +  SQRT( B**2  -4.0*C )  )/2.0
      IF(K .GT. ND) K = ND
      KF = K
      KM1 = K - 1
      KLEFT = N - KF + IINIT
      INITP1 = IINIT + 1
      NEND = (M-1)*N + IINIT
      DO 117 NPP = INITP1,KLEFT
      NEND = NEND + 1
  117 WRITE(NATAPE) ( A(J),J=NPP,NEND,N)
      REWIND NATAPE
!*** ***JPASS1 IS TRUE ON 1ST PASS THRU BACK SOLUTION
      JPASS1 = .TRUE.
!
!***   ***PUT REMAINING RHS IN CONTIGUOUS LOCATIONS BY COLUMNS
!***  FROM KORE - (M * KF) + 1  TO  KORE
!
      NNEW = KORE - KF + 1
      MM1 = M - 1
!
!***   ***IF M = 1, THE ELTS OF THE 1 RHS COLUMN ARE ALREADY IN CONTIGUO
!***   ***LOCATIONS
!
      IF (M.EQ.1)  GO TO 1118
      DO 118 I = 1,MM1
      NOLD = KORE - (I*N) + 1
      DO 118 J = 1,KF
      NNEW = NNEW - 1
      NOLD = NOLD - 1
      A(NNEW) = A(NOLD)
  118 CONTINUE
 1118 CONTINUE
!
!***   ***NOW NNEW = KORE - (M*KF) + 1
!***   ***NOW NOLD = KORE - (M - 1) * N + 1 - KF
!   ***   SKIP 1ST PART OF TRAPEZOIDAL MATRIX + READ LAST K ROWS
!*** ***ATTATCH RHS TO IT SO THAT EVERYTHING IS IN CONSECUTIVE ORDER
      NREMAN = ND - K
      IF(NREMAN .EQ. 0)GO TO 126
      DO 122 I = 1,NREMAN
  122 READ(MT) IDUMMY
  126 NEND = 0
      KCNT = K
      NNEW = NNEW - 1
!***   ***NOTE THAT K = KF WHICH IS ALREADY KNOWN IN CORE
      DO 121 JCNT = 1,K
      NBEG = NEND + 1
      KCNT = KCNT - 1
      NEND = NBEG + KCNT
      READ(MT)IDUMMY,(A(J),J=NBEG,NEND)
      NNEW = NNEW + 1
      KEND = (MM1 * KF) + NNEW
      DO 121 NPP=NNEW,KEND,KF
      NEND = NEND + 1
  121 A(NEND) = A(NPP)
      REWIND LTAPE
      REWIND MT
!
! - - THERE, NOW WE CAN START THE BACK-SOLUTION
! * * NOTE..THE FIRST AVAILABLE LOCATION FOR THE SOLUTIONS IS A(N1)
!
!
!***   ***NL IS THE LAST SUBSCRIPT + 1 OF THE TRAPEZOIDAL A MATRIX THAT
!***   ***CORE
!
      NL = NEND + 1
      NREM = N
      NPM = N + M
      NEL = NPM
      MP1 = M + 1
      LAST = K .EQ. N
      NPASS = 0
!
! - - SOLVE FOR THE ANSWERS CORRESPONDING TO @K@ ROWS
!
  119 KM1 = K - 1
      KP1 = K + 1
      NS = NL - MP1
      NPASS = NPASS + 1
      DO 130 MN = 1, M
      NF = NS + MN
      A(NF) = A(NF) / A(NS)
      NT = NS
      IF (KM1 .EQ. 0) GO  TO 130
      DO 125 IB = 1, KM1
      NF = NF - IB - M
      NT = NT - MP1 - IB
      SUM = 0.0
      NP = NF
      N2 = MP1 + IB
      DO 120 IO = 1, IB
      NN = NT + IO
      NP = NP + N2 - IO
  120 SUM = SUM + A(NN) * A(NP)
  125 A(NF) = (A(NF) - SUM) / A(NT)
  130 END DO
!
! - - MOVE THE SOLUTIONS TO CONTIGUOUS LOCATIONS STARTING AT A(N1)
!
      N1 = KORE + 1
      DO 140 NN = 1, K
      DO 135 MN = 1, M
      NL = NL - 1
      N1 = N1 - 1
  135 A(N1) = A(NL)
  140 NL = NL - NN
!
! - - WRITE THE SOLUTIONS ON TAPE
!
      WRITE (NIN) K
      NS = N1 - 1
      DO 145 MN = 1, M
      NT = NS + MN
  145 WRITE ( NIN ) (A(IO), IO = NT, KORE, M)
!
! - - TEST IF THIS IS THE LAST PASS
!
      IF (LAST) GO TO 200
!
! - - WE MUST NOW MODIFY THE TRIANGULAR MATRIX TO REFLECT THE EFFECT OF
!     THE SOLUTIONS OBTAINED SO FAR  (EQ 21)
! * * NOTE..LOCATIONS A(1) TO A(N1-1) ARE NOW FREE TO USE
!
! - - CALCULATE THE NEXT VALUES OF @NEL@ AND @NREM@
!
      NELOLD = NEL
      KOLD = K
      NEL = NEL - K
      NREM = NREM - K
!
      NROW = NREM - K + 1
      IF (K .LT. NREM) GO TO 150
      LAST = .TRUE.
      NROW = 1
      K = NREM
  150 NS = 1
      NT = NELOLD + 1
!
! - - READ IN THE ROWS TO BE MODIFIED
!
      DO 190 IB = 1, NREM
      NT = NT - 1
      IF (IB .LE. NROW) GO TO 160
      NS = NS + NN
      NT = NT + NN
  160 IF (.NOT. JPASS1)GO TO 161
      NBEG = NT - M + 1
!*** ***READ RHS FROM NATAPE
      READ (NATAPE) ( A(IO),IO = NBEG,NT)
      NT = NT - M
  161 READ(MT)NN,(A(IO),IO=NS,NT)
      IF( .NOT. JPASS1 ) GO TO 163
      NT = NT + M
      NN = NN + M
  163 NP = N1 - 1
      NF = NT - M - KM1
      NN = NN - KOLD
      DO 170 MN = 1, M
      N2 = NF
      NA = NP + MN
      NB = NA
      SUM = 0.0
      DO 165 IO = 1, KOLD
      SUM = SUM + A(N2) * A(NA)
      N2 = N2 + 1
  165 NA = NA + M
      N2 = N2 + MN - 1
  170 A(N2) = A(N2) - SUM
!
! - - WRITE THE MODIFIED ROW ON TAPE OR CONDENSE THE ROW
!
      NL = NT - M + 1
      IF (IB .GE. NROW) GO TO 175
      NF = NL - KP1
      WRITE (NOUT) NN, (A(IO), IO = NS, NF), (A(IO), IO = NL, NT)
      GO TO 190
  175 NF = NL - KOLD
      DO 180 MN = NL, NT
      A(NF) = A(MN)
  180 NF = NF + 1
  190 END DO
!*** ***IF 1ST TIME THRU BACK SOLN,SWITCH TAPES SO THAT MT WHICH HAS THE
!*** ***ORIGINAL TRAPEZOIDAL MATRIX ON IT BECOME NATAPE AND IS NOT TO
!*** ***TAPE PART IN ALTERNATING SHRINKING MATRICES.  NATAPE BECOMES MT
!*** ***AND THIS NOW DOES THE ALTERNATING WITH NOUT.
      IF( .NOT. JPASS1 ) GO TO 195
      NTEMP = MT
      MT = NATAPE
      NATAPE = NTEMP
      JPASS1= .FALSE.
      REWIND NATAPE
  195 REWIND MT
      REWIND NOUT
!
! - - SWITCH THE TAPES
!
      NT = MT
      MT = NOUT
      NOUT = NT
!
! - - LOOP BACK THRU THE SOLUTION
!
      NL = NF
      GO TO 119
!
! - - START TO WRAP IT UP
!
  200 REWIND NIN
      N2 = N
!
! * * NOTE.. AT THIS POINT ALL LOCATIONS A(1) THRU A(KORE) ARE FREE
!
      DO 220 IB = 1, NPASS
      READ (NIN) K
      N1 = N2 - K + 1
      NS = N1
      NT = N2
!
! - - READ IN THE SOLUTIONS
!
      DO 210 IO = 1, M
      READ (NIN) (A(NN), NN = NS, NT)
      NT = NT + N
  210 NS = NS + N
  220 N2 = N1 - 1
!
!  ---  REWIND ALL INPUT TAPES
      REWIND NIN
      REWIND MT
      REWIND NOUT
! - - WRITE THE SOLUTIONS ON TAPE
!
      NT = 0
      DO 230 IO = 1, M
      NS = NT + 1
      NT = NT + N
  230 WRITE (NW) (A(NN), NN = NS, NT)
!   ***   IF TAPE WAS NEVER SWITCHED IT WOULD BE FOOLISH TO SWITCH BACK
      IF(JPASS1)GO TO 290
!
!***   ***SWITCH TAPES
!*** ***BACK SO THAT  MT WILL CONAIN THE TRAPEZOIDAL MATRIX
!*** ***NATAPE WILL HAVE NOTHING USEFUL ON IT.
      NTEMP = NATAPE
      NATAPE = MT
      MT = NTEMP
      REWIND NATAPE
  290 IF( .NOT. LASTRS)GO TO 109
  295 REWIND LTAPE
      REWIND MT
      KRED = 0
      DO 297 I=1,NLCNT
      READ(LTAPE)KREAD
  304 CONTINUE
      KRED = KRED + KREAD
      KREAD = KREAD + ( N - KRED - 1)
      DO 297 LREAD=1,KREAD
  297 READ(LTAPE)
      DO 299 NROW = 1,ND
      READ(MT) ICNT,(A(I),I=1,ICNT)
  299 WRITE(LTAPE)ICNT,(A(I),I=1,ICNT)
!
!   ***   REWIND ALL FILES EXCEPT THE OUTPUT FILE   NW
!
      REWIND NI
      REWIND MM
      REWIND NO
      REWIND NAT
      IF( RHSTAP .NE. 0 ) REWIND RHSTAP
      MD = MTOTAL
 9999 CONTINUE
      RETURN
      END
      SUBROUTINE INV(NBOX,INPUT,LTAPE,NTP3,NTP4,NTP8,NTP9,NTP1,NTP2,    &
     &ISIZE,VEC,BUF,KEY)
      DIMENSION VEC(1), BUF(1)
!
!     INPUT = UNIT CONTAINING MATRIX TO BE INVERTED, POSITIONED
!             TO FIRST RECORD
!     NTP1 = UNIT CONTAINING MATRIX TO BE INVERTED( ROW SORT)
!     NTP3 = UNIT CONTAINING INVERTED MATRIX ( COLUMN SORT )
!     LTAPE = UNIT CONTAINING INVERTED MATRIX ( ROW SORT )
!     NBOX = ORDER OF MATRIX TO BE INVERTED ( MAX = 1200 )
!
      KD = 3700
      ND = NBOX
      M  =  0
      REWIND NTP1
      DO 5 I = 1, NBOX
      READ(INPUT) ( VEC(K) , K = 1, NBOX )
      WRITE( NTP1 ) ( VEC(K), K = 1, NBOX )
    5 END DO
      REWIND NTP1
      REWIND NTP2
      REWIND NTP3
      REWIND NTP4
      REWIND NTP8
      REWIND NTP9
      REWIND LTAPE
      WRITE(NTP4) NBOX, NBOX
      DO 20 I = 1, NBOX
      DO 10 J = 1, NBOX
      VEC(J) = 0.0
      IF( J .EQ. I ) VEC(J) = 1.0
   10 END DO
      WRITE(NTP4) ( VEC(K), K = 1, NBOX )
   20 END DO
      REWIND NTP4
      IWHICH = 1
      CALL QUAS (ND,M,KD,NTP1,NTP8,NTP9,NTP2,NTP3,LTAPE,NTP4)
      IF (M .EQ. 0) GO TO 100
      IF (M .EQ.  NBOX) IWHICH=2
   90 CONTINUE
      REWIND NTP3
      GO TO (100,105), IWHICH
  100 CONTINUE
      CALL FUTSOL (ND,M,KD,NTP1,NTP8,NTP9,NTP2,NTP3,LTAPE,NTP4)
      IF (M .EQ.  NBOX) IWHICH = 2
      GO TO 90
  105 CONTINUE
      REWIND NTP3
      IF( KEY .EQ. 0 ) RETURN
      REWIND LTAPE
      REWIND NTP8
      CALL TRPOSE(NTP3,BUF,VEC,NBOX,NBOX,LTAPE,NTP8,ISIZE)
      REWIND LTAPE
      REWIND NTP3
      RETURN
      END
      SUBROUTINE OUTW (NTAPEO,Z)
!     WING OUPUT FORMAT
      DIMENSION Z(1),OUT(30,20)
      COMMON /ROWS/ NWCOL, NWRPC(20)
      COMMON /SEG/ NSEG, NROW(20),NCOL(20),COSS(20),SINS(20),BTE(20)    &
     &,NWT(20),SPNW(20),XLEW(20),BLE(20),ZLEW(20)
      DO 30 I=1,30
      DO 30 J=1,20
   30 OUT(I,J) = 1.0E+30
      NCT=NWCOL
      KOUNT=0
      DO 20 KJ=1,NWCOL
      KI=NWRPC(KJ)
      DO 20 I=1,KI
      KOUNT=KOUNT+1
   20 OUT( I,KJ) = Z(KOUNT)
      LL2=1
      NEND = NCT/10 + 1
      DO 105 LJ=1,NEND
      IF(((1+10*(LJ-1))-NCT)*(NCT-10*LJ).GE.0) GO TO 106
  105 END DO
      L7=0
  106 DO 103 II=1,LJ
      NRMAX=0
      NC = MIN0(10*II,NCT)
      L7=1+10*(II-1)
      LL=LL2
      NCS=0
      DO 102 JJ=LL,NWCOL
      NCS=NCS+1
      IF(NRMAX.LT.NWRPC(JJ)) NRMAX=NWRPC(JJ)
      IF(NCS-NC)102,6,6
    6 LL2=JJ+1
      GO TO 4
  102 END DO
    4 WRITE(NTAPEO,6005) (MM,MM=L7,NC)
      WRITE(NTAPEO,6010)
      IGO = NC-L7+1
      IF(IGO.EQ.0) RETURN
      GO TO (51,52,53,54,55,56,57,58,59,60), IGO
   51 WRITE(6,510) (I,(OUT(I,J),J=L7,NC),I=1,NRMAX)
  510 FORMAT(1H ,I10,9X,  F10.5)
      GO TO 103
   52 WRITE(6,520) (I,(OUT(I,J),J=L7,NC),I=1,NRMAX)
  520 FORMAT(1H ,I10,9X, 2F10.5)
      GO TO 103
   53 WRITE(6,530) (I,(OUT(I,J),J=L7,NC),I=1,NRMAX)
  530 FORMAT(1H ,I10,9X, 3F10.5)
      GO TO 103
   54 WRITE(6,540) (I,(OUT(I,J),J=L7,NC),I=1,NRMAX)
  540 FORMAT(1H ,I10,9X, 4F10.5)
      GO TO 103
   55 WRITE(6,550) (I,(OUT(I,J),J=L7,NC),I=1,NRMAX)
  550 FORMAT(1H ,I10,9X, 5F10.5)
      GO TO 103
   56 WRITE(6,560) (I,(OUT(I,J),J=L7,NC),I=1,NRMAX)
  560 FORMAT(1H ,I10,9X, 6F10.5)
      GO TO 103
   57 WRITE(6,570) (I,(OUT(I,J),J=L7,NC),I=1,NRMAX)
  570 FORMAT(1H ,I10,9X, 7F10.5)
      GO TO 103
   58 WRITE(6,580) (I,(OUT(I,J),J=L7,NC),I=1,NRMAX)
  580 FORMAT(1H ,I10,9X, 8F10.5)
      GO TO 103
   59 WRITE(6,590) (I,(OUT(I,J),J=L7,NC),I=1,NRMAX)
  590 FORMAT(1H ,I10,9X, 9F10.5)
      GO TO 103
   60 WRITE(6,600) (I,(OUT(I,J),J=L7,NC),I=1,NRMAX)
  600 FORMAT(1H ,I10,9X,10F10.5)
  103 END DO
 6005 FORMAT(1H0,2X,16HSPANWISE STATION,I8,9I10/(18X,I8,9I10))
 6010 FORMAT(1H ,2X,17HCHORDWISE STATION)
      RETURN
      END
      SUBROUTINE WINGS(IWING)
!
!          KEEPS TRACK OF WING/TAIL AND NACELLE/SHELL PARAMETERS
!
!          IWING = +/-1 REFERS TO WING/TAIL
!
!                = +/-2 REFERS TO NACELLE/SHELL
!
!                =   +3 REFERS TO NACELLE/SHELL,
!                       READ IN AS BODY
!
!          IWING ) 0 WRITES DATA ON TAPE
!
!                # 0 READS DATA FROM TAPE
!
      DIMENSION ADUM(20),NADUM(20)
      COMMON /PARAM/ NBODY,NWING,NTAIL,LBC,THK,MACH,ALPHA,REF(7)
      COMMON /SEG/ NSEG,NROW(20),NCOL(20),COSS(20),SINS(20),BTE(20),    &
     & NWT(20),SPNW(20),XLEW(20),BLE(20),ZLEW(20)
      COMMON /NEWCOM/ KDUM(44),KOL,NCPT,LOCPT(20),XCPT(20)
      COMMON /TAPES/ ITAPES(20)
      COMMON /VELCOM/ NPOINT,NPART,IMAX,JMAX,NMAX,EM,PRINT,             &
     & NWTHK,NWBLOK,NWROW(20),NBBLOK,NBROW(60)
      INTEGER PRINT
      REAL MACH
      LOGICAL LBC,THK,LDUM
!
      IUNIT=0
      IF(IABS(IWING).EQ.1) IUNIT=ITAPES(13)
      IF(IABS(IWING).EQ.2) IUNIT=ITAPES(14)
      IF(IWING.EQ.3) IUNIT=ITAPES(14)
      IF(IUNIT.EQ.0) GO TO 999
      IF(IWING.LT.0) GO TO 10
      IF(IWING.GT.0) GO TO 20
      GO TO 999
   10 REWIND IUNIT
      WRITE(IUNIT) NBODY,NWING,NTAIL,LBC,THK
      WRITE(IUNIT) NSEG,NROW,NCOL,COSS,SINS,BTE,NWT,SPNW,XLEW,          &
     &             BLE,ZLEW
      WRITE(IUNIT) KOL,NCPT,LOCPT,XCPT
      WRITE(IUNIT) NWTHK,NWBLOK,NWROW,NBBLOK,NBROW
      RETURN
   20 REWIND IUNIT
      IF(IWING.EQ.3) GO TO 30
      READ(IUNIT) NBODY,NWING,NTAIL,LBC,THK
      READ(IUNIT) NSEG,NROW,NCOL,COSS,SINS,BTE,NWT,SPNW,XLEW,           &
     &             BLE,ZLEW
      READ(IUNIT) KOL,NCPT,LOCPT,XCPT
      READ(IUNIT) NWTHK,NWBLOK,NWROW,NBBLOK,NBROW
      RETURN
   30 READ(IUNIT) NDUM,NBODY,NDUM,LDUM,LDUM
      READ(IUNIT) NDUM,NADUM,NADUM,ADUM,ADUM,ADUM,NADUM,ADUM,ADUM,      &
     & ADUM,ADUM
      READ(IUNIT) NDUM,NDUM,NADUM,ADUM
      READ(IUNIT) NDUM,NBBLOK,(NBROW(N),N=1,20)
      RETURN
  999 WRITE(6,600) IWING
  600 FORMAT(1H1,26HERROR IN WINGS,     IWING=,I3)
      RETURN
      END


!!!     OVERLAY(LWB,1,0)
      SUBROUTINE GEOM()
!
!     *******************************************VERSION B00************
!
!
!     INPUT CONFIGURATION GEOMETRY AND COMPUTE PANELS
!
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAP&
!!!     &E9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      INCLUDE 'files.cmn'
      COMMON /BLANK/ J0,J1,J2,J3,J4,J5,J6,NWAF,NWAFOR,NFUS,NRADX(4),NFOR&
     &X(4),NP,NPODOR,NF,NFINOR,NCAN,NCANOR,J2TEST,NW,NC
      COMMON /BLANK2/ HORZ,VERT,TEST1,PHI,THETA,PSI,XF,YF,ZF,DIST,FMAG,P&
     &LOTSZ,TYPE,KODE,XMIN,XMAX,YMIN,YMAX,ZMIN,ZMAX,XMID,YMID,ZMID,BIGD,&
     &ISP
      COMMON /LWB/ LWB
      COMMON /PARAM/ NBODY,NWING,NTAIL,LBC,THK,MACH,ALPHA,REFA,REFB,REFC&
     &,REFD,REFL,REFX,REFZ

!!!      COMMON /HEAD/ TITLE1(14),TITLE2(14)
      INCLUDE 'head.cmn'

      COMMON /SEG/ NSEG,NROW(20),NCOL(20),COSS(20),SINS(20),BT(20)
      COMMON /SCRAT/ BLOCK(7500)
      COMMON /NEWCOM/K0,K1,K2,K3,K4,K5,K6,KWAF,KWAFOR,KFUS,KRADX(4),     &
     &              KFORX(4),KF(6),KFINOR(6),KAN(6),KANOR(6),KRAD,MAX,  &
     &              KOL,NCPT
      COMMON /VELCOM/ DUM1(5),EM,PRINT
      COMMON /ITERAT/ ITMAX,CCTEST,DCTEST,ITMETH,ALF1,ALF2
      COMMON /GRAPH/ PLOT
      COMMON /PRESS/ IKRAD(4),INRAD(4),IKFORX(4),INFORX(4),IKWAF,INWAF,I&
     &KWAFR,INWAFR,IKFUS
      COMMON /PTYPE/ ORT,VU3,STE,PER
      COMMON /STAR/XSTAR(3600),NSEGW
      COMMON /SHELL/ NPTSB,NSEGWT,NPTSWT,NSEGN,NPTSN,NSEGS,NPTSS,       &
     &               NSEGNS,NPTSNS,NNAC,NSHELL,NQ,IPART,iidum
!
      LOGICAL LBC,THK,TAIL
!
      INTEGER THICK,PRINT,PLOT   !!!,TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7&
!!!     &,TAPE8,TAPE9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
  INTEGER:: FFILE
      INTEGER:: errCode
!
      LBC=.FALSE.
      THK=.FALSE.
      EM=-1.
      PRINT=0
      NCPT=0
      NBODY=0
      NWING=0
      NTAIL=0
      NSEG=0
      KOL=0
      REFB=1.0
      REFC=1.0
      REFD=1.0
      REFL=1.0
      REFX=0.
      REFZ=0.
      ITMAX=0
      CCTEST=0.0
      DCTEST=0.0
      ITMETH=0
      REWIND TAPE3
      REWIND TAPE7
      REWIND TAPE8
      REWIND TAPE9
      REWIND TAPE10
      REWIND TAPE12
      REWIND TAPE13
!
!     INPUT CONFIGURATION PARAMETERS
!
      READ (TAPE5,'(A)',IOSTAT=errCode) TITLE1
!
!     END OF FILE TEST ON TAPE5
!
      IF (errCode < 0) GO TO 30
!!!      IF(EOF(TAPE5)) 30,10
!!!   10 CONTINUE
      WRITE (TAPE6,*) TITLE1
      READ (TAPE5,220) J0,J1,J2,J3,J4,J5,J6,NWAF,NWAFOR,NFUS,           &
     & (NRADX(I),NFORX(I),I=1,4),NP,NPODOR,NF,NFINOR,NCAN,NCANOR,PLOT
      DO 20 I=1,4
      INRAD(I)=NRADX(I)
      INFORX(I)=NFORX(I)
   20 END DO
      INWAF=NWAF
      INWAFR=NWAFOR
      IKFUS=NFUS
      REWIND TAPE11
      WRITE (TAPE11) J0,J1,J2,J3,J4,J5,J6
      GO TO 40
   30 CONTINUE
      CALL CALPLT (0.,0.,999)
      STOP   !!! CALL EXIT
!
!     INPUT DESCRIPTION AND INITIALIZATION
!
!!!   40 CALL OVERLAY (LWB,1,1)
   40 CALL Config()
!
!     INPUT GEOMETRY PANEL PLOT CONTROL CARDS
!
      IF (PLOT.GE.0) GO TO 60
   50 READ (TAPE5,240) HORZ,VERT,TEST1,PHI,THETA,PSI,XF,YF,ZF,DIST,FMAG,&
     &PLOTSZ,TYPE,KODE
      IF (TYPE.EQ.ORT.OR.TYPE.EQ.VU3.OR.TYPE.EQ.STE.OR.TYPE.EQ.PER) WRIT&
     &E (TAPE13,240) HORZ,VERT,TEST1,PHI,THETA,PSI,XF,YF,ZF,DIST,FMAG,PL&
     &OTSZ,TYPE,KODE
      IF (KODE.EQ.0) GO TO 50
   60 CONTINUE
!
!     SET BOUNDARY CONDITION AND WING THICKNESS OPTIONS
!
      READ (TAPE5,'(A)') TITLE2
      READ (TAPE5,180) LINBC,THICK,PRINT,LCPA,LCPB,ITMETH,ITMAX,CCTEST,D&
     &CTEST,ALF1,ALF2
      IF (LINBC.GT.0) LBC=.TRUE.
      IF (LBC.AND.THICK.GT.0) THK=.TRUE.
!
!     INPUT REVISED CONFIGURATION PANELING DESCRIPTION CONTROL INTEGERS
!
      READ (TAPE5,220) K0,K1,K2,K3,K4,K5,K6,KWAF,KWAFOR,KFUS,(KRADX(I),K&
     &FORX(I),I=1,4)
      DO 70 I=1,4
      IKRAD(I)=KRADX(I)
      IKFORX(I)=KFORX(I)
   70 END DO
      IKWAF=KWAF
      IKWAFR=KWAFOR
      WRITE (TAPE11) K0,K1,K2,K3,K4,K5,K6
      REWIND TAPE11
      TAIL=.FALSE.
      IF (K4.GT.0.OR.K5.GT.0) TAIL=.TRUE.
      IF (.NOT.TAIL) GO TO 80
      READ (TAPE5,220) (KF(I),KFINOR(I),I=1,6),(KAN(I),KANOR(I),I=1,6)
   80 CONTINUE
      READ (TAPE9) REFA
      IF (K0.EQ.0) GO TO 90
      READ (TAPE5,200) REFAR,REFB,REFC,REFD,REFL,REFX,REFZ
      IF (REFAR.NE.0.) REFA=REFAR
      IF (REFB.EQ.0.) REFB=1.0
      IF (REFC.EQ.0.) REFC=1.0
      IF (REFD.EQ.0.) REFD=1.0
      IF (REFL.EQ.0.) REFL=1.0
   90 CONTINUE
      READ (TAPE9) BLOCK
      IF (K1.EQ.0) GO TO 100
!
!     REVISE CHORDWISE PANEL SPACING ON WING AND COMPUTE NEW AIRFOIL
!     ORDINATES
!
!!!      CALL OVERLAY (LWB,1,2)
      CALL NewOrd()
!
!     REVISE SPANWISE PANEL SPACING ON WING AND COMPUTE NEW PANEL
!     GEOMETRY
!
!!!      CALL OVERLAY (LWB,1,3)
      CALL WngPan()
!
  100 CONTINUE
      READ (TAPE9) BLOCK
      IF (K2.EQ.0) GO TO 120
      IF (KRADX(1).LE.21) GO TO 110
      WRITE (TAPE6,170)
      WRITE (TAPE6,230)
      STOP !!! CALL EXIT
  110 CONTINUE
!
!     REVISE BODY (FUSELAGE) MERIDIAN LINE SPACING
!
!!!      CALL OVERLAY (LWB,1,4)
      CALL NewRad()
!
!     REVISE AXIAL PANEL SPACING ON BODY (FUSELAGE) AND COMPUTE NEW
!     PANEL GEOMETRY
!
!!!      CALL OVERLAY (LWB,1,5)
      CALL BodPan()
!
  120 CONTINUE
      READ (TAPE9) BLOCK
      IF (K4.EQ.0) GO TO 130
!
!     REVISE CHORDWISE PANEL SPACING ON FIN (VERTICAL TAIL) AND COMPUTE
!     NEW AIRFOIL ORDINATES
!
!!!      CALL OVERLAY (LWB,1,6)
      CALL NutOrd()
!
!     REVISE SPANWISE PANEL SPACING ON FIN (VERTICAL TAIL) AND COMPUTE
!     NEW PANEL GEOMETRY
!
!!!      CALL OVERLAY (LWB,1,7)
      CALL TalPan()
!
  130 CONTINUE
      READ (TAPE9) BLOCK
      IF (K5.EQ.0) GO TO 140
!
!     REVISE CHORDWISE PANEL SPACING ON CANARD (HORIZONTAL TAIL) AND
!     COMPUTE NEW AIRFOIL ORDINATES
!
!!!      CALL OVERLAY (LWB,1,6)
      CALL NutOrd()
!
!     REVISE SPANWISE PANEL SPACING ON CANARD (HORIZONTAL TAIL) AND
!     COMPUTE NEW PANEL GEOMETRY
!
!!!      CALL OVERLAY (LWB,1,7)
      CALL TalPan()
!
  140 CONTINUE
      NSEGWT=NSEG
      CALL WINGS(-1)
!
!          INPUT NACELLE AND/OR/SHELL GEOMETRY
!
      THK=.FALSE.
      LBC=.TRUE.
      NCPT=0
      NWING=0
      NTAIL=0
      NWTHK=0
      NSEG=0
      KOL=0
      NNAC=0
      NSHELL=0
      IF(K6.EQ.0) GO TO 145
      READ(TAPE5,220) NNAC,NSHELL
  145 NQ=NNAC+NSHELL
!
!          NACELLE
!
      KDUM=K6
      K6=0
!!!      CALL OVERLAY(LWB,1,8)
      CALL Nacshl()
      NSEGN=NSEG
!
!          SHELL
!
      K6=1
!!!      CALL OVERLAY(LWB,1,8)
      CALL Nacshl()
      NSEGNS=NSEG
      NSEGS=NSEGNS-NSEGN
      K6=KDUM
      CALL WINGS(-2)
!
!     INPUT SINGULARITY PANEL PLOT CONTROL CARDS
!
      IF (PLOT.EQ.0) GO TO 160
  150 CONTINUE
      READ (TAPE5,240) HORZ,VERT,TEST1,PHI,THETA,PSI,XF,YF,ZF,DIST,FMAG,&
     &PLOTSZ,TYPE,KODE
      IF (TYPE.EQ.ORT.OR.TYPE.EQ.VU3.OR.TYPE.EQ.STE.OR.TYPE.EQ.PER) WRIT&
     &E (TAPE13,240) HORZ,VERT,TEST1,PHI,THETA,PSI,XF,YF,ZF,DIST,FMAG,PL&
     &OTSZ,TYPE,KODE
      IF (KODE.EQ.0) GO TO 150
  160 CONTINUE
      REWIND TAPE9
!
!
!
  170 FORMAT (1H0,24H ERROR MESSAGE FROM GEOM)
  180 FORMAT (7I3,4F7.0)
  190 FORMAT (13A6,A2)
  200 FORMAT (10F7.0)
!!!  210 FORMAT (1H1,10X,13A6,A2)
  220 FORMAT (25I3)
  230 FORMAT (1H0,46HERROR- BODY HAS MORE THAN 20 COLUMNS OF PANELS)
  240 FORMAT (2(A1,1X),A3,9F5.0,A3,16X,I1)
      END
      SUBROUTINE PANEL (IP,IQ,J,K,L,NP,AP)
!
!     *******************************************VERSION B00************
!
!
!     CALCULATE PANEL GEOMETRY (BASED ON THE HYPERSONIC ARBITRARY BODY
!     PROGRAM OF A. E. GENTRY)
!
      COMMON /POINT/ ARRAY(6000)
      COMMON /SCRAT/ BLOCK(7500)
      COMMON /EPS/ EPSPAN,EPSWP1,EPSWP2,EPSTP,EPSVC,EPSSP1,EPSSP2,EPSLV,&
     &EPSSV,EPSVV,EPSWV,EPSVP,EPSIT1,EPSIT2,EPSFM, epssb1,epssb2
!
      DIMENSION XPT(600), YPT(600), ZPT(600), THET(600), DELTA(600)
      DIMENSION XC(30,20), YC(30,20), ZC(30,20), ZU(30,20)
      DIMENSION XIN(4), YIN(4), ZIN(4), XI(4), ETA(4)
!
      EQUIVALENCE (ARRAY(1),XPT(1)),      (ARRAY(601),YPT(1))
      EQUIVALENCE (ARRAY(1201),ZPT(1)),   (ARRAY(1801),THET(1))
      EQUIVALENCE (ARRAY(2401),DELTA(1)), (ARRAY(3001),XC(1,1)) 
      EQUIVALENCE (ARRAY(3601),YC(1,1)),  (ARRAY(4201),ZC(1,1))
      EQUIVALENCE (BLOCK(6901),ZU(1,1))
!
      REAL NX,NY,NZ
!
!     REORDER THE PANEL CORNER POINTS TO CORRESPOND TO GENTRY CONVENTION
!
      J1=J-1
      K1=K-1
      XIN(1)=XC(J1,K1)
      XIN(2)=XC(J,K1)
      XIN(3)=XC(J,K)
      XIN(4)=XC(J1,K)
      YIN(1)=YC(J1,K1)
      YIN(2)=YC(J,K1)
      YIN(3)=YC(J,K)
      YIN(4)=YC(J1,K)
      IF (L.EQ.1) GO TO 10
      ZIN(1)=ZC(J1,K1)
      ZIN(2)=ZC(J,K1)
      ZIN(3)=ZC(J,K)
      ZIN(4)=ZC(J1,K)
      GO TO 20
   10 ZIN(1)=ZU(J1,K1)
      ZIN(2)=ZU(J,K1)
      ZIN(3)=ZU(J,K)
      ZIN(4)=ZU(J1,K)
   20 CONTINUE
!
!     FORM DIAGONAL VECTORS
!
      T1X=XIN(3)-XIN(1)
      T2X=XIN(4)-XIN(2)
      IF (IP.EQ.1) T2X=-T2X
      T1Y=YIN(3)-YIN(1)
      T2Y=YIN(4)-YIN(2)
      IF (IP.EQ.1) T2Y=-T2Y
      T1Z=ZIN(3)-ZIN(1)
      T2Z=ZIN(4)-ZIN(2)
      IF (IP.EQ.1) T2Z=-T2Z
!
!     FORM VECTOR CROSS PRODUCT, N = T2 X T1
!
      NX=T2Y*T1Z-T1Y*T2Z
      NY=T1X*T2Z-T2X*T1Z
      NZ=T2X*T1Y-T1X*T2Y
      IF (ABS(NX).LE.EPSPAN) NX=0.
      IF (ABS(NY).LE.EPSPAN) NY=0.
      IF (ABS(NZ).LE.EPSPAN) NZ=0.
      VN=SQRT(NX*NX+NY*NY+NZ*NZ)
      IF (VN.EQ.0.) GO TO 30
!
!     FORM UNIT NORMAL VECTOR
!
      VND=1./VN
      NX=NX*VND
      NY=NY*VND
      NZ=NZ*VND
!
!     COMPUTE AVERAGE POINT
!
   30 AVX=0.25*(XIN(1)+XIN(2)+XIN(3)+XIN(4))
      AVY=0.25*(YIN(1)+YIN(2)+YIN(3)+YIN(4))
      AVZ=0.25*(ZIN(1)+ZIN(2)+ZIN(3)+ZIN(4))
!
!     COMPUTE PROJECTION DISTANCE
!
      D=NX*(AVX-XIN(1))+NY*(AVY-YIN(1))+NZ*(AVZ-ZIN(1))
      T=SQRT(T1X*T1X+T1Y*T1Y+T1Z*T1Z)
      IF (T.EQ.0.0) GO TO 40
      TD=1./T
      T1X=T1X*TD
      T1Y=T1Y*TD
      T1Z=T1Z*TD
   40 T2X=NY*T1Z-NZ*T1Y
      T2Y=NZ*T1X-NX*T1Z
      T2Z=NX*T1Y-NY*T1X
!
!     COMPUTE COORDINATES OF CORNER POINTS IN REFERENCE COORDINATE
!     SYSTEM
!
      DO 50 N=1,4
      XPA=XIN(N)+NX*D
      YPA=YIN(N)+NY*D
      ZPA=ZIN(N)+NZ*D
      D=-D
      XDIF=XPA-AVX
      YDIF=YPA-AVY
      ZDIF=ZPA-AVZ
!
!     TRANSFORM CORNER POINT TO ELEMENT COORDINATE SYSTEM (XI,ETA)
!     WITH AVERAGE POINT AS ORIGIN
!
      XI(N)=T1X*XDIF+T1Y*YDIF+T1Z*ZDIF
   50 ETA(N)=T2X*XDIF+T2Y*YDIF+T2Z*ZDIF
!
!     COMPUTE CENTROID
!
      ETACK=ETA(2)-ETA(4)
      IF (ETACK.NE.0.0) GO TO 60
      XI0=0.0
      GO TO 70
   60 XI0=(XI(4)*(ETA(1)-ETA(2))+XI(2)*(ETA(4)-ETA(1)))/(3.*ETACK)
   70 ETA0=-ETA(1)/3.
!
!     OBTAIN CORNER POINTS IN SYSTEM WITH CENTROID AS ORIGIN
!
      XI(1)=XI(1)-XI0
      XI(2)=XI(2)-XI0
      XI(3)=XI(3)-XI0
      XI(4)=XI(4)-XI0
      ETA(1)=ETA(1)-ETA0
      ETA(2)=ETA(2)-ETA0
      ETA(3)=ETA(3)-ETA0
      ETA(4)=ETA(4)-ETA0
!
!     TRANSFORM CENTROID TO REFERENCE COORDINATE SYSTEM
!
      XPT(NP)=AVX+T1X*XI0+T2X*ETA0
      YPT(NP)=AVY+T1Y*XI0+T2Y*ETA0
      ZPT(NP)=AVZ+T1Z*XI0+T2Z*ETA0
!
!     COMPUTE PANEL INCIDENCE AND INCLINATION ANGLE
!
      DELTA(NP)=0.
      THET(NP)=0.
      RN=SQRT(NY*NY+NZ*NZ)
      IF (L.EQ.0) GO TO 90
      SL=-1.0
      IF (L.EQ.2) SL=1.0
      IF (IQ.EQ.1.AND.YIN(4).LT.YIN(1)) SL=-SL
      IF (NX.NE.0.) DELTA(NP)=ATAN2(SL*NX,RN)
      SP=FLOAT(1-2*IP)
      IF (IQ.EQ.1) GO TO 80
      IF (NY.NE.0.) THET(NP)=ATAN2(SP*NY,-SP*NZ)
      GO TO 100
   80 IF (NZ.NE.0.) THET(NP)=-ATAN2(SP*NZ,-SP*NY)
      GO TO 100
   90 IF (NX.NE.0.) DELTA(NP)=ATAN2(-NX,RN)
      IF (NY.EQ.0..AND.NZ.EQ.0.) GO TO 100
      THET(NP)=ATAN2(-NY,NZ)
  100 CONTINUE
!
!     COMPUTE PANEL AREA
!
      AP=0.5*(XI(3)-XI(1))*ETACK
      IF (IP.EQ.1) AP=-AP
      RETURN
      END
      SUBROUTINE SCAMP4 (X,Y,N,NDA,NDB,DA,DB,C,S,M)
!
!     ********************************************VERSION B00***********
!
!
!     GIVEN A SET OF N POINTS WHOSE ABSCISSAE FORM A STRICTLY MONOTONIC
!     SEQUENCE, AND GIVEN A FIRST OR SECOND DERIVATIVE AT X(1) AND A
!     FIRST OR SECOND DERIVATIVE AT X(N), TO FIND THE SMOOTHEST POSSIBLE
!     CURVE PASSING RIGOROUSLY THROUGH THE GIVEN POINTS, SATISFYING THE
!     SPECIFIED BOUNDARY CONDITIONS, AND POSSESSING CONTINUOUS FIRST AND
!     SECOND DERIVATIVES. THE CRITERION OF SMOOTHNESS IS THE
!     MINIMIZATION OF THE INTEGRAL OF THE SQUARE OF THE SECOND
!     DERIVATIVE, AND THE CUBIC FOUND IS ACCORDINGLY A CHAIN OF CUBICS,
!     I. E., A SEPARATE CUBIC ON EACH INTERVAL (X(I),X(I+1))
!
      DIMENSION C(4,1), S(1), X(1), Y(1), Z(4)
!
      L=1
      KK=1
      D1=DA
      D2=DB
      IF (M-12) 20,10,20
   10 KK=2
   20 IF (NDA+1) 30,40,50
   30 D1=DERIV2(X,Y,X)
      GO TO 50
   40 D1=DERIV1(X,Y,1)
   50 NA=IABS(NDA)
      IF (NDB+1) 60,70,80
   60 D2=DERIV2(X(N-3),Y(N-3),X(N))
      GO TO 80
   70 D2=DERIV1(X(N-2),Y(N-2),3)
   80 NB=IABS(NDB)
      CALL COMCU (D1,D2,S,X,Y,L,M,N,NA,NB)
      IF (M) 160,90,160
   90 K=N-1
      DO 150 J=1,K
      CALL CUBIC2 (X(J),Y(J),S(J),Z,M)
      IF (M-1) 100,110,100
  100 M=100*J+M
      GO TO 160
  110 IF(KK.EQ.2) GO TO 140
  120 DO 130 I=1,4
  130 C(I,J)=Z(I)
      GO TO 150
  140 L=7*J
      C(L-6,1)=X(J)
      C(L-5,1)=X(J+1)
      C(L-4,1)=3.0
      C(L-3,1)=Z(1)
      C(L-2,1)=Z(2)
      C(L-1,1)=Z(3)
      C(L,1)=Z(4)
  150 END DO
      M=0
  160 RETURN
      END
      SUBROUTINE DERIV (X,Y,N,NDA,DA,FD)
!
!     *******************************************VERSION A00************
!
!
!     COMPANION TO SUBROUTINE SCAMP4
!
!
!     FIT A CHAIN OF CUBIC CURVES THROUGH A SET OF N POINTS HAVING
!     CONTINUOUS FIRST AND SECOND DERIVATIVES AT THE INTERMEDIATE POINTS
!     AND SPECIFIED FIRST OR SECOND DERIVATIVE AT THE END POINTS
!
      COMMON /COEF/ C(4,50),CC(4,50)
!
      DIMENSION X(1), Y(1), FD(1)
!
!     FIT CUBIC CHAIN AND GET FIRST DERIVATIVES
!
      CALL SCAMP4 (X,Y,N,NDA,-1,DA,0.,C,FD,0)
      RETURN
      END
      FUNCTION DERIV1 (X1,Y1,N)
!
!     *******************************************VERSION A00************
!
!
!     COMPANION TO SUBROUTINE SCAMP4
!
!
!     FIND THE FIRST DERIVATIVE OF THE QUADRATIC THROUGH THREE GIVEN
!     POINTS AT A SPECIFIED ONE OF THESE POINTS. THIS PROVIDES A GOOD
!     APPROXIMATION TO THE SLOPE OF A FUNCTION AT A POINT, PARTICULARLY
!     IF THE OTHER TWO POINTS USED ARE NEARBY.
!
      DIMENSION X(3), Y(3), X1(3), Y1(3)
!
      EQUIVALENCE (S,K)
!
      DO 10 I=1,3
      X(I)=X1(I)
   10 Y(I)=Y1(I)
      K=N
!
!     FIND COEFFICIENTS OF THE X-SQUARED TERM AND THE X TERM. NO NEED TO
!     FIND CONSTANT TERM, AS IT DISAPPEARS UNDER DIFFERENTIATION.
!
      E=Y(1)-Y(2)
      H=Y(1)-Y(3)
      A=X(1)-X(2)
      B=X(1)-X(3)
      C=A*(X(1)+X(2))
      DT=B*(X(1)+X(3))
      C3=(B*E-A*H)/(B*C-A*DT)
      C2=(E-C3*C)/A
!
!     TEST TO SEE WHETHER DERIVATIVE IS WANTED AT ONE OF THE INPUT
!     POINTS OR ELSEWHERE
!
      K1=IABS(K)
      DO 20 I=1,3
      IF (K1-I) 20,30,20
   20 END DO
      GO TO 40
   30 S=X(K1)
   40 DERIV1=C2+2.0*C3*S
      RETURN
      END
      FUNCTION DERIV2 (X,Y,XX)
!
!     *******************************************VERSION A00************
!
!
!     COMPANION TO SUBROUTINE SCAMP4
!
!
!     FIND THE SECOND DERIVATIVE OF THE CUBIC THROUGH FOUR GIVEN POINTS
!     AT AN ARBITRARY POINT WHOSE X COORDINATE IS SPECIFIED
!
      DIMENSION X(4), Y(4)
!
      DERIV2=0.0
      IF (X(4)-X(3)) 10,70,10
   10 IF (X(4)-X(2)) 20,70,20
   20 IF (X(4)-X(1)) 30,70,30
   30 IF (X(3)-X(2)) 40,70,40
   40 IF (X(3)-X(1)) 50,70,50
   50 IF (X(2)-X(1)) 60,70,60
   60 Q41=(Y(4)-Y(1))/(X(4)-X(1))
      Q31=(Y(3)-Y(1))/(X(3)-X(1))
      Q21=(Y(2)-Y(1))/(X(2)-X(1))
      E=(Q31-Q21)/(X(3)-X(2))
      D=((Q41-Q21)/(X(4)-X(2))-E)/(X(4)-X(3))
      C=E-D*(X(3)+X(2)+X(1))
      DERIV2=2.0*(C+3.0*D*XX)
   70 RETURN
      END
      SUBROUTINE CUBIC2 (X,Y,D,C,J)
!
!     ********************************************VERSION B00***********
!
!
!     COMPANION TO SUBROUTINE SCAMP4
!
!
!     FIT A CUBIC TO TWO POINTS GIVEN THE SLOPE AT EACH POINT
!
!     THIS ROUTINE CALLS THE SYSTEM ROUTINE OVERFL WITH ARGUMENT J.
!     IF A FLOATING POINT OVERFLOW CONDITION EXISTS, OVERFL SETS J=1.
!     IF NO OVERFLOW EXISTS, OVERFL SETS J=0.
!     IN EITHER CASE, OVERFL SETS THE COMPUTER TO A NO OVERFLOW
!     CONDITION.
!!! mod by RLC   J is always 0 unless x(1)==x(2)
!
      DIMENSION X(1), Y(1), D(1), C(1)
!
      X2=X(2)
      B=X(1)-X2
      IF (B) 20,10,20
   10 J=3
      GO TO 30
   20 CONTINUE
      A=(Y(1)-Y(2))/B
      E=X(1)+X2
      C(4) = (D(1) + D(2) - 2.*A)/(B*B)
      C(3)=(A-D(2))/B-C(4)*(E+X2)
      C(2)=A-E*C(3)-C(4)*(E*X2+X(1)**2)
      C(1)=Y(2)-X2*(C(2)+X2*(C(3)+X2*C(4)))
      j=0   !!! J = LEGVAR(C) + 1
   30 RETURN
      END Subroutine Cubic2

      SUBROUTINE COMCU (DA,DB,S,X,Y,L,M,N,NDA,NDB)
!
!     ********************************************VERSION B00***********
!
!
!     COMPANION TO SUBROUTINE SCAMP4
!
!
!     FIT A COMPOSITE CUBIC THROUGH N POINTS, I. E., A SEPARATE CUBIC
!     BETWEEN EACH PAIR OF ADJACENT POINTS, SUCH THAT N-1 CUBICS ARE SO
!     DETERMINED THAT EACH MATCHES ITS NEIGHBORS IN FUNCTION VALUE AND
!     IN THE FIRST AND SECOND DERIVATIVES.
!
      COMMON /COEF/ C(50),D(50),E(50), dummyVariable(250)   ! added by RLC
                                                            ! 9 Nov 2005
!
      DIMENSION S(1), X(1), Y(1)
!
      K=N-1
      KUE=0
      IF (N-2) 10,20,60
   10 M=-1
      GO TO 180
   20 IF (NDA-1) 50,30,50
   30 IF (NDB-1) 50,40,50
   40 S(1)=DA
      S(2)=DB
      M=0
      GO TO 180
   50 KUE=1
   60 M=0
      E(1)=0.0
      C(N)=0.0
      IF (NDA-1) 70,70,80
   70 D(1)=1.0
      C(1)=0.0
      S(1)=DA
      GO TO 90
   80 D(1)=4.0
      C(1)=2.0
      S(1)=6.0*(Y(2)-Y(1))/(X(2)-X(1))-DA*(X(2)-X(1))
   90 IF (KUE) 120,100,120
  100 DO 110 I=2,K
      U=X(I)-X(I-1)
      V=X(I+1)-X(I)
      C(I)=U
      D(I)=2.0*(U+V)
      E(I)=V
  110 S(I)=3.0/(U*V)*(U*U*(Y(I+1)-Y(I))+V*V*(Y(I)-Y(I-1)))
  120 IF (NDB-1) 130,130,140
  130 E(N)=0.0
      D(N)=1.0
      S(N)=DB
      GO TO 150
  140 E(N)=2.0
      D(N)=4.0
      S(N)=6.0*(Y(N)-Y(N-1))/(X(N)-X(N-1))+DB*(X(N)-X(N-1))
  150 C(1)=C(1)/D(1)
      S(1)=S(1)/D(1)
      DO 160 I=2,N
      F=D(I)-C(I-1)*E(I)
      C(I)=C(I)/F
  160 S(I)=(S(I)-S(I-1)*E(I))/F
      DO 170 J=1,K
      I=N-J
  170 S(I)=S(I)-S(I+1)*C(I)
  180 RETURN
      END



!!!     OVERLAY(LWB,1,1)
      SUBROUTINE CONFIG()
!
!     *******************************************VERSION B00************
!
!
!     INPUT AND INITIALIZE CONFIGURATION DESCRIPTION (BASED ON PROGRAM
!     START OF NASA TMX-2074)
!
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAP&
!!!     &E9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      INCLUDE 'files.cmn'
      COMMON /BLANK/ J0,J1,J2,J3,J4,J5,J6,NWAF,NWAFOR,NFUS,NRADX(4),NFOR&
     &X(4),NP,NPODOR,NF,NFINOR,NCAN,NCANOR,J2TEST,NW,NC
      COMMON /BLANK2/ HORZ,VERT,TEST1,PHI,THETA,PSI,XF,YF,ZF,DIST,FMAG,P&
     &LOTSZ,TYPE,KODE,XMIN,XMAX,YMIN,YMAX,ZMIN,ZMAX,XMID,YMID,ZMID,BIGD,&
     &ISP
      COMMON /SCRAT/ BLOCK(7500)
      COMMON /GRAPH/ PLOT
      COMMON /ONE/ NAN2
      COMMON /PI/ PI,RADEG,DEGRAD
!
      DIMENSION XAF(30), WAFORG(20,4), WAFORD(20,3,30)
      DIMENSION TZORD(20,30), WAFOR(20,30)
      DIMENSION XFUS(30,4), ZFUS(30,4), FUSARD(30,4), FUSRAD(30,4)
      DIMENSION SFUS(30,30,8)
      DIMENSION PODORG(9,3), XPOD(9,30), PODORD(9,30), XPOD1(9,30)
      DIMENSION FINORG(6,2,4), XFIN(6,10), FINORD(6,2,10), FINX2(6,2,10)
      DIMENSION FINX3(6,2,10), FINOR(6,10), FINCR(6,10)
      DIMENSION CANORG(6,2,4), XCAN(6,10), CANORD(6,2,10), CANOR1(6,2,10)
      DIMENSION CANORX(6,2,10), CANOR(6,10), CANCR(6,10)
!
      EQUIVALENCE (BLOCK(1),XAF(1))
      EQUIVALENCE (BLOCK(31),WAFORG(1,1))
      EQUIVALENCE (BLOCK(111),WAFORD(1,1,1))
      EQUIVALENCE (BLOCK(1911),TZORD(1,1))
      EQUIVALENCE (BLOCK(2511),WAFOR(1,1))
      EQUIVALENCE (BLOCK(1),XFUS(1,1))
      EQUIVALENCE (BLOCK(121),ZFUS(1,1))
      EQUIVALENCE (BLOCK(241),FUSARD(1,1))
      EQUIVALENCE (BLOCK(361),FUSRAD(1,1))
      EQUIVALENCE (BLOCK(241),SFUS(1,1,1))
      EQUIVALENCE (BLOCK(1),PODORG(1,1))
      EQUIVALENCE (BLOCK(28),XPOD(1,1))
      EQUIVALENCE (BLOCK(298),PODORD(1,1))
      EQUIVALENCE (BLOCK(568),XPOD1(1,1))
      EQUIVALENCE (BLOCK(1),FINORG(1,1,1))
      EQUIVALENCE (BLOCK(49),XFIN(1,1))
      EQUIVALENCE (BLOCK(109),FINORD(1,1,1))
      EQUIVALENCE (BLOCK(229),FINX2(1,1,1))
      EQUIVALENCE (BLOCK(349),FINX3(1,1,1))
      EQUIVALENCE (BLOCK(469),FINOR(1,1))
      EQUIVALENCE (BLOCK(529),FINCR(1,1))
      EQUIVALENCE (BLOCK(1),CANORG(1,1,1))
      EQUIVALENCE (BLOCK(49),XCAN(1,1))
      EQUIVALENCE (BLOCK(109),CANORD(1,1,1))
      EQUIVALENCE (BLOCK(229),CANOR1(1,1,1))
      EQUIVALENCE (BLOCK(349),CANORX(1,1,1))
      EQUIVALENCE (BLOCK(469),CANOR(1,1))
      EQUIVALENCE (BLOCK(529),CANCR(1,1))

      INTEGER:: plot
!!!      INTEGER PLOT,TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAPE9&
!!!     &,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
!
      REWIND TAPE9
      REWIND TAPE12
!
!     REFERENCE AREA
!
      REFA=1.0
      IF (J0.EQ.0) GO TO 10
      READ (TAPE5,470) REFA
   10 CONTINUE
      WRITE (TAPE9) REFA
!
!     WING
!
      IF (J1.EQ.0) GO TO 160
      N=IABS(NWAFOR)
      NREC=(N+9)/10
      I1=-9
      I2=0
      DO 20 NN=1,NREC
      I1=I1+10
      I2=I2+10
      READ (TAPE5,470) (XAF(I),I=I1,I2)
   20 END DO
      DO 30 I=1,NWAF
      READ (TAPE5,470) (WAFORG(I,J),J=1,4)
   30 END DO
!
!     J1 = -1 INDICATES UNCAMBERED WING DATA
!
      IF (J1.LT.0) GO TO 60
      DO 50 NN=1,NWAF
      I1=-9
      I2=0
      DO 40 N1=1,NREC
      I1=I1+10
      I2=I2+10
      READ (TAPE5,470) (TZORD(NN,I),I=I1,I2)
      DO 40 I=I1,I2
      IF (WAFORG(NN,4).EQ.0.) GO TO 40
      TZORD(NN,I)=TZORD(NN,I)*100./WAFORG(NN,4)
   40 CONTINUE
   50 END DO
      GO TO 80
   60 DO 70 I=1,NWAF
      DO 70 K=1,N
   70 TZORD(I,K)=0.
   80 L=1
!
!     NWAFOR POSITIVE INDICATES SYMMETRICAL ORDINATES
!     NWAFOR NEGATIVE INDICATES UPPER AND LOWER ORDINATES ARE GIVEN
!
      IF (NWAFOR.LT.0) L=2
      DO 100 NN=1,NWAF
      DO 100 K=1,L
      I1=-9
      I2=0
      DO 90 N1=1,NREC
      I1=I1+10
      I2=I2+10
      READ (TAPE5,470) (WAFORD(NN,K,I),I=I1,I2)
   90 END DO
  100 CONTINUE
      DO 110 NN=1,NWAF
      DO 110 K=1,N
      WAFOR(NN,K)=WAFORD(NN,1,K)
      IF (L.EQ.1) GO TO 110
      WAFOR(NN,K)=(WAFORD(NN,1,K)+WAFORD(NN,2,K))/2.
      TZORD(NN,K)=(WAFORD(NN,1,K)-WAFORD(NN,2,K))/2.+TZORD(NN,K)
  110 CONTINUE
      IF (NWAFOR.LT.0) GO TO 130
      DO 120 NN=1,NWAF
      DO 120 K=1,N
  120 WAFORD(NN,2,K)=WAFORD(NN,1,K)
  130 CONTINUE
      IF(NWAFOR.LT.0) J1=IABS(J1)
      NWAFOR=IABS(NWAFOR)
      NW=NWAFOR
!
!     CHANGE WING TO ACTUAL UNITS
!
      DO 150 I=1,NWAF
      E=.01*WAFORG(I,4)
      E3=WAFORG(I,3)
      DO 140 J=1,NWAFOR
      WAFORD(I,1,J)=E*WAFORD(I,1,J)+E3+TZORD(I,J)*E
      WAFORD(I,2,J)=-E*WAFORD(I,2,J)+E3+TZORD(I,J)*E
  140 WAFORD(I,3,J)=WAFORG(I,1)+E*XAF(J)
  150 END DO
!
!     IF PLOTTING INPUT GEOMETRY, SET UP WING PLOT VECTORS
!
      IF (PLOT.LT.0) WRITE (TAPE3) BLOCK
!
  160 WRITE (TAPE9) BLOCK
      N=IABS(NWAFOR)
!
!     FUSELAGE (BODY)
!
      IF (J2.EQ.0) GO TO 290
      J2TEST=3
!
!     J2 = -1 AND J6 = -1 INDICATE CIRCULAR FUSELAGE SYMMETRICAL WITH
!     THE XY-PLANE
!
      IF (J2.EQ.-1.AND.J6.EQ.-1) J2TEST=1
!
!     J2 = -1 AND J6 = 0 INDICATE CIRCULAR CAMBERED FUSELAGE
!
      IF (J2.EQ.-1.AND.J6.EQ.0) J2TEST=2
!
!     J6 = 1 INDICATES COMPLETE CONFIGURATION SYMMETRICAL WITH THE
!     XY-PLANE
!
      IF (J6.EQ.1) J2TEST=1
      J2=1
      DO 280 NFU=1,NFUS
      NRAD=NRADX(NFU)
      NFUSOR=NFORX(NFU)
      N=NFUSOR
      NREC=(N+9)/10
      I1=-9
      I2=0
      DO 170 N1=1,NREC
      I1=I1+10
      I2=I2+10
      READ (TAPE5,470) (XFUS(I,NFU),I=I1,I2)
  170 END DO
!
!     J2TEST = 2 INDICATES CIRCULAR CAMBERED FUSELAGE
!
      IF (J2TEST.NE.2) GO TO 190
      I1=-9
      I2=0
      DO 180 N1=1,NREC
      I1=I1+10
      I2=I2+10
      READ (TAPE5,470) (ZFUS(I,NFU),I=I1,I2)
  180 END DO
      GO TO 210
  190 DO 200 I=1,N
  200 ZFUS(I,NFU)=0.
!
!     J2TEST = 3 INDICATES ARBITRARY FUSELAGE
!
  210 IF (J2TEST.NE.3) GO TO 250
      NCARD=(NRAD+9)/10
      DO 240 LN=1,N
      DO 230 K=1,2
      KK=K+(NFU-1)*2
      II=10
      I1=-9
      I2=0
      DO 220 NN=1,NCARD
      IF (NN.EQ.NCARD) II=MOD(NRAD,10)
      IF (II.EQ.0) II=10
      I1=I1+10
      I2=I2+II
      READ (TAPE5,470) (SFUS(I,LN,KK),I=I1,I2)
  220 END DO
  230 END DO
  240 END DO
      GO TO 280
  250 I1=-9
      I2=0
      DO 260 N1=1,NREC
      I1=I1+10
      I2=I2+10
      READ (TAPE5,470) (FUSARD(I,NFU),I=I1,I2)
  260 END DO
      DO 270 I=1,N
  270 FUSRAD(I,NFU)=SQRT(FUSARD(I,NFU)/PI)
  280 END DO
!
!     IF PLOTTING INPUT GEOMETRY, SET UP FUSELAGE PLOT VECTORS
!
      IF (PLOT.LT.0) WRITE (TAPE3) BLOCK
!
  290 WRITE (TAPE9) BLOCK
!
!     POD GEOMETRY DUMMY READ STATEMENTS
!
      IF (J3.EQ.0) GO TO 330
      N=NPODOR
      NREC=(N+9)/10
      DO 320 NN=1,NP
      READ (TAPE5,470) (PODORG(NN,I),I=1,3)
      I1=-9
      I2=0
      DO 300 N1=1,NREC
      I1=I1+10
      I2=I2+10
      READ (TAPE5,470) (XPOD(NN,I),I=I1,I2)
  300 END DO
      I1=-9
      I2=0
      DO 310 N1=1,NREC
      I1=I1+10
      I2=I2+10
      READ (TAPE5,470) (PODORD(NN,I),I=I1,I2)
  310 END DO
  320 END DO
!
!     IF PLOTTING INPUT GEOMETRY, SET UP POD PLOT VECTORS
!
      IF (PLOT.LT.0) WRITE (TAPE3) BLOCK
!
  330 CONTINUE
!
!     FINS (VERTICAL TAILS)
!
      IF (J4.EQ.0) GO TO 380
      N=NFINOR
      DO 350 NN=1,NF
      READ (TAPE5,470) ((FINORG(NN,I,J),J=1,4),I=1,2)
      READ (TAPE5,470) (XFIN(NN,I),I=1,N)
      READ (TAPE5,470) (FINORD(NN,1,J),J=1,N)
      DO 340 J=1,N
      FINCR(NN,J)=0.
  340 FINOR(NN,J)=FINORD(NN,1,J)
  350 END DO
!
!     CHANGE FINS TO ACTUAL UNITS
!
      DO 370 LQ=1,NF
      DO 370 I=1,2
      J=3-I
      E=.01*FINORG(LQ,J,4)
      E2=FINORG(LQ,J,2)
      DO 360 K=1,NFINOR
      EE=FINORD(LQ,1,K)*E
      FINORD(LQ,J,K)=E2+EE
      FINX2(LQ,J,K)=E2-EE
  360 FINX3(LQ,J,K)=FINORG(LQ,J,1)+E*XFIN(LQ,K)
  370 CONTINUE
!
!     IF PLOTTING INPUT GEOMETRY, SET UP FIN PLOT VECTORS
!
      IF (PLOT.LT.0) WRITE (TAPE3) BLOCK
!
  380 WRITE (TAPE9) BLOCK
!
!     CANARDS (HORIZONTAL TAILS)
!
      IF (J5.EQ.0) GO TO 460
      N=IABS(NCANOR)
      DO 420 NN=1,NCAN
      READ (TAPE5,470) ((CANORG(NN,I,J),J=1,4),I=1,2)
      READ (TAPE5,470) (XCAN(NN,I),I=1,N)
      READ (TAPE5,470) (CANORD(NN,1,J),J=1,N)
!
!     NCANOR POSITIVE INDICATES SYMMETRICAL ORDINATES
!     NCANOR NEGATIVE INDICATES UPPER AND LOWER ORDINATES ARE GIVEN
!
      IF (NCANOR.LT.0) GO TO 400
      DO 390 J=1,N
      CANCR(NN,J)=0.
      CANOR(NN,J)=CANORD(NN,1,J)
  390 CANOR1(NN,1,J)=CANORD(NN,1,J)
      GO TO 420
  400 READ (TAPE5,470) (CANOR1(NN,1,J),J=1,N)
      DO 410 J=1,N
      CANOR(NN,J)=(CANORD(NN,1,J)+CANOR1(NN,1,J))/2.
  410 CANCR(NN,J)=(CANORD(NN,1,J)-CANOR1(NN,1,J))/2.
  420 END DO
      NC=IABS(NCANOR)
!
!     CHANGE CANARD TO ACTUAL UNITS
!
      DO 450 NN=1,NCAN
      DO 440 K=1,2
      I=3-K
      E=.01*CANORG(NN,I,4)
      E3=CANORG(NN,I,3)
      DO 430 J=1,N
      CANORD(NN,I,J)=E*CANORD(NN,1,J)+E3
      CANOR1(NN,I,J)=-E*CANOR1(NN,1,J)+E3
  430 CANORX(NN,I,J)=CANORG(NN,I,1)+E*XCAN(NN,J)
  440 END DO
  450 END DO
!
!     IF PLOTTING INPUT GEOMETRY, SET UP CANARD PLOT VECTORS
!
      IF (PLOT.LT.0) WRITE (TAPE3) BLOCK
!
  460 WRITE (TAPE9) BLOCK
      REWIND TAPE9
!
!
!
  470 FORMAT (10F7.0)
      END



!!!     OVERLAY(LWB,1,2)
      SUBROUTINE NEWORD()
!
!     ********************************************VERSION B00***********
!
!
!     REVISE CHORDWISE PANEL SPACING ON WING AND COMPUTE NEW AIRFOIL
!     ORDINATES
!
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAP&
!!!     &E9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      INCLUDE 'files.cmn'
      COMMON /BLANK/ J0,J1,J2,J3,J4,J5,J6,NWAF,NWAFOR
      COMMON /NEWCOM/K0,K1,KDUM(6),KWAFOR
      COMMON /COEF/ C(4,50),CC(4,50)
      COMMON /SCRAT/ BLOCK(7500)
!
      DIMENSION XAF(30), WAFORG(20,4), WAFORD(20,3,30), TZORD(20,30)
      DIMENSION TZORK(20,30), WAFORK(20,30), DZCDX(20,30), DZTDX(20,30)
      DIMENSION DZCDXK(20,30), WAFOR(20,30), DZTDXK(20,30), TORD(30)
      DIMENSION ZORD(30), DZC(30), DZT(30), RHO(20)
      DIMENSION A(20), B(20), R(20), XAT(30), XAFK(30)
!
      EQUIVALENCE (BLOCK(1),XAF(1))
      EQUIVALENCE (BLOCK(31),WAFORG(1,1))
      EQUIVALENCE (BLOCK(111),WAFORD(1,1,1))
      EQUIVALENCE (BLOCK(1911),TZORD(1,1))
      EQUIVALENCE (BLOCK(2511),WAFOR(1,1)) 
      EQUIVALENCE (BLOCK(3111),TZORK(1,1))
      EQUIVALENCE (BLOCK(3711),WAFORK(1,1))
      EQUIVALENCE (BLOCK(4311),DZCDX(1,1))
      EQUIVALENCE (BLOCK(4911),DZCDXK(1,1))
      EQUIVALENCE (BLOCK(5511),DZTDXK(1,1))
      EQUIVALENCE (BLOCK(6111),XAFK(1))
      EQUIVALENCE (BLOCK(6141),RHO(1))
      EQUIVALENCE (BLOCK(6161),A(1))
      EQUIVALENCE (BLOCK(6181),B(1))
      EQUIVALENCE (BLOCK(6201),R(1))
      EQUIVALENCE (BLOCK(6221),TORD(1))
      EQUIVALENCE (BLOCK(6251),ZORD(1))
      EQUIVALENCE (BLOCK(6281),DZC(1))
      EQUIVALENCE (BLOCK(6311),DZT(1))
      EQUIVALENCE (BLOCK(6341),XAT(1))
      EQUIVALENCE (BLOCK(6901),DZTDX(1,1))
!
!!!      INTEGER TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAPE9,TAPE&
!!!     &10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
!
!     IN LINE FUNCTION DEFINITION
!
!!!      SLOP1(I,XI,C)=C(2,I)+XI*(2.*C(3,I)+3.*XI*C(4,I))
!!!      SLOP2(I,XI,CC)=CC(2,I)+XI*(2.*CC(3,I)+3.*XI*CC(4,I))
!!!      VALU1(I,XI,C)=C(1,I)+XI*(C(2,I)+XI*(C(3,I)+XI*C(4,I)))
!!!      VALU2(I,XI,CC)=CC(1,I)+XI*(CC(2,I)+XI*(CC(3,I)+XI*CC(4,I)))
!
      IF (K1.EQ.3) READ (TAPE5,220) (RHO(N),N=1,NWAF)
      IF (KWAFOR.EQ.0) GO TO 10
      READ (TAPE5,220) (XAFK(K),K=1,KWAFOR)
      GO TO 30
   10 KWAFOR=NWAFOR
      DO 20 K=1,NWAFOR
   20 XAFK(K)=XAF(K)
   30 CONTINUE
      NWAR=NWAFOR-1
!
!     CALCULATE CAMBER AND THICKNESS SLOPES
!
      DO 210 N=1,NWAF
      NDA=-1
      DA=0.
      DO 40 L=1,NWAFOR
      ZORD(L)=TZORD(N,L)
   40 TORD(L)=WAFOR(N,L)
!
!     J1 = -1 INDICATES UNCAMBERED WING
!
      IF (J1.LT.0) GO TO 60
      CALL DERIV (XAF,ZORD,NWAFOR,NDA,DA,DZC)
      DO 50 L=1,NWAR
      DO 50 M=1,4
   50 CC(M,L)=C(M,L)
      GO TO 80
   60 DO 70 L=1,NWAR
      DZC(L)=0.
      DO 70 M=1,4
   70 CC(M,L)=0.
   80 NWA=NWAFOR
      IF (K1.LT.3) GO TO 100
!
!     CALCULATE INITIAL SLOPE FOR ROUND LEADING EDGE
!
      NWA=NWAR
      NDA=0
      R(N)=SQRT(2.*RHO(N))
      SAF2=SQRT(XAF(2))
      SAF3=SQRT(XAF(3))
      CON2=TORD(2)/XAF(2)-R(N)/SAF2
      CON3=TORD(3)/XAF(3)-R(N)/SAF3
      DX=XAF(3)-XAF(2)
      A(N)=(CON2*XAF(3)-CON3*XAF(2))/DX
      B(N)=(CON3-CON2)/DX
      DA=R(N)/(2.*SAF2)+A(N)+2.*B(N)*XAF(2)
      DO 90 L=1,NWAR
      XAT(L)=XAF(L+1)
   90 TORD(L)=TORD(L+1)
      GO TO 120
  100 DO 110 L=1,NWA
  110 XAT(L)=XAF(L)
  120 CALL DERIV (XAT,TORD,NWA,NDA,DA,DZT)
      DO 130 L=1,NWAFOR
      DZCDX(N,L)=DZC(L)
  130 DZTDX(N,L)=DZT(L)
      IF (K1.LT.3) GO TO 150
      DZTDX(N,1)=900.
      DO 140 L=2,NWAFOR
  140 DZTDX(N,L)=DZT(L-1)
  150 CONTINUE
      IF (KWAFOR.EQ.0) GO TO 210
!
!     INTERPOLATE FOR REVISED CAMBER AND THICKNESS ORDINATES AND SLOPES
!
      TZORK(N,1)=TZORD(N,1)
      DZCDXK(N,1)=DZCDX(N,1)
      WAFORK(N,1)=WAFOR(N,1)
      DZTDXK(N,1)=DZTDX(N,1)
      KI=2
      DO 200 J=1,NWAR
      DO 180 K=KI,KWAFOR
      IF (XAFK(K).GT.XAF(J+1)) GO TO 190
      XJ=XAFK(K)
      TZORK(N,K) = CC(1,J) + XJ*(CC(2,J) + XJ*(CC(3,J) + XJ*CC(4,J)))
      DZCDXK(N,K) = CC(2,J) + XJ*(2.*CC(3,J) + 3.*XJ*CC(4,J))
      L=J
      XL=XJ
      IF (K1.LT.3) GO TO 170
      IF (J.GT.1) GO TO 160
      SXJ=SQRT(XJ)
      DZTDXK(N,K)=R(N)/(2.*SXJ)+A(N)+2.*B(N)*XJ
      WAFORK(N,K)=R(N)*SXJ+XJ*(A(N)+B(N)*XJ)
      GO TO 180
  160 XL=XJ-XAF(1)
      L=J-1
  170 WAFORK(N,K) = C(1,L) + XL*(C(2,L) + XL*(C(3,L) + XL*C(4,L)))
      DZTDXK(N,K) = C(2,L) + XL*(2.*C(3,L) + 3.*XL*C(4,L))
  180 END DO
  190 KI=K
  200 END DO
  210 END DO
!
!
!
  220 FORMAT (10F7.0)
      END



!!!     OVERLAY(LWB,1,3)
      SUBROUTINE WNGPAN()
!
!     *******************************************VERSION B00************
!
!
!     REVISE SPANWISE PANEL SPACING ON WING AND COMPUTE NEW PANEL
!     GEOMETRY
!
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAP&
!!!     &E9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      INCLUDE 'files.cmn'
      COMMON /BLANK/ J0,J1,J2,J3,J4,J5,J6,NWAF,NWAFOR
      COMMON /PARAM/ NBODY,NWING,NTAIL,LBC,THK,MACH,ALPHA,REFA
      COMMON /NEWCOM/KDD,KL,KDUM(5),KWAF,KWAFOR,KDUMO(35),KOL,NCPT
      COMMON /SEG/ NSEG,NROW(20),NCOL(20),COSS(20),SINS(20),BTE(20),NWT(&
     &20),SPNW(20),XLEW(20),BLE(20),ZLEW(20),XS(20),YS(20),ZS(20)
      COMMON /SCRAT/ BLOCK(7500)
      COMMON /POINT/ ARRAY(6000)
      COMMON /VELCOM/ DUM1(6),PRINT
      COMMON /GRAPH/ PLOT
      COMMON /PI/ PI,RADEG,DEGRAD
      COMMON /EPS/ EPSPAN,EPSWP1,EPSWP2,EPSTP,EPSVC,EPSSP1,EPSSP2,EPSLV,&
     &EPSSV,EPSVV,EPSWV,EPSVP,EPSIT1,EPSIT2,EPSFM, epssb1,epssb2
!
      DIMENSION XPT(600), YPT(600), ZPT(600), THET(600), DELTA(600)
      DIMENSION XC(30,20), YC(30,20), ZC(30,20), ZU(30,20)
      DIMENSION AREA(600), XE(600), XAF(30), WAFORG(20,4)
      DIMENSION WAFORD(20,3,30), TZORK(20,30), WAFORK(20,30)
      DIMENSION DZCDX(20,30), DZTDX(20,30), DZCDXK(20,30), DZTDXK(20,30)
      DIMENSION SLOPE(600), XAFK(30), XK(20), YK(20), ZK(20), CK(20)
      DIMENSION BL(20), TH(20), BT(20), CHORD(600)
!
      EQUIVALENCE (BLOCK(1),XAF(1))
      EQUIVALENCE (BLOCK(31),WAFORG(1,1))
      EQUIVALENCE (BLOCK(111),WAFORD(1,1,1))
      EQUIVALENCE (BLOCK(2511),DZTDX(1,1))
      EQUIVALENCE (BLOCK(3111),TZORK(1,1))
      EQUIVALENCE (BLOCK(3711),WAFORK(1,1))
      EQUIVALENCE (BLOCK(4311),DZCDX(1,1))
      EQUIVALENCE (BLOCK(4911),DZCDXK(1,1))
      EQUIVALENCE (BLOCK(5511),DZTDXK(1,1))
      EQUIVALENCE (BLOCK(6111),XAFK(1))
      EQUIVALENCE (BLOCK(6141),XK(1))
      EQUIVALENCE (BLOCK(6161),YK(1))
      EQUIVALENCE (BLOCK(6181),ZK(1))
      EQUIVALENCE (BLOCK(6201),CK(1))
      EQUIVALENCE (BLOCK(6221),BL(1))
      EQUIVALENCE (BLOCK(6261),TH(1))
      EQUIVALENCE (BLOCK(6281),BT(1))
      EQUIVALENCE (BLOCK(6301),CHORD(1))
      EQUIVALENCE (BLOCK(6901),SLOPE(1))
      EQUIVALENCE (BLOCK(6901),ZU(1,1))
      EQUIVALENCE (ARRAY(1),XPT(1))
      EQUIVALENCE (ARRAY(601),YPT(1))
      EQUIVALENCE (ARRAY(1201),ZPT(1))
      EQUIVALENCE (ARRAY(1801),THET(1))
      EQUIVALENCE (ARRAY(2401),DELTA(1))
      EQUIVALENCE (ARRAY(3001),XC(1,1))
      EQUIVALENCE (ARRAY(3601),YC(1,1))
      EQUIVALENCE (ARRAY(4201),ZC(1,1))
      EQUIVALENCE (ARRAY(4801),AREA(1))
      EQUIVALENCE (ARRAY(5401),XE(1))

      COMMON /STAR/XSTAR(3600),NSEGW

      LOGICAL LBC,THK
      INTEGER PRINT,PLOT
!!!,TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8&
!!!     &,TAPE9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
!
!     IN LINE FUNCTION DEFINITION
!
      XIN(X1,Y1,X2,Y2,Y)=X1+(X2-X1)*(Y-Y1)/(Y2-Y1)
!
      IF (KWAF.EQ.0) GO TO 10
!
!     READ INTERMEDIATE SPAN STATIONS
!
      READ (TAPE5,350) (YK(K),K=1,KWAF)
      GO TO 30
   10 KWAF=NWAF
      DO 20 K=1,KWAF
   20 YK(K)=WAFORG(K,2)
   30 CONTINUE
      IF(KWAF.GT.20) GO TO 315
      KO=2
      KOL=KWAF
      KI=1
      NI=1
      NW1=NWAF-1
      NSEG=0
      NC=0
      NJ=0
      NP=0
      IF (PRINT.GE.0) GO TO 40
      WRITE (TAPE6,360)
      WRITE (TAPE6,370)
   40 DO 260 N=1,NW1
!
!     CALCULATE WING SEGMENTS
!
      M=NI+1
      DELY=WAFORG(M,2)-WAFORG(NI,2)
      IF (DELY.EQ.0.) GO TO 250
      DO 50 K=KO,KWAF
      IF (YK(K).GE.WAFORG(M,2)) GO TO 60
   50 END DO
   60 KO=K
      IF (KO.EQ.KI) GO TO 260
      K=KI
   70 IF (YK(K).LE.WAFORG(M,2)) GO TO 90
   80 NI=M
      M=M+1
      IF (YK(K).GT.WAFORG(M,2)) GO TO 80
   90 CONTINUE
      XK(K)=XIN(WAFORG(NI,1),WAFORG(NI,2),WAFORG(M,1),WAFORG(M,2),YK(K))
      ZK(K)=XIN(WAFORG(NI,3),WAFORG(NI,2),WAFORG(M,3),WAFORG(M,2),YK(K))
      CK(K)=XIN(WAFORG(NI,4),WAFORG(NI,2),WAFORG(M,4),WAFORG(M,2),YK(K))
      IF (K.EQ.KO) GO TO 100
      K=KO
      GO TO 70
  100 CONTINUE
      DELX=XK(KO)-XK(KI)
      DELY=YK(KO)-YK(KI)
      DELZ=ZK(KO)-ZK(KI)
      DELC=CK(KO)-CK(KI)
      DELYD=1./DELY                              ! never used ???
      DELYZD=1.0/SQRT(DELY*DELY+DELZ*DELZ)
      BL(N)=DELX/DELYZD
      BT(N)=(DELX+DELC)/DELYZD
      TH(N)=ATAN2(DELZ,DELY)
      IF (N.EQ.1) GO TO 110
      IF (YK(K).GT.WAFORG(M,2)) GO TO 110
      IF (ABS(BL(N)-BL(N-1)).GT.EPSWP2) GO TO 110
      IF (ABS(BT(N)-BT(N-1)).GT.EPSWP2) GO TO 110
      IF (ABS(TH(N)-TH(N-1)).GT.EPSWP2) GO TO 110
      GO TO 120
  110 NSEG=NSEG+1
      NCOL(NSEG)=0
      SINS(NSEG)=SIN(TH(N))
      COSS(NSEG)=COS(TH(N))
      BLE(NSEG)=BL(N)
      BTE(NSEG)=BT(N)
      NWT(NSEG)=0
!
!     COMPUTE NUMBER OF COLUMNS IN EACH WING SEGMENT
!
  120 NCOL(NSEG)=NCOL(NSEG)+KO-KI
!
!     CALCULATE ORIGINS OF INTERMEDIATE CHORDS
!
      DO 240 K=KI,KO
      IF (K.EQ.KI.OR.K.EQ.KO) GO TO 130
      XK(K)=XIN(WAFORG(NI,1),WAFORG(NI,2),WAFORG(M,1),WAFORG(M,2),YK(K))
      ZK(K)=XIN(WAFORG(NI,3),WAFORG(NI,2),WAFORG(M,3),WAFORG(M,2),YK(K))
      CK(K)=XIN(WAFORG(NI,4),WAFORG(NI,2),WAFORG(M,4),WAFORG(M,2),YK(K))
  130 CONTINUE
      CL=CK(K)/100.
      L=1
      SJ=1.0
  140 CONTINUE
!
!     CALCULATE COORDINATES OF PANEL CORNERS
!
      DO 190 J=1,KWAFOR
      XC(J,K)=XK(K)+CL*XAFK(J)
      YC(J,K)=YK(K)
      ZC(J,K)=ZK(K)
      IF (LBC) GO TO 150
      ZCAM=XIN(TZORK(NI,J),WAFORG(NI,2),TZORK(M,J),WAFORG(M,2),YK(K))
      ZTHK=XIN(WAFORK(NI,J),WAFORG(NI,2),WAFORK(M,J),WAFORG(M,2),YK(K))
      ZC(J,K)=ZK(K)+CL*(ZCAM+SJ*ZTHK)
      IF (L.EQ.1) ZU(J,K)=ZC(J,K)
  150 IF (K.EQ.KI) GO TO 190
      K1=K-1
      NJ=NJ+1
      IF (J.EQ.1) GO TO 190
      JJ=J-1
      NC=NC+1
      NP=NP+1
      IP=1
      IF (SJ.LT.0.) IP=0
      IQ=0
!
!     CALCULATE PANEL INCLINATIONS AND CENTROIDS ON WING SURFACE
!
      IF (.NOT.LBC) CALL PANEL (IP,IQ,J,K,L,NP,AP)
      AREA(NP)=AP
      CHORD(NP)=0.
      IF (PRINT.GE.0) GO TO 170
      IF (.NOT.LBC.AND.L.EQ.1) GO TO 160
      WRITE (TAPE6,380) NP,XC(JJ,K1),YK(K1),ZC(JJ,K1),XC(J,K1),YK(K1),ZC&
     &(J,K1),XC(JJ,K),YK(K),ZC(JJ,K),XC(J,K),YK(K),ZC(J,K)
      GO TO 170
  160 WRITE (TAPE6,380) NP,XC(JJ,K1),YK(K1),ZU(JJ,K1),XC(J,K1),YK(K1),ZU&
     &(J,K1),XC(JJ,K),YK(K),ZU(JJ,K),XC(J,K),YK(K),ZU(J,K)
  170 CONTINUE
!
!     CALCULATE PANEL CONTROL POINTS IN PLANE OF WING
!
      CR=XC(J,K1)-XC(JJ,K1)
      CT=XC(J,K)-XC(JJ,K)
      RI=(1.+CT/(CR+CT))/3.
      RO=1.-RI
      XLE=RI*XC(JJ,K)+RO*XC(JJ,K1)
      XTE=RI*XC(J,K)+RO*XC(J,K1)
      CHORD(NP)=XTE-XLE
      SPN=SQRT((YK(K)-YK(K1))*(YK(K)-YK(K1))+(ZK(K)-ZK(K1))*(ZK(K)-ZK(K1&
     &)))
      SPNW(K1)=SPN
      IF (J.EQ.2) XLEW(K1)=XLE
      YLE=RI*YK(K)+RO*YK(K1)
      ZLE=RI*ZK(K)+RO*ZK(K1)
      IF (J.EQ.2) ZLEW(K1)=ZLE
      IF (LBC) GO TO 180
      GO TO 190
  180 CONTINUE
      XPT(NC)=XLE
      XE(NC)=XPT(NC)
      YPT(NC)=YLE
      ZPT(NC)=ZLE
!
!     CALCULATE PANEL AREAS, CHORDS, AND INCLINATION ANGLES
!
      AREA(NP)=.5*SPN*(CR+CT)
      THET(NC)=TH(N)
!
!     INTERPOLATE FOR WING CAMBER AND THICKNESS AT PANEL TRAILING EDGES
!
!
!         REVISED BY B. ROSEN 081881
!
      MI=NI
  182 IF(YPT(NC).GE.WAFORG(MI,2)) GO TO 184
      MI=MI-1
      GO TO 182
  184 MO=MI+1
      DZCDX(K1,J)=XIN(DZCDXK(MI,J),WAFORG(MI,2),DZCDXK(MO,J),           &
     & WAFORG(MO,2),YPT(NC))
      IF (J.EQ.2) DZCDX(K1,1)=XIN(DZCDXK(MI,1),WAFORG(MI,2),            &
     & DZCDXK(MO,1),WAFORG(MO,2),YPT(NC))
      DZTDX(K1,J)=XIN(DZTDXK(MI,J),WAFORG(MI,2),DZTDXK(MO,J),           &
     & WAFORG(MO,2),YPT(NC))
      SLOPE(NJ)=DZTDX(K1,J)
      DELTA(NC)=DZCDX(K1,JJ)
      IF (J.NE.KWAFOR) GO TO 190
      NC=NC+1
      XPT(NC)=XTE-EPSWP1
      XE(NC)=XPT(NC)
      YPT(NC)=YPT(NC-1)
      ZPT(NC)=ZPT(NC-1)
      CD = 0.01*(XTE - XLEW(K1))
      ZTE = CD*XIN(WAFORK(MI,J),WAFORG(MI,2),WAFORK(MO,J),              &
     & WAFORG(MO,2),YPT(NC))
      DELTA(NC)=DZCDX(K1,J)
      THET(NC)=TH(N)
  190 END DO
      IF (LBC) GO TO 200
      IF (SJ.LT.0.) GO TO 200
      SJ=-1.0
      L=2
      GO TO 140
  200 CONTINUE
      IF (K.EQ.KI) GO TO 240
      IF (.NOT.LBC) GO TO 240
      IF (KL.EQ.3) GO TO 210
!
!     CALCULATE INITIAL SLOPE FOR SHARP LEADING EDGE AIRFOILS
!
      DZTDX(K1,1)=XIN(DZTDXK(MI,1),WAFORG(MI,2),DZTDXK(MO,1),           &
     & WAFORG(MO,2),YPT(NC))
      GO TO 230
!
!     CALCULATE INITIAL SLOPE FOR ROUND LEADING EDGE AIRFOILS
!
  210 NPJ=NP-JJ
      SLE = 2.*ZTE/CHORD(NPJ + 1) - DZTDX(K1,2)
      DO 220 I=2,JJ
  220 SLE=SLE-(DZTDX(K1,I)+DZTDX(K1,I+1))*CHORD(NPJ+I)/CHORD(NPJ+1)
      DZTDX(K1,1)=SLE
  230 NJJ=NJ-JJ
      SLOPE(NJJ)=DZTDX(K1,1)
  240 END DO
!
!     COMPUTE NUMBER OF ROWS IN EACH WING SEGMENT
!
      NROW(NSEG)=JJ
      NCPT=NC
      IF (NCPT.GT.600) GO TO 320
      NWING=NP
      NI=N+1
      KI=KO
      IF (KO.LT.KWAF) KO=KO+1
      GO TO 260
  250 KO=KO+1
      NI=NI+1
      BL(N)=0.
      BT(N)=0.
      TH(N)=0.
  260 END DO
      IF (PLOT.EQ.0) GO TO 280
      IF (LBC) GO TO 270
      WRITE (TAPE3) ((XC(J,K),YC(J,K),ZU(J,K),J=1,KWAFOR),K=1,KWAF)
      WRITE (TAPE3) ((XC(J,K),YC(J,K),ZC(J,K),J=1,KWAFOR),K=1,KWAF)
      GO TO 280
  270 WRITE (TAPE3) ((XC(J,K),YC(J,K),ZC(J,K),J=1,KWAFOR),K=1,KWAF)
      WRITE (TAPE3) ((XC(J,K),YC(J,K),ZC(J,K),J=1,KWAFOR),K=1,KWAF)
  280 CONTINUE
      IF (PRINT.GE.0) GO TO 310
      IF (LBC) WRITE (TAPE6,450)
      IF (.NOT.LBC) WRITE (TAPE6,390)
      IF (LBC) WRITE (TAPE6,460)
      IF (.NOT.LBC) WRITE (TAPE6,470)
      DO 290 NP=1,NCPT
      TDEG=THET(NP)*RADEG
      DDEG=DELTA(NP)*RADEG
      IF (LBC) WRITE (TAPE6,400) NP,XPT(NP),YPT(NP),ZPT(NP),THET(NP),DEL&
     &TA(NP),SLOPE(NP),TDEG
      IF (.NOT.LBC) WRITE (TAPE6,400) NP,XPT(NP),YPT(NP),ZPT(NP),THET(NP&
     &),DELTA(NP),TDEG,DDEG
  290 END DO
      WRITE (TAPE6,430)
      WRITE (TAPE6,410)
      DO 300 NP=1,NWING
  300 WRITE (TAPE6,420) NP,AREA(NP),CHORD(NP)
  310 CONTINUE
!
!     STORE WING GEOMETRY ON TAPE 7
!
      WRITE (TAPE7) ARRAY,CHORD,SLOPE
      GO TO 330
  320 CONTINUE
      WRITE (TAPE6,340)
      WRITE (TAPE6,440)
      NSEGW=NSEG
      STOP   !!! CALL EXIT
  315 WRITE(TAPE6,340)
      WRITE(TAPE6,445)
      NSEGW=NSEG
      STOP   !!! CALL EXIT
  330 CONTINUE
      NSEGW=NSEG
!
!
!
  340 FORMAT (1H0,26H ERROR MESSAGE FROM WNGPAN/1H0)
  350 FORMAT (10F7.0)
  360 FORMAT (1H1,9X,35HWING PANEL CORNER POINT COORDINATES/10X,86H1 AND&
     & 3 INDICATE WING PANEL LEADING-EDGE POINTS, 2 AND 4 INDICATE TRAIL&
     &ING-EDGE POINTS)
  370 FORMAT (1H0,5X,5HPANEL,4(8X,1HX,8X,1HY,8X,1HZ)/20X,3(1H1,8X),3(1H2&
     &,8X),3(1H3,8X),3(1H4,8X)//)
  380 FORMAT (1H ,4X,I3,4X,12F9.5)
  390 FORMAT (1H1,1X,49HWING PANEL CENTROID POINTS AND INCLINATION ANGLE&
     &S)
  400 FORMAT (1H ,1X,I3,4X,8F11.5)
  410 FORMAT (1H0,5HPANEL,6X,4HAREA,8X,5HCHORD)
  420 FORMAT (1H ,1X,I3,4X,2F11.5)
  430 FORMAT (1H1,9X,27HWING PANEL AREAS AND CHORDS)
  440 FORMAT (51H  ERROR - NUMBER OF WING CONTROL POINTS EXCEEDS 600)
  445 FORMAT(1X,48HINPUT ERROR - NUMBER OF SPAN STATIONS EXCEEDS 20)
  450 FORMAT (1H1,1X,80HWING PANEL EDGE POINTS ON CHORD PASSING THROUGH &
     &CENTROID, AND INCLINATION ANGLES)
  460 FORMAT (1H0,5HPOINT,8X,1HX,10X,1HY,10X,1HZ,10X,5HTHETA,6X,6HCAMBER&
     &,4X,9HTHICKNESS,2X,5HTHETA/15X,3(2HCP,9X),3HRAD,8X,5HSLOPE,5X,5HSL&
     &OPE,6X,3HDEG//)
  470 FORMAT (1H0,5HPOINT,8X,1HX,10X,1HY,10X,1HZ,10X,5HTHETA,6X,5HDELTA,&
     &6X,5HTHETA,6X,5HDELTA/15X,3(2HCP,9X),3HRAD,8X,3HRAD,8X,3HDEG,8X,3H&
     &DEG//)
      END



!!!     OVERLAY(LWB,1,4)
      SUBROUTINE NEWRAD()
!
!     *******************************************VERSION B00************
!
!
!     REVISE BODY (FUSELAGE) MERIDIAN LINE SPACING
!
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAP&
!!!     &E9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      INCLUDE 'files.cmn'
      COMMON /BLANK/ J0,J1,J2,J3,J4,J5,J6,NWAF,NWAFOR,NFUS,NRADX(4),NFOR&
     &X(4),DUM(6),J2TEST
      COMMON /NEWCOM/KDUM(9),KFUS,KRADX(4),KFORX(4),KDUMY(25),MAX
      COMMON /SCRAT/ BLOCK(7500)
!!!      COMMON /POINT/ ARRAY(4800)
      COMMON /POINT/ ARRAY(6000)
      COMMON /PI/ PI,RADEG,DEGRAD
!
      DIMENSION XFUS(30,4), ZFUS(30,4), FUSARD(30,4), FUSRAD(30,4)
      DIMENSION SFUS(30,30,8), ANSIN(30), ANCOS(30), PHIN(30)
      DIMENSION PHIK(30), XB(30), YB(30,30), ZB(30,30), YF(30), ZF(30)
!
      EQUIVALENCE (BLOCK(1),XFUS(1,1))
      EQUIVALENCE (BLOCK(121),ZFUS(1,1))
      EQUIVALENCE (BLOCK(241),FUSARD(1,1))
      EQUIVALENCE (BLOCK(361),FUSRAD(1,1))
      EQUIVALENCE (BLOCK(241),SFUS(1,1,1))
      EQUIVALENCE (ARRAY(1),YB(1,1))
      EQUIVALENCE (ARRAY(1801),ZB(1,1))
      EQUIVALENCE (ARRAY(3601),XB(1))
      EQUIVALENCE (ARRAY(3661),ANSIN(1))
      EQUIVALENCE (ARRAY(3691),ANCOS(1))
      EQUIVALENCE (ARRAY(3721),PHIN(1))
      EQUIVALENCE (ARRAY(3751),PHIK(1))
!
!!!      INTEGER TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAPE9,TAPE&
!!!     &10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      LOGICAL NEWPHI
!
!     IN LINE FUNCTION DEFINITION
!
      XIN(X1,Y1,X2,Y2,Y)=X1+(X2-X1)*(Y-Y1)/(Y2-Y1)
!
      RADD=DEGRAD
      PITWO=2.*PI
      PI3HAF=PI*(3./2.)
      NEWPHI=.FALSE.
      M=1
      KFUS=NFUS
      KTEST=0
      REWIND TAPE10
      DO 110 NFU=1,NFUS
      NRAD=NRADX(NFU)
      KRAD=KRADX(NFU)
!
!     J2TEST IS SET IN PROGRAM CONFIG
!
!     J2TEST = 3 AND KRAD = 0 INDICATE AN ARBITRARY FUSELAGE WITH
!     MERIDIAN LINES DEFINED BY NRAD IN THE GEOMETRY INPUT
!
      IF (J2TEST.EQ.3.AND.KRAD.EQ.0) KTEST=1
      IF (KRAD.EQ.0) KRAD=NRAD
      IF (KRAD.GT.20) GO TO 130
      IF (KRAD.LT.0) NEWPHI=.TRUE.
      KRAD=IABS(KRAD)
      KRADX(NFU)=KRAD
      NFUSOR=NFORX(NFU)
      FANG=FLOAT(2*(KRAD-1))
      DELE=PITWO/FANG
!
!     READ NEW MERIDIAN ANGLES
!
      IF (NEWPHI) READ (TAPE5,170) (PHIK(K),K=1,KRAD)
      DO 30 K=1,KRAD
      E=FLOAT(K-1)
      IF (NEWPHI) GO TO 10
      PHIR=E*DELE
      GO TO 20
   10 PHIR=PHIK(K)*RADD
   20 PHIK(K)=PHIR
!
!     J2TEST = 3 INDICATES ARBITRARY FUSELAGE
!
      IF (J2TEST.EQ.3) GO TO 30
      PHIR4=PHIR+PI3HAF
      ANSIN(K)=SIN(PHIR4)
      ANCOS(K)=COS(PHIR4)
   30 END DO
      KK=1+(NFU-1)*2
      NF=NFU
      K2=KK+1
      DO 100 N=1,NFUSOR
      IF (N.GT.1) M=M+1
      IF(M.GT.30) GO TO 120
      XB(N)=XFUS(N,NF)
!
!     J2TEST = 3 INDICATES ARBITRARY FUSELAGE
!
      IF (J2TEST.EQ.3) GO TO 50
      RAD=FUSRAD(N,NF)
      CAM=ZFUS(N,NF)
!
!     COMPUTE SECTION Y AND Z COORDINATES FOR CIRCULAR BODY (FUSELAGE)
!
      DO 40 K=1,KRAD
      YB(N,K)=RAD*ANCOS(K)
   40 ZB(N,K)=RAD*ANSIN(K)+CAM
      GO TO 100
   50 CONTINUE
!
!     COMPUTE SECTION Y AND Z ORDINATES FOR NONCIRCULAR BODY (FUSELAGE)
!     BY LINEAR INTERPOLATION
!
      KI=2
      PHIN(1)=0.
      YB(N,1)=SFUS(1,N,KK)
      ZB(N,1)=SFUS(1,N,K2)
      YF(1)=YB(N,1)
      ZF(1)=ZB(N,1)
      ZC=(SFUS(1,N,K2)+SFUS(NRAD,N,K2))/2.
      DO 90 NN=2,NRAD
      IF (KTEST.EQ.1) GO TO 80
      YF(NN)=SFUS(NN,N,KK)
      ZF(NN)=SFUS(NN,N,K2)-ZC
      N1=NN-1
      IF (YF(NN).EQ.0..AND.ZF(NN).EQ.0.) GO TO 80
      PHIN(NN)=ATAN2(YF(NN),-ZF(NN))
      DO 60 K=KI,KRAD
      IF (PHIK(K).GT.PHIN(NN)) GO TO 70
      YB(N,K)=XIN(YF(N1),PHIN(N1),YF(NN),PHIN(NN),PHIK(K))
      ZB(N,K)=XIN(ZF(N1),PHIN(N1),ZF(NN),PHIN(NN),PHIK(K))+ZC
   60 END DO
   70 KI=K
      GO TO 90
   80 YB(N,NN)=SFUS(NN,N,KK)
      ZB(N,NN)=SFUS(NN,N,K2)
   90 END DO
  100 END DO
      MAX=M
      WRITE (TAPE10) XB,YB,ZB
  110 END DO
      GO TO 150
  120 CONTINUE
      WRITE (TAPE6,160)
      WRITE (TAPE6,190)
      GO TO 140
  130 CONTINUE
      WRITE (TAPE6,160)
      WRITE (TAPE6,180)
  140 STOP   !!! CALL EXIT
  150 CONTINUE
!
!
!
  160 FORMAT (1H0,26H ERROR MESSAGE FROM NEWRAD/1H0)
  170 FORMAT (10F7.0)
  180 FORMAT (1H ,39HERROR - BODY HAS MORE THAN 20 MERIDIANS)
  190 FORMAT (1H ,44HERROR - BODY HAS MORE THAN 60 AXIAL STATIONS)
      END



!!!     OVERLAY(LWB,1,5)
      SUBROUTINE BODPAN()
!
!     *******************************************VERSION B00************
!
!
!     REVISE AXIAL SPACING ON BODY (FUSELAGE) AND COMPUTE NEW PANEL
!     GEOMETRY
!
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAP&
!!!     &E9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      INCLUDE 'files.cmn'
      COMMON /BLANK/ DUM(9),NFUS,NRADX(4),NFORX(4)
      COMMON /PARAM/ NBODY,NWING,NTAIL,LBC,THK,MACH,ALPHA,REFA
      COMMON /NEWCOM/KDUM(9),KFUS,KRADX(4),KFORX(4)
      COMMON /SCRAT/ BLOCK(7500)
      COMMON /POINT/ ARRAY(6000)
      COMMON /BTHET/ THETA(600)
      COMMON /VELCOM/ DUM1(6),PRINT
      COMMON /GRAPH/ PLOT
      COMMON /PI/ PI,RADEG,DEGRAD
      COMMON /CLINE/CCL(30),XX(30)

      DIMENSION XB(30), YB(30,30), ZB(30,30)
      DIMENSION XJ(60), AREA(600), XPT(600), YPT(600), ZPT(600)
      DIMENSION THET(600), DELTA(600), XC(30,20), YC(30,20),ZC(30,20),XFUS(30,4)

      EQUIVALENCE (BLOCK(1),XFUS(1,1))
      EQUIVALENCE (BLOCK(121),YB(1,1))
      EQUIVALENCE (BLOCK(1921),ZB(1,1))
      EQUIVALENCE (BLOCK(3721),XB(1))
      EQUIVALENCE (ARRAY(1),XPT(1))
      EQUIVALENCE (ARRAY(601),YPT(1))
      EQUIVALENCE (ARRAY(1201),ZPT(1))
      EQUIVALENCE (ARRAY(1801),THET(1))
      EQUIVALENCE (ARRAY(2401),DELTA(1))
      EQUIVALENCE (ARRAY(3001),XC(1,1))
      EQUIVALENCE (ARRAY(3601),YC(1,1))
      EQUIVALENCE (ARRAY(4201),ZC(1,1))
      EQUIVALENCE (ARRAY(4801),AREA(1))
!
      INTEGER PRINT,PLOT
!!!,TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8&
!!!     &,TAPE9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
!
!     IN LINE FUNCTION DEFINITION
!
      XIN(X1,Y1,X2,Y2,Y)=X1+(X2-X1)*(Y-Y1)/(Y2-Y1)
!
      REWIND TAPE10
      IF (PRINT.GE.0) GO TO 10
      WRITE (TAPE6,240)
      WRITE (TAPE6,250)
   10 CONTINUE
!
!     CALCULATE COORDINATES OF PANEL CORNERS
!
      IL = 0
      IP=0
      IQ=0
      J=1
      JI=1
      NP=0
      JM=0
      L=0
      DO 150 NFU=1,NFUS
      JMAX=KFORX(NFU)
      NFUSOR=NFORX(NFU)
      KRAD=KRADX(NFU)
      KRAD=IABS(KRAD)
      IF (KRAD.EQ.0) KRAD=NRADX(NFU)
      READ (TAPE10) XB,YB,ZB
      IF (JMAX.EQ.0) GO TO 40
!
!     READ IN NEW AXIAL STATIONS FOR BODY (FUSELAGE)
!
      READ (TAPE5,230) (XJ(K),K=1,JMAX)
      GO TO 70
   40 JMAX=NFORX(NFU)
      IF (JMAX.LE.30) GO TO 50
      WRITE (TAPE6,220)
      WRITE (TAPE6,330) NFU
      STOP   !!! CALL EXIT
   50 CONTINUE
      KFORX(NFU)=JMAX
      DO 60 K=1,JMAX
   60 XJ(K)=XFUS(K,NFU)
   70 CONTINUE
      JM=JM+JMAX
      IF (JM.LE.30) GO TO 80
      WRITE (TAPE6,220)
      WRITE (TAPE6,340)
      STOP   !!! CALL EXIT
   80 CONTINUE
      DO 90 K=1,KRAD
      XC(J,K)=XB(1)
      YC(J,K)=YB(1,K)
   90 ZC(J,K)=ZB(1,K)
      IF(NFU.NE.1) GO TO 95
      IL = IL + 1
      CCL(IL) = (ZC(J,1) + ZC(J,KRAD))/2.
      XX(IL) = XB(1)
   95 CONTINUE
      DO 130 JJ=2,JMAX
      J1=J
      J=J+1
      DO 120 M=2,NFUSOR
      M1=M-1
      IF (XB(M).LT.XJ(JJ)) GO TO 120
      DO 110 K=1,KRAD
      XC(J,K)=XJ(JJ)
      YC(J,K)=XIN(YB(M1,K),XB(M1),YB(M,K),XB(M),XJ(JJ))
      ZC(J,K)=XIN(ZB(M1,K),XB(M1),ZB(M,K),XB(M),XJ(JJ))
      IF (K.EQ.1) GO TO 110
      K1=K-1
      NP=NP+1
!
!     CALCULATE PANEL INCLINATION AND CENTROID
!
      CALL PANEL (IP,IQ,J,K,L,NP,AP)
!
      IF (PRINT.GE.0) GO TO 100
      WRITE (TAPE6,260) NP,XC(J1,K1),YC(J1,K1),ZC(J1,K1),XC(J,K1),YC(J,K&
     &1),ZC(J,K1),XC(J1,K),YC(J1,K),ZC(J1,K),XC(J,K),YC(J,K),ZC(J,K)
  100 AREA(NP)=AP
  110 END DO
      IL = IL + 1
      CCL(IL) = (ZC(J,1) + ZC(J,KRAD))/2.
      XX(IL) = XJ(JJ)
      GO TO 130
  120 END DO
  130 END DO
      J=J+1
      IF (PLOT.EQ.0) GO TO 140
      JF=JMAX+JI-1
      WRITE (TAPE3) ((XC(JJ,K),YC(JJ,K),ZC(JJ,K),JJ=JI,JF),K=1,KRAD)
      JI=J
  140 CONTINUE
  150 END DO
      NBODY=NP
      IF (NBODY.GT.600) GO TO 200
      IF (PRINT.GE.0) GO TO 180
      WRITE (TAPE6,270)
      WRITE (TAPE6,280)
      DO 160 NP=1,NBODY
      WRITE (TAPE6,290) NP,XPT(NP),YPT(NP),ZPT(NP)
  160 END DO
      WRITE (TAPE6,300)
      WRITE (TAPE6,310)
      DO 170 NP=1,NBODY
      DDEG=DELTA(NP)*RADEG
      TDEG=THET(NP)*RADEG
  170 WRITE (TAPE6,290) NP,AREA(NP),DELTA(NP),THET(NP),DDEG,TDEG
  180 CONTINUE
      DO 190 NP=1,NBODY
  190 THETA(NP)=THET(NP)
      REWIND TAPE10
      WRITE (TAPE10) ARRAY
      REWIND TAPE10
      GO TO 210
  200 CONTINUE
      WRITE (TAPE6,220)
      WRITE (TAPE6,320)
      STOP   !!! CALL EXIT
  210 CONTINUE
!
!
!
  220 FORMAT (1H0,26H ERROR MESSAGE FROM BODPAN/1H0)
  230 FORMAT (10F7.0)
  240 FORMAT (1H1,9X,35HBODY PANEL CORNER POINT COORDINATES/10X,86H1 AND&
     & 3 INDICATE BODY PANEL LEADING-EDGE POINTS, 2 AND 4 INDICATE TRAIL&
     &ING-EDGE POINTS)
  250 FORMAT (1H0,5X,5HPANEL,4(8X,1HX,8X,1HY,8X,1HZ)/20X,3(1H1,8X),3(1H2&
     &,8X),3(1H3,8X),3(1H4,8X)//)
  260 FORMAT (1H ,4X,I3,4X,12F9.5)
  270 FORMAT (1H1,1X,37HBODY PANEL CENTROID POINT COORDINATES)
  280 FORMAT (1H0,5HPOINT,6X,1HX,10X,1HY,10X,1HZ/15X,3(2HCP,9X)//)
  290 FORMAT (1H ,1X,I3,4X,5F11.5)
  300 FORMAT (1H1,4X,39HBODY PANEL AREAS AND INCLINATION ANGLES)
  310 FORMAT (1H0,5HPANEL,6X,4HAREA,7X,5HDELTA,6X,5HTHETA,6X,5HDELTA,6X,&
     &5HTHETA/24X,3HRAD,8X,3HRAD,8X,3HDEG,8X,3HDEG//)
  320 FORMAT (43H  ERROR - NUMBER OF BODY PANELS EXCEEDS 600)
  330 FORMAT(1X,33HINPUT ERROR - THE VALUE OF KFORX(,I2,12H) EXCEEDS 30)
  340 FORMAT(1X,61HINPUT ERROR - THE SUM OF VALUES OF THE KFORX ARRAY EX&
     &CEEDS 30)
      END



!!!     OVERLAY(LWB,1,6)
      SUBROUTINE NUTORD()
!
!     ********************************************VERSION B00***********
!
!
!     REVISE CHORDWISE PANEL SPACING ON FIN (VERTICAL TAIL) OR CANARD
!     (HORIZONTAL TAIL) AND COMPUTE NEW AIRFOIL ORDINATES
!
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAP&
!!!     &E9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      INCLUDE 'files.cmn'
      COMMON /BLANK/DUM(20),NF,NFINOR,NC,NCANOR,DUMMW(3),NWFR,KWF,KWFR
      COMMON /NEWCOM/KDUM(4),K4,K5,KDUMO(12),KF(6),KFINOR(6),KAN(6),     &
     &              KANOR(6)
      COMMON /COEF/ C(4,50),CC(4,50)
      COMMON /SCRAT/ BLOCK(7500)

      DIMENSION TALORG(6,2,4), XT(6,10), TALORD(6,2,10), TALCR(6,10)
      DIMENSION TALOR(6,10), TORD(30), ZORD(30)
      DIMENSION DZC(30), DZT(30), XAF(30), TZORK(20,30), WAFORK(20,30)
      DIMENSION DZCDX(20,30), DZTDX(20,30), DZCDXK(20,30), WAFOR(20,30)
      DIMENSION DZTDXK(20,30), RHO(20), A(20), B(20), R(20), XAT(30), XAFK(6,30)

      EQUIVALENCE (BLOCK(1),TALORG(1,1,1))
      EQUIVALENCE (BLOCK(49),XT(1,1))
      EQUIVALENCE (BLOCK(109),TALORD(1,1,1))
      EQUIVALENCE (BLOCK(469),TALOR(1,1))
      EQUIVALENCE (BLOCK(529),TALCR(1,1))
      EQUIVALENCE (BLOCK(589),WAFOR(1,1))
      EQUIVALENCE (BLOCK(3111),TZORK(1,1))
      EQUIVALENCE (BLOCK(3711),WAFORK(1,1))
      EQUIVALENCE (BLOCK(4311),DZCDX(1,1))
      EQUIVALENCE (BLOCK(4911),DZCDXK(1,1))
      EQUIVALENCE (BLOCK(5511),DZTDXK(1,1))
      EQUIVALENCE (BLOCK(6111),XAFK(1,1))
      EQUIVALENCE (BLOCK(1189),DZC(1))
      EQUIVALENCE (BLOCK(1219),DZT(1))
      EQUIVALENCE (BLOCK(1249),XAT(1))
      EQUIVALENCE (BLOCK(1279),RHO(1))
      EQUIVALENCE (BLOCK(1299),R(1))
      EQUIVALENCE (BLOCK(1309),A(1))
      EQUIVALENCE (BLOCK(1329),B(1))
      EQUIVALENCE (BLOCK(1349),TORD(1))
      EQUIVALENCE (BLOCK(1379),ZORD(1))
      EQUIVALENCE (BLOCK(2511),DZTDX(1,1))
!
!!!      INTEGER TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAPE9,TAPE&
!!!     &10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      LOGICAL FIN
!
      PRINT 5
    5 FORMAT(10X,'  NUTORD   - COMMON /NEWCOM/  ',//)
!!!      CALL PDUMP(KDUM(1),KANOR(6),2)
      WRITE(2,*) KDUM,K4,K5,KDUMO,KF,KFINOR,KAN,KANOR
!
!
      FIN=.FALSE.
      IF (K4.LE.0) GO TO 10
      FIN=.TRUE.
      NT=NF
      NWFR = NFINOR
      I1 = -1
      KL=K4
      GO TO 20
   10 IF (K5.LE.0) GO TO 240
      NT=NC
      NWFR = IABS(NCANOR)
      I1 = -1
!
!     NCANOR NEGATIVE INDICATES UPPER AND LOWER ORDINATES GIVEN
!
      IF(NCANOR.LT.0)I1 = 1
      KL=K5
   20 CONTINUE
      IF(KL.EQ.3) GO TO 26
      DO 25 I=1,NT
      RHO(I)=0.0
   25 END DO
      GO TO 27
   26 READ(TAPE5,250) (RHO(I),I=1,NT)
   27 CONTINUE
!
!     CALCULATE REVISED ORDINATES FOR EACH TAIL SEGMENT
!
      DO 230 N=1,NT
      KWFR = 0
      IF(FIN.AND.KFINOR(N).GT.0) KWFR = KFINOR(N)
      IF(.NOT.FIN.AND.KANOR(N).GT.0) KWFR = KANOR(N)
      IF(KWFR.EQ.0) GO TO 30
      READ(TAPE5,250)(XAFK(N,K),K=1,KWFR)
      GO TO 50
   30 KWFR = NWFR
      DO 40 K=1,NWFR
   40 XAFK(N,K)=XT(N,K)
   50 CONTINUE
      NWAR = NWFR - 1
!
!     CALCULATE CAMBER AND THICKNESS SLOPES
!
      NDA=-1
      DA=0.
      DO 60 L=1,NWFR
      XAF(L)=XT(N,L)
      ZORD(L)=TALCR(N,L)
   60 TORD(L)=TALOR(N,L)
      IF(I1.LT.0) GO TO 80
      CALL DERIV(XAF,ZORD,NWFR,NDA,DA,DZC)
      DO 70 L=1,NWAR
      DO 70 M=1,4
   70 CC(M,L)=C(M,L)
      GO TO 100
   80 DO 90 L=1,NWAR
      DZC(L)=0.
      DO 90 M=1,4
   90 CC(M,L)=0.
      DZC(NWFR) = 0.
  100 NWA = NWFR
      IF (KL.LT.3.OR.RHO(N).EQ.0.) GO TO 120
!
!     CALCULATE INITIAL SLOPE FOR ROUND LEADING EDGE
!
      NWA=NWAR
      NDA=0
      R(N)=SQRT(2.*RHO(N))
      SAF2=SQRT(XAF(2))
      SAF3=SQRT(XAF(3))
      CON2=TORD(2)/XAF(2)-R(N)/SAF2
      CON3=TORD(3)/XAF(3)-R(N)/SAF3
      DX=XAF(3)-XAF(2)
      A(N)=(CON2*XAF(3)-CON3*XAF(2))/DX
      B(N)=(CON3-CON2)/DX
      DA=R(N)/(2.*SAF2)+A(N)+2.*B(N)*XAF(2)
      DO 110 L=1,NWAR
      XAT(L)=XAF(L+1)
  110 TORD(L)=TORD(L+1)
      GO TO 140
  120 DO 130 L=1,NWA
  130 XAT(L)=XAF(L)
  140 CALL DERIV (XAT,TORD,NWA,NDA,DA,DZT)
      DO 150 L=1,NWFR
      DZCDX(N,L)=DZC(L)
  150 DZTDX(N,L)=DZT(L)
      IF (KL.LT.3.OR.RHO(N).EQ.0.) GO TO 170
      DZTDX(N,1)=900.
      DO 160 L=2,NWFR
  160 DZTDX(N,L)=DZT(L-1)
  170 CONTINUE
      IF(KWFR.EQ.0)GO TO 230
!
!     INTERPOLATE FOR REVISED CAMBER AND THICKNESS ORDINATES AND SLOPES
!
      TZORK(N,1)=TALCR(N,1)
      DZCDXK(N,1)=DZCDX(N,1)
      WAFORK(N,1)=TALOR(N,1)
      DZTDXK(N,1)=DZTDX(N,1)
      KI=2
      DO 220 J=1,NWAR
      DO 200 K=KI,KWFR
      IF (XAFK(N,K).GT.XAF(J+1)) GO TO 210
      XJ=XAFK(N,K)
      TZORK(N,K) = CC(1,J) + XJ*(CC(2,J) + XJ*(CC(3,J) + XJ*CC(4,J)))
      DZCDXK(N,K) = CC(2,J) + XJ*(2.*CC(3,J) + 3.*XJ*CC(4,J))
      L=J
      XL=XJ
      IF (KL.LT.3.OR.RHO(N).EQ.0.) GO TO 190
      IF (J.GT.1) GO TO 180
      SXJ=SQRT(XJ)
      DZTDXK(N,K)=R(N)/(2.*SXJ)+A(N)+2.*B(N)*XJ
      WAFORK(N,K)=R(N)*SXJ+XJ*(A(N)+B(N)*XJ)
      GO TO 200
  180 XL=XJ-XAF(1)
      L=J-1
  190 WAFORK(N,K) = C(1,L) + XL*(C(2,L) + XL*(C(3,L)+ XL*C(4,L)))
      DZTDXK(N,K) = C(2,L) + XL*(2.*C(3,L) + 3.*XL*C(4,L))
  200 END DO
  210 KI=K
  220 END DO
  230 END DO
  240 CONTINUE
!
!
!
  250 FORMAT (10F7.0)
      END



!!!     OVERLAY(LWB,1,7)
      SUBROUTINE TALPAN
!
!     *******************************************VERSION B00************
!
!
!     REVISE SPANWISE PANEL SPACING ON FIN (VERTICAL TAIL) OR CANARD
!     (HORIZONTAL TAIL) AND COMPUTE NEW PANEL GEOMETRY
!
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAP&
!!!     &E9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      INCLUDE 'files.cmn'
      COMMON /BLANK/DUM(20),NF,NFINOR,NK,NCANOR,DUMW(3),NWFR,KWF,KWFR
      COMMON /PARAM/ NBODY,NWING,NTAIL,LBC,THK,MACH,ALPHA,REFA
      COMMON /NEWCOM/KDUM(4),K4,K5,KDUMO(12),KF(6),KFINOR(6),KAN(6),     &
     &              KANOR(6),LDUMO(2),KOL,NCPT
      COMMON /SEG/ NSEG,NROW(20),NCOL(20),COSS(20),SINS(20),BTE(20),NWT(&
     &20),SPNW(20),XLEW(20),BLE(20),ZLEW(20),XS(20),YS(20),ZS(20)
      COMMON /SCRAT/ BLOCK(7500)
      COMMON /POINT/ ARRAY(6000)
      COMMON /VELCOM/ DUM1(6),PRINT
      COMMON /GRAPH/ PLOT
      COMMON /PI/ PI,RADEG,DEGRAD
      COMMON /EPS/ EPSPAN,EPSWP1,EPSWP2,EPSTP,EPSVC,EPSSP1,EPSSP2,EPSLV,&
     &EPSSV,EPSVV,EPSWV,EPSVP,EPSIT1,EPSIT2,EPSFM, epssb1,epssb2
!
      DIMENSION XPT(600), YPT(600), ZPT(600), THET(600), DELTA(600)
      DIMENSION XC(30,20), YC(30,20), ZC(30,20), ZU(30,20)
      DIMENSION AREA(600), XE(600)
      DIMENSION TALORG(6,2,4), XT(6,10), TALORD(6,2,10), TALCR(6,10), TALOR(6,10)
      DIMENSION WAFORG(2,4), TZORK(20,30), WAFORK(20,30)
      DIMENSION DZCDX(20,30), DZTDX(20,30), DZCDXK(20,30), DZTDXK(20,30)
      DIMENSION SLOPE(600), XAFK(6,30), XK(20), YK(20), ZK(20)
      DIMENSION CK(20), CD(20), ZY(20), BL(20), TH(20), BT(20), CHORD(600)
!
      EQUIVALENCE (BLOCK(1),TALORG(1,1,1))
      EQUIVALENCE (BLOCK(49),XT(1,1))
      EQUIVALENCE (BLOCK(109),TALORD(1,1,1))
      EQUIVALENCE (BLOCK(469),TALOR(1,1))
      EQUIVALENCE (BLOCK(529),TALCR(1,1))
      EQUIVALENCE (BLOCK(589),ZY(1))
      EQUIVALENCE (BLOCK(609),XK(1))
      EQUIVALENCE (BLOCK(629),YK(1))
      EQUIVALENCE (BLOCK(649),ZK(1))
      EQUIVALENCE (BLOCK(669),CK(1))
      EQUIVALENCE (BLOCK(689),BL(1))
      EQUIVALENCE (BLOCK(709),BT(1))
      EQUIVALENCE (BLOCK(729),TH(1))
      EQUIVALENCE (BLOCK(2511),DZTDX(1,1))
      EQUIVALENCE (BLOCK(3111),TZORK(1,1))
      EQUIVALENCE (BLOCK(3711),WAFORK(1,1))
      EQUIVALENCE (BLOCK(4311),DZCDX(1,1))
      EQUIVALENCE (BLOCK(4911),DZCDXK(1,1))
      EQUIVALENCE (BLOCK(5511),DZTDXK(1,1))
      EQUIVALENCE (BLOCK(6111),XAFK(1,1))
      EQUIVALENCE (BLOCK(6301),CHORD(1))
      EQUIVALENCE (BLOCK(6901),SLOPE(1))
      EQUIVALENCE (BLOCK(6901),ZU(1,1))
      EQUIVALENCE (ARRAY(1),XPT(1))
      EQUIVALENCE (ARRAY(601),YPT(1))
      EQUIVALENCE (ARRAY(1201),ZPT(1))
      EQUIVALENCE (ARRAY(1801),THET(1))
      EQUIVALENCE (ARRAY(2401),DELTA(1))
      EQUIVALENCE (ARRAY(3001),XC(1,1))
      EQUIVALENCE (ARRAY(3601),YC(1,1))
      EQUIVALENCE (ARRAY(4201),ZC(1,1))
      EQUIVALENCE (ARRAY(4801),AREA(1))
      EQUIVALENCE (ARRAY(5401),XE(1))
!
      LOGICAL LBC,THK,FIN
      INTEGER PRINT,PLOT
!!!,TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8&
!!!     &,TAPE9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
!
!     IN LINE FUNCTION DEFINITION
!
      XIN(X1,Y1,X2,Y2,Y)=X1+(X2-X1)*(Y-Y1)/(Y2-Y1)
!
!
!
      PRINT 5
    5 FORMAT(10X,'  TALPAN   - COMMON /NEWCOM/  ',//)
!!!      CALL PDUMP(KDUM(1),NCPT,2)
      WRITE(2,*) KDUM,K4,K5,KDUMO,KF,KFINOR,KAN,KANOR,LDUMO,KOL,NCPT

      FIN=.FALSE.
      IF (K4.LE.0) GO TO 10
      FIN=.TRUE.
      IF (PRINT.LT.0) WRITE (TAPE6,440)
      NTAL=NF
      KK=K4
      KL=K4
      K4=0
      GO TO 20
   10 IF (K5.LE.0) GO TO 360
      KK=0
      KL=K5
      NTAL=NK
      IF (PRINT.LT.0) WRITE (TAPE6,450)
   20 CONTINUE
      IF (PRINT.LT.0) WRITE (TAPE6,460)
      REWIND TAPE7
!
!     READ WING GEOMETRY FROM TAPE 7
!
      READ (TAPE7) ARRAY,CHORD,SLOPE
      REWIND TAPE7
      KI=1
      NI=1
      NC=NCPT
      NJ=NCPT
      NINIT=NWING
      NP=NWING
      NC1=NC+1
      NP1=NP+1
      NKOL=0
      NKIL=0
      IF (NSEG.EQ.0) GO TO 40
      DO 30 N=1,NSEG
   30 NKOL=NKOL+NCOL(N)
      NKIL=NKOL
   40 CONTINUE
!
!     CALCULATE PANEL GEOMETRY FOR EACH TAIL SEGMENT
!
      DO 290 NT=1,NTAL
      IF(FIN)KWF=KF(NT)
      IF(.NOT.FIN) KWF=KAN(NT)
      KWF = IABS(KWF)
      IF(KWF.EQ.0) GO TO 50
!
!     READ INTERMEDIATE SPAN STATIONS
!
      READ(TAPE5,430)(YK(K),K=1,KWF)
   50 KWFR = NWFR
      IF(FIN.AND.KFINOR(NT).GT.0) KWFR=KFINOR(NT)
      IF(.NOT.FIN.AND.KANOR(NT).GT.0) KWFR=KANOR(NT)
      DO 70 N=1,2
      WAFORG(N,1)=TALORG(NT,N,1)
      IF (KK.GT.0) GO TO 60
      WAFORG(N,2)=TALORG(NT,N,2)
      WAFORG(N,3)=TALORG(NT,N,3)
      GO TO 70
   60 WAFORG(N,2)=TALORG(NT,N,3)
      WAFORG(N,3)=TALORG(NT,N,2)
   70 WAFORG(N,4)=TALORG(NT,N,4)
      IF(KWF.NE.0) GO TO 90
      KWF = 2
      DO 80 K=1,2
   80 YK(K) = WAFORG(K,2)
   90 LLI = KOL + KI
      LLF = KOL + KWF
      N=1
      M=2
      DELY=WAFORG(N+1,2)-WAFORG(N,2)
      IF (DELY.EQ.0.) GO TO 290
      DELX=WAFORG(N+1,1)-WAFORG(N,1)
      DELZ=WAFORG(N+1,3)-WAFORG(N,3)
      DELC=WAFORG(N+1,4)-WAFORG(N,4)
      DELYZD=1.0/SQRT(DELY*DELY+DELZ*DELZ)
      BL(N)=DELX/DELYZD
      BT(N)=(DELX+DELC)/DELYZD
      CD(N)=WAFORG(N,4)                          ! never used ???
      IF (FIN) TH(N)=ATAN2(DELY,DELZ)
      IF (.NOT.FIN) TH(N)=ATAN2(DELZ,DELY)
      NSEG=NSEG+1
      SINS(NSEG)=SIN(TH(N))
      COSS(NSEG)=COS(TH(N))
      BLE(NSEG)=BL(N)
      BTE(NSEG)=BT(N)
      NWT(NSEG)=1
      IF (FIN) NWT(NSEG)=-1
      IF (.NOT.FIN.AND.KAN(NT).LT.0) NWT(NSEG)=-1
!
!     CALCULATE ORIGINS OF INTERMEDIATE CHORDS
!
      DO 240 K = KI,KWF
      K1=K-1
      NKOL=NKIL+K1
      L=K+KOL
      L1=L-1
      XK(K)=XIN(WAFORG(NI,1),WAFORG(NI,2),WAFORG(M,1),WAFORG(M,2),YK(K))
      ZK(K)=XIN(WAFORG(NI,3),WAFORG(NI,2),WAFORG(M,3),WAFORG(M,2),YK(K))
      CK(K)=XIN(WAFORG(NI,4),WAFORG(NI,2),WAFORG(M,4),WAFORG(M,2),YK(K))
      CL=CK(K)/100.
      LP=1
      SJ=1.0
      ZY(K)=ZK(K)
      IF (FIN) ZK(K)=YK(K)
  100 CONTINUE
!
!     CALCULATE COORDINATES OF PANEL CORNERS
!
      DO 190 J=1,KWFR
      XC(J,L)=XK(K)+CL*XAFK(NT,J)
      ZC(J,L)=ZK(K)
      IF (LBC) GO TO 110
      ZCAM=TZORK(NT,J)
      ZTHK=WAFORK(NT,J)
      ZC(J,L)=ZY(K)+CL*(ZCAM+SJ*ZTHK)
      IF (LP.EQ.1) ZU(J,L)=ZC(J,L)
      IF (FIN) YK(K)=ZK(K)
      GO TO 120
  110 IF (FIN) YK(K)=ZY(K)
  120 YC(J,L)=YK(K)
      IF (K.EQ.KI) GO TO 190
      NJ=NJ+1
      IF (J.EQ.1) GO TO 190
      JJ=J-1
      NC=NC+1
      NP=NP+1
      IP=1
      IF (SJ.LT.0.) IP=0
      IQ=0
      IF (FIN) IQ=1
!
!     CALCULATE PANEL INCLINATIONS AND CENTROIDS ON TAIL SURFACE
!
      IF (.NOT.LBC) CALL PANEL (IP,IQ,J,L,LP,NP,AP)
      AREA(NP)=AP
      CHORD(NP)=0.
      IF (PRINT.GE.0) GO TO 160
      IF (.NOT.LBC.AND.FIN) GO TO 140
      IF (.NOT.LBC.AND.LP.EQ.1) GO TO 130
      WRITE (TAPE6,470) NP,XC(JJ,L1),YK(K1),ZC(JJ,L1),XC(J,L1),YK(K1),ZC&
     &(J,L1),XC(JJ,L),YK(K),ZC(JJ,L),XC(J,L),YK(K),ZC(J,L)
      GO TO 160
  130 WRITE (TAPE6,470) NP,XC(JJ,L1),YK(K1),ZU(JJ,L1),XC(J,L1),YK(K1),ZU&
     &(J,L1),XC(JJ,L),YK(K),ZU(JJ,L),XC(J,L),YK(K),ZU(J,L)
      GO TO 160
  140 IF (LP.EQ.1) GO TO 150
      WRITE (TAPE6,470) NP,XC(JJ,L1),ZC(JJ,L1),YK(K1),XC(J,L1),ZC(J,L1),&
     &YK(K1),XC(JJ,L),ZC(JJ,L),YK(K),XC(J,L),ZC(J,L),YK(K)
      GO TO 160
  150 WRITE (TAPE6,470) NP,XC(JJ,L1),ZU(JJ,L1),YK(K1),XC(J,L1),ZU(J,L1),&
     &YK(K1),XC(JJ,L),ZU(JJ,L),YK(K),XC(J,L),ZC(J,L),YK(K)
!
!     CALCULATE PANEL CONTROL POINTS IN PLANE OF TAIL
!
  160 CONTINUE
      CR=XC(J,L1)-XC(JJ,L1)
      CT=XC(J,L)-XC(JJ,L)
      RI=(1.+CT/(CR+CT))/3.
      RO=1.-RI
      XLE=RI*XC(JJ,L)+RO*XC(JJ,L1)
      XTE=RI*XC(J,L)+RO*XC(J,L1)
      CHORD(NP)=XTE-XLE
      SPN=SQRT((YK(K)-YK(K1))*(YK(K)-YK(K1))+(ZK(K)-ZK(K1))*(ZK(K)-ZK(K1&
     &)))
      SPNW(NKOL)=SPN
      IF (J.EQ.2) XLEW(NKOL)=XLE
      YLE=RI*YK(K)+RO*YK(K1)
      ZLE=RI*ZK(K)+RO*ZK(K1)
      IF (J.EQ.2) ZLEW(NKOL)=ZLE
      IF (LBC) GO TO 170
      GO TO 190
  170 CONTINUE
      XPT(NC)=XLE
      XE(NC)=XPT(NC)
      YPT(NC)=YLE
      ZPT(NC)=ZLE
!
!     CALCULATE PANEL AREAS, CHORDS, AND INCLINATION ANGLES
!
      AREA(NP)=.5*SPN*(CR+CT)
      THET(NC)=TH(N)
!
!     CALCULATE CAMBER AND THICKNESS SLOPES
!
      KJ=KI+1
      IF (K.GT.KJ) GO TO 180
      DZCDX(NT,J)=DZCDXK(NT,J)
      IF (J.EQ.2) DZCDX(NT,1)=DZCDXK(NT,1)
      DZTDX(NT,J)=DZTDXK(NT,J)
  180 DELTA(NC) = DZCDX(NT,JJ)
      SLOPE(NJ)=DZTDX(NT,J)
      IF(J.NE.KWFR)GO TO 190
      NC=NC+1
      XPT(NC)=XTE-EPSTP
      XE(NC)=XPT(NC)
      YPT(NC)=YPT(NC-1)
      ZPT(NC)=ZPT(NC-1)
      DELTA(NC)=DZCDXK(NT,J)
      THET(NC)=TH(N)
      CDC = 0.01*(XTE - XLEW(NKOL))
  190 END DO
      IF (LBC) GO TO 200
      IF (SJ.LT.0.) GO TO 200
      SJ=-1.0
      LP=2
      GO TO 100
  200 CONTINUE
      IF (K.EQ.KI) GO TO 240
      IF (K.GT.KJ) GO TO 230
      IF (.NOT.LBC) GO TO 240
      IF (KL.EQ.3) GO TO 210
!
!     CALCULATE INITIAL SLOPE FOR SHARP LEADING EDGE AIRFOILS
!
      DZTDX(NT,1)=DZTDXK(NT,1)
      GO TO 230
!
!     CALCULATE INITIAL SLOPE FOR ROUND LEADING EDGE AIRFOIL
!
  210 NPJ=NP-JJ
      ZTE = CDC*WAFORK(NT,KWFR)/CHORD(NPJ + 1)
      SLE=2.*ZTE-DZTDX(NT,2)
      DO 220 I=2,JJ
  220 SLE=SLE-(DZTDX(NT,I)+DZTDX(NT,I+1))*CHORD(NPJ+I)/CHORD(NPJ+1)
      DZTDX(NT,1)=SLE
  230 NJJ=NJ-JJ
      SLOPE(NJJ)=DZTDX(NT,1)
  240 END DO
      IF (PLOT.EQ.0) GO TO 280
      IF (.NOT.LBC.AND.FIN) GO TO 260
      IF (.NOT.LBC) GO TO 250
      GO TO 270
  250 WRITE(TAPE3)((XC(J,L),YC(J,L),ZU(J,L),J=1,KWFR),L=LLI,LLF)
      WRITE(TAPE3)((XC(J,L),YC(J,L),ZC(J,L),J=1,KWFR),L=LLI,LLF)
      GO TO 280
  260 WRITE(TAPE3)((XC(J,L),ZU(J,L),YC(J,L),J=1,KWFR),L=LLI,LLF)
      WRITE(TAPE3)((XC(J,L),ZC(J,L),YC(J,L),J=1,KWFR),L=LLI,LLF)
      GO TO 280
  270 WRITE(TAPE3)((XC(J,L),YC(J,L),ZC(J,L),J=1,KWFR),L=LLI,LLF)
      WRITE(TAPE3)((XC(J,L),YC(J,L),ZC(J,L),J=1,KWFR),L=LLI,LLF)
      WRITE(TAPE6,400)((XC(J,L),YC(J,L),ZC(J,L),J=1,KWFR),L=LLI,LLF)
  280 CONTINUE
!
!     COMPUTE NUMBER OF ROWS AND COLUMNS IN EACH TAIL SEGMENT
!
      NROW(NSEG)=JJ
      NCOL(NSEG) = KWF - KI
      NCPT=NC
      NKIL=NKOL
      IF (NCPT.GT.600) GO TO 340
      NWING=NP
      NTAIL=NWING-NINIT
      KOL = KOL + KWF
  290 END DO
      IF (PRINT.GE.0) GO TO 330
      IF (FIN.AND.LBC) WRITE (TAPE6,410)
      IF (FIN.AND..NOT.LBC) WRITE (TAPE6,480)
      IF (.NOT.FIN.AND.LBC) WRITE (TAPE6,420)
      IF (.NOT.FIN.AND..NOT.LBC) WRITE (TAPE6,490)
      IF (LBC) WRITE (TAPE6,380)
      IF (.NOT.LBC) WRITE (TAPE6,390)
      DO 310 NP=NC1,NCPT
      TDEG=THET(NP)*RADEG
      DDEG=DELTA(NP)*RADEG
      IF (LBC) WRITE (TAPE6,500) NP,XPT(NP),YPT(NP),ZPT(NP),THET(NP),DEL&
     &TA(NP),SLOPE(NP),TDEG
      IF (FIN.AND..NOT.LBC) GO TO 300
      IF (.NOT.LBC) WRITE (TAPE6,500) NP,XPT(NP),YPT(NP),ZPT(NP),THET(NP&
     &),DELTA(NP),DDEG,TDEG
      GO TO 310
  300 WRITE (TAPE6,500) NP,XPT(NP),YPT(NP),ZPT(NP),THET(NP),DELTA(NP),DD&
     &EG,TDEG
  310 END DO
      IF (FIN) WRITE (TAPE6,540)
      IF (.NOT.FIN) WRITE (TAPE6,530)
      WRITE (TAPE6,510)
      DO 320 NP=NP1,NWING
  320 WRITE (TAPE6,520) NP,AREA(NP),CHORD(NP)
  330 CONTINUE
!
!     STORE WING AND TAIL GEOMETRY ON TAPE 7
!
      WRITE (TAPE7) ARRAY,CHORD,SLOPE
      GO TO 350
  340 CONTINUE
      WRITE (TAPE6,370)
      WRITE (TAPE6,550)
      STOP   !!! CALL EXIT
  350 CONTINUE
  360 CONTINUE
!
!
!
  370 FORMAT (1H0,26H ERROR MESSAGE FROM TALPAN/1H0)
  380 FORMAT (1H0,5HPOINT,8X,1HX,10X,1HY,10X,1HZ,10X,5HTHETA,6X,6HCAMBER&
     &,5X,9HTHICKNESS,2X,5HTHETA/15X,3(2HCP,9X),3HRAD,8X,5HSLOPE,6X,5HSL&
     &OPE,6X,3HDEG//)
  390 FORMAT (1H0,5HPOINT,8X,1HX,10X,1HY,10X,1HZ,10X,5HTHETA,6X,5HDELTA,&
     &6X,5HDELTA,6X,5HTHETA/15X,3(2HCP,9X),3HRAD,8X,3HRAD,8X,3HDEG,8X,3H&
     &DEG//)
  400 FORMAT (10X,3(E14.7,2X))
  410 FORMAT (1H1,1X,79HFIN PANEL EDGE POINTS ON CHORD PASSING THROUGH C&
     &ENTROID, AND INCLINATION ANGLES)
  420 FORMAT (1H1,1X,80HTAIL PANEL EDGE POINTS ON CHORD PASSING THROUGH &
     &CENTROID, AND INCLINATION ANGLES)
  430 FORMAT (10F7.0)
  440 FORMAT (1H1,9X,35H FIN PANEL CORNER POINT COORDINATES/10X,86H 1 AN&
     &D 3 INDICATE FIN PANEL LEADING-EDGE POINTS, 2 AND 4 INDICATE TRAIL&
     &ING-EDGE POINTS)
  450 FORMAT (1H1,9X,35HTAIL PANEL CORNER POINT COORDINATES/10X,86H1 AND&
     & 3 INDICATE TAIL PANEL LEADING-EDGE POINTS, 2 AND 4 INDICATE TRAIL&
     &ING-EDGE POINTS)
  460 FORMAT (1H0,5X,5HPANEL,4(8X,1HX,8X,1HY,8X,1HZ)/20X,3(1H1,8X),3(1H2&
     &,8X),3(1H3,8X),3(1H4,8X)//)
  470 FORMAT (1H ,4X,I3,4X,12F9.5)
  480 FORMAT (1H1,1X,49H FIN PANEL CENTROID POINTS AND INCLINATION ANGLE&
     &S)
  490 FORMAT (1H1,1X,49HTAIL PANEL CENTROID POINTS AND INCLINATION ANGLE&
     &S)
  500 FORMAT (1H ,1X,I3,4X,8F11.5)
  510 FORMAT (1H0,5HPANEL,6X,4HAREA,8X,5HCHORD)
  520 FORMAT (1H ,1X,I3,4X,2F11.5)
  530 FORMAT (1H1,9X,27HTAIL PANEL AREAS AND CHORDS)
  540 FORMAT (1H1,9X,27H FIN PANEL AREAS AND CHORDS)
  550 FORMAT (65H ERROR - NUMBER OF WING AND TAIL PANEL CONTROL POINTS E&
     &XCEEDS 600)
      END


!!!      OVERLAY(LWB,1,10)
      SUBROUTINE NACSHL
!
!***********************************************************************
!
!          READ NACELLE OR SHELL INPUTS
!
!          AND CALCULATE PANEL GEOMETRY
!
!***********************************************************************
!
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,   &
!!!     &               TAPE9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      INCLUDE 'files.cmn'
      COMMON /PARAM/ NBODY,NWING,NTAIL,LBC,THK,MACH,ALPHA,REFA
      COMMON /NEWCOM/ KDUM(6),K6,KDUMO(37),KOL,NCPT
      COMMON /SEG/ NSEG,NROW(20),NCOL(20),COSS(20),SINS(20),BTE(20),NWT(&
     &20),SPNW(20),XLEW(20),BLE(20),ZLEW(20),XS(20),YS(20),ZS(20)
      COMMON /SCRAT/ BLOCK(7500)
      COMMON /POINT/ ARRAY(6000)
      COMMON /VELCOM/ DUM1(6),PRINT
      COMMON /GRAPH/ PLOT
      COMMON /PI/ PI,RADEG,DEGRAD
      COMMON /EPS/ EPSPAN,EPSWP1,EPSWP2,EPSTP,EPSVC,EPSSP1,EPSSP2,EPSLV,&
     &EPSSV,EPSVV,EPSWV,EPSVP,EPSIT1,EPSIT2,EPSFM, epssb1,epssb2
      COMMON /SHELL/ NPTSB,NSEGWT,NPTSWT,NSEGN,NPTSN,NSEGS,NPTSS,       &
     &               NSEGNS,NPTSNS,NNAC,NSHELL,NQ,IPART, iidum
!
      DIMENSION XPT(600), YPT(600), ZPT(600), THET(600), DELTA(600)
      DIMENSION XC(30,20), YC(30,20), ZC(30,20), AREA(600), XE(600)
      DIMENSION CHORD(600), SLOPE(600), X(20), Y(20), Z(20), C(20), TH(20)
      DIMENSION BL(20), BT(20), XOC(30), DZCDX(20,30), DZTDX(20,30)
!!!      DIMENSION TITLE(14)
      CHARACTER(LEN=80):: title   ! note /HEAD/ is not in scope
!
      EQUIVALENCE (ARRAY(1),XPT(1))
      EQUIVALENCE (ARRAY(601),YPT(1))
      EQUIVALENCE (ARRAY(1201),ZPT(1))
      EQUIVALENCE (ARRAY(1801),THET(1))
      EQUIVALENCE (ARRAY(2401),DELTA(1))
      EQUIVALENCE (ARRAY(3001),XC(1,1))
      EQUIVALENCE (ARRAY(3601),YC(1,1))
      EQUIVALENCE (ARRAY(4201),ZC(1,1))
      EQUIVALENCE (ARRAY(4801),AREA(1))
      EQUIVALENCE (ARRAY(5401),XE(1))
      EQUIVALENCE (BLOCK(1),X(1))
      EQUIVALENCE (BLOCK(21),Y(1))
      EQUIVALENCE (BLOCK(41),Z(1))
      EQUIVALENCE (BLOCK(61),C(1))
      EQUIVALENCE (BLOCK(81),TH(1))
      EQUIVALENCE (BLOCK(101),BL(1))
      EQUIVALENCE (BLOCK(121),BT(1))
      EQUIVALENCE (BLOCK(141),XOC(1))
      EQUIVALENCE (BLOCK(171),DZCDX(1,1))
      EQUIVALENCE (BLOCK(771),DZTDX(1,1))
      EQUIVALENCE (BLOCK(6301),CHORD(1))
      EQUIVALENCE (BLOCK(6901),SLOPE(1))
!
      LOGICAL LBC,THK,NAC,SHELL
      INTEGER PRINT,PLOT
!!!,TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,     &
!!!     & TAPE8,TAPE9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
!
!          INITIALIZE CONSTANTS
!
      NAC=.FALSE.
      SHELL=.FALSE.
      IF(K6.EQ.0) GO TO 10
      IF(K6.EQ.1) GO TO 20
   10 NN=NNAC
      NAC=.TRUE.
      GO TO 30
   20 NN=NSHELL
      SHELL=.TRUE.
   30 CONTINUE
      IF(NN.EQ.0) GO TO 130
      NC=NCPT
      NC1=NC+1
      NP=NWING
      NP1=NP+1
      NKOL=0
      IF(NSEG.EQ.0) GO TO 50
      DO 40 N=1,NSEG
   40 NKOL=NKOL+NCOL(N)
   50 CONTINUE
!
!          CALCULATE GEOMETRY FOR EACH NACELLE OR SHELL
!
      IF (NAC) WRITE(TAPE6,600)
      IF (SHELL) WRITE(TAPE6,605)
      WRITE(TAPE6,610)
      DO 100 NS=1,NN
!
!          INPUT NACELLE OR SHELL GEOMETRY
!
      READ(TAPE5,'(A)') TITLE
      READ(TAPE5,500) IAXI,IRAD
      IF(NAC.AND.IRAD.LT.0) THK=.TRUE.
      NAXI=IABS(IAXI)
      NRAD=IABS(IRAD)
      READ(TAPE5,520) (XOC(J),J=1,NAXI)
      READ(TAPE5,510) (X(K),Y(K),Z(K),C(K),K=1,NRAD)
      IF(IAXI.GE.0) GO TO 54
      DO 52 K=2,NRAD
   52 READ(TAPE5,520) (DZCDX(K,J),J=1,NAXI)
   54 IF(IRAD.GE.0) GO TO 58
      DO 56 K=2,NRAD
   56 READ(TAPE5,520) (DZTDX(K,J),J=1,NAXI)
   58 CONTINUE
!
!          CALCULATE SEGMENT AND COLUMN GEOMETRY
!
      DO 100 K=1,NRAD
      K1=K-1
      KOL=KOL+1
      L=KOL
      L1=L-1
      IF(K.EQ.1) GO TO 70
      NKOL=NKOL+1
      DELX=X(K)-X(K1)
      DELC=C(K)-C(K1)
      DELY=Y(K)-Y(K1)
      DELZ=Z(K)-Z(K1)
      SPN=SQRT(DELY*DELY+DELZ*DELZ)
      TH(NKOL)=ATAN2(DELZ,DELY)
      BL(NKOL)=DELX/SPN
      BT(NKOL)=(DELX+DELC)/SPN
      SPNW(NKOL)=SPN
      IF(K.EQ.2) GO TO 60
      IF(ABS(TH(NKOL)-TH(NKOL-1)).GT.EPSWP2) GO TO 60
      IF(ABS(BL(NKOL)-BL(NKOL-1)).GT.EPSWP2) GO TO 60
      IF(ABS(BT(NKOL)-BT(NKOL-1)).GT.EPSWP2) GO TO 60
      NCOL(NSEG)=NCOL(NSEG)+1
      GO TO 70
   60 NSEG=NSEG+1
      NROW(NSEG)=NAXI-1
      NCOL(NSEG)=1
      COSS(NSEG)=COS(TH(NKOL))
      SINS(NSEG)=SIN(TH(NKOL))
      BTE(NSEG)=BT(NKOL)
      BLE(NSEG)=BL(NKOL)
      IF (NAC) NWT(NSEG)=2
      IF (NAC.AND.K.EQ.2) NWT(NSEG)=-2
      IF (SHELL) NWT(NSEG)=3
      IF (SHELL.AND.K.EQ.2) NWT(NSEG)=-3
   70 CONTINUE
!
!          CALCULATE PANEL CORNER POINTS
!
      DO 100 J=1,NAXI
      J1=J-1
      XC(J,L)=X(K)+C(K)*XOC(J)/100.0
      YC(J,L)=Y(K)
      ZC(J,L)=Z(K)
!
!          CALCULATE PANEL EDGE POINTS THROUGH CENTROID
!
      IF(K.EQ.1) GO TO 100
      NC=NC+1
      IF(J.EQ.1) GO TO 80
      CR=XC(J,L1)-XC(J1,L1)
      CT=XC(J,L)-XC(J1,L)
      RI=(1.0+CT/(CR+CT))/3.0
      RO=1.0-RI
      XLE=RI*XC(J1,L)+RO*XC(J1,L1)
      XTE=RI*XC(J,L)+RO*XC(J,L1)
      IF(J.EQ.2) XLEW(NKOL)=XLE
      YLE=RI*Y(K)+RO*Y(K1)
      ZLE=RI*Z(K)+RO*Z(K1)
      IF(J.EQ.2) ZLEW(NKOL)=ZLE
   80 CONTINUE
      XPT(NC)=XTE
      IF(J.EQ.NAXI) XPT(NC)=XTE-EPSTP
      XE(NC)=XPT(NC)
      YPT(NC)=YLE
      ZPT(NC)=ZLE
      IF(J.EQ.2) XPT(NC-1)=XLE
      IF(J.EQ.2) XE(NC-1)=XPT(NC-1)
      IF(J.EQ.2) YPT(NC-1)=YLE
      IF(J.EQ.2) ZPT(NC-1)=ZLE
!
!          CALCULATE PANEL ANGLES
!
      SLOPE(NC)=0.0
      THET(NC)=TH(NKOL)
      DELTA(NC)=0.0
      IF(IAXI.LT.0) DELTA(NC)=DZCDX(K,J)
      IF(NAC.AND.IRAD.LT.0) SLOPE(NC)=DZTDX(K,J)
      IF(J.EQ.1) GO TO 100
!
!          OUTPUT CORNER POINTS
!
      NP=NP+1
      IF(PRINT.GE.0) GO TO 90
      WRITE(TAPE6,620) NP,XC(J1,L1),YC(J1,L1),ZC(J1,L1),XC(J,L1),       &
     & YC(J,L1),ZC(J,L1),XC(J1,L),YC(J1,L),ZC(J1,L),XC(J,L),YC(J,L),    &
     & ZC(J,L)
   90 CONTINUE
!
!          CALCULATE PANEL AREAS AND CHORDS
!
      CHORD(NP)=XTE-XLE
      AREA(NP)=0.5*SPN*(CR+CT)
  100 CONTINUE
!
!          REDEFINE NUMBER OF PANELS AND CONTROL POINTS
!
      NCPT=NC
      NWING=NP
      IF(NCPT.GT.600) GO TO 140
      IF(PRINT.GE.0) GO TO 130
!
!          OUTPUT PANEL EDGE POINTS ON CHORDS THROUGH CENTROIDS
!
      IF (NAC) WRITE(TAPE6,630)
      IF (SHELL) WRITE(TAPE6,635)
      WRITE(TAPE6,640)
      DO 110 NC=NC1,NCPT
      TDEG=THET(NC)*RADEG
  110 WRITE(TAPE6,650) NC,XPT(NC),YPT(NC),ZPT(NC),THET(NC),DELTA(NC),   &
     &                 SLOPE(NC),TDEG
!
!          OUTPUT PANEL AREAS AND CHORDS
!
      IF (NAC) WRITE(TAPE6,660)
      IF (SHELL) WRITE(TAPE6,665)
      WRITE(TAPE6,670)
      DO 120 NP=NP1,NWING
  120 WRITE(TAPE6,680) NP,AREA(NP),CHORD(NP)
!
!          STORE WING, TAIL, AND/OR SHELL GEOMETRY ON TAPE7
!
  130 CONTINUE
      IF(SHELL.AND.NSEG.NE.0) WRITE (TAPE7) ARRAY,CHORD,SLOPE
      GO TO 150
!
  140 WRITE(TAPE6,690)
      IF (NAC) WRITE(TAPE6,700)
      IF (SHELL) WRITE(TAPE6,705)
      STOP   !!! CALL EXIT
  150 CONTINUE
!
!          FORMATS
!
  500 FORMAT(2I3)
  510 FORMAT(4F7.0)
  520 FORMAT(10F7.0)
  530 FORMAT(13A6,A2)
  600 FORMAT(1H1,8X,38HNACELLE PANEL CORNER POINT COORDINATES/10X,89H1 A&
     &ND 3 INDICATE NACELLE PANEL LEADING EDGE POINTS, 2 AND 4 INDICATE &
     &TRAILING EDGE POINTS)
  605 FORMAT(1H1,8X,36HSHELL PANEL CORNER POINT COORDINATES/10X,87H1 AND&
     & 3 INDICATE SHELL PANEL LEADING EDGE POINTS, 2 AND 4 INDICATE TRAI&
     &LING EDGE POINTS)
  610 FORMAT(1H0,5X,5HPANEL,4(8X,1HX,8X,1HY,8X,1HZ)/20X,3(1H1,8X),3(1H2,&
     &8X),3(1H3,8X),3(1H4,8X)//)
  620 FORMAT(1H ,4X,I3,4X,12F9.5)
  630 FORMAT(1H1,1X,83HNACELLE PANEL EDGE POINTS ON CHORD PASSING THROUG&
     &H CENTROID, AND INCLINATION ANGLES)
  635 FORMAT(1H1,1X,81HSHELL PANEL EDGE POINTS ON CHORD PASSING THROUGH &
     &CENTROID, AND INCLINATION ANGLES)
  640 FORMAT(1H0,5HPOINT,8X,1HX,10X,1HY,10X,1HZ,10X,5HTHETA,6X,6HCAMBER,&
     &5X,9HTHICKNESS,2X,5HTHETA/15X,3(2HCP,9X),3HRAD,8X,5HSLOPE,6X,5HSLO&
     &PE,6X,3HDEG//)
  650 FORMAT(1H ,1X,I3,4X,8F11.5)
  660 FORMAT(1H1,9X,30HNACELLE PANEL AREAS AND CHORDS)
  665 FORMAT(1H1,9X,28HSHELL PANEL AREAS AND CHORDS)
  670 FORMAT(1H0,5HPANEL,6X,4HAREA,8X,5HCHORD//)
  680 FORMAT(1H ,1X,I3,4X,2F11.5)
  690 FORMAT(1H0,25H ERROR MESSAGE FROM SHELL/1H0)
  700 FORMAT(77H ERROR - NUMBER OF WING AND TAIL AND NACELLE PANEL CONTR&
     &OL POINTS EXCEEDS 600)
  705 FORMAT(90H ERROR - NUMBER OF WING AND TAIL AND NACELLE AND/OR SHEL&
     &L PANEL CONTROL POINTS EXCEEDS 600)
      END



!!!     OVERLAY(LWB,2,0)
      SUBROUTINE VELCMP()
!
!     *******************************************VERSION B00************
!
!
!     COMPUTE THE VELOCITY COMPONENTS (U,V,W) AT THE PANEL CONTROL
!     POINTS AND FORM THE AERODYNAMIC INFLUENCE COEFFICIENT MATRICES
!
!*
      COMMON /DES/ CLBAR,CMBAR
!!!      COMMON /OPTION/ NOPT(10),NXSTNS,XSTN(20),XAPEX,YAPEX,XLAMDA
      INCLUDE 'option.cmn'
      COMMON /TAPES/ ITAPES(20)
      COMMON /ROWS/ NWCOL, NWRPC(20)
      COMMON /CYCLE/ NEWAIC, IPREP, NDRAG1, NDRAG2, IDIR
!*
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAP&
!!!     &E9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15,TAPE16,TAPE17
      INCLUDE 'files.cmn'
      COMMON /LWB/ LWB
      COMMON /PARAM/ NBODY,NWING,NTAIL,LBC,THK,MACH,ALPHA,REFA
      COMMON /VELCOM/ NPOINT,NPART,IMAX,JMAX,NMAX,EM,PRINT,NWTHK,NWBLOK,&
     &NWROW(20),NBBLOK,NBROW(60)
      COMMON /NEWCOM/KDUM(9),KFUS,KRADX(4),KFORX(4),KDUMW(27),NCPT,      &
     &              LOCPT(20),XCPT(20)
      COMMON /SCRAT/ BLOCK(7500)
      COMMON /POINT/ ARRAY(6000)
      COMMON /SEG/ NSEG,NROW(20),NCOL(20),COSS(20),SINS(20),BT(20),DUM(6&
     &0),BL(20)
      COMMON /MATCOM/ MATIN
      COMMON /EPS/ EPSPAN,EPSWP1,EPSWP2,EPSTP,EPSVC,EPSSP1,EPSSP2,EPSLV,&
     &EPSSV,EPSVV,EPSWV,EPSVP,EPSIT1,EPSIT2,EPSFM, epssb1,epssb2
      COMMON /SHELL/ NPTSB,NSEGWT,NPTSWT,NSEGN,NPTSN,NSEGS,NPTSS,       &
     &               NSEGNS,NPTSNS,NNAC,NSHELL,NQ,IPART,IIDUM
!
      DIMENSION XLE(600), XPT(600), DEL(600), COSTH(600)
      DIMENSION XBT(600), YBT(600), ZBT(600), YPT(600), ZPT(600)
      DIMENSION CHORD(600), DZTDX(600), TI(600)
      DIMENSION D(60,60)
      DIMENSION DELTA(600), DELTI(600), A(600)
!
      EQUIVALENCE (BLOCK(1),DEL(1))
      EQUIVALENCE (BLOCK(601),COSTH(1))
      EQUIVALENCE (BLOCK(3901),XBT(1))
      EQUIVALENCE (BLOCK(4501),YBT(1))
      EQUIVALENCE (BLOCK(5101),ZBT(1))
      EQUIVALENCE (BLOCK(5701),TI(1))
      EQUIVALENCE (BLOCK(6301),CHORD(1))
      EQUIVALENCE (BLOCK(6901),DZTDX(1))
      EQUIVALENCE (ARRAY(1),XPT(1))
      EQUIVALENCE (ARRAY(601),YPT(1))
      EQUIVALENCE (ARRAY(1201),ZPT(1))
      EQUIVALENCE (ARRAY(1801),D(1,1))
      EQUIVALENCE (ARRAY(2401),DELTA(1))
      EQUIVALENCE (ARRAY(4801),DELTI(1))
      EQUIVALENCE (ARRAY(5401),XLE(1))
      EQUIVALENCE (BLOCK(5701),A(1))
!
      COMMON /STAR/XSTAR(3600),NSEGW
      DIMENSION ZCSTR(600),ZTSTR(600),CPSTR(600)
      DIMENSION DCNSTR(600),DCTSTR(600),DCMSTR(600)
      EQUIVALENCE (XSTAR(1)   , ZCSTR(1))
      EQUIVALENCE (XSTAR(601) , ZTSTR(1))
      EQUIVALENCE (XSTAR(1201), CPSTR(1))
      EQUIVALENCE (XSTAR(1801),DCNSTR(1))
      EQUIVALENCE (XSTAR(2401),DCTSTR(1))
      EQUIVALENCE (XSTAR(3001),DCMSTR(1))

      COMMON /DESIGN/CAMBER(600)
!*
      DIMENSION XBAR(600)
      EQUIVALENCE ( ARRAY(1801),XBAR(1) )
!*
!
      REAL MACH
      LOGICAL LBC,SUB,SUPLE,SUPTE
!!!      INTEGER TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAPE9,TAPE&
!!!     &10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15,TAPE16,TAPE17,FFFILE
!
      MATIN=0
!
!     SET MATRIX BLOCK SIZE AT 60 BY 60
!
      NMAX=60
!
!     READ IN MACH NUMBER AND ANGLE OF ATTACK
!
!*
      NEWAIC = NEWAIC + 1
      READ(TAPE5,290) MACH,ALPHA,CLBAR,CMBAR
!*
      IF (MACH.LT.0..OR.MACH.EQ.EM) GO TO 240
      NEWAIC = 1
      IPREP = 0
      NDRAG1 = 0
      NDRAG2 = 0
      IDIR = 0
      SUB=MACH.LT.1.0
      BETA=SQRT(ABS(MACH*MACH-1.0))
      BETAD=1./BETA
      NBBLOK=0
      NWBLOK=0
      REWIND TAPE8
      REWIND TAPE9
      REWIND TAPE10
      REWIND TAPE11
      N16=ITAPES(16)
      REWIND N16
      CALL WINGS(1)
      NPANEL=NBODY+NWING
      CALL WINGS(2)
      NPANEL=NPANEL+NWING
      IF (NPANEL.EQ.0) GO TO 240
      REWIND TAPE7
      NPTSWT=0
      NPTSN=0
      NPTSS=0
      NPTSNS=0
      DO 60 IP=1,2
      IPART=IP
      CALL WINGS(IPART)
      IF(IPART.EQ.1) NCPT1=NCPT
      IF(IPART.EQ.2) NCPT2=NCPT
      IF(NWING.EQ.0) GO TO 60
      IF(.NOT.LBC) GO TO 10
      READ(TAPE7) ARRAY,CHORD,DZTDX
      IF(IPART.EQ.2) GO TO 10
!*
      N11 = ITAPES(11)
      REWIND N11
!
!      STORE WING GEOMETRY ON N11
!
      WRITE(N11) ARRAY
!
!     SAVE INFO. ALREADY IN ARRAY(1801) TO ARRAY(2400) ON TAPE11
      REWIND TAPE11
      WRITE(TAPE11) XBAR
   10 CONTINUE
      I=0
      J=0
      K=0
      NWBLOK=0
      DO 50 N=1,NSEG
      NC=NCOL(N)
      NR=NROW(N)
      NR1=NR+1
      NWBLOK=NWBLOK+NC
      BLE=BL(N)*BETAD
      SUPLE=.FALSE.
      IF (.NOT.SUB.AND.ABS(BLE).LT.1.0) SUPLE=.TRUE.
      BTE=BT(N)*BETAD
      SUPTE=.FALSE.
      IF (.NOT.SUB.AND.ABS(BTE).LT.1.0) SUPTE=.TRUE.
!*
      IF( NOPT(1) .EQ. 2 ) SUPLE = .FALSE.
      IF( NOPT(1) .EQ. 2 ) SUPTE = .FALSE.
      IF( NOPT(1).EQ.1 .AND. NOPT(4).EQ.2 ) SUPLE = .FALSE.
      IF( NOPT(1).EQ.1 .AND. NOPT(4).EQ.2 ) SUPTE = .FALSE.
!*
      DO 50 M=1,NC
      K=K+1
      NK=NR
      IF (LBC.AND.SUPTE) NK=NR1
      IF (.NOT.LBC) NK=2*NR
      NWROW(K)=NK
      DO 50 L=1,NR1
      I=I+1
      TI(I)=0.
      IF (L.LT.NR1) GO TO 30
      IF (LBC) XPT(I)=XLE(I)
      IF (SUPTE) GO TO 20
      TI(I)=I
      GO TO 46
   20 J=J+1
      IF (LBC) DEL(J)=DELTA(I)
      IF(IPART.EQ.2) GO TO 25
      IF(LBC) ZCSTR(J)=DELTA(I)
      IF(LBC) ZTSTR(J)=DZTDX(I)
      IF(LBC) CAMBER(J)=ZCSTR(J)
   25 CONTINUE
      IF (LBC) COSTH(J)=COSS(N)
      GO TO 46
   30 IF(.NOT.LBC) GO TO 46
      J=J+1
      XF=.50
!*
      IF( NOPT(1) .EQ. 1 ) GO TO 35
      XF = .95
      IF(IPART.EQ.2) GO TO 35
      XBAR(J) = ( XLE( I + 1 ) + XLE ( I ) ) / 2.
   35 CONTINUE
      IF( NOPT(1).EQ.1 .AND. NOPT(4).EQ. 2 ) XF = 0.95
      IF(SUB.AND.NOPT(4).EQ.2) XF=0.85
!*
      XS=XF
      LOCPT(N)=0
      IF (.NOT.SUPTE) GO TO 40
      LOCPT(N)=1
      IF (SUPLE) XS=EPSVC
      XF=XS*FLOAT(NR1-L)/FLOAT(NR1-1)
   40 XPT(I)=XF*XLE(I+1)+(1.-XF)*XLE(I)
      DEL(J)=XF*DELTA(I+1)+(1.-XF)*DELTA(I)
      IF(IPART.EQ.2) GO TO 44
      ZCSTR(J)=XF*DELTA(I+1)+(1.-XF)*DELTA(I)
      ZTSTR(J)=XF*DZTDX(I+1)+(1.-XF)*DZTDX(I)
      CAMBER(J)=ZCSTR(J)
   44 CONTINUE
      COSTH(J)=COSS(N)
      XCPT(N)=XS
   46 CONTINUE
      IF(IPART.EQ.1) NPTSWT=J
      IF(IPART.EQ.2) NPTSNS=J
      IF(IPART.EQ.2.AND.N.LE.NSEGN) NPTSN=J
      IF(IPART.EQ.2.AND.N.GT.NSEGN) NPTSS=NPTSNS-NPTSN
   50 CONTINUE
      WRITE(N16) TI
      IF(IPART.EQ.2) GO TO 55
!*
!
!     STORE XBAR ON N11
!
      WRITE(N11) XBAR
!
!     RESTORE INFO. ON TAPE11 TO COMMON BLOCK / POINT /
!
      REWIND TAPE11
      READ(TAPE11) XBAR
      REWIND TAPE11
!*
   55 CONTINUE
      IF(LBC) NCPT=I
      IF(.NOT.LBC) GO TO 60
      WRITE(TAPE11) DEL,COSTH
      WRITE(TAPE10) ARRAY,CHORD,DZTDX
      CALL WINGS(-IPART)
      WRITE(TAPE6,250) IPART,NWING,NCPT,NSEG
      IF(NSEG.NE.0) WRITE(TAPE6,260) NSEG,(NROW(N),N=1,NSEG)
      IF(NSEG.NE.0) WRITE(TAPE6,270) NSEG,(NCOL(N),N=1,NSEG)
   60 END DO
      REWIND TAPE11
      IF(NBODY.EQ.0) GO TO 65
      WRITE(TAPE6,275) NBODY
      READ(TAPE7) ARRAY
      WRITE(TAPE10) ARRAY
   65 REWIND TAPE7
      REWIND TAPE10
      IF(NPTSWT.NE.0) READ(TAPE10) ARRAY,CHORD,DZTDX
      IF(NPTSWT.NE.0) WRITE(TAPE7) ARRAY,CHORD,DZTDX
      IF(NPTSNS.NE.0) READ(TAPE10) ARRAY,CHORD,DZTDX
      IF(NPTSNS.NE.0) WRITE(TAPE7) ARRAY,CHORD,DZTDX
      IF(NBODY.NE.0) READ(TAPE10) ARRAY
      IF(NBODY.NE.0) WRITE(TAPE7) ARRAY
      EM=MACH
      NPART=0
      N15=ITAPES(15)
      N17=ITAPES(17)
      N18=ITAPES(18)
      N19=ITAPES(19)
      N20=ITAPES(20)
      REWIND N17
      REWIND N18
      REWIND N19
      REWIND N20
      REWIND TAPE10
      REWIND TAPE15
      REWIND TAPE16
      REWIND TAPE17
!
!          CALCULATE VELOCITY COMPONENTS FOR THE MATRIX PARTITIONS
!
!          NSEGNS EQUAL TO ZERO %  IMPLIES NO SHELL
!                  AND THE WING AND NACELLE INFLUENCE THE BODY
!
!          NSEGNS NOT EQUAL TO ZERO %  IMPLIES A SHELL
!                  AND ONLY WING AND NACELLE THICKNESSES
!                  INFLUENCE THE BODY
!
!
      DO 110 I=1,3
      REWIND TAPE7
      REWIND N15
      REWIND N16
      IF(NCPT1.NE.0) READ(TAPE7) ARRAY,CHORD,DZTDX
      IF(NCPT1.NE.0) READ(N16) TI
      NPOINT=NCPT1
      IF(NCPT1.NE.0.AND.I.EQ.2) GO TO 70
      IF(NCPT2.NE.0) READ(TAPE7) ARRAY,CHORD,DZTDX
      IF(NCPT2.NE.0) READ(N16) TI
      NPOINT=NCPT2
      IF(NCPT2.NE.0.AND.I.EQ.3) GO TO 70
      IF(NBODY.NE.0) READ(TAPE7) ARRAY
      NPOINT=NBODY
      IF(NBODY.NE.0.AND.I.EQ.1) GO TO 70
      GO TO 75
   70 WRITE(N15) (ARRAY(II),II=1,3000)
   75 CONTINUE
      DO 110 J=1,3
      NPART=NPART+1
      IF(NBODY.EQ.0.AND.(I.EQ.1.OR.J.EQ.1)) GO TO 110
      IF(NCPT1.EQ.0.AND.(I.EQ.2.OR.J.EQ.2)) GO TO 110
      IF(NCPT2.EQ.0.AND.(I.EQ.3.OR.J.EQ.3)) GO TO 110
      CALL SECOND(TIME)
      WRITE(TAPE6,310) NPART,TIME
      IF(NPART.EQ.1) WRITE(TAPE6,320)
      IF(NPART.EQ.2) WRITE(TAPE6,330)
      IF(NPART.EQ.3) WRITE(TAPE6,400)
      IF(NPART.EQ.4) WRITE(TAPE6,340)
      IF(NPART.EQ.5) WRITE(TAPE6,350)
      IF(NPART.EQ.6) WRITE(TAPE6,390)
      IF(NPART.EQ.7) WRITE(TAPE6,380)
      IF(NPART.EQ.8) WRITE(TAPE6,370)
      IF(NPART.EQ.9) WRITE(TAPE6,360)
      REWIND TAPE7
      IF(NCPT1.EQ.0) GO TO 80
      READ(TAPE7) ARRAY,CHORD,DZTDX
      CALL WINGS(1)
      IF(J.EQ.2) GO TO 95
   80 IF(NCPT2.EQ.0) GO TO 85
      READ(TAPE7) ARRAY,CHORD,DZTDX
      CALL WINGS(2)
      IF(J.EQ.3) GO TO 95
   85 IF(NBODY.EQ.0) GO TO 95
      READ(TAPE7) ARRAY
      DO 90 N=1,NBODY
      XBT(N)=XPT(N)
      YBT(N)=YPT(N)
   90 ZBT(N)=ZPT(N)
   95 REWIND N15
      READ(N15) (ARRAY(II),II=1,2400),(DELTI(II),II=1,600)
      IF(J.EQ.2.OR.J.EQ.3) GO TO 100
!
!          COMPUTE VELOCITY COMPONENTS INDUCED BY BODY PANELS
!
!!!      CALL OVERLAY(LWB,2,1)
      CALL BodVel()
      GO TO 110
!
!          COMPUTE VELOCITY COMPONENTS INDUCED BY WING PANELS
!
!!!  100 IF (LBC) CALL OVERLAY(LWB,2,2)
!!!      IF (.NOT.LBC) CALL OVERLAY(LWB,2,3)
  100 IF (LBC) CALL LinVel()
      IF (.NOT.LBC) CALL WngVel()
      IF(I.EQ.2.AND.J.EQ.2) CALL WINGS(-1)
      IF(I.EQ.3.AND.J.EQ.3) CALL WINGS(-2)
!
  110 CONTINUE
!
      CALL SECOND(TIME)
      WRITE(TAPE6,410) TIME
!
!          FOR CASES WITH NACELLES AND SHELLS
!          WING AND SHELL ARRAYS MUST BE FLIPPED
!
      IF(NPTSNS.EQ.0) GO TO 530
      IREC1=5
      DO 520 K=1,4
      REWIND TAPE15
      REWIND TAPE16
      IREC1=IREC1-1
      IREC2=0
      DO 510 I=1,2
      IPART=I
      CALL WINGS(IPART)
      NCPTS=NWING
      DO 510 J=1,2
      JPART=J
      CALL WINGS(JPART)
      NSING=NWING
      IREC2=IREC2+1
      IF(NSING.EQ.0.OR.NWING.EQ.0) GO TO 505
      DO 500 N=1,NCPTS
      READ(TAPE15) (A(NN),NN=1,NSING)
      IF(IREC1.EQ.IREC2) WRITE(TAPE9) (A(NN),NN=1,NSING)
      READ(TAPE16) (A(NN),NN=1,NSING)
      IF(IREC1.EQ.IREC2) WRITE(N11) (A(NN),NN=1,NSING)
  500 END DO
  505 IF(IREC1.EQ.IREC2) GO TO 520
  510 CONTINUE
  520 END DO
!
  530 CONTINUE
      REWIND TAPE7
      REWIND TAPE8
      REWIND TAPE9
      REWIND TAPE10
      MATIN=1
      IF(NCPT1.NE.0) READ(TAPE7) ARRAY,CHORD,DZTDX
      IF(NCPT2.NE.0) READ(TAPE7) ARRAY,CHORD,DZTDX
      IF(NBODY.NE.0) READ(TAPE7) ARRAY
!
!          WRITE DIAGONAL BLOCKS OF MATRICES
!
      IIN=TAPE10
      IOUT=TAPE7
      IF(NPTSNS.EQ.0) GO TO 115
      IIN=TAPE17
      IOUT=N17
      REWIND IIN
      REWIND IOUT
  115 CONTINUE
      DO 200 IP=1,2
      IPART=3-IP
      CALL WINGS(IPART)
      IF(NWING.LE.NMAX) GO TO 150
      IF(IPART.EQ.1.AND.NPTSNS.NE.0) REWIND TAPE10
      DO 140 JP=1,IPART
      JPART=JP
      CALL WINGS(JPART)
      IF(NWING.LE.NMAX) GO TO 140
      DO 130 NW=1,NWBLOK
      NR=NWROW(NW)
      DO 120 M=1,NR
  120 READ(TAPE10) (D(M,N),N=1,NR)
  130 IF(JPART.EQ.IPART) WRITE(TAPE7) D
  140 END DO
      GO TO 160
  150 NWBLOK=1
      NWROW(1)=NWING
  160 NBBLOK=1
      NBROW(1)=NBODY
      IF(NBODY.LE.NMAX) GO TO 190
      NBBLOK=0
      DO 180 KF=1,KFUS
      NR=KRADX(KF)-1
      NC=KFORX(KF)-1
      DO 180 NN=1,NC
      NBBLOK=NBBLOK+1
      NBROW(NBBLOK)=NR
      IF(IPART.EQ.1) GO TO 180
      DO 170 M=1,NR
  170 READ(IIN) (D(M,N),N=1,NR)
      WRITE(IOUT) D
  180 CONTINUE
  190 CALL WINGS(-IPART)
  200 END DO
      DO 220 IP=1,2
      IPART=IP
      CALL WINGS(IPART)
      IF(IPART.EQ.2) GO TO 210
      WRITE(TAPE6,275) NBODY
      WRITE(TAPE6,285) NBBLOK
  210 WRITE(TAPE6,250) IPART,NWING,NCPT,NSEG
      IF(NSEG.NE.0) WRITE(TAPE6,260) NSEG,(NROW(N),N=1,NSEG)
      IF(NSEG.NE.0) WRITE(TAPE6,270) NSEG,(NCOL(N),N=1,NSEG)
  220 WRITE(TAPE6,280) NWBLOK
      REWIND TAPE7
      REWIND TAPE10
      IF(NPTSNS.NE.0) REWIND IOUT
!
      CALL SECOND(TIME)
      WRITE(TAPE6,300) TIME
!
      CALL WINGS(1)
      NWCOL = 0
      ICT = 0
      DO 175 I = 1, NSEG
      NR = NROW(I)
      NC = NCOL(I)
      DO 173 J = 1, NC
      ICT = ICT + 1
  173 NWRPC(ICT) = NR
  175 NWCOL = NWCOL + NCOL(I)
  240 CONTINUE
!
!
!
  250 FORMAT(//' IPART=',I6,//,                                      &
     & ' NWING=',I6,'  NCPT=',I6,'  NSEG=',I6)
  260 FORMAT(/' NROW(N),N=1,',I4/(1X,20I6))
  270 FORMAT(/' NCOL(N),N=1,',I4/(1X,20I6))
  275 FORMAT(//' NBODY=',I6)
  280 FORMAT(/' NWBLOK=',I6)
  285 FORMAT(/' NBBLOK=',I6)
  290 FORMAT(10F7.0)
  300 FORMAT(/' VELCMP, TIME =',F10.5)
  310 FORMAT(1H1,' PARTITION =',I3,'  TIME =',F10.5)
  320 FORMAT(' INFLUENCE OF BODY ON BODY')
  330 FORMAT(' INFLUENCE OF WING ON BODY')
  340 FORMAT(' INFLUENCE OF BODY ON WING')
  350 FORMAT(' INFLUENCE OF WING ON WING')
  360 FORMAT(' INFLUENCE OF NACELLE/SHELL ON NACELLE/SHELL')
  370 FORMAT(' INFLUENCE OF WING ON NACELLE/SHELL')
  380 FORMAT(' INFLUENCE OF BODY ON NACELLE/SHELL')
  390 FORMAT(' INFLUENCE OF NACELLE/SHELL ON WING')
  400 FORMAT(' INFLUENCE OF NACELLE/SHELL ON BODY')
  410 FORMAT(1H1,' END OF AIC CALCULATIONS, TIME =',F10.5)
      END

      SUBROUTINE TRAP (XT,YT,SUM,NT)
!
!     *******************************************VERSION A00************
!
!
!     EVALUATE AN INTEGRAL BY THE TRAPEZOIDAL RULE.
!
      DIMENSION XT(1), YT(1)
!
      SUM=0.
      DO 10 I=2,NT
   10 SUM=SUM+.5*(XT(I)-XT(I-1))*(YT(I)+YT(I-1))
      RETURN
      END


!!!     OVERLAY(LWB,2,1)
      SUBROUTINE BODVEL()
!
!     *******************************************VERSION B00************
!
!
!     COMPUTE THE THREE COMPONENTS OF VELOCITY INDUCED AT SPECIFIED
!     CONTROL POINTS BY THE BODY PANELS
!
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAP&
!!!     &E9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15,TAP16,TAPE17
      INCLUDE 'files.cmn'
      COMMON /PARAM/ NBODY,NWING,NPANEL,LBC,THK,MACH,ALPHA,REFA
      COMMON /VELCOM/ NPOINT,NPART,IMAX,JMAX,NMAX,EX,PRINT,NWTHK
      COMMON /NEWCOM/KDUM(9),KFUS,KRADX(4),KFORX(4),KDUMN(25),MAX
      COMMON /POINT/ ARRAY(6000)
      COMMON /SCRAT/ BLOCK(7500)
      COMMON /BODCOM/ EM,DA,CX,XCOR(4),YCOR(4),ZCOR(4),XI,YI,ZI,XJ,ZJ   &
     &,BTA
      COMMON /BTHET/ THETA(600)
      COMMON /SHELL/ NPTSB,NSEGWT,NPTSWT,NSEGN,NPTSN,NSEGS,NPTSS,       &
     &               NSEGNS,NPTSNS,NNAC,NSHELL,NQ,IPART, iidum
!
      DIMENSION UB(600), VB(600), WB(600), VI(600), WI(600), AN(600)
      DIMENSION DN(60), XBT(600), YBT(600), ZBT(600), TI(600), CD(600)
      DIMENSION XPT(600), YPT(600), ZPT(600), THET(600), DELTA(600)
      DIMENSION XC(30,20), YC(30,20), ZC(30,20), DELTI(600)
!
      EQUIVALENCE (BLOCK(1),UB(1))
      EQUIVALENCE (BLOCK(601),VB(1))
      EQUIVALENCE (BLOCK(1201),WB(1))
      EQUIVALENCE (BLOCK(1801),VI(1))
      EQUIVALENCE (BLOCK(2401),WI(1))
      EQUIVALENCE (BLOCK(3001),AN(1))
      EQUIVALENCE (BLOCK(3601),DN(1))
      EQUIVALENCE (BLOCK(3901),XBT(1))
      EQUIVALENCE (BLOCK(4501),YBT(1))
      EQUIVALENCE (BLOCK(5101),ZBT(1))
      EQUIVALENCE (BLOCK(5701),TI(1))
      EQUIVALENCE (BLOCK(6301),CD(1))
      EQUIVALENCE (ARRAY(1),XPT(1))
      EQUIVALENCE (ARRAY(601),YPT(1))
      EQUIVALENCE (ARRAY(1201),ZPT(1))
      EQUIVALENCE (ARRAY(1801),THET(1))
      EQUIVALENCE (ARRAY(2401),DELTA(1))
      EQUIVALENCE (ARRAY(3001),XC(1,1))
      EQUIVALENCE (ARRAY(3601),YC(1,1))
      EQUIVALENCE (ARRAY(4201),ZC(1,1))
      EQUIVALENCE (ARRAY(4801),DELTI(1))
!*
      COMMON /TAPES/ ITAPES(20)
!*
!
      LOGICAL LBC                                                       &
     &,SUB,SUP
      REAL MACH
      INTEGER PRINT
!!!,TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAPE&
!!!     &9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15,TAPE16,TAPE17
!
!*
      N11 = ITAPES(11)
      N17 = ITAPES(17)
      N18 = ITAPES(18)
      N20 = ITAPES(20)
!*
      EM=MACH
      BTA = SQRT(ABS(EM*EM - 1.0))
      SUB = .FALSE.
      IF(MACH.LT.1.0) SUB = .TRUE.
      SUP = .NOT.SUB
      JMAX=MAX
      II=0
!
!     I IS THE INDEX OF THE CONTROL POINT
!     J IS THE INDEX OF THE INFLUENCING PANEL
!
      DO 110 I=1,NPOINT
      ISKIP=TI(I)
      IF(LBC.AND.I.EQ.ISKIP.AND.NPART.NE.1) GO TO 110
      II=II+1
      SINTI=SIN(THET(I))
      COSTI=COS(THET(I))
      XPTI=XPT(I)
      YPTI=YPT(I)
      ZPTI=ZPT(I)
      IF(NPART.NE.1) GO TO 10
      SIND=SIN(DELTA(I))
      COSD=COS(DELTA(I))
      GO TO 20
   10 IF (LBC) DELT=0.
      IF (.NOT.LBC) DELT=DELTI(I)
      SIND=SIN(DELT)
      COSD=COS(DELT)
   20 CONTINUE
      DO 30 J=1,NBODY
      UB(J)=0.
      VI(J)=0.
   30 WI(J)=0.
      J=0
      J2=0
      L=0
      DO 70 KF=1,KFUS
      NROW=KRADX(KF)-1
      NCOL=KFORX(KF)-1
      DO 60 NC=1,NCOL
      L=L+1
      J1=1+J2
      J2=J1+NROW-1
      DO 50 N=1,NROW
      J=J+1
      DA=TAN(DELTA(J))
      COST=COS(THETA(J))
      SINT=SIN(THETA(J))
      XW=SINT*COSTI
      XX=COST*SINTI
      XY=COST*COSTI
      XZ=SINT*SINTI
      SINTR=XW-XX
      SINTL=XW+XX
      COSTR=XY+XZ
      COSTL=XY-XZ
      N1=N+1
      XC1=XC(L,N1)
      YC1=YC(L,N1)
      ZC1=ZC(L,N1)
!
!     CALCULATION OF PANEL CORNER POINTS IN PANEL COORDINATE SYSTEM
!
      XCOR(1)=0.
      YCOR(1)=0.
      ZCOR(1)=0.
      XCOR(2)=XC(L+1,N+1)-XC1
      XCOR(3)=0.
      XCOR(4)=XCOR(2)
      DO 40 K=2,4
      L1=L+1
      N1=N+1
      IF (K.GE.3) N1=N
      IF (K.EQ.3) L1=L
      DELY=YC(L1,N1)-YC1
      DELZ=ZC(L1,N1)-ZC1
      YCOR(K)=DELY*COST+DELZ*SINT
   40 ZCOR(K)=DELZ*COST-DELY*SINT
      CX=XCOR(2)
!
!     CALCULATION OF CONTROL POINT IN PANEL COORDINATE SYSTEM
!
      XI=XPTI-XC1
      DY=YPTI-YC1
      DZ=ZPTI-ZC1
      YI=DY*COST+DZ*SINT
      ZI=DZ*COST-DY*SINT
      XJ=XBT(J)-XC1
      DYJ=YBT(J)-YC1
      DZJ=ZBT(J)-ZC1
      ZJ=DZJ*COST-DYJ*SINT
!
!     CALCULATE VELOCITY COMPONENTS INDUCED BY CONSTANT SOURCE
!     DISTRIBUTION PANELS
!
      IF(NPART.EQ.7.AND.II.GT.NPTSN) GO TO 42
      IF(SUB) CALL SUBPAN(UR,VR,WR)
      IF(SUP) CALL SUPPAN(UR,VR,WR)
      DY=-YPTI-YC1
      YI=DY*COST+DZ*SINT
      ZI=DZ*COST-DY*SINT
      IF(SUB) CALL SUBPAN(UL,VL,WL)
      IF(SUP) CALL SUPPAN(UL,VL,WL)
      GO TO 44
   42 UR=0.0
      VR=0.0
      WR=0.0
      UL=0.0
      VL=0.0
      WL=0.0
   44 CONTINUE
!
!     CALCULATE VELOCITY COMPONENTS IN ORIGINAL COORDINATE SYSTEM
!
      UB(J)=UL+UR+UB(J)
      VI(J)=VR*COSTR-WR*SINTR-VL*COSTL+WL*SINTL+VI(J)
      WI(J)=VR*SINTR+VL*SINTL+WR*COSTR+WL*COSTL+WI(J)
      VB(J)=VI(J)*COSTI-WI(J)*SINTI
      WB(J)=WI(J)*COSTI+VI(J)*SINTI
      AN(J)=WI(J)*COSD-UB(J)*SIND
      IF(NPART.NE.1) GO TO 50
      IF (NBODY.LE.NMAX) GO TO 50
      IF (II.LT.J1.OR.II.GT.J2) GO TO 50
      JS1=J1
      JS2=J2
      NS=NROW
   50 END DO
   60 END DO
      L=L+1
   70 END DO
      JMAX=L
!*
      IF(NPTSNS.EQ.0) WRITE(N11) (AN(J),J=1,NBODY)
      IF(NPTSNS.NE.0) WRITE(N20) (AN(J),J=1,NBODY)
!*
      IF(NBODY.LE.NMAX.OR.NPART.NE.1) GO TO 90
!
!     STORE DIAGONAL BLOCKS OF AERODYNAMIC MATRIX IN DN ARRAY
!
      DO 80 J=1,NBODY
      IF (J.LT.JS1.OR.J.GT.JS2) GO TO 80
      K=J-JS1+1
      DN(K)=AN(J)
      AN(J)=0.
   80 END DO
      IF(NPTSNS.EQ.0) WRITE(TAPE10) (DN(J),J=1,NS)
      IF(NPTSNS.NE.0) WRITE(TAPE17) (DN(J),J=1,NS)
   90 CONTINUE
      IF (IABS(PRINT).LT.4) GO TO 100
      WRITE (TAPE6,180) II
      WRITE (TAPE6,140) NBODY
      WRITE (TAPE6,170) (UB(J),J=1,NBODY)
      WRITE (TAPE6,120) NBODY
      WRITE (TAPE6,170) (VB(J),J=1,NBODY)
      WRITE (TAPE6,130) NBODY
      WRITE (TAPE6,170) (WB(J),J=1,NBODY)
      WRITE (TAPE6,150) NBODY
      WRITE (TAPE6,170) (AN(J),J=1,NBODY)
      IF(NBODY.GT.NMAX.AND.NPART.EQ.1) WRITE(TAPE6,160) NS
      IF(NBODY.GT.NMAX.AND.NPART.EQ.1) WRITE(TAPE6,170) (DN(J),J=1,NS)
  100 WRITE (TAPE8) (UB(J),VB(J),WB(J),J=1,NBODY)
      IF(NPTSNS.EQ.0) WRITE(TAPE9) (AN(J),J=1,NBODY)
      IF(NPTSNS.NE.0) WRITE(N18) (AN(J),J=1,NBODY)
  110 END DO
!
!
!
  120 FORMAT (2X,10HVB(J),J=1,,I3)
  130 FORMAT (2X,10HWB(J),J=1,,I3)
  140 FORMAT (2X,10HUB(J),J=1,,I3)
  150 FORMAT (2X,10HAN(J),J=1,,I3)
  160 FORMAT (2X,10HDN(J),J=1,,I3)
  170 FORMAT (1X,10F10.5)
  180 FORMAT (1H0,22HAERODYNAMIC MATRIX, I=,I3)
      END
      SUBROUTINE SUBPAN(UPM,VPM,WPM)
!
!     ********************************************VERSION B00***********
!
!
!     COMPUTE THE THREE COMPONENTS OF VELOCITY INDUCED AT A SPECIFIED
!     CONTROL POINT BY A CONSTANT SOURCE DISTRIBUTION ON A
!     QUADRILATERAL PANEL HAVING LONGITUDINAL TAPER AND INCLINED AT AN
!     ANGLE DELTA TO THE FREE STREAM DIRECTION
!
!
!     SUBSONIC SOURCE INFLUENCE FUNCTIONS
!
!
      COMMON /BODCOM/EM,SA,CX,XC(4),YC(4),ZC(4),XJ,YJ,ZJ,XI,ZI,BETA
      COMMON /PI/ PI,RADEG,DEGRAD
      COMMON /EPS/ EPSPAN,EPSWP1,EPSWP2,EPSTP,EPSVC,EPSSP1,EPSSP2,EPSLV,&
     &             EPSSV,EPSVV,EPSWV,EPSVP,EPSIT1,EPSIT2,EPSFM,EPSSB1,  &
     &             EPSSB2
!
      DIMENSION ZK(2)
!
      REAL INF
!
      EQUIVALENCE (EPSSB2,INF)
!
      R4PI=1./(4.*PI)
      IS=0
      IF(ABS(SA).GE.INF) IS=1
      IF(IS.NE.0) SA=0.
      A=BETA*SA
      BT2=BETA*BETA
      BA2 = BT2*SA*SA
      TA= 1.0 + BA2
      RTA=1./TA
      DO 5 I=1,2
    5 ZK(I)=ZI-SA*(XI-XC(I))
      CL=CX
      IF(IS.NE.0) CL=BETA*ZC(2)
      BI=BETA*(YC(2)-YC(1))/CL
      BO=BETA*(YC(4)-YC(3))/CL
      SABI=SQRT(TA+BI*BI)
      SABO=SQRT(TA+BO*BO)
      XL=XJ-XC(1)
      XT=XJ-XC(2)
      IF(IS.EQ.0) GO TO 8
      XL=BETA*(ZJ-ZC(1))
      XT=BETA*(ZJ-ZC(2))
    8 CONTINUE
      IF(ABS(XL).LE.EPSSB1)XL=0.
      IF(ABS(XT).LE.EPSSB1)XT=0.
      YIL=(YJ-YC(1))*BETA
      YIT=(YJ-YC(2))*BETA
      YOL=(YJ-YC(3))*BETA
      YOT=(YJ-YC(4))*BETA
      ZL=(ZJ-ZK(1))*BETA
      ZT=(ZJ-ZK(2))*BETA
      IF(IS.EQ.0) GO TO 9
      ZL=XI-XJ
      ZT=ZL
    9 CONTINUE
      ZAX=ZL-A*XL
      IF(ABS(ZAX).LE..001) ZAX=0.
      ZAX2=ZAX*ZAX
      RL2=XL*XL+ZL*ZL
      RT2=XT*XT+ZT*ZT
      DIL=SQRT(YIL*YIL+RL2)
      DIT=SQRT(YIT*YIT+RT2)
      DOL=SQRT(YOL*YOL+RL2)
      DOT=SQRT(YOT*YOT+RT2)
      YDIL=YIL+DIL
      YDOL=YOL+DOL
      YDOT=YOT+DOT
      YDIT=YIT+DIT
      YBXI=YIL-BI*XL
      YBXO=YOL-BO*XL
      IF(ABS(YBXI).LE.EPSSB1)YBXI=0.
      IF(ABS(YBXO).LE.EPSSB1)YBXO=0.
      XAZL=XL+A*ZL
      XAZT=XT+A*ZT
      XIL=XAZL+BI*YIL
      XOL=XAZL+BO*YOL
      XIT=XAZT+BI*YIT
      XOT=XAZT+BO*YOT
      IF(ZAX.EQ.0.) GO TO 10
      AYZI=A*YIL-BI*ZL
      AYZO=A*YOL-BO*ZL
      EIL=-XL*YBXI-ZL*AYZI
      EOL=-XL*YBXO-ZL*AYZO
      EIT=-XT*YBXI-ZT*AYZI
      EOT=-XT*YBXO-ZT*AYZO
      FI=ATAN2(ZAX*(DIL*EIT-DIT*EIL),EIL*EIT+ZAX2*(DIL*DIT))
      FO=ATAN2(ZAX*(DOL*EOT-DOT*EOL),EOL*EOT+ZAX2*(DOL*DOT))
      F=FI-FO
      GO TO 20
   10 F=0.
      IF(YBXI*YBXO.LE.0..AND.XL*XT.LE.0.) F=2.*PI
      IF(YBXI.EQ.0.) GO TO 30
   20 GI=ALOG((XIL+SABI*DIL)/(XIT+SABI*DIT))/SABI
      GO TO 40
   30 IF(XIL.GT.0..AND.XIT.GT.0.) GO TO 20
      IF(XIL.GE.0..AND.XIT.LE.0.) GI=INF
      IF(XIL.LT.0..AND.XIT.LE.0.) GI=ALOG(XIT/XIL)/SABI
   40 IF(ZAX.EQ.0..AND.YBXO.EQ.0.) GO TO 50
   45 GO=ALOG((XOL+SABO*DOL)/(XOT+SABO*DOT))/SABO
      GO TO 60
   50 IF(XOL.GT.0..AND.XOT.GT.0.) GO TO 45
      IF(XOL.GE.0..AND.XOT.LE.0.) GO=INF
      IF(XOL.LT.0..AND.XOT.LE.0.) GO=ALOG(XOT/XOL)/SABO
   60 IF(RL2.GT.EPSSB1)GO TO 70
      IF(YIL.GT.0..AND.YOL.GT.0.) GO TO 70
      IF(YIL.GE.0..AND.YOL.LE.0.) HL=INF
      IF(YIL.LT.0..AND.YOL.LT.0.) HL=ALOG(YOL/YIL)
      GO TO 80
   70 HL=ALOG(YDIL/YDOL)
   80 IF(RT2.GT.EPSSB1)GO TO 90
      IF(YIT.GT.0..AND.YOT.GT.0.) GO TO 90
      IF(YIT.GE.0..AND.YOT.LE.0.) HT=INF
      IF(YIT.LT.0..AND.YOT.LT.0.) HT=ALOG(YOT/YIT)
      GO TO 100
   90 HT=ALOG(YDIT/YDOT)
  100 G=BI*GI-BO*GO-HL+HT
      VPM=(GO-GI)*R4PI
      IF(IS.EQ.0) GO TO 110
      UPM=-F*R4PI/BETA
      WPM= G*R4PI
      RETURN
  110 CONTINUE
      UPM=(G-A*F)*R4PI*RTA/BETA
      WPM=(F+A*G)*R4PI*RTA
      RETURN
      END
      SUBROUTINE SUPPAN(UPM,VPM,WPM)
!
!     *******************************************VERSION A01************
!
!
!     COMPUTE THE THREE COMPONENTS OF VELOCITY INDUCED AT A SPECIFIED
!     CONTROL POINT BY A CONSTANT SOURCE DISTRIBUTION ON A
!     QUADRILATERAL PANEL HAVING LONGITUDINAL TAPER AND INCLINED AT AN
!     ANGLE DELTA TO THE FREE STREAM DIRECTION
!
!     SUPERSONIC SOURCE PANEL INFLUENCE FUNCTIONS
!
!
!!!      COMMON /FILES/TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,     &
!!!     &             TAPE9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      INCLUDE 'files.cmn'
      COMMON /BODCOM/ EM,SA,CX,XC(4),YC(4),ZC(4),XI,YI,ZI,XJ,ZJ         &
     &,BTA
      COMMON /PI/ PI,RADEG,DEGRAD
      COMMON /EPS/EPSPAN,EPSWP1,EPSWP2,EPSTP,EPSVC,EPSSP1,EPSSP2,EPSLV,  &
     &           EPSSV,EPSVV,EPSWV,EPSVP,EPSIT1,EPSIT2,EPSFM, epssb1,epssb2
!
      DIMENSION B(4), SX(4), SM(4), DX(4), DY(4), DZ(4), D(4), E(4), F(4&
     &), G(4), H(4), XPM(4), YMX(4), ZAX(4), AYM(4), RPM2(4)
!
!!!      INTEGER TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAPE9,    &
!!!     &        TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      REAL NUM
!
      EPS=1.0E-5
      EP2=EPS*EPS
      PI=3.14159265
      BT2=1.-EM*EM
      BTA=SQRT(ABS(BT2))
      BA2=BT2*SA*SA
      TA=1.0+BA2
      IF (TA.LT.0.) GO TO 200
      SM(3)=0.0
      DO 190 I=1,4
      ZC(I)=ZJ-SA*(XJ-XC(I))
      IF (I.LE.2) SM(1)=(YC(2)-YC(1))/CX
      IF (I.GT.2) SM(3)=(YC(4)-YC(3))/CX
      SM(2)=SM(1)
      SM(4)=SM(3)
      SSM=SIGN(1.,SM(I))
      BM2=BT2*SM(I)*SM(I)
      TAM=TA+BM2
      IF (ABS(TAM).LE.EPS) TAM=0.
      SAM=SQRT(ABS(TAM))
      SAMD=1./SAM
      CPM=CX*SAM
      DX(I)=XI-XC(I)
      DY(I)=YI-YC(I)
      DZ(I)=ZI-ZC(I)
      IF (ABS(DX(I)).LE.EPS) DX(I)=0.
      IF (ABS(DY(I)).LE.EPS) DY(I)=0.
      IF (ABS(DZ(I)).LE.EPS) DZ(I)=0.
      RPM2(I)=0.
      DX2=DX(I)*DX(I)
      DY2=DY(I)*DY(I)
      DZ2=DZ(I)*DZ(I)
      DR2=DY2+DZ2
      D2=DX2+BT2*DR2
      D(I)=0.0
      IF (EM.GE.1.) DXZ=DX(I)-BTA*ABS(DZ(I))
      IF (EM.GE.1..AND.DXZ.LT.0.) GO TO 170
      IF (D2.GT.0.0) D(I)=SQRT(D2)
      XPM(I)=DX(I)+BT2*(SM(I)*DY(I)+SA*DZ(I))
      YMX(I)=DY(I)-SM(I)*DX(I)
      ZAX(I)=DZ(I)-SA*DX(I)
      AYM(I)=SA*DY(I)-SM(I)*DZ(I)
      IF (ABS(XPM(I)).LE.EPS) XPM(I)=0.
      IF (ABS(YMX(I)).LE.EPS) YMX(I)=0.
      IF (ABS(ZAX(I)).LE.EPS) ZAX(I)=0.
      IF (ABS(AYM(I)).LE.EPS) AYM(I)=0.
      IF (I.LE.2) RPM2(1)=YMX(1)*YMX(1)+ZAX(1)*ZAX(1)+BT2*(AYM(1)*AYM(1)&
     &)
      RPM2(2)=RPM2(1)
      IF (I.GT.2) RPM2(3)=YMX(3)*YMX(3)+ZAX(3)*ZAX(3)+BT2*(AYM(3)*AYM(3)&
     &)
      RPM2(4)=RPM2(3)
      IF (ABS(RPM2(I)).LE.EP2) RPM2(I)=0.
      RPM=SQRT(ABS(RPM2(I)))
      IF (RPM.LE.EPS) RPM=0.
      DPM=SAM*D(I)
      F(I)=0.
      DNOM=-DX(I)*YMX(I)-BT2*DZ(I)*AYM(I)
      FNUM=D(I)*ZAX(I)
      IF (FNUM.EQ.0..AND.DNOM.EQ.0.) GO TO 10
      F(I)=ATAN2(FNUM,DNOM)
      IF (D(I).EQ.0.) F(I)=F(I)*SIGN(1.,ZAX(I))
   10 IF (TAM) 100,90,20
   20 IF (EM.GT.1..AND.D(I).EQ.0.) GO TO 70
      IF (RPM-EPS) 40,40,30
   30 NUM=XPM(I)+DPM
      G(I)=ALOG(NUM/(BTA*RPM))*SAMD
      GO TO 150
   40 SX(I)=SIGN(1.,XPM(I))
      IF (EM.LT.1.0) GO TO 50
      IF (I.EQ.1.AND.XPM(1).LT.CPM) GO TO 130
      IF (I.EQ.3.AND.XPM(3).LT.CPM) GO TO 140
   50 IF (I.EQ.2) SGN12=SX(1)*SX(2)
      IF (I.EQ.4) SGN34=SX(3)*SX(4)
      IF (XPM(I)) 60,70,80
   60 IF (I.EQ.2.AND.SGN12.LT.0.) GO TO 130
      IF (I.EQ.4.AND.SGN34.LT.0.) GO TO 140
      G(I)=-ALOG(ABS(XPM(I)))*SAMD
      GO TO 150
   70 G(I)=0.
      GO TO 150
   80 G(I)=ALOG(XPM(I))*SAMD
      GO TO 150
   90 G(I)=0.
      IF (XPM(I).GT.BTA*RPM) G(I)=D(I)/XPM(I)
      GO TO 150
  100 G(I)=0.0
      ARG=SIGN(1.,XPM(I))
      IF (RPM.NE.0.) ARG=XPM(I)/(BTA*RPM)
      IF (ARG.GE.1.) GO TO 150
      IF (ARG.LE.-1.) GO TO 110
      IF (D(I).GT.0) G(I)=ACOS(ARG)*SAMD
      GO TO 150
  110 AM2=SA*SA+SM(I)*SM(I)
      TRM1=(SM(I)*DY(I)+SA*DZ(I)+ABS(AYM(I))*SAM)/AM2
      IF (DX(I).GT.TRM1) GO TO 120
      F(I)=0.
      IF (SSM.GT.0.) F(I)=PI*SIGN(1.,ZAX(I))
      GO TO 150
  120 IF (SSM*YMX(I).GE.0.) GO TO 150
      G(I)=PI*SAMD
      GO TO 150
  130 G(1)=500.*SAMD
      IF (EM.LT.1.0) G(2)=-G(1)
      GO TO 160
  140 G(3)=500.*SAMD
      IF (EM.LT.1.0) G(4)=-G(3)
  150 CONTINUE
  160 H(I)=0.
      HARG=-BTA*DY(I)
      IF (D(I).EQ.0.0.AND.HARG.EQ.0.0) GO TO 180
      IF (EM.LT.1.0) H(I)=BTA*.5*ALOG((D(I)+HARG)/(D(I)-HARG))
      IF (EM.GT.1.) H(I)=BTA*ATAN2(D(I),HARG)
      GO TO 180
  170 F(I)=0.
      G(I)=0.
      H(I)=0.
      AYM(I)=0.
      YMX(I)=0.
      ZAX(I)=0.
      XPM(I)=0.
      DPM=0.
      RPM=0.
      RPM2(2)=RPM2(1)
      RPM2(4)=RPM2(3)
  180 E(I)=H(I)+BT2*SM(I)*G(I)
  190 END DO
      TAD=1./TA
      E14=(E(1)-E(2)-E(3)+E(4))*TAD
      F14=(F(1)-F(2)-F(3)+F(4))*TAD
      G14=G(1)-G(2)-G(3)+G(4)
      R4PI=1.0/(4.*PI)
      IF (EM.GT.1.) R4PI=2.*R4PI
      UPM=R4PI*(E14/BT2-SA*F14)
      VPM=-R4PI*G14
      WPM=R4PI*(F14+SA*E14)
      RETURN
  200 WRITE (6,210)
      STOP   !!! CALL EXIT
!
!
!
  210 FORMAT (1H0,43HERROR - BODY PANEL SLOPE EXCEEDS MACH ANGLE)
      END



!!!     OVERLAY(LWB,2,2)
      SUBROUTINE LINVEL()
!
!     *******************************************VERSION B00************
!
!     COMPUTE THE THREE COMPONENTS OF VELOCITY INDUCED AT SPECIFIED
!     CONTROL POINTS BY SOURCE AND VORTEX DISTRIBUTIONS ON PANELS
!     LOCATED IN THE PLANE OF THE WING, FIN (VERTICAL TAIL), OR CANARD
!     (HORIZONTAL TAIL) SURFACE
!
!
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAP&
!!!     &E9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15,TAPE16,TAPE17
      INCLUDE 'files.cmn'
      COMMON /PARAM/ NBODY,NWING,NTAIL,LBC,THIK,MACH,ALPHA,REFA
      COMMON /COMPS/ DX,DY,DZ,AL,BL,CL,SUB,BPOS,BCOS,BSIN,ML
      COMMON /SEG/ NSEG,NROW(20),NCOL(20),COSS(20),SINS(20),TT(20),NWT(2&
     &0)
      COMMON /VELCOM/ NPOINT,NPART,IMAX,JMAX,NMAX,EM,PRINT,NWTHK
      COMMON /POINT/ ARRAY(6000)
      COMMON /SCRAT/ BLOCK(7500)
      COMMON /PI/ PI,RADEG,DEGRAD
!*
      COMMON /TAPES/ ITAPES(20)
!!!      COMMON /OPTION/ NOPT(10),NXSTNS,XSTN(20),XAPEX,YAPEX,XLAMDA
      INCLUDE 'option.cmn'
!*
      COMMON /EPS/ EPSPAN,EPSWP1,EPSWP2,EPSTP,EPSVC,EPSSP1,EPSSP2,EPSLV,&
     &EPSSV,EPSVV,EPSWV,EPSVP,EPSIT1,EPSIT2,EPSFM, epssb1,epssb2
      COMMON /SHELL/ NPTSB,NSEGWT,NPTSWT,NSEGN,NPTSN,NSEGS,NPTSS,       &
     &               NSEGNS,NPTSNS,NNAC,NSHELL,NQ,IPART,II
!
      DIMENSION XPT(600), YPT(600), ZPT(600), THET(600), DELTA(600), XC(&
     &30,20), YC(30,20), ZC(30,20), BLE(30), UTOR(30), VTOR(30), WTOR(30&
     &), UTOL(30), VTOL(30), WTOL(30), UTIR(30), VTIR(30), WTIR(30), UTI&
     &L(30), VTIL(30), WTIL(30), ASAVE(30), USAVE(30), VSAVE(30), WSAVE(&
     &30), UCOR(30), VCOR(30), WCOR(30), ULOR(30), VLOR(30), WLOR(30), U&
     &COL(30), VCOL(30), WCOL(30), ULOL(30), VLOL(30), WLOL(30), UCIR(30&
     &), VCIR(30), WCIR(30), ULIR(30), VLIR(30), WLIR(30), UCIL(30), VCI&
     &L(30), WCIL(30), ULIL(30), VLIL(30), WLIL(30), RCOR(30), SCOR(30),&
     & TCOR(30), RLOR(30), SLOR(30), TLOR(30), RCOL(30), SCOL(30), TCOL(&
     &30), RLOL(30), SLOL(30), TLOL(30), RCIR(30), SCIR(30), TCIR(30), R&
     &LIR(30), SLIR(30), TLIR(30), RCIL(30), SCIL(30), TCIL(30), RLIL(30&
     &), SLIL(30), TLIL(30), AC(600), UC(600), VC(600), WC(600), UT(600)&
     &, VT(600), WT(600), DC(60), TI(600), CHORD(600), DZTDX(600)
!
      EQUIVALENCE (ARRAY(1),XPT(1))
      EQUIVALENCE (ARRAY(601),YPT(1))
      EQUIVALENCE (ARRAY(1201),ZPT(1))
      EQUIVALENCE (ARRAY(1801),THET(1))
      EQUIVALENCE (ARRAY(3001),XC(1,1))
      EQUIVALENCE (ARRAY(2401),UTOR(1))
      EQUIVALENCE (ARRAY(2431),VTOR(1))
      EQUIVALENCE (ARRAY(2461),WTOR(1))
      EQUIVALENCE (ARRAY(2491),UTOL(1))
      EQUIVALENCE (ARRAY(2521),VTOL(1))
      EQUIVALENCE (ARRAY(2551),WTOL(1))
      EQUIVALENCE (ARRAY(2581),UTIR(1))
      EQUIVALENCE (ARRAY(2611),VTIR(1))
      EQUIVALENCE (ARRAY(2641),WTIR(1))
      EQUIVALENCE (ARRAY(2671),UTIL(1))
      EQUIVALENCE (ARRAY(2701),VTIL(1))
      EQUIVALENCE (ARRAY(2731),WTIL(1))
      EQUIVALENCE (ARRAY(3601),YC(1,1))
      EQUIVALENCE (ARRAY(4201),ZC(1,1))
      EQUIVALENCE (ARRAY(4801),DELTA(1))
      EQUIVALENCE (ARRAY(2761),USAVE(1))
      EQUIVALENCE (ARRAY(2791),VSAVE(1))
      EQUIVALENCE (ARRAY(2821),WSAVE(1))
      EQUIVALENCE (ARRAY(2851),ASAVE(1))
      EQUIVALENCE (BLOCK(1),UCOR(1))
      EQUIVALENCE (BLOCK(31),VCOR(1))
      EQUIVALENCE (BLOCK(61),WCOR(1))
      EQUIVALENCE (BLOCK(91),ULOR(1))
      EQUIVALENCE (BLOCK(121),VLOR(1))
      EQUIVALENCE (BLOCK(151),WLOR(1))
      EQUIVALENCE (BLOCK(181),UCOL(1))
      EQUIVALENCE (BLOCK(211),VCOL(1))
      EQUIVALENCE (BLOCK(241),WCOL(1))
      EQUIVALENCE (BLOCK(271),ULOL(1))
      EQUIVALENCE (BLOCK(301),VLOL(1))
      EQUIVALENCE (BLOCK(331),WLOL(1))
      EQUIVALENCE (BLOCK(361),UCIR(1))
      EQUIVALENCE (BLOCK(391),VCIR(1))
      EQUIVALENCE (BLOCK(421),WCIR(1))
      EQUIVALENCE (BLOCK(451),ULIR(1))
      EQUIVALENCE (BLOCK(481),VLIR(1))
      EQUIVALENCE (BLOCK(511),WLIR(1))
      EQUIVALENCE (BLOCK(541),UCIL(1))
      EQUIVALENCE (BLOCK(571),VCIL(1))
      EQUIVALENCE (BLOCK(601),WCIL(1))
      EQUIVALENCE (BLOCK(631),ULIL(1))
      EQUIVALENCE (BLOCK(661),VLIL(1))
      EQUIVALENCE (BLOCK(691),WLIL(1))
      EQUIVALENCE (BLOCK(721),RCOR(1))
      EQUIVALENCE (BLOCK(751),SCOR(1))
      EQUIVALENCE (BLOCK(781),TCOR(1))
      EQUIVALENCE (BLOCK(811),RLOR(1))
      EQUIVALENCE (BLOCK(841),SLOR(1))
      EQUIVALENCE (BLOCK(871),TLOR(1))
      EQUIVALENCE (BLOCK(901),RCOL(1))
      EQUIVALENCE (BLOCK(931),SCOL(1))
      EQUIVALENCE (BLOCK(961),TCOL(1))
      EQUIVALENCE (BLOCK(991),RLOL(1))
      EQUIVALENCE (BLOCK(1021),SLOL(1))
      EQUIVALENCE (BLOCK(1051),TLOL(1))
      EQUIVALENCE (BLOCK(1081),RCIR(1))
      EQUIVALENCE (BLOCK(1111),SCIR(1))
      EQUIVALENCE (BLOCK(1141),TCIR(1))
      EQUIVALENCE (BLOCK(1171),RLIR(1))
      EQUIVALENCE (BLOCK(1201),SLIR(1))
      EQUIVALENCE (BLOCK(1231),TLIR(1))
      EQUIVALENCE (BLOCK(1261),RCIL(1))
      EQUIVALENCE (BLOCK(1291),SCIL(1))
      EQUIVALENCE (BLOCK(1321),TCIL(1))
      EQUIVALENCE (BLOCK(1351),RLIL(1))
      EQUIVALENCE (BLOCK(1381),SLIL(1))
      EQUIVALENCE (BLOCK(1411),TLIL(1))
      EQUIVALENCE (BLOCK(1441),AC(1))
      EQUIVALENCE (BLOCK(2041),UC(1))
      EQUIVALENCE (BLOCK(2641),VC(1))
      EQUIVALENCE (BLOCK(3241),WC(1))
      EQUIVALENCE (BLOCK(3841),UT(1))
      EQUIVALENCE (BLOCK(4441),VT(1))
      EQUIVALENCE (BLOCK(5041),WT(1))
      EQUIVALENCE (BLOCK(5641),DC(1))
      EQUIVALENCE (BLOCK(5701),TI(1))
      EQUIVALENCE (BLOCK(6301),CHORD(1))
      EQUIVALENCE (BLOCK(6901),DZTDX(1))
!
      DIMENSION UCF(30),VCF(30),WCF(30),UTF(30),VTF(30),WTF(30)
      EQUIVALENCE (ARRAY(5401),UCF(1))
      EQUIVALENCE (ARRAY(5431),VCF(1))
      EQUIVALENCE (ARRAY(5461),WCF(1))
      EQUIVALENCE (ARRAY(5491),UTF(1))
      EQUIVALENCE (ARRAY(5521),VTF(1))
      EQUIVALENCE (ARRAY(5551),WTF(1))
!
      LOGICAL THK,THIK,LBC,SUB,BPOS,SUPTE,FLAG,FIN,MIDFIN
      INTEGER PRINT
!!!,TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAPE&
!!!     &9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15,TAPE16,TAPE17
      REAL MACH
!*
      N11 = ITAPES(11)
!*
!
      SUB=MACH.LT.1.0
      THK=THIK
      FLAG=.FALSE.
      NYC=1
      IF (ABS(YC(1,1)).LE.EPSLV) NYC=0
      BETA=SQRT(ABS(MACH*MACH-1.0))
      CON=1./(2.*PI)
      IF (SUB) CON=CON/2.
      CONT=2.*CON/BETA
      BCON=BETA*CON
      BCONT=BETA*CONT
!
!     I IS THE CONTROL POINT INDEX
!     J IS THE INFLUENCING PANEL INDEX
!     L IS THE PANEL ROW INDEX
!     M IS THE PANEL COLUMN INDEX
!     N IS THE WING SEGMENT INDEX
!
      II=0
      DO 400 I=1,NPOINT
      ISKIP=TI(I)
      IF(NPART.GT.3.AND.I.EQ.ISKIP) GO TO 400
      II=II+1
      SINTI=SIN(THET(I))
      COSTI=COS(THET(I))
      XI=XPT(I)
      YI=YPT(I)
      ZI=ZPT(I)
      IF(NPART.LE.3) GO TO 10
      SIND=0.
      COSD=1.
      GO TO 20
   10 SIND=SIN(DELTA(I))
      COSD=COS(DELTA(I))
   20 CONTINUE
      J=0
      K=0
      J2=0
      J2T=0
      M2=1
      NP=0
      DO 340 N=1,NSEG
      NR=NROW(N)
      NC=NCOL(N)
      NT=NWT(N)
      NR1=NR+1
      NC1=NC+1
      DO 30 L=1,NR1
      USAVE(L)=0.
      VSAVE(L)=0.
      WSAVE(L)=0.
   30 ASAVE(L)=0.
      M1=M2
      IF (N.GT.1.AND.NT.NE.0) M1=M2+1
      IF(NT.GE.2) M1=M2
!
!     IF FLAG IS TRUE, AN ADDITIONAL COLUMN OF VORTEX PANELS
!     EXTENDS FROM THE CENTER LINE TO THE INBOARD EDGE OF THE
!     WING, HORIZONTAL TAIL, OR CANARD
!
!     THIS EXTENSION DEPENDS ON THE VALUE OF NOPT(7) %
!
!     ( B. ROSEN 081881)
!
!
!       - VORTEX PANELS EXTEND HORIZONTALLY
!         FROM WING-BODY JUNCTION
!         TO Y=0
!         (ZERO DIHEDRAL)
!
!           VORTEX PANELS UNSWEPT                 NOPT(7)=0
!           VORTEX PANELS SWEPT AT MACH ANGLE     NOPT(7)=1
!
!       - VORTEX PANELS EXTEND
!         FROM WING-BODY JUNCTION
!         TO Y=Z=0
!         (DIHEDRAL CALCULATED IN LINVEL)
!
!           VORTEX PANELS UNSWEPT                 NOPT(7)=2
!           VORTEX PANELS SWEPT AT MACH ANGLE     NOPT(7)=3
!
!       - VORTEX PANELS EXTEND
!         FROM WING-BODY JUNCTION
!         TO Y=0
!         WITH SAME DIHEDRAL AS INBOARD WING (CANARD) SEGMENT
!
!           VORTEX PANELS UNSWEPT                 NOPT(7)=4
!           VORTEX PANELS SWEPT AT MACH ANGLE     NOPT(7)=5
!
!
      IF (N.EQ.1.AND.NYC.NE.0) FLAG=.TRUE.
      MYC=1
      IF (ABS(YC(1,M1)).LE.EPSLV) MYC=0
      IF (N.GT.1.AND.NT.EQ.1.AND.MYC.NE.0) FLAG=.TRUE.
      IF(NPTSNS.NE.0) FLAG=.FALSE.
      IF (FLAG) THK=.FALSE.
   40 M2=M1+NC
      IF (FLAG) M2=M1
      COST=COSS(N)
      SINT=SINS(N)
      IF(FLAG) DELTAY=YC(1,M1)
      IF(FLAG) DELTAZ=0.0
      IF(FLAG.AND.NOPT(7).GE.2) DELTAZ=ZC(1,M1)
      IF(FLAG.AND.NOPT(7).GE.4) DELTAZ=DELTAY*SINT/COST
      IF(FLAG) REFF=SQRT(DELTAY**2+DELTAZ**2)
      IF(FLAG) COST=DELTAY/REFF
      IF(FLAG) SINT=DELTAZ/REFF
      FIN=.FALSE.
      MIDFIN=.FALSE.
      IF (NT.NE.0.AND.COST.LE.0.70710678) FIN=.TRUE.
      IF(NPART.EQ.3.OR.NPART.EQ.6.OR.NPART.EQ.9) FIN=.FALSE.
      IF (.NOT.FIN) GO TO 50
      IF (NYC.EQ.0.AND.ABS(COST).LE.EPSLV) MIDFIN=.TRUE.
   50 CONTINUE
!
!     CALCULATE PANEL LEADING EDGE SLOPES
!
      DO 70 L=1,NR1
      IF (FLAG) GO TO 60
      DY=YC(L,M2)-YC(L,M1)
      DZ=ZC(L,M2)-ZC(L,M1)
      DYZ=DY*COST+DZ*SINT
      BLE(L)=(XC(L,M2)-XC(L,M1))/(BETA*DYZ)
      GO TO 70
   60 BLE(L)=0.
      IF(.NOT.SUB.AND.2*(NOPT(7)/2).EQ.NOPT(7)) BLE(L)=0.0
      IF(.NOT.SUB.AND.2*((NOPT(7)+1)/2).EQ.(NOPT(7)+1)) BLE(L)=-1.0
   70 END DO
      BTE=BLE(NR1)
      SUPTE=.FALSE.
      IF (.NOT.SUB.AND.ABS(BTE).LT.1.0) SUPTE=.TRUE.
!*
      IF( NOPT(1) .EQ. 1 .AND. NOPT(4) .LE. 1 ) GO TO 75
      SUPTE = .FALSE.
   75 CONTINUE
!*
      BCOS=BETA*COST
      BSIN=BETA*SINT
      XW=SINT*COSTI
      XX=COST*SINTI
      XY=COST*COSTI
      XZ=SINT*SINTI
      SINTR=XW-XX
      SINTL=XW+XX
      COSTR=XY+XZ
      COSTL=XY-XZ
!
!     CALCULATE INFLUENCE OF INBOARD CORNERS OF FIRST COLUMN OF PANELS
!
      DO 120 L=1,NR1
      DX=XI-XC(L,M1)
      DY=YI-YC(L,M1)
      DZ=ZI-ZC(L,M1)
      IF(FLAG) DX=XI-(XC(L,M1)-BLE(L)*BETA*REFF)
      IF(FLAG) DY=YI
      IF(FLAG) DZ=ZI-(ZC(L,M1)-REFF*SINT)
      AT=AL
      BL=BLE(L)
      CT=CL
      ML=1
      IF (L.EQ.NR1) GO TO 80
      BL1=BLE(L+1)
      AB=BL-BL1
      CC=XC(L+1,M1)-XC(L,M1)
   80 CONTINUE
      BPOS=BL.GE.0.
      AL=AB
      CL=CC
      CALL VORVEL (UCOR(L),VCOR(L),WCOR(L),ULOR(L),VLOR(L),WLOR(L),UTOR(&
     &L),VTOR(L),WTOR(L))
      IF (L.EQ.1) GO TO 90
      ABA=ABS(AL-AT)
      ACL=ABS(CL-CT)
      IF (ABA.LE.EPSLV.AND.ACL.LE.EPSLV) GO TO 90
      AL=AT
      CL=CT
      ML=2
      CALL VORVEL (X,X,X,X,X,X,UTOR(L),VTOR(L),WTOR(L))
      AL=AB
      CL=CC
      ML=1
   90 IF (.NOT.THK) GO TO 100
      CALL SORVEL (RCOR(L),SCOR(L),TCOR(L),RLOR(L),SLOR(L),TLOR(L))
  100 DY=-YI-YC(L,M1)
      IF (FLAG) DY=-YI
      CALL VORVEL (UCOL(L),VCOL(L),WCOL(L),ULOL(L),VLOL(L),WLOL(L),UTOL(&
     &L),VTOL(L),WTOL(L))
      IF (L.EQ.1) GO TO 110
      IF (ABA.LE.EPSLV.AND.ACL.LE.EPSLV) GO TO 110
      AL=AT
      CL=CT
      ML=2
      CALL VORVEL (X,X,X,X,X,X,UTOL(L),VTOL(L),WTOL(L))
      AL=AB
      CL=CC
      ML=1
  110 IF (.NOT.THK) GO TO 120
      CALL SORVEL (RCOL(L),SCOL(L),TCOL(L),RLOL(L),SLOL(L),TLOL(L))
  120 END DO
!
!     CALCULATE INFLUENCE OF CORNERS OF REMAINING COLUMNS OF PANELS
!
      IF (.NOT.FLAG) M1=M1+1
      DO 330 M=M1,M2
      NS=NR
      IF (SUPTE) NS=NR1
      IF (FLAG) GO TO 130
      J1=1+J2
      J2=J1+NS-1
      NST=NR1
      J1T=1+J2T
      J2T=J1T+NST-1
  130 DO 190 L=1,NR1
      UCIR(L)=UCOR(L)
      VCIR(L)=VCOR(L)
      WCIR(L)=WCOR(L)
      UCIL(L)=UCOL(L)
      VCIL(L)=VCOL(L)
      WCIL(L)=WCOL(L)
      ULIR(L)=ULOR(L)
      VLIR(L)=VLOR(L)
      WLIR(L)=WLOR(L)
      ULIL(L)=ULOL(L)
      VLIL(L)=VLOL(L)
      WLIL(L)=WLOL(L)
      UTIR(L)=UTOR(L)
      VTIR(L)=VTOR(L)
      WTIR(L)=WTOR(L)
      UTIL(L)=UTOL(L)
      VTIL(L)=VTOL(L)
      WTIL(L)=WTOL(L)
      IF (.NOT.THK) GO TO 140
      RCIR(L)=RCOR(L)
      SCIR(L)=SCOR(L)
      TCIR(L)=TCOR(L)
      RCIL(L)=RCOL(L)
      SCIL(L)=SCOL(L)
      TCIL(L)=TCOL(L)
      RLIR(L)=RLOR(L)
      SLIR(L)=SLOR(L)
      TLIR(L)=TLOR(L)
      RLIL(L)=RLOL(L)
      SLIL(L)=SLOL(L)
      TLIL(L)=TLOL(L)
  140 DX=XI-XC(L,M)
      DY=YI-YC(L,M)
      DZ=ZI-ZC(L,M)
      AT=AL
      BL=BLE(L)
      CT=CL
      IF (L.EQ.NR1) GO TO 150
      BL1=BLE(L+1)
      AB=BL-BL1
      CC=XC(L+1,M)-XC(L,M)
  150 CONTINUE
      BPOS=BL.GE.0.
      AL=AB
      CL=CC
      CALL VORVEL (UCOR(L),VCOR(L),WCOR(L),ULOR(L),VLOR(L),WLOR(L),UTOR(&
     &L),VTOR(L),WTOR(L))
      IF (L.EQ.1) GO TO 160
      ABA=ABS(AL-AT)
      ACL=ABS(CL-CT)
      IF (ABA.LE.EPSLV.AND.ACL.LE.EPSLV) GO TO 160
      AL=AT
      CL=CT
      ML=1
      CALL VORVEL (X,X,X,X,X,X,UTOR(L),VTOR(L),WTOR(L))
      AL=AB
      CL=CC
      ML=1
  160 IF (.NOT.THK) GO TO 170
      CALL SORVEL (RCOR(L),SCOR(L),TCOR(L),RLOR(L),SLOR(L),TLOR(L))
  170 DY=-YI-YC(L,M)
      CALL VORVEL (UCOL(L),VCOL(L),WCOL(L),ULOL(L),VLOL(L),WLOL(L),UTOL(&
     &L),VTOL(L),WTOL(L))
      IF (L.EQ.1) GO TO 180
      IF (ABA.LE.EPSLV.AND.ACL.LE.EPSLV) GO TO 180
      AL=AT
      CL=CT
      ML=2
      CALL VORVEL (X,X,X,X,X,X,UTOL(L),VTOL(L),WTOL(L))
      AL=AB
      CL=CC
      ML=1
  180 IF (.NOT.THK) GO TO 190
      CALL SORVEL (RCOL(L),SCOL(L),TCOL(L),RLOL(L),SLOL(L),TLOL(L))
  190 END DO
!
!     COMBINE CORNER INFLUENCES TO OBTAIN PANEL VELOCITY COMPONENTS
!
      DO 320 L=1,NR1
      IF (.NOT.FLAG.OR.L.GT.1) GO TO 200
      JSAVE=J
      KSAVE=K
      NPSAVE=NP
  200 K=K+1
      IF (SUPTE.OR.L.LT.NR1) GO TO 210
      IF (.NOT.THK) GO TO 320
      GO TO 250
  210 CONTINUE
      J=J+1
      IF (L.EQ.NR1) GO TO 240
      NP=NP+1
      AMP=1.0/CHORD(NP)
!
!     VELOCITY COMPONENTS INDUCED BY PANEL VORTEX DISTRIBUTIONS
!
      ULR=ULIR(L)-UTIR(L+1)-ULOR(L)+UTOR(L+1)
      ULL=ULIL(L)-UTIL(L+1)-ULOL(L)+UTOL(L+1)
      VLR=VLIR(L)-VTIR(L+1)-VLOR(L)+VTOR(L+1)
      VLL=VLIL(L)-VTIL(L+1)-VLOL(L)+VTOL(L+1)
      WLR=WLIR(L)-WTIR(L+1)-WLOR(L)+WTOR(L+1)
      WLL=WLIL(L)-WTIL(L+1)-WLOL(L)+WTOL(L+1)
      UCR=UCIR(L)-UCOR(L)-UCIR(L+1)+UCOR(L+1)-ULR
      UCL=UCIL(L)-UCOL(L)-UCIL(L+1)+UCOL(L+1)-ULL
      VCR=VCIR(L)-VCOR(L)-VCIR(L+1)+VCOR(L+1)-VLR
      VCL=VCIL(L)-VCOL(L)-VCIL(L+1)+VCOL(L+1)-VLL
      WCR=WCIR(L)-WCOR(L)-WCIR(L+1)+WCOR(L+1)-WLR
      WCL=WCIL(L)-WCOL(L)-WCIL(L+1)+WCOL(L+1)-WLL
      IF (.NOT.THK) GO TO 220
!
!     VELOCITY COMPONENTS INDUCED BY PANEL SOURCE DISTRIBUTIONS
!
      RLR=(RLIR(L)-RLIR(L+1)-RLOR(L)+RLOR(L+1))*AMP
      RLL=(RLIL(L)-RLIL(L+1)-RLOL(L)+RLOL(L+1))*AMP
      SLR=(SLIR(L)-SLIR(L+1)-SLOR(L)+SLOR(L+1))*AMP
      SLL=(SLIL(L)-SLIL(L+1)-SLOL(L)+SLOL(L+1))*AMP
      TLR=(TLIR(L)-TLIR(L+1)-TLOR(L)+TLOR(L+1))*AMP
      TLL=(TLIL(L)-TLIL(L+1)-TLOL(L)+TLOL(L+1))*AMP
  220 IF (L.EQ.1) GO TO 230
      UCR=RCR+UCR
      UCL=RCL+UCL
      VCR=SCR+VCR
      VCL=SCL+VCL
      WCR=TCR+WCR
      WCL=TCL+WCL
      IF (.NOT.THK) GO TO 260
      UTR=RTR-RLR
      UTL=RTL-RLL
      VTR=STR-SLR
      VTL=STL-SLL
      WTR=TTR-TLR
      WTL=TTL-TLL
      GO TO 260
!
!     SPECIAL CASE FOR LEADING EDGE PANELS
!
  230 IF (.NOT.THK) GO TO 260
      UTR=RCIR(L)-RCOR(L)-RLR
      UTL=RCIL(L)-RCOL(L)-RLL
      VTR=SCIR(L)-SCOR(L)-SLR
      VTL=SCIL(L)-SCOL(L)-SLL
      WTR=TCIR(L)-TCOR(L)-TLR
      WTL=TCIL(L)-TCOL(L)-TLL
      GO TO 260
!
!     SPECIAL CASE FOR TRAILING EDGE PANELS
!
  240 UCR=RCR
      UCL=RCL
      VCR=SCR
      VCL=SCL
      WCR=TCR
      WCL=TCL
      IF (.NOT.THK) GO TO 270
  250 UTR=RLR-RCIR(L)+RCOR(L)
      UTL=RLL-RCIL(L)+RCOL(L)
      VTR=SLR-SCIR(L)+SCOR(L)
      VTL=SLL-SCIL(L)+SCOL(L)
      WTR=TLR-TCIR(L)+TCOR(L)
      WTL=TLL-TCIL(L)+TCOL(L)
      GO TO 270
  260 RCR=ULR
      RCL=ULL
      SCR=VLR
      SCL=VLL
      TCR=WLR
      TCL=WLL
      IF (.NOT.THK) GO TO 270
      RTR=RLR
      RTL=RLL
      STR=SLR
      STL=SLL
      TTR=TLR
      TTL=TLL
!
!     COMBINE CONTRIBUTIONS OF LEFT AND RIGHT WING PANELS AND TRANSFORM
!     VELOCITY COMPONENTS BACK TO ORIGINAL COORDINATE SYSTEM
!
  270 CONTINUE
      CONR=1.0
      CONL=1.0
      IF (MIDFIN) CONR=.5
      IF (MIDFIN) CONL=.5
      IF (ABS(YI).GT..001) GO TO 280
      IF (ABS(COSTI).GT..001) GO TO 280
      IF (ABS(COST).GT.0.001) GO TO 280
      CONR=1.0
      CONL=1.0
      IF (SINTI.GT.0.) CONR=0.
      IF (SINTI.LT.0.) CONL=0.
  280 CONTINUE
      IF (.NOT.SUPTE.AND.L.EQ.NR1) GO TO 310
      ACR=WCR*COSTR+VCR*SINTR
      ACL=WCL*COSTL+VCL*SINTL
      BCR=VCR*COSTR-WCR*SINTR
      BCL=VCL*COSTL-WCL*SINTL
      AT=(ACR*CONR+ACL*CONL)*BCON
      BT=(BCR*CONR-BCL*CONL)*BCON
      UC(J)=(UCR*CONR+UCL*CONL)*CON
      VC(J)=BT*COSTI-AT*SINTI
      WC(J)=AT*COSTI+BT*SINTI
      AC(J)=AT*COSD-UC(J)*SIND
      IF (M.GT.M1) GO TO 300
      IF (.NOT.FLAG) GO TO 290
      USAVE(L)=UC(J)
      VSAVE(L)=VC(J)
      WSAVE(L)=WC(J)
      ASAVE(L)=AC(J)
      GO TO 320
  290 UC(J)=UC(J)+USAVE(L)
      VC(J)=VC(J)+VSAVE(L)
      WC(J)=WC(J)+WSAVE(L)
      AC(J)=AC(J)+ASAVE(L)
  300 CONTINUE
      IF(NPART.NE.5.AND.NPART.NE.9) GO TO 310
      IF (II.LT.J1.OR.II.GT.J2) GO TO 310
      JS1=J1
      JS2=J2
      NSS=NS
      ACF=VCR*SINTR
      BCF=VCR*COSTR
      ATF=ACF*CONR*BCON
      BTF=BCF*CONR*BCON
      UCF(L)=UCR*CONR*CON
      VCF(L)=BTF*COSTI-ATF*SINTI
      WCF(L)=ATF*COSTI+BTF*SINTI
  310 IF (.NOT.THK) GO TO 320
      ATR=WTR*COSTR+VTR*SINTR
      ATL=WTL*COSTL+VTL*SINTL
      BTR=VTR*COSTR-WTR*SINTR
      BTL=VTL*COSTL-WTL*SINTL
      AT=(ATR*CONR+ATL*CONL)*BCONT
      BT=(BTR*CONR-BTL*CONL)*BCONT
      UT(K)=(UTR*CONR+UTL*CONL)*CONT
      VT(K)=BT*COSTI-AT*SINTI
      WT(K)=AT*COSTI+BT*SINTI
      IF(NPART.NE.5.AND.NPART.NE.9) GO TO 320
      IF(I.LT.J1T.OR.I.GT.J2T) GO TO 320
      JS1T=J1T
      JS2T=J2T
      NSST=NST
      ATRF=WTR*COSTR
      BTRF=-WTR*SINTR
      ATF=ATRF*CONR*BCONT
      BTF=BTRF*CONR*BCONT
      UTF(L)=0.0
      VTF(L)=BTF*COSTI-ATF*SINTI
      WTF(L)=ATF*COSTI+BTF*SINTI
  320 END DO
  330 END DO
      IF (.NOT.FLAG) GO TO 340
      FLAG=.FALSE.
      THK=THIK
      J=JSAVE
      K=KSAVE
      NP=NPSAVE
      GO TO 40
  340 END DO
      NWING=J
      NWTHK=K
!*
      IF(NSEGNS.NE.0.AND.NPART.LE.3) GO TO 345
      IF(NPTSNS.EQ.0) WRITE(N11) (AC(J),J=1,NWING)
      IF(NPTSNS.NE.0) WRITE(TAPE16) (AC(J),J=1,NWING)
  345 CONTINUE
!*
      IF(NWING.LE.NMAX.OR.(NPART.NE.5.AND.NPART.NE.9)) GO TO 360
!
!     STORE DIAGONAL BLOCKS OF AERODYNAMIC MATRIX IN DC ARRAY
!
      DO 350 J=1,NWING
      IF (J.LT.JS1.OR.J.GT.JS2) GO TO 350
      K=J-JS1+1
      DC(K)=AC(J)
      AC(J)=0.
  350 END DO
      WRITE (TAPE10) (DC(J),J=1,NSS)
  360 CONTINUE
      IF (IABS(PRINT).LT.4) GO TO 380
      WRITE (TAPE6,450) II
      WRITE (TAPE6,460) NWING
      WRITE (TAPE6,440) (UC(J),J=1,NWING)
      WRITE (TAPE6,420) NWING
      WRITE (TAPE6,440) (VC(J),J=1,NWING)
      WRITE (TAPE6,410) NWING
      WRITE (TAPE6,440) (WC(J),J=1,NWING)
      WRITE(TAPE6,425) NWING
      WRITE (TAPE6,440) (AC(J),J=1,NWING)
      IF (.NOT.THK) GO TO 370
      WRITE (TAPE6,470) NWTHK
      WRITE (TAPE6,440) (UT(K),K=1,NWTHK)
      WRITE (TAPE6,430) NWTHK
      WRITE (TAPE6,440) (VT(K),K=1,NWTHK)
      WRITE (TAPE6,480) NWTHK
      WRITE (TAPE6,440) (WT(K),K=1,NWTHK)
  370 CONTINUE
      IF(NWING.GT.NMAX.AND.(NPART.EQ.5.OR.NPART.EQ.9))                  &
     & WRITE(TAPE6,490) NSS
      IF(NWING.GT.NMAX.AND.(NPART.EQ.5.OR.NPART.EQ.9))                  &
     & WRITE(TAPE6,440) (DC(J),J=1,NSS)
  380 IF (.NOT.THK) GO TO 390
      WRITE (TAPE8) (UT(K),VT(K),WT(K),K=1,NWTHK)
      IF(NPART.NE.5.AND.NPART.NE.9) GO TO 390
      WRITE(TAPE8) (UTF(K),VTF(K),WTF(K),K=1,NSST)
  390 IF(NSEGNS.NE.0.AND.NPART.LE.3) GO TO 395
      WRITE(TAPE8) (UC(J),VC(J),WC(J),J=1,NWING)
      IF(NPTSNS.EQ.0) WRITE(TAPE9) (AC(J),J=1,NWING)
      IF(NPTSNS.NE.0) WRITE(TAPE15) (AC(J),J=1,NWING)
      IF(NPART.NE.5.AND.NPART.NE.9) GO TO 395
      WRITE(TAPE8) (UCF(J),VCF(J),WCF(J),J=1,NSS)
  395 CONTINUE
  400 END DO
!
!
!
  410 FORMAT('  WC(J),J=1,', I3)
  420 FORMAT('  VC(J),J=1,', I3)
  425 FORMAT('  AC(J),J=1,', I3)
  430 FORMAT('  VT(K),K=1,', I3)
  440 FORMAT(1X,10F10.5)
  450 FORMAT(/' AERODYNAMIC MATRIX, I=', I3)
  460 FORMAT('  UC(J),J=1,', I3)
  470 FORMAT('  UT(K),K=1,', I3)
  480 FORMAT('  WT(K),K=1,', I3)
  490 FORMAT('  DC(J),J=1,', I3)
      END
      SUBROUTINE SORVEL (UC,VC,WC,UL,VL,WL)
!
!     ********************************************VERSION B00***********
!
!
!     COMPUTE THE THREE COMPONENTS OF VELOCITY INDUCED AT A SPECIFIED
!     CONTROL POINT BY CONSTANT AND LINEARLY VARYING SOURCE
!     DISTRIBUTIONS ON A SWEPT QUADRILATERAL PANEL. SORVEL CALCULATES
!     THE VELOCITY INDUCED BY ONE CORNER OF THE PANEL.
!
!     UC, VC, WC ARE VELOCITY COMPONENTS INDUCED BY CONSTANT SOURCE
!     DISTRIBUTION
!
!     UL, VL, WL ARE VELOCITY COMPONENTS INDUCED BY LINEAR CHORDWISE,
!     LINEAR SPANWISE SOURCE DISTRIBUTION
!
      COMMON /COMPS/X,DELTAY,DELTAZ,A,BB,C,SUB,BPOS,COST,SINT, ml
      COMMON /PI/ PI,RADEG,DEGRAD
      COMMON /EPS/ EPSPAN,EPSWP1,EPSWP2,EPSTP,EPSVC,EPSSP1,EPSSP2,EPSLV,&
     &EPSSV,EPSVV,EPSWV,EPSVP,EPSIT1,EPSIT2,EPSFM, epssb1,epssb2
      COMMON /SHELL/ NPTSB,NSEGWT,NPTSWT,NSEGN,NPTSN,NSEGS,NPTSS,       &
     &               NSEGNS,NPTSNS,NNAC,NSHELL,NQ,IPART,II
      COMMON /VELCOM/ NPOINT,NPART
!
      LOGICAL SUB,SUP,BPOS,BNEG,SUPLE
!
      IF(NSEGNS.NE.0.AND.NPART.GE.7.AND.II.GT.NPTSN) GO TO 170
      SUP=.NOT.SUB
      SUPLE=.FALSE.
      BNEG=.NOT.BPOS
      B = ABS(BB)
      IF(B.LE.EPSSV) B = 0.0
      SGN=1.0
      IF (SUP) SGN=-1.0
      BT1=SGN+B*B
      BTERM=SQRT(ABS(BT1))
      BTERMD=1./BTERM
      Y=DELTAY*COST+DELTAZ*SINT
      IF (BNEG) Y=-Y
      Z=DELTAZ*COST-DELTAY*SINT
      IF (ABS(Y).LE.EPSSV) Y=0.
      IF (ABS(Z).LE.EPSSV) Z=0.
      X2=X*X
      Y2=Y*Y
      Z2=Z*Z
      R2=Y2+Z2
      R=SQRT(R2)
      IF (SUB) GO TO 10
      IF (B.LT.1.0) SUPLE=.TRUE.
      IF (X.LE.0.) GO TO 170
      D=0.
      IF (X2.GT.R2) D=SQRT(X2-R2)
      GO TO 20
   10 D=SQRT(X2+R2)
   20 CONTINUE
      T2=B*X+SGN*Y
      T3=X-B*Y
      AT3=ABS(T3)
      IF (AT3.LE.EPSSV) AT3=0.
      UC=-PI*BTERMD
!
!     SPECIAL CASE FOR SUPERSONIC LEADING EDGE
!
      IF (D.GT.0.) GO TO 30
      IF (Y.LE.B*X) GO TO 170
      IF (T3.LE.0.) GO TO 170
      IF (X.LE.(B*Y+BTERM*ABS(Z))) GO TO 170
      SZ=SIGN(1.0,Z)
      UC=-PI/BTERM
      VC=-B*UC
      WC=SZ*PI
      UL=-PI*(T3*BTERMD-Z*SZ)
      VL=-B*UL
      WL=-SZ*BTERM*UL
      GO TO 160
   30 IF (SUP.AND.X2.LE.R2) GO TO 170
      IF (Z.EQ.0.) GO TO 80
!
!     GENERAL EQUATIONS
!
      DENOM=B*R2-X*Y
      F1=ATAN2(Z*D,DENOM)
      IF (SUB) F1=F1-ATAN2(Z,Y)
      G1=0.
      IF (BTERM.EQ.0.) GO TO 60
      ARG=T2
      IF (SUB) GO TO 40
      TZ=T3*T3+BT1*Z2
      IF (TZ.GT.0.) ST3=SQRT(TZ)
      IF (SUPLE) GO TO 50
   40 ARG=ARG+D*BTERM
      IF (SUP) ARG=ARG/ST3
      IF (ARG.GT.0.) G1=ALOG(ARG)*BTERMD
      GO TO 70
   50 ARG1=ARG/ST3
      IF (ABS(ARG1).LE.1.0) G1=ACOS(ARG1)*BTERMD
      GO TO 70
   60 IF (T2.NE.0.) G1=D/T2
   70 G2=ALOG((X+D)/R)
      G3=0.
      IF (SUB) G3=ALOG(R)
      C1=D
      IF (SUB) C1=X+D
      G=BT1*G1-B*G2
      H=B*G1-G2+G3
      UC=-G1
      VC=H
      WC=F1
      UL=Z*F1-T3*G1-Y*G2
      VL=T3*H+C1-B*Z*F1
      WL=T3*F1+Z*G
      GO TO 160
!
!     SPECIAL EQUATIONS FOR Z=0
!
   80 CONTINUE
      F1=0.
      DENOM=-Y*T3
      IF (DENOM.NE.0.) F1=ATAN2(0.,DENOM)
      IF (SUB.AND.Y.NE.0.) F1=F1-ATAN2(0.,Y)
      IF (SUPLE) GO TO 100
      G1=0.
      IF (BTERM.EQ.0.) GO TO 110
      IF (AT3.GT.0.) GO TO 90
      G1=(100.+ALOG(2.*BT1*ABS(Y)))*BTERMD
      IF (SUB.AND.Y.LT.0.) G1=-G1
      GO TO 120
   90 ARG=T2+D*BTERM
      IF (SUP) ARG=ARG/AT3
      IF (ARG.GT.0.) G1=ALOG(ARG)*BTERMD
      GO TO 120
  100 ARG1=T2/AT3
      IF (ABS(ARG1).LE.1.0) G1=ACOS(ARG1)*BTERMD
      GO TO 120
  110 IF (T2.NE.0.) G1=D/T2
  120 G2=100.
      IF (Y.EQ.0.) GO TO 130
      G2=ALOG((X+D)/ABS(Y))
      GO TO 140
  130 IF (X.NE.0.) G2=G2+ALOG(2.*ABS(X))
      IF (X.LT.0.) G2=-G2
  140 C1=D
      G3=0.
      IF (.NOT.SUB) GO TO 150
      C1=X+D
      IF (Y.NE.0.) G3=ALOG(ABS(Y))
      IF (Y.EQ.0.) G3=-100.
  150 H=B*G1-G2+G3
      UC=-G1
      VC=H
      WC=F1
      UL=-T3*G1-Y*G2
      VL=T3*H+C1
      WL=T3*F1
  160 IF (BPOS) RETURN
      UC=-UC
      WC=-WC
      UL=-UL
      WL=-WL
      RETURN
  170 UC=0.
      VC=0.
      WC=0.
      UL=0.
      VL=0.
      WL=0.
      RETURN
      END
      SUBROUTINE VORVEL (UC,VC,WC,UL,VL,WL,ULT,VLT,WLT)
!
!     *******************************************VERSION B00************
!
!     COMPUTE THE THREE COMPONENTS OF VELOCITY INDUCED AT A SPECIFIED
!     CONTROL POINT BY CONSTANT AND LINEARLY VARYING VORTEX
!     DISTRIBUTIONS ON A SWEPT QUADRILATERAL PANEL. VORVEL CALCULATES
!     THE VELOCITY INDUCED BY THE LEADING AND TRAILING CORNERS OF ONE
!     EDGE OF THE PANEL.
!
!     UC, VC, WC ARE VELOCITY COMPONENTS INDUCED BY CONSTANT CHORDWISE
!     AND SPANWISE VORTEX DISTRIBUTION
!
!     UL, VL, WL, ARE VELOCITY COMPONENTS INDUCED BY LEADING EDGE OF
!     LINEAR CHORDWISE, CONSTANT SPANWISE VORTEX DISTRIBUTION
!
!     ULT, VLT, WLT ARE VELOCITY COMPONENTS INDUCED BY TRAILING EDGE OF
!     LINEAR CHORDWISE, CONSTANT SPANWISE VORTEX DISTRIBUTION
!
!*
!!!      COMMON /OPTION/ NOPT(10),NXSTNS,XSTN(20),XAPEX,YAPEX,XLAMDA
      INCLUDE 'option.cmn'
!*
      COMMON /PARAM/ NBODY,NWING,NTAIL,LBC,THK,MACH,ALPHA,REFA
      COMMON /COMPS/ X,DELTAY,DELTAZ,AL,BL,C,SUB,BPOS,COST,SINT,ML
      COMMON /PI/ PI,RADEG,DEGRAD
      COMMON /EPS/ EPSPAN,EPSWP1,EPSWP2,EPSTP,EPSVC,EPSSP1,EPSSP2,EPSLV,&
     &EPSSV,EPSVV,EPSWV,EPSVP,EPSIT1,EPSIT2,EPSFM, epssb1,epssb2
      COMMON /SHELL/ NPTSB,NSEGWT,NPTSWT,NSEGN,NPTSN,NSEGS,NPTSS,       &
     &               NSEGNS,NPTSNS,NNAC,NSHELL,NQ,IPART,II
      COMMON /VELCOM/ NPOINT,NPART
!
      DIMENSION Q(51), XI(51), QX(51)
!
      LOGICAL SUB,SUP,BPOS,SUPLE,LBC
!
      IF(NSEGNS.NE.0.AND.NPART.LE.3) GO TO 400
      A=AL
      B=ABS(BL)
      BPOS=BL.GE.0.
      IF (BL.LT.0.) A=-AL
      IF (ABS(C).LE.EPSVV) C=0.
      CC=C*C
      SUP=.NOT.SUB
      SUPLE=.FALSE.
      IF (ABS(B).LE.EPSVV) B=0.
      AB=A+B
      SGN=1.0
      IF (SUP) SGN=-1.0
      B1=SGN+B*B
      SB1=SQRT(ABS(B1))
      Y=DELTAY*COST+DELTAZ*SINT
      Z=DELTAZ*COST-DELTAY*SINT
      IF (BL.LT.0.) Y=-Y
      IF (ABS(Y).LE.EPSVV) Y=0.
      IF (ABS(Z).LE.EPSVV) Z=0.
      X2=X*X
      Y2=Y*Y
      Z2=Z*Z
      R2=Y2+Z2
      R=SQRT(R2)
      IF (SUB) GO TO 10
      IF (ABS(B).LT.1.0) SUPLE=.TRUE.
      IF (X.LT.0.) GO TO 400
   10 D=0.
      D2=X2+SGN*R2
      IF (D2.GT.0.) D=SQRT(D2)
      AZ=A*Z
      T1=C-A*Y
      IF (ABS(T1).LE.EPSVV) T1=0.
      T2=T1*T1
      T3=X-B*Y
      AT3=ABS(T3)
      IF (AT3.LE.EPSVV) AT3=0.
      IF (AT3.EQ.0.) T3=0.
      T4=AZ*AZ
      T5=T2+T4
      IF (T5.NE.0.) T5=1./T5
      T6=B*C-A*X
      T7=T6*T6
      T8=T7+SGN*(T2+T4)
      T9=T1*T3+A*B*Z2
      E=SQRT(ABS(T8))
      B2=SGN*(C*Y-A*R2)
      B3=B*X+SGN*Y
      B4=T5*T6
      TZ=T3*T3+B1*Z2
      IF (TZ.GT.0.) ST3=SQRT(TZ)
      WQ=0.
!
!     EVALUATION OF DOWNWASH INDUCED BY TRAILING VORTEX SHEET
!
      IF (A.EQ.0..OR.ML.EQ.2) GO TO 160
      INT=0
      MAX=11
      XI(1)=0.
      XO=0.
      IF (T1.NE.0.) XO=T3/T1
      BO=B-A*XO
      SBO=SGN+BO*BO
      EL=1.0
      IF (SUP.AND.Y.GT.0..AND.ABS(BO).LE.1.) EL=T5*(T9-E*ABS(Z))
      IF (EL.LE.0.) GO TO 150
      IF (EL.GT.1.0) EL=1.0
      DXI=EL/FLOAT(MAX-1)
      EPX=.005*EL
      IF (Y.EQ.0..OR.ABS(Z).GT..10) GO TO 60
      IF (XO.LT.-EPX.OR.XO.GT.EL+EPX) GO TO 60
      INT=1
      MAX=13
      QM=0.
      IF (XO.LT..6*EL) MAX=8
      IF (XO.LT..1*EL) MAX=4
      IF (XO.LT.EPX) MAX=1
      IF (MAX.EQ.1) GO TO 20
      DXI=(XO-EPX)/FLOAT(MAX-3)
      GO TO 60
   20 INT=2
      IF (XO.GE.EL-EPX) MAX=1
      COEF=2.
      IF (MAX.EQ.1) COEF=1.
      WO=0.
      IF (SBO.EQ.0.) GO TO 30
      IF (MAX.GT.1.AND.QM.EQ.0.) GO TO 30
      ARG=EPX*EPX+SBO*Z2
      IF (ARG.LE.0.) GO TO 30
      ARG=SQRT(ARG)/(2.*SBO*ABS(Y))
      IF (Y.GT.0.) ARG=ARG+1./ARG
      IF (ARG.GT.0.) WO=ALOG(ARG)
      WO=COEF*BO*EPX*XO*WO/SQRT(ABS(SBO))
   30 WQ=WQ+WO
      IF (MAX.EQ.1.AND.XO.GT.EPX) GO TO 60
      IF (XO.LE..9*EL) GO TO 40
      MAX=4
      GO TO 50
   40 MAX=16-MAX
      IF (MAX.LT.8) MAX=8
   50 DXI=(EL-XO-EPX)/FLOAT(MAX-3)*.125
      XI(1)=XO+EPX
   60 CONTINUE
      IF (MAX.EQ.1) GO TO 150
      DO 140 M=1,MAX
      Q(M)=0.
      IF (INT.EQ.0) GO TO 80
      IF (INT.EQ.2) GO TO 70
      IF (M.EQ.MAX) GO TO 80
      IF (M.GT.MAX-3) DXI=.5*DXI
      GO TO 80
   70 IF (M.EQ.1.OR.M.EQ.3) GO TO 80
      IF (M.LE.5) DXI=2.*DXI
   80 CONTINUE
      IF (M.GT.1) XI(M)=XI(M-1)+DXI
      DX=X-XI(M)*C
      DX2=DX*DX
      BX=B-A*XI(M)
      BX2=BX*BX
      BX1=SGN+BX2
      SBX=SQRT(ABS(BX1))
      SDX=0.
      DXR=DX2+SGN*R2
      IF (DXR.GT.0.) SDX=SQRT(DXR)
      IF (SDX.EQ.0.) GO TO 90
      IF (SUP.AND.DX.LT.0.) GO TO 130
      ARG=SGN*Y+BX*DX
      IF (SBX.EQ.0.) GO TO 110
      TZI=(T3-XI(M)*T1)**2+BX1*Z2
      IF (TZI.EQ.0.) GO TO 120
      STZ=SQRT(TZI)
      IF (SUP.AND.ABS(BX).LT.1.) GO TO 100
      ARG=(ARG+SBX*SDX)/STZ
      IF (SUP) ARG=ABS(ARG)
      IF (ARG.GT.0.) Q(M)=ALOG(ARG)*BX/SBX
      GO TO 130
   90 IF (T1.LT.BX*T6.AND.T8.LT.0.) GO TO 130
      IF (ABS(BX).GE.1.) GO TO 130
      IF (Y.LE.BX*DX) GO TO 130
      IF (DX.LT.(BX*Y+SBX*ABS(Z))) GO TO 130
      Q(M)=PI*BX/SBX
      GO TO 130
  100 ARG=ARG/STZ
      IF (ARG.GT.1.0) GO TO 130
      IF (ARG.LE.-1.0) GO TO 90
      Q(M)=ACOS(ARG)*BX/SBX
      GO TO 130
  110 IF (ABS(ARG).GT.EPSVV) Q(M)=SDX*BX/ARG
      GO TO 130
  120 IF (Y.GE.0.) GO TO 140
      Q(M)=-ALOG(ABS(Y))*BX/SBX
  130 CONTINUE
      QX(M)=Q(M)*XI(M)
  140 END DO
      CALL TRAP (XI,QX,DW,MAX)
      WQ=WQ+DW
  150 CONTINUE
      QM=Q(MAX)
      IF (INT.EQ.1) GO TO 20
  160 CONTINUE
      IF (.NOT.SUPLE) GO TO 180
!
!     SPECIAL EQUATIONS FOR SUPERSONIC LEADING EDGE
!
      IF (D.GT.0.) GO TO 180
      IF (Y.LE.B*X) GO TO 400
      IF (X.LT.(B*Y+SB1*ABS(Z))) GO TO 400
      SZ=SIGN(1.0,Z)
      PZ=PI*SZ
      UC=PZ
      VC=-B*PZ
      WC=-PI*SB1
      IF (T8.GT.0.) E=0.
      SL=PI*T5*(SZ*T9-Z*E)
      TL=SZ*E*T5*SL
      IF (T8.GT.0.) TL=PI*T5*T5*T8*ABS(Z)
      IF (ML.EQ.2) GO TO 170
      UL=SL
      VL=-((B+T1*B4)*SL-AZ*TL)/2.
      WL=AZ*B4*SL-T1*TL+A*WQ
  170 ULS=SL+PZ
      ULT=ULS
      TT=SZ*E*T5*ULS
      IF (T8.GT.0.) TT=TL
      VLT=(A*PZ-(AB+T1*B4)*ULS+AZ*TT)/2.
      WLT=AZ*B4*ULS-T1*TT
      GO TO 390
  180 IF (SUP.AND.D.EQ.0.) GO TO 400
      IF (Z.EQ.0.) GO TO 260
!
!     GENERAL EQUATIONS
!
      DENOM=B*R2-X*Y
      F1=ATAN2(Z*D,DENOM)
      IF (SUB) F1=F1-ATAN2(Z,Y)
      G1=0.
      IF (T8.EQ.0.) GO TO 210
      IF (C.EQ.0.) GO TO 190
      ARG=X*T6+B2
      IF (T8.LT.0.) GO TO 200
      ARG=(ARG+D*E)/(ST3*C)
      IF (SUP) ARG=ABS(ARG)
      IF (ARG.GT.0.) G1=ALOG(ARG)
      GO TO 210
  190 IF (ST3.NE.0.) G1=ALOG(ST3)
      GO TO 210
  200 ARG=ARG/(ST3*C)
      IF (ABS(ARG).GT.1.0) GO TO 210
      G1=-ACOS(ARG)
  210 H1=0.
      IF (C.EQ.0..AND.A.LT.0.) G1=-G1
      IF (ML.EQ.2) GO TO 230
      IF (SB1.EQ.0.) GO TO 230
      ARH=B3
      IF (SUPLE) GO TO 220
      ARH=(ARH+D*SB1)/ST3
      IF (SUP) ARH=ABS(ARH)
      IF (ARH.GT.0.) H1=ALOG(ARH)
      GO TO 230
  220 H1=-ACOS(ARH/ST3)
  230 G2=ALOG((X+D)/R)
      G3=0.
      IF (SUB) G3=ALOG(R)
      C1=D
      IF (SUB) C1=X+C1
      C2=C1/R2
      H=SB1*H1-B*(G2-G3)
      IF (SB1.EQ.0.) H2=B*D/B3-G2+G3
      IF (SB1.NE.0.) H2=B*H1/SB1-G2+G3
      UC=F1
      VS=-B*F1+Z*C2
      WS=H-Y*C2
      VC=VS
      WC=WS
      IF (C.EQ.0.) C2=0.
      C3=0.
      C4=0.
      C5=G2/2.
      C6=0.
      IF (C.EQ.0.) GO TO 240
      C3=(X*C2+SGN*G2)/(2.*C)
      C4=((X2-SGN*R2/2.)*G2-1.5*X*D)/(2.*CC)
      C5=(D-X*G2)/C
  240 CONTINUE
      WQ=WQ-C4
      G=E*G1-T6*G2
      SL=T5*(T9*F1+Z*G)
      TL=-B*D
      IF (C.NE.0.) TL=(B2*G2+T6*D)/C
      TL=-T5*(T5*(G*T9-Z*T8*F1)+TL)
      IF (ML.EQ.2) GO TO 250
      UL=SL
      VL=-((B+T1*B4)*SL-AZ*TL)/2.+Z*C3
      WL=AZ*B4*SL-T1*TL-Y*C3+A*WQ
  250 ULS=SL+F1
      ULT=ULS
      TLT=TL-T5*G
      WQT=C5-C4-G3/2.
      VLS=(A*F1-(AB+T1*B4)*ULS+AZ*TLT)/2.+Z*(C2+C3)
      VLT=VLS
      WLS=AZ*B4*ULS-T1*TLT-Y*(C2+C3)+A*WQT
      WLT=WLS
      GO TO 390
!
!     SPECIAL EQUATIONS FOR Z=0
!
  260 CONTINUE
      F1=0.
      DENOM=-Y*T3
      IF (DENOM.NE.0.) F1=ATAN2(0.,DENOM)
      IF (SUB.AND.Y.NE.0.) F1=F1-ATAN2(0.,Y)
      G1=0.
      IF (T8.EQ.0.) GO TO 300
      IF (C.EQ.0.) GO TO 280
      IF (T8.LT.0.) GO TO 290
      IF (AT3.GT.0.) GO TO 270
      IF (Y.EQ.0..OR.T1.EQ.0.) GO TO 300
      ARG=2.*Y*T1*B1/C
      IF (ARG.GT.0.) G1=ALOG(ARG)
      IF (Y.GT.0.) G1=100.+G1
      GO TO 300
  270 ARG=(X*T6+SGN*Y*T1+D*E)/(AT3*C)
      IF (SUP) ARG=ABS(ARG)
      IF (ARG.GT.0.) G1=ALOG(ARG)
      GO TO 300
  280 IF (Y.GT.0.) G1=-100.
      IF (AT3.NE.0.) G1=ALOG(AT3)
      GO TO 300
  290 ARG=(X*T6-Y*T1)/(AT3*C)
      IF (ABS(ARG).GT.1.0) GO TO 300
      G1=-ACOS(ARG)
  300 H1=0.
      IF (C.EQ.0..AND.A.LT.0.) G1=-G1
      IF (ML.EQ.2) GO TO 330
      IF (SB1.EQ.0.) GO TO 330
      IF (SUPLE) GO TO 320
      IF (AT3.GT.0.) GO TO 310
      IF (Y.EQ.0.) GO TO 330
      H1=ALOG(ABS(Y))
      IF (SUB.AND.Y.LT.0.) H1=-H1
      IF (Y.GT.0.) H1=100.+H1
      GO TO 330
  310 CONTINUE
      ARH=(B3+D*SB1)/AT3
      IF (SUP) ARH=ABS(ARH)
      IF (ARH.GT.0.) H1=ALOG(ARH)
      GO TO 330
  320 H1=-ACOS(B3/AT3)
  330 G2=100.
      IF (Y.NE.0.) GO TO 340
      IF (X.NE.0.) G2=G2+ALOG(2.*ABS(X))
      IF (X.LT.0.) G2=-G2
      GO TO 350
  340 G2=0.
      ARG=(X+D)/ABS(Y)
      IF (ARG.GT.0.) G2=ALOG(ARG)
  350 G3=0.
      C1=D
      IF (.NOT.SUB) GO TO 360
      C1=X+D
      G3=-100.
      IF (Y.NE.0.) G3=ALOG(ABS(Y))
  360 C2=0.
      IF (Y.NE.0.) C2=C1/Y2
      H=SB1*H1-B*(G2-G3)
      IF (SB1.EQ.0.) H2=B*D/B3-G2+G3
      IF (SB1.NE.0.) H2=B*H1/SB1-G2+G3
      UC=F1
      VS=-B*F1
      WS=H-Y*C2
      VC=VS
      WC=WS
      IF (C.EQ.0.) C2=0.
      C4=0.
      C5=G2/2.
      C6=0.
      IF (C.EQ.0.) GO TO 370
      C3=(X*C2+SGN*G2)/2.
      C4=((X2-SGN*Y2/2.)*G2-1.5*X*D)/(2.*CC)
      C5=(D-X*G2)/C
  370 CONTINUE
      WQ=WQ-C4
      WQT=C5-C4-G3/2.
      IF (T1.NE.0.) GO TO 380
      WL=A*WQ
      WLS=A*WQT
      WLT=WLS
      GO TO 410
  380 SL=T3*F1/T1
      UL=SL
      VL=-(B+T6/T1)*SL/2.
      G=E*G1-T6*G2
      TL=T3*T5*G
      IF (C.EQ.0.) TL=TL-B*D/T1
      IF (C.NE.0.) TL=TL+(T6*D/T1+Y*(SGN*G2-C3))/C
      WL=TL+A*WQ
      ULS=SL+F1
      VLS=(A*F1-(AB+T6/T1)*ULS)/2.
      WLS=TL+G/T1-Y*C2+A*WQT
      ULT=ULS
      VLT=VLS
      WLT=WLS
  390 IF (BPOS) GO TO 420
      UC=-UC
      WC=-WC
      UL=-UL
      WL=-WL
      ULT=-ULT
      WLT=-WLT
      GO TO 420
  400 UC=0.
      VC=0.
      WC=0.
      WL=0.
      WLT=0.
  410 UL=0.
      VL=0.
      ULT=0.
      VLT=0.
      GO TO 390
  420 CONTINUE
!*
      IF( NOPT(1).EQ.1 .AND. NOPT(4).EQ.0 ) GO TO 430
      UL = .0
      VL = 0.
      WL = 0.
      ULT = 0.
      VLT = 0.
      WLT = 0.
  430 CONTINUE
!*
      RETURN
      END


!!!     OVERLAY(LWB,2,3)
      SUBROUTINE WNGVEL()
!
!     *******************************************VERSION B00************
!
!     COMPUTE THE THREE COMPONENTS OF VELOCITY INDUCED AT SPECIFIED
!     CONTROL POINTS BY VORTEX PANELS LOCATED ON WING, FIN (VERTICAL
!     TAIL), OR CANARD (HORIZONTAL TAIL) SURFACES.
!
!
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAP&
!!!     &E9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      INCLUDE 'files.cmn'
      COMMON /PARAM/ NBODY,NWING,NPANEL,LBC,THK,MACH,ALPHA,REFA
      COMMON /VELCOM/ NPOINT,NPART,IMAX,JMAX,NMAX,EM,PRINT,NWTHK
      COMMON /SEG/ NSEG,NROW(20),NCOL(20),COSS(20),SINS(20),TT(20),NWT(2&
     &0),SPNW(20),XLEW(20),BEL(20),ZLEW(20),XS(20),YS(20),ZS(20)
      COMMON /COMPV/ XJ,YJ,ZJ,AL,BL,CL,SUB,BPOS,M,NSIDE
      COMMON /POINT/ ARRAY(6000)
      COMMON /SCRAT/ BLOCK(7500)
      COMMON /TRAN/ SIND,COSD,TAND,SINT,COST,CONTD,SINTI,COSTI,CON,BCON,&
     &SINDI,COSDI,MIDFIN,FIN,VENTRL
      COMMON /BTHET/ THETI(600)
      COMMON /PI/ PI,RADEG,DEGRAD
      COMMON /EPS/ EPSPAN,EPSWP1,EPSWP2,EPSTP,EPSVC,EPSSP1,EPSSP2,EPSLV,&
     &EPSSV,EPSVV,EPSWV,EPSVP,EPSIT1,EPSIT2,EPSFM, epssb1,epssb2
!
      DIMENSION XPT(600), YPT(600), ZPT(600), THET(600), DELTA(600)
      DIMENSION XC(30,20), YC(30,20), ZC(30,20)
      DIMENSION DELTI(600), USAVE(60), VSAVE(60), WSAVE(60), ASAVE(60)
      DIMENSION XCPT(600), YCPT(600), ZCPT(600), ZK(2,2), A(30), CI(30)
      DIMENSION CO(30), AC(600), UC(600), VC(600), WC(600), COSBD(600)
      DIMENSION SINBD(600), TANBD(600), DC(60), UL(30), VL(30), WL(30), AN(30), ZU(30,20)
!
      EQUIVALENCE (ARRAY(1),XPT(1))
      EQUIVALENCE (ARRAY(601),YPT(1))
      EQUIVALENCE (ARRAY(1201),ZPT(1))
      EQUIVALENCE (ARRAY(1801),THET(1))
      EQUIVALENCE (ARRAY(2401),DELTA(1))
      EQUIVALENCE (ARRAY(3001),XC(1,1))
      EQUIVALENCE (ARRAY(3601),YC(1,1))
      EQUIVALENCE (ARRAY(4201),ZC(1,1))
      EQUIVALENCE (ARRAY(4801),DELTI(1))
      EQUIVALENCE (ARRAY(5401),USAVE(1))
      EQUIVALENCE (ARRAY(5461),VSAVE(1))
      EQUIVALENCE (ARRAY(5521),WSAVE(1))
      EQUIVALENCE (ARRAY(5581),ASAVE(1))
      EQUIVALENCE (BLOCK(1),XCPT(1))
      EQUIVALENCE (BLOCK(601),YCPT(1))
      EQUIVALENCE (BLOCK(1441),A(1))
      EQUIVALENCE (BLOCK(1471),CI(1))
      EQUIVALENCE (BLOCK(1501),CO(1))
      EQUIVALENCE (BLOCK(1531),AC(1))
      EQUIVALENCE (BLOCK(2131),UC(1))
      EQUIVALENCE (BLOCK(2731),VC(1))
      EQUIVALENCE (BLOCK(3331),WC(1))
      EQUIVALENCE (BLOCK(3931),COSBD(1))
      EQUIVALENCE (BLOCK(4531),SINBD(1))
      EQUIVALENCE (BLOCK(5131),TANBD(1))
      EQUIVALENCE (BLOCK(5731),DC(1))
      EQUIVALENCE (BLOCK(5791),ZCPT(1))
      EQUIVALENCE (BLOCK(6781),UL(1))
      EQUIVALENCE (BLOCK(6811),VL(1))
      EQUIVALENCE (BLOCK(6841),WL(1))
      EQUIVALENCE (BLOCK(6871),AN(1))
      EQUIVALENCE (BLOCK(6901),ZU(1,1))
!
      LOGICAL THK,LBC,SUB,BPOS,FLAG,MIDFIN,FIN,VENTRL
      INTEGER PRINT
!!!,TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAPE&
!!!     &9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      REAL MACH
!
      IF (MACH.GT.1.) GO TO 510
      SUB=MACH.LT.1.0
      SGN=-1.0
      FLAG=.FALSE.
      NYC=1
      IF (ABS(YC(1,1)).LE.EPSWV) NYC=0
      IF (SUB) SGN=1.0
      BETA=SQRT(ABS(MACH*MACH-1.0))
      CON=1./(2.*PI)
      IF (SUB) CON=CON/2.
      BCON=BETA*CON
      IF (NPART.NE.2) GO TO 10
      REWIND TAPE7
      READ (TAPE7) (XCPT(N),N=1,600),(YCPT(N),N=1,600),(ZCPT(N),N=1,600)&
     &,(THET(N),N=1,600),(DELTA(N),N=1,600)
      REWIND TAPE7
      GO TO 30
   10 DO 20 N=1,NWING
      XCPT(N)=XPT(N)
      YCPT(N)=YPT(N)
   20 ZCPT(N)=ZPT(N)
   30 CONTINUE
      DO 40 N=1,NWING
      BD=BETA*TAN(DELTA(N))
      TANBD(N)=BD
      ARG=1.+SGN*BD*BD
      IF (ARG.LT.0.) GO TO 500
      COSBD(N)=1./SQRT(ARG)
   40 SINBD(N)=BD*COSBD(N)
!
!     I IS THE INDEX OF THE CONTROL POINT
!     J IS THE INDEX OF THE INFLUENCING PANEL
!
      DO 490 I=1,NPOINT
      IF (NPART.EQ.2) GO TO 50
      SINTI=SIN(THET(I))
      COSTI=COS(THET(I))
      SINDI=SIN(DELTA(I))
      COSDI=COS(DELTA(I))
      GO TO 60
   50 SINTI=SIN(THETI(I))
      COSTI=COS(THETI(I))
      SINDI=SIN(DELTI(I))
      COSDI=COS(DELTI(I))
   60 XI=XPT(I)
      YI=YPT(I)
      ZI=ZPT(I)
      J=0
      JJ=0
      J2=0
      N2=0
!
!     COMPUTE INFLUENCE OF EACH WING SEGMENT
!
      DO 450 NS=1,NSEG
      NR=NROW(NS)
      NC=NCOL(NS)
      NR1=NR+1
      NR2=2*NR
      NT=NWT(NS)
      FIN=.FALSE.
      MIDFIN=.FALSE.
      VENTRL=.FALSE.
      IF (NT.NE.0.AND.COSS(NS).LE..70711) FIN=.TRUE.
      IF (.NOT.FIN) GO TO 70
      IF (NYC.EQ.0.AND.ABS(COSS(NS)).LE.EPSWV) MIDFIN=.TRUE.
      IF (SINS(NS).LT.0.) VENTRL=.TRUE.
   70 CONTINUE
      JSAVE=J
      JJSAVE=JJ
      J2SAVE=J2
      NI=N2+1
      IF (NS.GT.1.AND.NT.NE.0) NI=NI+1
      DO 80 K=1,NR2
      USAVE(K)=0.
      VSAVE(K)=0.
      WSAVE(K)=0.
   80 ASAVE(K)=0.
!
!     IF FLAG IS TRUE, AN ADDITIONAL COLUMN OF VORTEX PANELS
!     EXTENDS FROM THE CENTER LINE TO THE INBOARD EDGE OF THE
!     WING, HORIZONTAL TAIL, OR CANARD
!
      IF (NS.EQ.1.AND.NYC.NE.0) FLAG=.TRUE.
      MYC=1
      IF (ABS(YC(1,NI)).LE.EPSWV) MYC=0
      IF (NS.GT.1.AND.NT.EQ.1.AND.MYC.NE.0) FLAG=.TRUE.
   90 N2=NI+NC-1
      IF (FLAG) N2=NI
!
!     COMPUTE INFLUENCE OF EACH COLUMN OF PANELS
!
      DO 410 N=NI,N2
      J1=1+J2
      J2=J1+NR2-1
      JL=J1
      JT=J1+NR
      US=0.
      VS=0.
      WS=0.
      AS=0.
!
!     COMPUTE INFLUENCE OF UPPER AND LOWER SURFACES
!
      DO 400 NN=1,2
      NSIDE=NN
!
!     COMPUTE INFLUENCE OF EACH PANEL
!
      DO 400 L=1,NR1
      J=J+1
      IF (L.EQ.NR1) GO TO 270
      JJ=JJ+1
      IF (FLAG) GO TO 120
      SIND=SINBD(JJ)
      COSD=COSBD(JJ)
      TAND=TANBD(JJ)
      TANB=TAND/BETA
      THETA=THET(JJ)
      IF (FIN) THETA=THETA-.5*PI
      IF (VENTRL) THETA=-THETA-PI
      COST=COS(THETA)
      SINT=SIN(THETA)
      DCOST=1./COST
      DO 110 K=1,2
      N1=N+K-1
      DO 110 MM=1,2
      M=MM
      M1=L+M-1
      IF (FIN) GO TO 100
      DX=XCPT(JJ)-XC(M1,N1)
      DY=YCPT(JJ)-YC(M1,N1)
      ZK(M,K)=ZCPT(JJ)-DCOST*(DX*TANB+DY*SINT)
      GO TO 110
  100 IF (NSIDE.EQ.1) ZK(M,K)=ZU(M1,N1)
      IF (NSIDE.EQ.2) ZK(M,K)=ZC(M1,N1)
  110 CONTINUE
      GO TO 130
  120 THETA=0.
      DXC=XC(L+1,NI)-XC(L,NI)
      IF (NSIDE.EQ.1) DZC=ZU(L+1,NI)-ZU(L,NI)
      IF (NSIDE.EQ.2) DZC=ZC(L+1,NI)-ZC(L,NI)
      TAND=BETA*DZC/DXC
      ARG=1.+SGN*TAND*TAND
      IF (ARG.LT.0.) GO TO 500
      SARG=SQRT(ARG)
      COSD=1./SARG
      SIND=TAND*COSD
      CL=DXC*SARG
      COST=1.0
      SINT=0.
  130 CONTINUE
      COSTD=COST*COSD
      CONTD=SQRT(SGN*TAND*TAND+COST*COST)
      COSDTD=1./(COSD*CONTD)
      CONTDD=1./CONTD
!
!     COMPUTE PANEL LEADING AND TRAILING EDGE SLOPES
!
      BL=0.
      DO 150 MM=1,2
      M=MM
      IF (FLAG) GO TO 140
      M1=L+M-1
      DXC=XC(M1,N+1)-XC(M1,N)
      DYC=YC(M1,N+1)-YC(M1,N)
      IF (VENTRL) DYC=-DYC
      DZC=ZK(M,2)-ZK(M,1)
      DYC=BETA*DYC
      DZC=BETA*DZC
      DZL=DZC*COSTD-DXC*SIND
      DYL=DYC*COSD*CONTD+SINT*DZL*CONTDD
      DXL=(DXC*COSTD+DZC*SIND*SGN)*COSDTD
      BL=DXL/DYL
  140 IF (M.EQ.1) BLE=BL
      IF (M.EQ.2) BTE=BL
  150 END DO
      AL=BLE-BTE
      A(L)=AL
!
!     COMPUTE PANEL CHORD LENGTHS
!
      DO 260 K=1,2
      N1=N+K-1
      IF (FLAG) N1=NI
      IF (FLAG) GO TO 160
      DXC=XC(L+1,N1)-XC(L,N1)
      DZC=ZK(2,K)-ZK(1,K)
      CL=(DXC*COSTD+BETA*DZC*SIND*SGN)*COSDTD
  160 IF (K.EQ.1) CI(L)=CL
      IF (K.EQ.2) CO(L)=CL
!
!     COMPUTE INFLUENCE OF PANEL CORNERS
!
      DO 260 MM=1,2
      M=MM
      M1=L+M-1
!
!     COMPUTE CONTROL POINT IN PANEL COORDINATE SYSTEM
!
      DX=XI-XC(M1,N1)
      DY=YI-YC(M1,N1)
      IF (VENTRL) DY=-DY
      IF (FLAG) GO TO 170
      DZ=ZI-ZK(M,K)
      GO TO 180
  170 IF (K.EQ.1) DY=YI
      IF (NSIDE.EQ.1) DZ=ZI-ZU(M1,N1)
      IF (NSIDE.EQ.2) DZ=ZI-ZC(M1,N1)
  180 CONTINUE
      DY=BETA*DY
      DZ=BETA*DZ
      XJ=(DX*COSTD+DZ*SIND*SGN)*COSDTD
      ZJ=DZ*COSTD-DX*SIND
      YJ=DY*COSD*CONTD+SINT*ZJ*CONTDD
      ZJ=ZJ-DY*COSD*SINT
      IF (M.EQ.1) BL=BLE
      IF (M.EQ.2) BL=BTE
      IF (K.EQ.2) GO TO 220
      IF (M.EQ.2) GO TO 190
      CALL VORPAN (UCIR,VCIR,WCIR,ULIR,VLIR,WLIR,X,X,X,VEIR,WEIR,VAIR,WA&
     &IR,USIR,VSIR)
      GO TO 200
  190 CALL VORPAN (RCIR,SCIR,TCIR,X,X,X,RLIR,SLIR,TLIR,SEIR,TEIR,SAIR,TA&
     &IR,RSIR,SSIR)
  200 IF (MIDFIN) GO TO 260
      DY=-YI-YC(M1,N1)
      IF (FLAG.AND.K.EQ.1) DY=-YI
      IF (VENTRL) DY=-DY
      DY=BETA*DY
      ZJ=DZ*COSTD-DX*SIND
      YJ=DY*COSD*CONTD+SINT*ZJ*CONTDD
      ZJ=ZJ-DY*COSD*SINT
      IF (M.EQ.2) GO TO 210
      CALL VORPAN (UCIL,VCIL,WCIL,ULIL,VLIL,WLIL,X,X,X,VEIL,WEIL,VAIL,WA&
     &IL,USIL,VSIL)
      GO TO 260
  210 CALL VORPAN (RCIL,SCIL,TCIL,X,X,X,RLIL,SLIL,TLIL,SEIL,TEIL,SAIL,TA&
     &IL,RSIL,SSIL)
      GO TO 260
  220 IF (M.EQ.2) GO TO 230
      CALL VORPAN (UCOR,VCOR,WCOR,ULOR,VLOR,WLOR,X,X,X,VEOR,WEOR,VAOR,WA&
     &OR,USOR,VSOR)
      GO TO 240
  230 CALL VORPAN (RCOR,SCOR,TCOR,X,X,X,RLOR,SLOR,TLOR,SEOR,TEOR,SAOR,TA&
     &OR,RSOR,SSOR)
  240 IF (MIDFIN) GO TO 260
      DY=-YI-YC(M1,N1)
      IF (FLAG.AND.K.EQ.1) DY=-YI
      IF (VENTRL) DY=-DY
      DY=BETA*DY
      ZJ=DZ*COSTD-DX*SIND
      YJ=DY*COSD*CONTD+SINT*ZJ*CONTDD
      ZJ=ZJ-DY*COSD*SINT
      IF (M.EQ.2) GO TO 250
      CALL VORPAN (UCOL,VCOL,WCOL,ULOL,VLOL,WLOL,X,X,X,VEOL,WEOL,VAOL,WA&
     &OL,USOL,VSOL)
      GO TO 260
  250 CALL VORPAN (RCOL,SCOL,TCOL,X,X,X,RLOL,SLOL,TLOL,SEOL,TEOL,SAOL,TA&
     &OL,RSOL,SSOL)
  260 CONTINUE
      GO TO 320
!
!     COMPUTE INFLUENCE OF WAKE- NOTE- PLANAR WAKE ASSUMED
!
  270 SINT=SINS(NS)
      COST=COSS(NS)
      IF (.NOT.FIN) GO TO 280
      SINT=0.
      COST=1.0
  280 CONTINUE
      TAND=0.
      SIND=0.
      COSD=1.0
      BCOS=BETA*COST
      BSIN=BETA*SINT
      CONTD=COST
      BL=0.
      IF (FLAG) GO TO 290
      DXC=XC(NR1,N+1)-XC(NR1,N)
      DYC=YC(NR1,N+1)-YC(NR1,N)
      DZC=ZC(NR1,N+1)-ZC(NR1,N)
      IF (VENTRL) DYC=-DYC
      DYL=DYC*COST+DZC*SINT
      BL=DXC/(BETA*DYL)
  290 AL=0.
      CL=1.0
      M=1
      DO 310 K=1,2
      N1=N+K-1
      IF (FLAG) N1=NI
      XJ=XI-XC(NR1,N1)
      DY=YI-YC(NR1,N1)
      IF (FLAG.AND.K.EQ.1) DY=YI
      IF (VENTRL) DY=-DY
      DZ=ZI-ZC(NR1,N1)
      YJ=DY*BCOS+DZ*BSIN
      ZJ=DZ*BCOS-DY*BSIN
      IF (K.EQ.2) GO TO 300
      CALL VORPAN (X,X,X,X,X,X,X,X,X,VEIR,WEIR,VAIR,WAIR,X,X)
      IF (MIDFIN) GO TO 310
      DY=-YI-YC(NR1,N1)
      IF (FLAG.AND.K.EQ.1) DY=-YI
      IF (VENTRL) DY=-DY
      YJ=DY*BCOS+DZ*BSIN
      ZJ=DZ*BCOS-DY*BSIN
      CALL VORPAN (X,X,X,X,X,X,X,X,X,VEIL,WEIL,VAIL,WAIL,X,X)
      GO TO 310
  300 CALL VORPAN (X,X,X,X,X,X,X,X,X,VEOR,WEOR,VAOR,WAOR,X,X)
      IF (MIDFIN) GO TO 310
      DY=-YI-YC(NR1,N1)
      IF (FLAG.AND.K.EQ.1) DY=-YI
      IF (VENTRL) DY=-DY
      YJ=DY*BCOS+DZ*BSIN
      ZJ=DZ*BCOS-DY*BSIN
      CALL VORPAN (X,X,X,X,X,X,X,X,X,VEOL,WEOL,VAOL,WAOL,X,X)
      SEIR=0.
      SEIL=0.
      SEOL=0.
      SEOR=0.
      SAIR=0.
      SAIL=0.
      SAOL=0.
      SAOR=0.
      TAIR=0.
      TAIL=0.
      TAOL=0.
      TAOR=0.
      TEIR=0.
      TEIL=0.
      TEOR=0.
      TEOL=0.
  310 END DO
  320 CONTINUE
!
!     COMBINE CORNER INFLUENCES TO OBTAIN PANEL VELOCITY COMPONENTS
!
      UAR=0.
      VAR=VAIR-VAOR-SAIR+SAOR
      WAR=WAIR-WAOR-TAIR+TAOR
      UAL=0.
      VAL=VAIL-VAOL-SAIL+SAOL
      WAL=WAIL-WAOL-TAIL+TAOL
      UIR=0.
      VIR=VEIR-SEIR
      WIR=WEIR-TEIR
      UIL=0.
      VIL=VEIL-SEIL
      WIL=WEIL-TEIL
      UOR=0.
      VOR=VEOR-SEOR
      WOR=WEOR-TEOR
      UOL=0.
      VOL=VEOL-SEOL
      WOL=WEOL-TEOL
      IF (L.EQ.NR1) GO TO 330
      ULR=ULIR-ULOR-RLIR+RLOR
      ULL=ULIL-ULOL-RLIL+RLOL
      VLR=VLIR-VLOR-SLIR+SLOR
      VLL=VLIL-VLOL-SLIL+SLOL
      WLR=WLIR-WLOR-TLIR+TLOR
      WLL=WLIL-WLOL-TLIL+TLOL
      USR=USIR-USOR-RSIR+RSOR
      USL=USIL-USOL-RSIL+RSOL
      VSR=VSIR-VSOR-SSIR+SSOR
      VSL=VSIL-VSOL-SSIL+SSOL
      WSR=UCIR-UCOR-RCIR+RCOR
      WSL=UCIL-UCOL-RCIL+RCOL
      UCR=WSR-ULR
      UCL=WSL-ULL
      VCR=VCIR-VCOR-SCIR+SCOR-VLR
      VCL=VCIL-VCOL-SCIL+SCOL-VLL
      WCR=WCIR-WCOR-TCIR+TCOR-WLR
      WCL=WCIL-WCOL-TCIL+TCOL-WLL
!
!     COMBINE CONTRIBUTIONS OF LEFT AND RIGHT WING PANELS AND TRANSFORM
!     VELOCITY COMPONENTS BACK TO ORIGINAL COORDINATE SYSTEM
!
      CALL TRANS (UCR,VCR,WCR,UCL,VCL,WCL,UC(J),VC(J),WC(J),AC(J))
      CALL TRANS (ULR,VLR,WLR,ULL,VLL,WLL,UL(L+1),VL(L+1),WL(L+1),AN(L+1&
     &))
      CALL TRANS (USR,VSR,WSR,USL,VSL,WSL,USJ,VSJ,WSJ,ASJ)
      US=US+USJ
      VS=VS+VSJ
      WS=WS+WSJ
      AS=AS+ASJ
      IF (L.EQ.1) GO TO 360
  330 CALL TRANS (UIR,VIR,WIR,UIL,VIL,WIL,UI,VI,WI,AI)
      CALL TRANS (UOR,VOR,WOR,UOL,VOL,WOL,UO,VO,WO,AO)
      CALL TRANS (UAR,VAR,WAR,UAL,VAL,WAL,UA,VA,WA,BA)
      IF (L.EQ.NR1) GO TO 340
      UC(J)=UC(J)+UL(L)
      VC(J)=VC(J)+VL(L)
      WC(J)=WC(J)+WL(L)
      AC(J)=AC(J)+AN(L)
  340 CONTINUE
!
!     ADD CONTRIBUTION OF THE WAKE
!
      DO 350 K=2,L
      K1=K-1
      UW=UI*CI(K1)-UO*CO(K1)+UA*A(K1)
      VW=VI*CI(K1)-VO*CO(K1)+VA*A(K1)
      WW=WI*CI(K1)-WO*CO(K1)+WA*A(K1)
      AW=AI*CI(K1)-AO*CO(K1)+BA*A(K1)
      JK=JL+K-2
      IF (NSIDE.EQ.2.AND.K.GT.2) JK=JK+NR
      UC(JK)=UC(JK)+UW
      VC(JK)=VC(JK)+VW
      WC(JK)=WC(JK)+WW
      AC(JK)=AC(JK)+AW
      IF (L.EQ.NR1.AND.K.EQ.L) GO TO 350
      JM=JK+1
      IF (NSIDE.EQ.2.AND.K.EQ.2) JM=JM+NR
      UC(JM)=UC(JM)+UW
      VC(JM)=VC(JM)+VW
      WC(JM)=WC(JM)+WW
      AC(JM)=AC(JM)+AW
  350 END DO
  360 CONTINUE
      IF (NSIDE.EQ.1) GO TO 380
      IF (L.NE.1) GO TO 370
!
!     SPECIAL CASE FOR LEADING EDGE PANELS
!
      UC(JL)=UC(JL)+UC(J)
      VC(JL)=VC(JL)+VC(J)
      WC(JL)=WC(JL)+WC(J)
      AC(JL)=AC(JL)+AC(J)
      J=J-1
  370 IF (L.NE.NR1) GO TO 380
      J=J-1
  380 IF (NWING.LE.NMAX) GO TO 390
      IF (NPART.EQ.2) GO TO 390
      IF (I.LT.J1.OR.I.GT.J2) GO TO 390
      JS1=J1
      JS2=J2
      NRS=NR2
  390 CONTINUE
  400 CONTINUE
      UC(JT)=US
      VC(JT)=VS
      WC(JT)=WS
      AC(JT)=AS
  410 END DO
      IF (N.GT.NI) GO TO 450
      IF (.NOT.FLAG) GO TO 430
      FLAG=.FALSE.
      J=JSAVE
      JJ=JJSAVE
      J2=J2SAVE
      DO 420 K=1,NR2
      JK=JSAVE+K
      USAVE(K)=UC(JK)
      VSAVE(K)=VC(JK)
      WSAVE(K)=WC(JK)
  420 ASAVE(K)=AC(JK)
      GO TO 90
  430 DO 440 K=1,NR2
      JK=JSAVE+K
      UC(K)=UC(JK)+USAVE(K)
      VC(K)=VC(JK)+VSAVE(K)
      WC(K)=WC(JK)+WSAVE(K)
  440 AC(K)=AC(JK)+ASAVE(K)
  450 END DO
      NWING=J
      NWTHK=NWING
      IF (NWING.LE.NMAX) GO TO 470
      IF (NPART.EQ.2) GO TO 470
!
!     STORE DIAGONAL BLOCKS OF AERODYNAMIC MATRIX IN DC ARRAY
!
      DO 460 J=1,NWING
      IF (J.LT.JS1.OR.J.GT.JS2) GO TO 460
      M=J-JS1+1
      DC(M)=AC(J)
      AC(J)=0.
  460 END DO
      WRITE (TAPE10) (DC(J),J=1,NRS)
  470 CONTINUE
      IF (IABS(PRINT).LT.4) GO TO 480
      WRITE (TAPE6,600) I
      WRITE (TAPE6,560) NWING
      WRITE (TAPE6,590) (UC(J),J=1,NWING)
      WRITE (TAPE6,540) NWING
      WRITE (TAPE6,590) (VC(J),J=1,NWING)
      WRITE (TAPE6,550) NWING
      WRITE (TAPE6,590) (WC(J),J=1,NWING)
      WRITE (TAPE6,570) NWING
      WRITE (TAPE6,590) (AC(J),J=1,NWING)
      IF (NWING.GT.NMAX) WRITE (TAPE6,580) NR
      IF (NWING.GT.NMAX) WRITE (TAPE6,590) (DC(J),J=1,NRS)
  480 WRITE (TAPE8) (UC(J),VC(J),WC(J),J=1,NWING)
      WRITE (TAPE9) (AC(J),J=1,NWING)
  490 END DO
      GO TO 520
  500 CONTINUE
      WRITE (TAPE6,530)
      WRITE (TAPE6,610)
      STOP   !!! CALL EXIT
  510 CONTINUE
      WRITE (TAPE6,530)
      WRITE (TAPE6,620)
      STOP   !!! CALL EXIT
  520 CONTINUE
!
!
!
  530 FORMAT (1H0,26H ERROR MESSAGE FROM WNGVEL)
  540 FORMAT (2X,10HVC(J),J=1,,I3)
  550 FORMAT (2X,10HWC(J),J=1,,I3)
  560 FORMAT (2X,10HUC(J),J=1,,I3)
  570 FORMAT (2X,10HAC(J),J=1,,I3)
  580 FORMAT (2X,10HDC(J),J=1,,I3)
  590 FORMAT (1X,10F10.5)
  600 FORMAT (/' AERODYNAMIC MATRIX, I=',I3)
  610 FORMAT (/' ERROR - WING PANEL SLOPE EXCEEDS MACH ANGLE')
  620 FORMAT (/'THE NONPLANAR BOUNDARY CONDITION OPTION IS NOT '/         &
     & ' AVAILABLE AT SUPERSONIC SPEEDS IN THIS VERSION OF THE PROGRAM'/  &
     & ' RERUN USING PLANAR BOUNDARY CONDITION OPTION')
      END
      SUBROUTINE VORPAN (UC,VC,WC,UL,VL,WL,ULT,VLT,WLT,VE,WE,VA,WA,UCS,V&
     &CS)
!
!     *******************************************VERSION B00************
!
!     COMPUTE THE THREE COMPONENTS OF VELOCITY INDUCED AT A SPECIFIED
!     CONTROL POINT BY CONSTANT AND LINEARLY VARYING VORTEX
!     DISTRIBUTIONS ON A SWEPT QUADRILATERAL PANEL. ALSO COMPUTE THE
!     THREE COMPONENTS OF VELOCITY INDUCED BY THE CONCENTRATED VORTEX
!     LYING ALONG THE DOWNSTREAM EXTENSION OF THE INBOARD EDGE AS WELL
!     AS THE VORTEX SHEET LOCATED DOWNSTREAM OF THE TRAILING EDGE
!     BETWEEN THE INBOARD EDGE AND THE INTERSECTION OF THE LEADING AND
!     TRAILING EDGES OF THE PANEL.
!
!     UC, VC, WC ARE VELOCITY COMPONENTS INDUCED BY CONSTANT CHORDWISE
!     AND SPANWISE VORTEX DISTRIBUTION
!
!     UL, VL, WL, ARE VELOCITY COMPONENTS INDUCED BY LEADING EDGE OF
!     LINEAR CHORDWISE, CONSTANT SPANWISE VORTEX DISTRIBUTION
!
!     ULT, VLT, WLT ARE VELOCITY COMPONENTS INDUCED BY TRAILING EDGE OF
!     LINEAR CHORDWISE, CONSTANT SPANWISE VORTEX DISTRIBUTION
!
!     UCS,VCS ARE VELOCITY COMPONENTS INDUCED BY CONSTANT CHORDWISE
!     AND SPANWISE SOURCE DISTRIBUTIONS
!
!
!     VE, WE ARE VELOCITY COMPONENTS INDUCED BY CONCENTRATED VORTICES
!     IN THE WAKE
!
!     VA, WA ARE VELOCITY COMPONENTS INDUCED BY VORTEX SHEET IN THE
!     WAKE
!
      COMMON /PARAM/ NBODY,NWING,NTAIL,LBC,THK,MACH,ALPHA,REFA
      COMMON /COMPV/ X,YL,Z,AL,BL,C,SUB,BPOS,ML,NS
      COMMON /PI/ PI,RADEG,DEGRAD
      COMMON /EPS/ EPSPAN,EPSWP1,EPSWP2,EPSTP,EPSVC,EPSSP1,EPSSP2,EPSLV,&
     &EPSSV,EPSVV,EPSWV,EPSVP,EPSIT1,EPSIT2,EPSFM, epssb1,epssb2
!
      DIMENSION Q(51), XI(51), QX(51)
!
      LOGICAL SUB,SUP,BPOS,SUPLE,LBC
!
      A=AL
      B=ABS(BL)
      BPOS=BL.GE.0.
      IF (BL.LT.0.) A=-AL
      Y=YL
      IF (BL.LT.0.) Y=-YL
      IF (ABS(C).LE.EPSVP) C=0.
      CC=C*C
      SUP=.NOT.SUB
      SUPLE=.FALSE.
      IF (ABS(B).LE.EPSVP) B=0.
      AB=A+B
      SGN=1.0
      IF (SUP) SGN=-1.0
      B1=SGN+B*B
      SB1=SQRT(ABS(B1))
      IF (ABS(Y).LE.EPSVP) Y=0.
      IF (ABS(Z).LE.EPSVP) Z=0.
      X2=X*X
      Y2=Y*Y
      Z2=Z*Z
      R2=Y2+Z2
      R=SQRT(R2)
      VA=0.
      VE=0.
      WA=0.
      WE=0.
      IF (SUB) GO TO 10
      IF (ABS(B).LT.1.0) SUPLE=.TRUE.
      IF (X.LT.0.) GO TO 440
   10 D=0.
      D2=X2+SGN*R2
      IF (D2.GT.0.) D=SQRT(D2)
      AZ=A*Z
      T1=C-A*Y
      IF (ABS(T1).LE.EPSVP) T1=0.
      T2=T1*T1
      T3=X-B*Y
      AT3=ABS(T3)
      IF (AT3.LE.EPSVP) AT3=0.
      T4=AZ*AZ
      T5=T2+T4
      IF (T5.NE.0.) T5=1./T5
      T6=B*C-A*X
      T7=T6*T6
      T8=T7+SGN*(T2+T4)
      T9=T1*T3+A*B*Z2
      E=SQRT(ABS(T8))
      B2=SGN*(C*Y-A*R2)
      B3=B*X+SGN*Y
      B4=T5*T6
      TZ=T3*T3+B1*Z2
      IF (TZ.GT.0.) ST3=SQRT(TZ)
      WQ=0.
!
!     EVALUATION OF DOWNWASH INDUCED BY TRAILING VORTEX SHEET
!
      IF (A.EQ.0..OR.ML.EQ.2) GO TO 160
      INT=0
      MAX=11
      XI(1)=0.
      XO=0.
      IF (T1.NE.0.) XO=T3/T1
      BO=B-A*XO
      SBO=SGN+BO*BO
      EL=1.0
      IF (SUP.AND.Y.GT.0..AND.ABS(BO).LE.1.) EL=T5*(T9-E*ABS(Z))
      IF (EL.LE.0.) GO TO 150
      IF (EL.GT.1.0) EL=1.0
      DXI=EL/FLOAT(MAX-1)
      EPX=.005*EL
      IF (Y.EQ.0..OR.ABS(Z).GT..10) GO TO 60
      IF(XO.LT.-EPX.OR.XO.GT.EL+EPX) GO TO 60
      INT=1
      MAX=13
      QM=0.
      IF (XO.LT..6*EL) MAX=8
      IF (XO.LT..1*EL) MAX=4
      IF (XO.LT.EPX) MAX=1
      IF (MAX.EQ.1) GO TO 20
      DXI=(XO-EPX)/FLOAT(MAX-3)
      GO TO 60
   20 INT=2
      IF (XO.GE.EL-EPX) MAX=1
      COEF=2.
      IF (MAX.EQ.1) COEF=1.
      WO=0.
      IF (SBO.EQ.0.) GO TO 30
      IF (MAX.GT.1.AND.QM.EQ.0.) GO TO 30
      ARG=EPX*EPX+SBO*Z2
      IF (ARG.LE.0.) GO TO 30
      ARG=SQRT(ARG)/(2.*SBO*ABS(Y))
      IF (Y.GT.0.) ARG=ARG+1./ARG
      IF (ARG.GT.0.) WO=ALOG(ARG)
      WO=COEF*BO*EPX*XO*WO/SQRT(ABS(SBO))
   30 WQ=WQ+WO
      IF (MAX.EQ.1.AND.XO.GT.EPX) GO TO 60
      IF (XO.LE..9*EL) GO TO 40
      MAX=4
      GO TO 50
   40 MAX=16-MAX
      IF (MAX.LT.8) MAX=8
   50 DXI=(EL-XO-EPX)/FLOAT(MAX-3)*.125
      XI(1)=XO+EPX
   60 CONTINUE
      IF (MAX.EQ.1) GO TO 150
      DO 140 M=1,MAX
      Q(M)=0.
      IF (INT.EQ.0) GO TO 80
      IF (INT.EQ.2) GO TO 70
      IF (M.EQ.MAX) GO TO 80
      IF (M.GT.MAX-3) DXI=.5*DXI
      GO TO 80
   70 IF (M.EQ.1.OR.M.EQ.3) GO TO 80
      IF (M.LE.5) DXI=2.*DXI
   80 CONTINUE
      IF (M.GT.1) XI(M)=XI(M-1)+DXI
      DX=X-XI(M)*C
      DX2=DX*DX
      BX=B-A*XI(M)
      BX2=BX*BX
      BX1=SGN+BX2
      SBX=SQRT(ABS(BX1))
      SDX=0.
      DXR=DX2+SGN*R2
      IF (DXR.GT.0.) SDX=SQRT(DXR)
      IF (SDX.EQ.0.) GO TO 90
      IF (SUP.AND.DX.LT.0.) GO TO 130
      ARG=SGN*Y+BX*DX
      IF (SBX.EQ.0.) GO TO 110
      TZI=(T3-XI(M)*T1)**2+BX1*Z2
      IF (TZI.EQ.0.) GO TO 120
      STZ=SQRT(TZI)
      IF (SUP.AND.ABS(BX).LT.1.) GO TO 100
      ARG=(ARG+SBX*SDX)/STZ
      IF (SUP) ARG=ABS(ARG)
      IF (ARG.GT.0.) Q(M)=ALOG(ARG)*BX/SBX
      GO TO 130
   90 IF (T1.LT.BX*T6.AND.T8.LT.0.) GO TO 130
      IF (ABS(BX).GE.1.) GO TO 130
      IF (Y.LE.BX*DX) GO TO 130
      IF (DX.LT.(BX*Y+SBX*ABS(Z))) GO TO 130
      Q(M)=PI*BX/SBX
      GO TO 130
  100 ARG=ARG/STZ
      IF (ARG.GT.1.0) GO TO 130
      IF (ARG.LE.-1.0) GO TO 90
      Q(M)=ACOS(ARG)*BX/SBX
      GO TO 130
  110 IF (ABS(ARG).GT.EPSVP) Q(M)=SDX*BX/ARG
      GO TO 130
  120 IF (Y.GE.0.) GO TO 140
      Q(M)=-ALOG(ABS(Y))*BX/SBX
  130 CONTINUE
      QX(M)=Q(M)*XI(M)
  140 END DO
      CALL TRAP (XI,QX,DW,MAX)
      WQ=WQ+DW
  150 CONTINUE
      IF (INT.EQ.1) GO TO 20
  160 CONTINUE
      IF (.NOT.SUPLE) GO TO 180
!
!     SPECIAL EQUATIONS FOR SUPERSONIC LEADING EDGE
!
      IF (D.GT.0.) GO TO 180
      IF (Y.LE.B*X) GO TO 440
      IF (X.LT.(B*Y+SB1*ABS(Z))) GO TO 440
      SZ=SIGN(1.0,Z)
      PZ=PI*SZ
      IF (NS.EQ.2.AND.Z.EQ.0.) PZ=-PZ
      UC=PZ
      VC=-B*PZ
      WC=-PI*SB1
      UCS=-PI/SB1
      VCS=-B*UCS
      IF (T8.GT.0.) E=0.
      SL=PI*T5*(SZ*T9-Z*E)
      TL=SZ*E*T5*SL
      IF (T8.GT.0.) TL=PI*T5*T5*T8*ABS(Z)
      IF (NS.EQ.2.AND.Z.EQ.0.) SL=-SL
      IF (ML.EQ.2) GO TO 170
      UL=SL
      VL=-((B+T1*B4)*SL-AZ*TL)*0.5
      WL=AZ*B4*SL-T1*TL+A*WQ
      IF (ML.EQ.1) GO TO 430
  170 ULS=SL+PZ
      ULT=ULS
      TT=SZ*E*T5*ULS
      IF (NS.EQ.2.AND.Z.EQ.0.) TT=-TT
      IF (T8.GT.0.) TT=TL
      VLT=(A*PZ-(AB+T1*B4)*ULS+AZ*TT)*0.5
      WLT=AZ*B4*ULS-T1*TT
      GO TO 430
  180 IF (SUP.AND.D.EQ.0.) GO TO 440
      IF (Z.EQ.0.) GO TO 280
!
!     GENERAL EQUATIONS
!
      DENOM=B*R2-X*Y
      F1=ATAN2(Z*D,DENOM)
      IF (SUB) F1=F1-ATAN2(Z,Y)
      G1=0.
      IF (T8.EQ.0.) GO TO 210
      IF (C.EQ.0.) GO TO 190
      ARG=X*T6+B2
      IF (T8.LT.0.) GO TO 200
      ARG=(ARG+D*E)/(ST3*C)
      IF (SUP) ARG=ABS(ARG)
      IF (ARG.GT.0.) G1=ALOG(ARG)
      GO TO 210
  190 IF (ST3.NE.0.) G1=ALOG(ST3)
      GO TO 210
  200 ARG=ARG/(ST3*C)
      IF (ABS(ARG).GT.1.0) GO TO 210
      G1=-ACOS(ARG)
  210 H1=0.
      GS=0.
      IF (C.EQ.0..AND.A.LT.0.) G1=-G1
      IF (SB1.EQ.0.) GO TO 230
      ARH=B3
      IF (SUPLE) GO TO 220
      ARH=(ARH+D*SB1)/ST3
      IF (ARH.GT.0.) H1=ALOG(ARH)
      GS=H1/SB1
      GO TO 240
  220 ARG=ARH/ST3
      IF (ABS(ARG).LE.1.0) H1=-ACOS(ARG)
      GS=-H1/SB1
      GO TO 240
  230 IF (B3.NE.0.) GS=D/B3
  240 G2=ALOG((X+D)/R)
      G3=0.
      IF (SUB) G3=ALOG(R)
      C1=D
      IF (SUB) C1=X+C1
      C2=C1/R2
      H=SB1*H1-B*(G2-G3)
      IF (SB1.EQ.0.) H2=B*D/B3-G2+G3
      IF (SB1.NE.0.) H2=B*H1/SB1-G2+G3
      UC=F1
      VS=-B*F1+Z*C2
      WS=H-Y*C2
      VC=VS
      WC=WS
      UCS=-GS
      VCS=-B*UCS-G2
      IF (C.EQ.0.) C2=0.
      C3=0.
      C4=0.
      C5=G2/2.
      IF (C.EQ.0.) GO TO 250
      C3=(X*C2+SGN*G2)/(2.*C)
      C4=((X2-SGN*R2*0.5)*G2-1.5*X*D)/(2.*CC)
      C5=(D-X*G2)/C
  250 CONTINUE
      C6=.50/R2
      IF (SUB) C6=C6*C1/D
      C2=C2+C6*C
      VB=-F1*0.5
      WB=H2*0.5
      VD=Z*C6
      WD=-Y*C6
      VA=VB
      WA=WB
      VE=VD
      WE=WD
      IF (ML.EQ.1) GO TO 260
      VC=VS+2.0*(C*VD+A*VB)
      WC=WS+2.0*(C*WD+A*WB)
  260 WQ=WQ-C4
      G=E*G1-T6*G2
      SL=T5*(T9*F1+Z*G)
      TL=-B*D
      IF (C.NE.0.) TL=(B2*G2+T6*D)/C
      TL=-T5*(T5*(G*T9-Z*T8*F1)+TL)
      IF (ML.EQ.2) GO TO 270
      UL=SL
      VL=-((B+T1*B4)*SL-AZ*TL)*0.5+Z*C3
      WL=AZ*B4*SL-T1*TL-Y*C3+A*WQ
      IF (ML.EQ.1) GO TO 430
  270 ULS=SL+F1
      ULT=ULS
      TLT=TL-T5*G
      WQT=C5-C4-G3*0.5
      VLS=(A*F1-(AB+T1*B4)*ULS+AZ*TLT)*0.5+Z*(C2+C3)
      WLS=AZ*B4*ULS-T1*TLT-Y*(C2+C3)+A*WQT
      VLT=VLS+A*VB
      WLT=WLS+A*WB
      GO TO 430
!
!     SPECIAL EQUATIONS FOR Z=0
!
  280 CONTINUE
      F1=0.
      DENOM=-Y*T3
      IF (DENOM.NE.0.) F1=ATAN2(0.,DENOM)
      IF (SUB.AND.Y.NE.0.) F1=F1-ATAN2(0.,Y)
      IF (NS.EQ.2) F1=-F1
      G1=0.
      IF (T8.EQ.0.) GO TO 320
      IF (C.EQ.0.) GO TO 300
      IF (T8.LT.0.) GO TO 310
      IF (AT3.GT.0.) GO TO 290
      IF (Y.EQ.0..OR.T1.EQ.0.) GO TO 320
      ARG=2.*Y*T1*B1/C
      IF (ARG.GT.0.) G1=ALOG(ARG)
      IF (Y.GT.0.) G1=100.+G1
      GO TO 320
  290 ARG=(X*T6+SGN*Y*T1+D*E)/(AT3*C)
      IF (SUP) ARG=ABS(ARG)
      IF (ARG.GT.0.) G1=ALOG(ARG)
      GO TO 320
  300 IF (Y.GT.0.) G1=-100.
      IF (AT3.NE.0.) G1=ALOG(AT3)
      GO TO 320
  310 ARG=(X*T6-Y*T1)/(AT3*C)
      IF (ABS(ARG).GT.1.0) GO TO 320
      G1=-ACOS(ARG)
  320 H1=0.
      GS=0.
      IF (C.EQ.0..AND.A.LT.0.) G1=-G1
      IF (SB1.EQ.0.) GO TO 350
      IF (SUPLE) GO TO 340
      IF (AT3.GT.0.) GO TO 330
      IF (Y.EQ.0.) GO TO 360
      H1=ALOG(ABS(Y))
      IF (SUB.AND.Y.LT.0.) H1=-H1
      IF (Y.GT.0.) H1=100.+H1
      GS=H1/SB1
      GO TO 360
  330 CONTINUE
      ARH=(B3+D*SB1)/AT3
      IF (SUP) ARH=ABS(ARH)
      IF (ARH.GT.0.) H1=ALOG(ARH)
      GS=H1/SB1
      GO TO 360
  340 ARG=B3/AT3
      IF (ABS(ARG).LE.1.0) H1=-ACOS(ARG)
      GS=-H1/SB1
      GO TO 360
  350 IF (B3.NE.0.) GS=D/B3
  360 G2=100.
      IF (Y.NE.0.) GO TO 370
      IF (X.NE.0.) G2=G2+ALOG(2.*ABS(X))
      IF (X.LT.0.) G2=-G2
      GO TO 380
  370 G2=0.
      ARG=(X+D)/ABS(Y)
      IF (ARG.GT.0.) G2=ALOG(ARG)
  380 G3=0.
      C1=D
      IF (.NOT.SUB) GO TO 390
      C1=X+D
      G3=-100.
      IF (Y.NE.0.) G3=ALOG(ABS(Y))
  390 C2=0.
      IF (Y.NE.0.) C2=C1/Y2
      H=SB1*H1-B*(G2-G3)
      IF (SB1.EQ.0.) H2=B*D/B3-G2+G3
      IF (SB1.NE.0.) H2=B*H1/SB1-G2+G3
      UC=F1
      VS=-B*F1
      WS=H-Y*C2
      VC=VS
      WC=WS
      UCS=-GS
      VCS=-B*UCS-G2
      IF (C.EQ.0.) C2=0.
      C4=0.
      C5=G2*0.5
      IF (C.EQ.0.) GO TO 400
      C3=(X*C2+SGN*G2)*0.5
      C4=((X2-SGN*Y2*0.5)*G2-1.5*X*D)/(2.*CC)
      C5=(D-X*G2)/C
  400 CONTINUE
      C6=0.
      IF (Y.NE.0.) C6=.50/Y2
      IF (SUB.AND.D.NE.0.) C6=C6*C1/D
      C2=C2+C6*C
      VB=-F1*0.5
      WB=H2*0.5
      WD=-Y*C6
      VA=VB
      WA=WB
      WE=WD
      IF (ML.EQ.1) GO TO 410
      VC=VS+2.0*(A*VB)
      WC=WS+2.0*(A*WB+C*WD)
  410 WQ=WQ-C4
      WQT=C5-C4-G3*0.5
      IF (T1.NE.0.) GO TO 420
      WL=A*WQ
      WLS=A*WQT
      WLT=WLS+A*WB
      GO TO 450
  420 SL=T3*F1/T1
      UL=SL
      VL=-(B+T6/T1)*SL*0.5
      G=E*G1-T6*G2
      TL=T3*T5*G
      IF (C.EQ.0.) TL=TL-B*D/T1
      IF (C.NE.0.) TL=TL+(T6*D/T1+Y*(SGN*G2-C3))/C
      WL=TL+A*WQ
      IF (ML.EQ.1) GO TO 430
      ULS=SL+F1
      VLS=(A*F1-(AB+T6/T1)*ULS)*0.5
      WLS=TL+G/T1-Y*C2+A*WQT
      ULT=ULS
      VLT=VLS
      WLT=WLS
      VLT=VLS+A*VB
      WLT=WLS+A*WB
  430 IF (BPOS) GO TO 460
      UC=-UC
      WC=-WC
      UL=-UL
      WL=-WL
      ULT=-ULT
      WLT=-WLT
      UCS=-UCS
      GO TO 460
  440 UC=0.
      VC=0.
      WC=0.
      UCS=0.
      VCS=0.
      WL=0.
      WLT=0.
  450 UL=0.
      VL=0.
      ULT=0.
      VLT=0.
      GO TO 430
  460 CONTINUE
      RETURN
      END
      SUBROUTINE TRANS (UR,VR,WR,UL,VL,WL,U,V,W,A)
!
!     *******************************************VERSION B00************
!
!     TRANSFORM THE THREE COMPONENTS OF VELOCITY FROM THE PANEL
!     COORDINATE SYSTEM TO THE REFERENCE COORDINATE SYSTEM. ALSO COMBINE
!     THE CONTRIBUTIONS OF THE LEFT AND RIGHT WING PANELS AND CALCULATE
!     THE NORMAL VELOCITY AT THE CONTROL POINT.
!
      COMMON /TRAN/ SIND,COSD,TAND,SINT,COST,CONTD,SINTI,COSTI,CON,BCON,&
     &SINDI,COSDI,MIDFIN,FIN,VENTRL
!
      LOGICAL MIDFIN,FIN,VENTRL
!
      IF (.NOT.MIDFIN) GO TO 10
      UL=0.
      VL=0.
      WL=0.
   10 CONTINUE
      VW=SINT*(VR+VL)+CONTD*(WR+WL)
      U=CON*(COST*(UR+UL)-SIND*VW)/CONTD
      V=BCON*COSD*(CONTD*(VR-VL)-SINT*(WR-WL))
      W=BCON*(TAND*(UR+UL)+COST*COSD*VW)/CONTD
      SGN=1.0
      IF (.NOT.FIN) GO TO 20
      VA=V
      V=W
      W=VA
      IF (.NOT.VENTRL) SGN=-1.0
   20 CONTINUE
      A=(W*COSTI-SGN*V*SINTI)*COSDI-U*SINDI
      RETURN
      END

!!!      OVERLAY(LWB,5,0)
      SUBROUTINE OLAY50()
      COMMON /PARAM/ NBODY,NWING,NTAIL,LBC,THK,MACH,ALPHA,REFA
      COMMON /POINT/ ARRAY(6000)
      COMMON /SCRAT/ BLOCK(7500)
      COMMON /ROWS/ NWCOL, NWRPC(20)
      COMMON /SEG/ NSEG,NROW(20),NCOL(20),COSS(20),SINS(20),BTE(20),NWT(&
     &20),SPNW(20),XLEW(20),BLE(20),ZLEW(20)
      DIMENSION AREA(600), XBAR(600)
      DIMENSION A(1202), B(1202), C(1202)
      DIMENSION CHORD(600), DELTA(600), DZTDX(600)
      EQUIVALENCE ( ARRAY(4801),AREA(1)), ( ARRAY(1), XBAR(1) )         &
     &, ( BLOCK(1), A(1)), ( BLOCK(1203),B(1)), ( BLOCK(2405),C(1))     &
     &, ( ARRAY(2401),DELTA(1)), ( ARRAY(3601), CHORD(1))               &
     &, ( BLOCK(6901), DZTDX(1))
      COMMON /TAPES/ ITAPES(20)
      COMMON /DES/ CLBAR, CMBAR
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAP&
!!!     &E9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      INCLUDE 'files.cmn'
      INTEGER COMPT,PRINT
!!!,TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE&
!!!     &8,TAPE9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
  INTEGER:: FFILE
!!!      COMMON /OPTION/ NOPT(10),NXSTNS,XSTN(20),XAPEX,YAPEX,XLAMDA
      INCLUDE 'option.cmn'
      COMMON /CYCLE/ NEWAIC, IPREP, NDRAG1, NDRAG2, IDIR
!
!     CHECK FOR NEW MACH NUMBER OR CONFIGURATION
!
      IF( IPREP .NE. 0 ) GO TO 50
      IF( NOPT(1).EQ.1 .AND. NOPT(2).GE.2 ) GO TO 1
      IF( NOPT(1).EQ.1 .AND. NOPT(3).EQ.0) GO TO 50
!
!     ASSIGN SCRATCH UNITS
!
    1 CONTINUE
      N1 = ITAPES(1)
      N2 = ITAPES(2)
      N3 = ITAPES(3)
      N4 = ITAPES(4)
      N5 = ITAPES(5)
      N6 = ITAPES(6)
      N7 = ITAPES(7)
      N8 = ITAPES(8)
      N9 = ITAPES(9)
      N10 = ITAPES(10)
      IN = ITAPES(11)
      REWIND IN
!
!     DUMMY READ STATEMENTS TO SKIP GEOMETRY FILES
!
      IF( NWING .EQ. 0 ) GO TO 5
      READ(IN)
      READ(IN)
    5 CONTINUE
      IPREP = IPREP + 1
!
!
!     SET STORAGE FOR SUBROUTINE TRIXY
!
      NMAX = 1202
      IF( NBODY .NE. 0 .AND. NWING .NE. 0 ) GO TO 20
      IF( NWING .NE. 0 ) CALL XFER( IN, N5, NWING, NWING, A )
      IF( NBODY .NE. 0 ) CALL XFER( IN, N5, NBODY, NBODY, A )
      GO TO 50
!
!     INVERT MATRIX (ABB)
   20 CONTINUE
!
      CALL INV(NBODY,IN,N6,N3,N4,N8,N9,N1,N2,NMAX,A,B,0)
!
!     (ABB) INVERSE IS ON N3
!
      CALL XFER( IN, N1, NBODY, NWING, A )
!
!     TRANSPOSE (ABW) TO OBTAIN COLUMN SORT MATRIX ON N8
!
      REWIND N8
      REWIND N9
      REWIND N1
      CALL TRPOSE(N1,A,B,NWING,NBODY,N8,N9,NMAX)
!
!     TRANSFER (AWB) TO UNIT N1
!
      CALL XFER( IN, N1, NWING, NBODY, A )
!
!     TRANSPOSE (AWB) TO OBTAIN COLUMN SORT MATRIX ON N2
!
      REWIND N2
      REWIND N9
      CALL TRPOSE(N1,A,B,NBODY,NWING,N2,N9,NMAX)
!
!     FORM MATRIX PRODUCT (AWB) * INV(ABB)
!     RESULT WILL BE IN COLUMN SORT ON UNIT N1
!
      REWIND N1
      CALL TRIXY(N2,N3,N1,NWING,NBODY,NBODY,A,B,C,NMAX,IRET)
!
!     SAVE AWB * INV(ABB) ON UNIT N10
!
      REWIND N1
      CALL XFER( N1, N10, NBODY, NWING, A )
!
!     FORM AWB * INV( ABB ) * ABW IN COLUMN FORM ON UNIT N2
!
      REWIND N1
      REWIND N8
      REWIND N2
      CALL TRIXY(N1,N8,N2,NWING,NBODY,NWING,A,B,C,NMAX,IRET)
!
!     TAKE TRANSPOSE ( ONE MORE TIME ) TO OBTAIN
!     MATRIX IN ROW SORT ON UNIT N7
!
      REWIND N7
      REWIND N9
      REWIND N2
      CALL TRPOSE(N2,A,B,NWING,NWING,N7,N9,NMAX)
!
!     READ IN (AWW)
!
      CALL XFER( IN, N9, NWING, NWING, A )
!
!     FORM REDUCED MATRIX (AR) = (AWW) - (AWB) * INV(ABB) * (ABW)
!
      REWIND N5
      DO 40 I = 1, NWING
      READ(N9) ( A(J), J = 1, NWING )
      READ(N7) ( B(J), J = 1, NWING )
      DO 30 J = 1, NWING
   30 C(J) = A(J) - B(J)
      WRITE(N5) ( C(J) , J = 1, NWING )
   40 END DO
!
!     TRANSFER THE FOLLOWING MATRICES IN COLUMN FORM TO UNIT N5
!
!     AWB * INV(ABB)
!     ABW
!     INV(ABB)
!
      REWIND N10
      REWIND N8
      REWIND N3
      DO 60 I = 1, NBODY
      READ(N10) ( A(J), J = 1, NWING )
   60 WRITE(N5) ( A(J), J = 1, NWING )
      DO 70 I = 1, NWING
      READ(N8) ( A(J), J = 1, NBODY )
   70 WRITE(N5) ( A(J), J = 1, NBODY )
      DO 80 I = 1, NBODY
      READ(N3) ( A(J), J = 1, NBODY )
   80 WRITE(N5) ( A(J), J = 1, NBODY )
   50 CONTINUE
      END



!!!     OVERLAY(LWB,3,0)
      SUBROUTINE SOLVE()
!
!     *******************************************VERSION B00************
!
!
!     SOLVE FOR THE STRENGTHS OF THE BODY SOURCES AND WING VORTICES
!     WHICH SATISFY THE BOUNDARY CONDITION OF TANGENTIAL FLOW AT THE
!     PANEL CONTROL POINTS. ALSO DETERMINE THE CORRESPONDING PRESSURE
!     DISTRIBUTION AND THE FORCES AND MOMENTS ON THE CONFIGURATION.
!
!
!     THE PROGRAM MUST SOLVE A SYSTEM OF LINEAR EQUATIONS OF MAXIMUM
!     ORDER 1200. THE SOLUTION TECHNIQUE SELECTED CAN BE DESCRIBED AS A
!     BLOCKED JACOBI ITERATIVE METHOD. THE 1200 BY 1200 MATRIX IS
!     NATURALLY PARTITIONED INTO FOUR 600 BY 600 BLOCKS. EACH PARTITION
!     IS FURTHER SUBDIVIDED INTO BLOCKS OF MAXIMUM SIZE 60 BY 60. THE
!     MATRIX ELEMENTS IN EACH BLOCK ARE CAREFULLY CHOSEN TO REPRESENT
!     SOME WELL DEFINED FEATURE OF THE ORIGINAL CONFIGURATION. FOR
!     EXAMPLE, A BODY BLOCK REPRESENTS THE INFLUENCE OF ONE RING OF
!     PANELS AROUND THE BODY, WHILE A WING BLOCK REPRESENTS THE
!     INFLUENCE OF ONE CHORDWISE COLUMN OF WING PANELS. FOR WINGS USING
!     THE NON-PLANAR BOUNDARY CONDITION OPTION, THE BLOCK SIZE
!     CORRESPONDS TO THE TOTAL NUMBER OF PANELS ON THE UPPER AND LOWER
!     SURFACES OF THE COLUMN.
!
!     THE INITIAL ITERATION CALCULATES THE SINGULARITY STRENGTHS
!     CORRESPONDING TO EACH BLOCK IN ISOLATION. FOR THIS STEP, ONLY THE
!     DIAGONAL BLOCKS ARE PRESENT IN THE AERODYNAMIC MATRIX. ONCE THE
!     INITIAL APPROXIMATION TO THE SINGULARITY STRENGTHS IS DETERMINED,
!     THE INTERFERENCE EFFECT OF EACH BLOCK ON ALL THE OTHERS IS
!     CALCULATED BY MATRIX MULTIPLICATION. THE INCREMENTAL NORMAL
!     VELOCITIES OBTAINED ARE SUBTRACTED FROM THE NORMAL VELOCITIES
!     SPECIFIED BY THE BOUNDARY CONDITIONS. THIS PROCESS IS ITERATED
!     UNTIL A CONVERGENCE TEST ON THE MAXIMUM RESIDUAL IS SATISFIED OR A
!     MAXIMUM NUMBER OF ITERATIONS IS REACHED. THE CONVERGENCE CRITERION
!     AND THE ITERATION LIMIT ARE SET IN SUBROUTINE ITRATE.
!
!
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAP&
!!!     &E9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15,TAPE16,TAPE17
      INCLUDE 'files.cmn'
      COMMON /LWB/ LWB
      COMMON /PARAM/ NBODY,NWING,NTAIL,LBC,THK,MACH,ALPHA,REFA
      COMMON /POINT/ ARRAY(6000)
      COMMON /SCRAT/ BLOCK(7500)
      COMMON /SEG/ NSEG,NR(20),NC(20),COSS(20),SINS(20),BTE(20),NWT(20),&
     &             SPNW(20),XLEW(20),BLE(20),ZLEW(20)
      COMMON /NEWCOM/ KDUM(46),LOCPT(20),XCPT(20)
      COMMON /VELCOM/ NPOINT,NPART,IMAX,JMAX,NMAX,EX,PRINT,NWTHK
      COMMON /FORM/ CN,CT,CM,CNB,CTB,CMB,CNS(20),CTS(20),CMS(20)
      COMMON /MATCOM/ MATIN
      COMMON /PRESS/ KRADX(4),NRADX(4),KFORX(4),NFORX(4),KWAF,NWAF,KWAFO&
     &R,NWAFOR,NFUS
      COMMON /PI/ PI,RADEG,DEGRAD
!
      DIMENSION UA(600), VA(600), WA(600), CP(600), NS(600), CHORD(600)
      DIMENSION THET(600), DELTA(600), NB(600), NW(600), NT(600), U(600), V(600)
      DIMENSION W(600),A(60,60),GW(600),GB(600),DZTDX(600),X(600),Y(600),Z(600)
      DIMENSION CHD(20),DEL(600),COSTH(600)
!
      EQUIVALENCE (BLOCK(1),U(1))
      EQUIVALENCE (BLOCK(601),V(1))
      EQUIVALENCE (BLOCK(1201),W(1))
      EQUIVALENCE (BLOCK(1801),A(1,1))
      EQUIVALENCE (BLOCK(5701),GW(1))
      EQUIVALENCE (BLOCK(6301),GB(1))
      EQUIVALENCE (BLOCK(6901),DZTDX(1))
      EQUIVALENCE (BLOCK(1801),UA(1))
      EQUIVALENCE (BLOCK(2401),VA(1))
      EQUIVALENCE (BLOCK(3001),WA(1))
      EQUIVALENCE (BLOCK(3601),CP(1))
      EQUIVALENCE (BLOCK(4201),NS(1))
      EQUIVALENCE (BLOCK(1),NW(1))
      EQUIVALENCE (BLOCK(601),NB(1))
      EQUIVALENCE (BLOCK(1201),NT(1))
      EQUIVALENCE (ARRAY(1801),THET(1))
      EQUIVALENCE (ARRAY(2401),DELTA(1))
      EQUIVALENCE (ARRAY(3601),CHORD(1))
      EQUIVALENCE (ARRAY(1),X(1))
      EQUIVALENCE (ARRAY(601),Y(1))
      EQUIVALENCE (ARRAY(1201),Z(1))
      EQUIVALENCE (BLOCK(5701),DEL(1))
      EQUIVALENCE (BLOCK(6301),COSTH(1))
!
      COMMON /STAR/XSTAR(3600),NSEGW
      DIMENSION ZCSTR(600),ZTSTR(600),CPSTR(600),                       &
     &          DCNSTR(600),DCTSTR(600),DCMSTR(600)
      EQUIVALENCE (XSTAR(1)   , ZCSTR(1)),(XSTAR(601) , ZTSTR(1)),      &
     &            (XSTAR(1201), CPSTR(1)),(XSTAR(1801),DCNSTR(1)),      &
     &            (XSTAR(2401),DCTSTR(1)),(XSTAR(3001),DCMSTR(1))
!*
!!!      COMMON /OPTION/ NOPT(10),NXSTNS,XSTN(20),XAPEX,YAPEX,XLAMDA
      INCLUDE 'option.cmn'
      COMMON /TAPES/ ITAPES(20)
      COMMON /SAVZLE/ ZLE(600)
      COMMON /DESIGN/ CAMBER(600)
      COMMON /NEWCAM/ CAMMID(600)
      COMMON /POLAR/NCASE,CDBP(10),CLBP(10),CMBP(10),CDWP(10),CLWP(10), &
     &CMWP(10),ALPHAP(10),MACHP(10),NSPAN,XSPAN(20)
      COMMON /SHELL/ NPTSB,NSEGWT,NPTSWT,NSEGN,NPTSN,NSEGS,NPTSS,       &
     &               NSEGNS,NPTSNS,NNAC,NSHELL,NQ,IPART, iidum
      DIMENSION UF(30),VF(30),WF(30)
      EQUIVALENCE (BLOCK(4801),UF(1)), (BLOCK(4831),VF(1)),             &
     &            (BLOCK(4861),WF(1))
!
      REAL MACH,NB,NW,NT,NS
      INTEGER COMPT,PRINT
!!!,TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE&
!!!     &8,TAPE9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15,TAPE16,TAPE17,  &
  INTEGER:: FFILE
      LOGICAL LBC,THK
!
      NCASE=NCASE+1
      EM=MACH
      REWIND TAPE7
      REWIND TAPE8
      REWIND TAPE12
      REWIND TAPE13
      REWIND TAPE15
      REWIND TAPE16
      REWIND TAPE17
      ALP=ALPHA*DEGRAD
      SINAL=SIN(ALP)
      COSAL=COS(ALP)
!
!**********************************************************
!
!          CALCULATE BOUNDARY CONDITIONS ON
!          BODY, WING, TAIL, NACELLE, AND SHELL PANELS
!
!**********************************************************
!
      N15=ITAPES(15)
      N16=ITAPES(16)
      N17=ITAPES(17)
      N18=ITAPES(18)
      N19=ITAPES(19)
      REWIND TAPE7
      REWIND TAPE11
      DO 50 IP=1,3
      IPART=IP
      IF(IPART.NE.3) CALL WINGS(IPART)
      NPTS=NWING
      IF(IPART.EQ.3) NPTS=NBODY
      IF(NPTS.EQ.0) GO TO 50
      IF(IPART.NE.3) READ(TAPE7) ARRAY,CHORD,DZTDX
      IF(IPART.EQ.3) READ(TAPE7) ARRAY
      IF(LBC.AND.IPART.NE.3) READ(TAPE11) DEL,COSTH
      IF(IPART.NE.1) GO TO 30
      IF(NOPT(1).EQ.2) GO TO 30
      IF(NOPT(6).NE.1) GO TO 10
      READ(TAPE5,425) (CAMBER(J),J=1,NPTS)
      WRITE(TAPE6,426)
      WRITE(TAPE6,427)
  425 FORMAT(7F10.3)
  426 FORMAT(1H1,/,2X,25H***   ANALYSIS CASE   ***)
  427 FORMAT(1H ,2X,43HCAMBER SLOPES INPUT AT PANEL CONTROL POINTS,//)
      CALL OUTW(TAPE6,CAMBER)
      CALL SHAPE
   10 IF(.NOT.LBC) GO TO 30
      IF(NOPT(2).NE.1.AND.NOPT(6).NE.1) GO TO 30
      DO 20 I=1,NPTS
   20 DEL(I)=CAMBER(I)
   30 DO 40 I=1,NPTS
      IF(LBC.AND.IPART.NE.3)                                            &
     & NS(I)=COSAL*DEL(I)-SINAL*COSTH(I)
      IF(.NOT.LBC.OR.IPART.EQ.3)                                        &
     & NS(I)=COSAL*SIN(DELTA(I))-SINAL*COS(THET(I))*COS(DELTA(I))
      IF(IPART.EQ.2.AND.I.GT.NPTSN)                                     &
     & NS(I)=0.0
      IF(IPART.EQ.1) NW(I)=NS(I)
      IF(IPART.EQ.2) NT(I)=NS(I)
   40 IF(IPART.EQ.3) NB(I)=NS(I)
   50 END DO
!
!          MODIFY BOUNDARY CONDITIONS
!          ON BODY, WING, TAIL, NACELLE, AND SHELL
!          TO ACCOUNT FOR THE EFFECTS OF
!          WING, TAIL, AND NACELLE THICKNESS
!
      REWIND TAPE16
      DO 125 IP=1,3
      IPART=IP-1
      IF(IPART.NE.0) CALL WINGS(IPART)
      LLBC=0
      IF(LBC) LLBC=1
      NPTS=NWING
      IF(IPART.EQ.0) NPTS=NBODY
      IF(NPTS.EQ.0) GO TO 125
      REWIND TAPE7
      IF(NPTSWT.NE.0) READ(TAPE7) ARRAY,CHORD,DZTDX
      IF(IPART.EQ.1) GO TO 60
      IF(NPTSNS.NE.0) READ(TAPE7) ARRAY,CHORD,DZTDX
      IF(IPART.EQ.2) GO TO 60
      IF(NBODY.NE.0) READ(TAPE7) ARRAY
   60 REWIND TAPE15
      WRITE(TAPE15) THET,DELTA
      IF(NBODY.EQ.0) GO TO 80
      DO 70 I=1,NPTS
   70 READ(TAPE8)
   80 REWIND TAPE7
      DO 120 JP=1,2
      JPART=JP
      CALL WINGS(JPART)
      IF(NWING.EQ.0) GO TO 120
      READ(TAPE7) ARRAY,CHORD,DZTDX
      REWIND TAPE15
      READ(TAPE15) THET,DELTA
      DO 110 I=1,NPTS
      IF(THK) READ(TAPE8) (UA(J),VA(J),WA(J),J=1,NWTHK)
      IF(THK.AND.JPART.EQ.IPART) READ(TAPE8)
      IF(IPART.EQ.0.AND.NPTSNS.NE.0) GO TO 85
      READ(TAPE8)
      IF(JPART.EQ.IPART) READ(TAPE8)
   85 IF(.NOT.THK) GO TO 110
      US=0.0
      VS=0.0
      WS=0.0
      JC2=0
      JT=0
      DO 100 N=1,NSEG
      NCOL=NC(N)
      NROW=NR(N)
      NR1=NROW+1
      IF(LOCPT(N).NE.0) NROW=NR1
      DO 100 NN=1,NCOL
      JC1=JC2+1
      JC2=JC1+NROW-1
      IF(I.GE.JC1.AND.I.LE.JC2.AND.JPART.EQ.IPART)                      &
     & JT=JT+NR1
      IF(I.GE.JC1.AND.I.LE.JC2.AND.JPART.EQ.IPART)                      &
     & GO TO 100
      DO 90 NNN=1,NR1
      JT=JT+1
      US=US+UA(JT)*DZTDX(JT)
      VS=VS+VA(JT)*DZTDX(JT)
   90 WS=WS+WA(JT)*DZTDX(JT)
  100 CONTINUE
      NS(I)=WS*COS(THET(I))-VS*SIN(THET(I))
      IF(LLBC.EQ.0.OR.IPART.EQ.0)                                       &
     & NS(I)=NS(I)*COS(DELTA(I))-US*SIN(DELTA(I))
      IF(IPART.EQ.0) NB(I)=NB(I)-NS(I)
      IF(IPART.EQ.1) NW(I)=NW(I)-NS(I)
      IF(IPART.EQ.2) NT(I)=NT(I)-NS(I)
  110 END DO
      IF(THK.AND.IPART.EQ.1) WRITE(TAPE16) NS
  120 END DO
  125 END DO
      REWIND TAPE15
      IF(NBODY.NE.0) WRITE(TAPE15) NB
      IF(NPTSWT.NE.0) WRITE(TAPE15) NW
      IF(NPTSNS.NE.0) WRITE(TAPE15) NT
!
!**********************************************************
!
!          SOLVE GOVERNING EQUATIONS
!
!**********************************************************
!
      NPTSB=NBODY
      REWIND N15
!
!          SOLUTION FOR WING/BODY
!
      IF(NPTSNS.NE.0) GO TO 130
      CALL WINGS(1)
      IPART=1
      WRITE(TAPE6,620)
  620 FORMAT(1H1,/,1X,18HWING/BODY SOLUTION,/)
      GO TO 190
!
!          SOLUTION FOR ISOLATED BODY
!
  130 IF(NPTSB.EQ.0) GO TO 140
      CALL WINGS(1)
      NWING=0
      IPART=0
      WRITE(TAPE6,630)
  630 FORMAT(1H1,/,1X,22HISOLATED BODY SOLUTION,/)
      GO TO 210
!
!          SOLUTION FOR WING/SHELL
!          IN PRESENCE OF BODY
!
  140 REWIND TAPE15
      IF(NPTSB.NE.0) READ(TAPE15) NB
      IF(NPTSWT.NE.0) READ(TAPE15) NW
      IF(NPTSNS.NE.0) READ(TAPE15) NB
      IF(NPTSB.EQ.0) GO TO 180
      REWIND N18
      DO 150 I=1,NPTSB
  150 READ(N18) (NT(J),J=1,NPTSB)
      DO 170 IP=1,2
      IPART=IP
      CALL WINGS(IPART)
      IF(NWING.EQ.0) GO TO 170
      DO 160 I=1,NWING
      READ(N18) (NT(J),J=1,NPTSB)
      SUM=0.0
      NS(I)=SOSCAP(NT,GB,SUM,NPTSB,1,1,1,1)
      IF(IPART.EQ.1) NW(I)=NW(I)-NS(I)
      IF(IPART.EQ.2) NB(I)=NB(I)-NS(I)
  160 END DO
      IF(IPART.EQ.1) WRITE(TAPE16) NS
  170 END DO
  180 CALL WINGS(1)
      CALL WINGS(3)
      IPART=-1
      WRITE(TAPE6,640)
  640 FORMAT(1H1,/,1X,39HWING/SHELL SOLUTION IN PRESENCE OF BODY,/)
!
!          BRANCH TO APPROPRIATE SOLUTION METHOD
!
  190 IF(NOPT(1).EQ.2) GO TO 200
      IF(NOPT(2).LE.1) GO TO 210
!
!          INVERSE ANALYSIS, FULL OR MIXED
!
      CALL MIXED
      CALL SHAPE
      GO TO 260
!
!          DRAG OPTIMIZATION, FULL OR CONSTRAINED
!
  200 CALL DRAG
      CALL SHAPE
      GO TO 260
!
!          ANALYSIS
!
  210 IF(IPART.NE.0) GO TO 220
      IDUM7=TAPE7
      IDUM9=TAPE9
      IDUM10=TAPE10
      IDUM24=ITAPES(5)
      TAPE7=N17
      TAPE9=N18
      TAPE10=N19
      ITAPES(5)=ITAPES(20)
  220 REWIND TAPE7
      REWIND TAPE9
      REWIND TAPE10
      IF(NOPT(3).EQ.1) GO TO 245
      IF(IPART.EQ.0) GO TO 230
      IF(NPTSWT.NE.0) READ(TAPE7) ARRAY,CHORD,DZTDX
      IF(NPTSNS.NE.0) READ(TAPE7) ARRAY,CHORD,DZTDX
      IF(NPTSB.NE.0) READ(TAPE7) ARRAY
  230 IF(NBODY.LE.NMAX.AND.NWING.LE.NMAX) GO TO 240
      CALL DIAGIN
      REWIND TAPE10
      GO TO 245
  240 CALL PARTIN
      IF(NBODY.EQ.0.OR.NWING.EQ.0) GO TO 250
  245 IF(NOPT(3).EQ.0) CALL ITRATE
      IF(NOPT(3).EQ.1) CALL DIRECT
  250 IF(IPART.NE.0) GO TO 260
      TAPE7=IDUM7
      TAPE9=IDUM9
      TAPE10=IDUM10
      ITAPES(5)=IDUM24
      WRITE(N15) GB
      GO TO 140
  260 IF(IPART.EQ.1.AND.NPTSB.NE.0) WRITE(N15) GB
      IF(NPTSWT.NE.0) WRITE(N15) GW
      IF(IPART.EQ.-1) WRITE(N15) GB
      REWIND TAPE15
      REWIND TAPE16
      REWIND TAPE17
      IF(NPTSWT.EQ.0.OR.NOPT(4).EQ.0) GO TO 270
      REWIND TAPE7
      READ(TAPE7) ARRAY,CHORD,DZTDX
      IF(MACH.GE.1.0) CALL INTPOL(.95,.75)
      IF(MACH.LT.1.0) CALL INTPOL(.85,.75)
  270 CONTINUE
!
!**********************************************************
!
!          CALCULATE VELOCITITES ON BODY
!
!**********************************************************
!
      REWIND TAPE8
      IF(NPTSB.EQ.0) GO TO 360
      COMPT=1
      NPASS=1
      REWIND N15
      READ(N15) GB
!
!          BODY ON BODY
!
      DO 280 I=1,NPTSB
      U(I)=0.0
      V(I)=0.0
  280 W(I)=0.0
      DO 290 I=1,NPTSB
      READ(TAPE8) (UA(J),VA(J),WA(J),J=1,NPTSB)
      DO 290 J=1,NPTSB
      U(I)=U(I)+UA(J)*GB(J)
      V(I)=V(I)+VA(J)*GB(J)
  290 W(I)=W(I)+WA(J)*GB(J)
!
!          WING, TAIL, NACELLE AND SHELL ON BODY
!
      REWIND TAPE7
      DO 335 IP=1,2
      IPART=IP
      CALL WINGS(IPART)
      IF(NWING.EQ.0) GO TO 335
      READ(N15) GW
      READ(TAPE7) ARRAY,CHORD,DZTDX
      DO 330 I=1,NPTSB
      IF(.NOT.THK) GO TO 310
      READ(TAPE8) (UA(J),VA(J),WA(J),J=1,NWTHK)
      DO 300 J=1,NWTHK
      U(I)=U(I)+UA(J)*DZTDX(J)
      V(I)=V(I)+VA(J)*DZTDX(J)
  300 W(I)=W(I)+WA(J)*DZTDX(J)
  310 IF(NPTSNS.NE.0) GO TO 330
      READ(TAPE8) (UA(J),VA(J),WA(J),J=1,NWING)
      DO 320 J=1,NWING
      U(I)=U(I)+UA(J)*GW(J)
      V(I)=V(I)+VA(J)*GW(J)
  320 W(I)=W(I)+WA(J)*GW(J)
  330 END DO
  335 END DO
!
      READ(TAPE7) ARRAY
      IF(IABS(PRINT).LT.2) GO TO 350
      WRITE(TAPE6,800) EM,ALPHA
      WRITE(TAPE6,810)
      DO 340 N=1,NBODY
      NS(N)=(W(N)*COS(THET(N))-V(N)*SIN(THET(N)))*COS(DELTA(N))         &
     &      -U(N)*SIN(DELTA(N))
  340 WRITE(TAPE6,820) N,GB(N),U(N),V(N),W(N),NS(N)
  350 CONTINUE
!
!          CALCULATE PRESSURES ON BODY
!
      CALL PressureCoeff(NPTSB,EM,ALP,U,V,W,CP,CPSTAG,CPCRIT,CPVAC,     &
     & COMPT,VFIX,SGN)
!
!          CALCULATE FORCES AND MOMENT ON BODY
!
      CALL FORMOM(NPTSB,NPASS,ALP,COMPT)
!
!**********************************************************
!
!          CALCULATE VELOCITIES ON WING/TAIL AND NACELLE/SHELL
!
!**********************************************************
!
  360 IF(NPTSWT.EQ.0.AND.NPTSNS.EQ.0) GO TO 610
      DO 600 IP=1,2
      IPART=IP
      COMPT=IP+1
      NPASS=1
  370 REWIND TAPE7
      REWIND N15
      REWIND N16
      CALL WINGS(IPART)
      IF(NWING.EQ.0) GO TO 600
      DO 380 I=1,NWING
      U(I)=0.0
      V(I)=0.0
  380 W(I)=0.0
!
!          BODY ON WING, TAIL, NACELLE, AND SHELL
!
      IF(NPTSB.EQ.0) GO TO 400
      READ(N15) GB
      DO 395 I=1,NWING
      IF(NPASS.EQ.2) GO TO 385
      READ(TAPE8) (UA(J),VA(J),WA(J),J=1,NPTSB)
      WRITE(N16) (UA(J),VA(J),WA(J),J=1,NPTSB)
      GO TO 386
  385 READ(N16) (UA(J),VA(J),WA(J),J=1,NPTSB)
  386 IF(IPART.EQ.2.AND.NPASS.EQ.2.AND.I.GT.NPTSN) GO TO 395
      DO 390 J=1,NPTSB
      U(I)=U(I)+UA(J)*GB(J)
      V(I)=V(I)+VA(J)*GB(J)
  390 W(I)=W(I)+WA(J)*GB(J)
  395 END DO
!
!          WING, TAIL, NACELLE, AND SHELL
!                       ON
!          WING, TAIL, NACELLE, AND SHELL
!
  400 SGN=1.0
      IF(LBC.AND.NPASS.EQ.2) SGN=-1.0
      DO 500 JP=1,2
      JPART=JP
      CALL WINGS(JPART)
      THICK=0.0
      IF(THK) THICK=1.0
      NSING=NWING
      NTING=NWTHK
      CALL WINGS(IPART)
      IF(NSING.EQ.0) GO TO 500
      READ(TAPE7) ARRAY,CHORD,DZTDX
      READ(N15) GW
      JT2=0
      JC2=0
      I=0
      DO 495 N=1,NSEG
      NCOL=NC(N)
      NROW=NR(N)
      NR1=NROW+1
      IF(LBC.AND.LOCPT(N).NE.0) NROW=NROW+1
      IF(.NOT.LBC) NROW=2*NROW
      DO 495 NN=1,NCOL
      JT1=JT2+1
      JT2=JT1+NR1-1
      JC1=JC2+1
      JC2=JC1+NROW-1
      DO 495 NNN=1,NROW
      I=I+1
!
!          SOURCE PANELS
!
      IF(THICK.EQ.0.0) GO TO 450
      IF(NPASS.EQ.2) GO TO 410
      READ(TAPE8) (UA(J),VA(J),WA(J),J=1,NTING)
      WRITE(N16) (UA(J),VA(J),WA(J),J=1,NTING)
      IF(JPART.NE.IPART) GO TO 420
      READ(TAPE8) (UF(J),VF(J),WF(J),J=1,NR1)
      WRITE(N16) (UF(J),VF(J),WF(J),J=1,NR1)
      GO TO 420
  410 READ(N16) (UA(J),VA(J),WA(J),J=1,NTING)
      IF(JPART.NE.IPART) GO TO 420
      READ(N16) (UF(J),VF(J),WF(J),J=1,NR1)
  420 IF(IPART.EQ.2.AND.NPASS.EQ.2.AND.I.GT.NPTSN) GO TO 450
      DO 440 J=1,NTING
      IF(JPART.EQ.IPART.AND.J.GE.JT1.AND.J.LE.JT2) GO TO 430
      U(I)=U(I)+UA(J)*DZTDX(J)
      V(I)=V(I)+VA(J)*DZTDX(J)
      W(I)=W(I)+WA(J)*DZTDX(J)
      GO TO 440
  430 JF=J-JT1+1
      U(I)=U(I)+(UA(J)-UF(JF)*(1.0-SGN))*DZTDX(J)
      V(I)=V(I)+(VA(J)-VF(JF)*(1.0-SGN))*DZTDX(J)
      W(I)=W(I)+(WA(J)-WF(JF)*(1.0-SGN))*DZTDX(J)
  440 END DO
!
!          VORTEX PANELS
!
  450 CONTINUE
      IF(NPASS.EQ.2) GO TO 460
      READ(TAPE8) (UA(J),VA(J),WA(J),J=1,NSING)
      WRITE(N16) (UA(J),VA(J),WA(J),J=1,NSING)
      IF(JPART.NE.IPART) GO TO 470
      READ(TAPE8) (UF(J),VF(J),WF(J),J=1,NROW)
      WRITE(N16) (UF(J),VF(J),WF(J),J=1,NROW)
      GO TO 470
  460 READ(N16) (UA(J),VA(J),WA(J),J=1,NSING)
      IF(JPART.NE.IPART) GO TO 470
      READ(N16) (UF(J),VF(J),WF(J),J=1,NROW)
  470 IF(IPART.EQ.2.AND.NPASS.EQ.2.AND.I.GT.NPTSN) GO TO 495
      DO 490 J=1,NSING
      IF(JPART.EQ.IPART.AND.J.GE.JC1.AND.J.LE.JC2) GO TO 480
      U(I)=U(I)+UA(J)*GW(J)
      V(I)=V(I)+VA(J)*GW(J)
      W(I)=W(I)+WA(J)*GW(J)
      GO TO 490
  480 JF=J-JC1+1
      U(I)=U(I)+(UA(J)-UF(JF)*(1.0-SGN))*GW(J)
      V(I)=V(I)+(VA(J)-VF(JF)*(1.0-SGN))*GW(J)
      W(I)=W(I)+(WA(J)-WF(JF)*(1.0-SGN))*GW(J)
  490 END DO
  495 CONTINUE
  500 END DO
      REWIND N15
      IF(NPTSB.NE.0) READ(N15) GB
      IF(NPTSWT.NE.0) READ(N15) GW
      IF(IPART.EQ.2) READ(N15) GW
!
      IF(IPART.EQ.2) GO TO 560
      REWIND 31
      DO 505 N=1,NWING
  505 WRITE(31,830) U(N),V(N),W(N)
      IF(IABS(PRINT).LT.2) GO TO 520
      IF(LBC) GO TO 520
      IF(IPART.EQ.1) WRITE(TAPE6,840) EM,ALPHA
      IF(IPART.EQ.2) WRITE(TAPE6,845) EM,ALPHA
      WRITE(TAPE6,850)
      DO 510 N=1,NWING
  510 WRITE(TAPE6,820) N,GW(N),U(N),V(N),W(N)
  520 CONTINUE
!
!          CALCULATE VFIX FOR THE CARLSON CORRECTION
!
      REWIND TAPE11
      READ(TAPE11) DEL,COSTH
      TANLES=TAN(XLAMDA)
      IF(TANLES.EQ.0) GO TO 530
      IF(MACH.LE.1.0) GO TO 530
      XNUINF=SQRT(6.0)*ATAN(SQRT((MACH**2-1.0)/6.0))-ACOS(1.0/MACH)
      BETA=SQRT(MACH**2-1.0)
      BCOTLM=BETA/TANLES
      ETAVFX=1.226+0.15*PI**(1.0-SQRT(BCOTLM))
      EEK=1.0+(PI/2.0-1.0)*BCOTLM**ETAVFX
      VFIX=-SGN*ALP/EEK
      GO TO 540
  530 VFIX=0.0
  540 DO 550 NCAM=1,NWING
  550 ZCSTR(NCAM)=CAMBER(NCAM)
  560 CONTINUE
!
!          CALCULATE PRESSURES ON WING, TAIL, NACELLE, AND SHELL
!
      CALL PressureCoeff(NWING,EM,ALP,U,V,W,CP,CPSTAG,CPCRIT,CPVAC,     &
     & COMPT,VFIX,SGN)
!
      REWIND TAPE7
      IF(NPTSWT.NE.0) READ(TAPE7) ARRAY,CHORD,DZTDX
      IF(IPART.EQ.2) READ(TAPE7) ARRAY,CHORD,DZTDX
      REWIND N15
      IF(NPTSB.NE.0) READ(N15) GB
      IF(NPTSWT.NE.0) READ(N15) GW
      IF(IPART.EQ.2) READ(N15) GW
      IF(IABS(PRINT).LT.1) GO TO 580
      IF(.NOT.LBC) GO TO 580
      IF(IPART.EQ.1.AND.NPASS.EQ.1) WRITE(TAPE6,860) EM,ALPHA,VFIX
      IF(IPART.EQ.1.AND.NPASS.EQ.2) WRITE(TAPE6,870) EM,ALPHA,VFIX
      IF(IPART.EQ.2.AND.NPASS.EQ.1) WRITE(TAPE6,880) EM,ALPHA
      IF(IPART.EQ.2.AND.NPASS.EQ.2) WRITE(TAPE6,890) EM,ALPHA
      WRITE(TAPE6,900)
      NX=0
      NCP=0
      DO 570 N=1,NSEG
      NCOL=NC(N)
      NROW=NR(N)
      NR1=NROW+1
      DO 570 NN=1,NCOL
      DO 570 NNN=1,NR1
      NX=NX+1
      IF(NNN.EQ.NR1.AND.LOCPT(N).EQ.0) GO TO 570
      NCP=NCP+1
      IF(IPART.EQ.1) WRITE(TAPE6,910)                                   &
     & NCP,X(NX),Y(NX),Z(NX),GW(NCP),U(NCP),V(NCP),W(NCP),ZCSTR(NCP),   &
     & ZTSTR(NCP),CP(NCP),CPSTR(NCP),NCP
      IF(IPART.EQ.2) WRITE(TAPE6,920)                                   &
     & NCP,X(NX),Y(NX),Z(NX),GW(NCP),U(NCP),V(NCP),W(NCP),CP(NCP),NCP
  570 CONTINUE
  580 CONTINUE
!
!          CALCULATE FORCES AND MOMENT
!          ON WING, TAIL, NACELLE, AND SHELL
!
      CALL FORMOM(NWING,NPASS,ALP,COMPT)
      IF(LBC.AND.NPASS.EQ.1) GO TO 590
      GO TO 600
  590 NPASS=NPASS+1
      GO TO 370
  600 END DO
!
!**********************************************************
!
!          CALCULATIONS COMPLETED
!
!**********************************************************
!
  610 WRITE(TAPE6,930) CPSTAG,CPCRIT,CPVAC
      CALL SECOND(TIME)
      WRITE(TAPE6,940) TIME
!
!          FORMATS
!
  800 FORMAT(1H1,25HVELOCITIES ON BODY, MACH=,F5.3,3X,6HALPHA=,F7.3//)
  810 FORMAT(1X,5HPANEL,10X,6HSOURCE,10X,5HAXIAL,10X,7HLATERAL,10X,     &
     & 8HVERTICAL,10X,6HNORMAL/2X,3HNO.,10X,8HSTRENGTH,7X,8HVELOCITY,   &
     & 9X,8HVELOCITY,9X,8HVELOCITY,9X,8HVELOCITY//)
  820 FORMAT(1H ,I4,7X,F10.5,5X,F10.5,3(7X,F10.5))
  830 FORMAT(3E14.7)
  840 FORMAT(1H1,25HVELOCITIES ON WING, MACH=,F5.3,3X,6HALPHA=,F7.3//)
  845 FORMAT(1H1,26HVELOCITIES ON SHELL, MACH=,F5.3,3X,6HALPHA=,F7.3//)
  850 FORMAT(1X,5HPANEL,10X,6HVORTEX,10X,5HAXIAL,10X,7HLATERAL,10X,     &
     & 8HVERTICAL/2X,3HNO.,10X,8HSTRENGTH,7X,8HVELOCITY,9X,8HVELOCITY,  &
     & 9X,8HVELOCITY//)
  860 FORMAT(1H1,40HWING UPPER SURFACE CONTROL POINTS, MACH=,F5.3,      &
     & 3X,6HALPHA=,F7.3,3X,5HVFIX=,F12.5,//)
  870 FORMAT(1H1,40HWING LOWER SURFACE CONTROL POINTS, MACH=,F5.3,      &
     & 3X,6HALPHA=,F7.3,3X,5HVFIX=,F12.5,//)
  880 FORMAT(1H1,43HNACELLE/SHELL UPPER SURFACE CONTROL POINTS,,        &
     & 3X,5HMACH=,F6.3,3X,6HALPHA=,F7.3,//)
  890 FORMAT(1H1,43HNACELLE/SHELL LOWER SURFACE CONTROL POINTS,,        &
     & 3X,5HMACH=,F6.3,3X,6HALPHA=,F7.3,//)
  900 FORMAT(3X,5HPOINT,4X,1HX,9X,1HY,9X,1HZ,9X,1HG,9X,1HU,9X,1HV,      &
     & 9X,1HW,7X,5HDZCDX,5X,5HDZTDX,6X,2HCP,7X,5HCPSTR,3X,5HPOINT,/)
  910 FORMAT(1X,I6,11F10.5,I6)
  920 FORMAT(1X,I6,7F10.5,20X,F10.5,10X,I6)
  930 FORMAT(/'CPSTAG =',F10.5,3X,'CPCRIT =',F10.5,3X,'CPVAC =',F10.5)
  940 FORMAT(/'SOLVE, TIME =',F10.5)
      END

      SUBROUTINE INVERT (A,IA,NROWS)
!
!     ********************************************VERSION B00***********
!
!
!     SIMPLE MATRIX INVERSION ROUTINE BASED ON GAUSS-JORDAN ELIMINATION
!     WITHOUT PIVOTING
!
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAP&
!!!     &E9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      INCLUDE 'files.cmn'
!
      DIMENSION IPIVOT(115), INDXR(115), INDXC(115), A(NROWS,NROWS)
!
!!!      INTEGER TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAPE9,TAPE&
!!!     &10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
!
      N=IA
      DO 10 J=1,N
   10 IPIVOT(J)=0
      DO 100 I=1,N
      T=0.0
      DO 30 J=1,N
      IF (IPIVOT(J).EQ.1) GO TO 30
      DO 20 K=1,N
      IF (IPIVOT(K).EQ.1) GO TO 20
      IF (.NOT.(ABS(A(J,K))-ABS(T).GT.0.0)) GO TO 20
      IROW=J
      ICOL=K
      T=A(J,K)
   20 END DO
   30 END DO
      IPIVOT(ICOL)=IPIVOT(ICOL)+1
      IF (IROW.EQ.ICOL) GO TO 50
      DO 40 L=1,N
      T=A(IROW,L)
      A(IROW,L)=A(ICOL,L)
   40 A(ICOL,L)=T
   50 INDXR(I)=IROW
      INDXC(I)=ICOL
      PIVOT=A(ICOL,ICOL)
      IF (PIVOT) 60,130,60
   60 A(ICOL,ICOL)=1.0
      DO 70 L=1,N
   70 A(ICOL,L)=A(ICOL,L)/PIVOT
      DO 90 L=1,N
      IF (L.EQ.ICOL) GO TO 90
      T=A(L,ICOL)
      A(L,ICOL)=0.0
      DO 80 M=1,N
   80 A(L,M)=A(L,M)-A(ICOL,M)*T
   90 END DO
  100 END DO
      DO 120 I=1,N
      L=N+1-I
      IF (INDXR(L).EQ.INDXC(L)) GO TO 120
      IROW=INDXR(L)
      ICOL=INDXC(L)
      DO 110 K=1,N
      T=A(K,IROW)
      A(K,IROW)=A(K,ICOL)
  110 A(K,ICOL)=T
  120 END DO
!
!     SUCCESSFUL SOLUTION
!
      RETURN
  130 CONTINUE
!
!     SINGULAR MATRIX
!
      WRITE (TAPE6,140)
      WRITE (TAPE6,150)
      STOP   !!! CALL EXIT
!!!      RETURN
!
!
  140 FORMAT (1H0,26H ERROR MESSAGE FROM INVERT)
  150 FORMAT (1H0,30HERROR - THE MATRIX IS SINGULAR)
      END
      SUBROUTINE PARTIN
!
!     ********************************************VERSION B00***********
!
!
!     FOR WING-BODY COMBINATIONS, INVERT THE MATRIX PATITIONS (PROVIDED
!     THE ORDER DOES NOT EXCEED 60).
!
!     FOR ISOLATED WINGS OR BODIES, ALSO SOLVE THE BOUNDARY CONDITION
!     EQUATIONS AND DETERMINE THE WING VORTEX STRENGTHS OR BODY SOURCE
!     STRENGTHS.
!
!
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAP&
!!!     &E9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      INCLUDE 'files.cmn'
      COMMON /PARAM/ NBODY,NWING,NTAIL,LBC,THK,MACH,ALPHA,REFA
      COMMON /SEG/ NSEG,NR(20)
      COMMON /VELCOM/ NPOINT,NPART,IMAX,JMAX,NMAX,EX,PRINT,NWTHK
      COMMON /POINT/ ARRAY(6000)
      COMMON /SCRAT/ BLOCK(7500)
      COMMON /MATCOM/ MATIN
      COMMON /SHELL/ NPTSB,NSEGWT,NPTSWT,NSEGN,NPTSN,NSEGS,NPTSS,       &
     &               NSEGNS,NPTSNS,NNAC,NSHELL,NQ,IPART, iidum
!
      DIMENSION D(60,60), NW(600), NB(600), NT(600), A(60,60), GW(600)
      DIMENSION GB(600), GT(600)
!
      EQUIVALENCE (ARRAY(1),D(1,1))
      EQUIVALENCE (BLOCK(1),NW(1))
      EQUIVALENCE (BLOCK(601),NB(1))
      EQUIVALENCE (BLOCK(1201),NT(1))
      EQUIVALENCE (BLOCK(1801),A(1,1))
      EQUIVALENCE (BLOCK(5701),GW(1))
      EQUIVALENCE (BLOCK(6301),GB(1))
      EQUIVALENCE (BLOCK(6901),GT(1))
!
      REAL NW,NB,NT
!!!      INTEGER TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAPE9,TAPE&
!!!     &10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
!
      CALL SECOND (TIME)
      WRITE (TAPE6,110) TIME
!
!     SET MATRIX BLOCK SIZE AT 60 BY 60
!
      NDIM=60
!
      REWIND TAPE9
      REWIND TAPE10
      NPANEL=NBODY+NWING
      IF (NWING.EQ.0.OR.NBODY.EQ.0) GO TO 50
      IF(MATIN.EQ.0) GO TO 100
      DO 10 I=1,NBODY
   10 READ (TAPE9) (D(I,J),J=1,NBODY)
      CALL INVERT (D,NBODY,NDIM)
      WRITE (TAPE10) D
      DO 20 I = 1,NPANEL
   20 READ(TAPE9)
      DO 40 I=1,NWING
   40 READ (TAPE9) (D(I,J),J=1,NWING)
      CALL INVERT (D,NWING,NDIM)
      WRITE (TAPE10) D
      REWIND TAPE9
      REWIND TAPE10
      MATIN=0
      GO TO 100
   50 CONTINUE
      IF(MATIN.EQ.1) GO TO 55
      READ(TAPE10) A
      GO TO 65
   55 CONTINUE
      DO 60 I=1,NPANEL
   60 READ (TAPE9) (A(I,J),J=1,NPANEL)
      REWIND TAPE9
      CALL INVERT (A,NPANEL,NDIM)
      WRITE(TAPE10) A
      IF(IPART.NE.0) MATIN=0
   65 CONTINUE
      IF (NWING.EQ.0) GO TO 80
      DO 70 I=1,NWING
      GW(I)=0.
      DO 70 J=1,NWING
      GW(I)=GW(I)+A(I,J)*NW(J)
   70 CONTINUE
      GO TO 100
   80 DO 90 I=1,NBODY
      GB(I)=0.
      DO 90 J=1,NBODY
      GB(I)=GB(I)+A(I,J)*NB(J)
   90 CONTINUE
  100 CALL SECOND (TIME)
      WRITE (TAPE6,110) TIME
      REWIND TAPE9
      RETURN
!
!
  110 FORMAT (1H0,14HPARTIN, TIME =,F10.5)
      END
      SUBROUTINE DIAGIN
!
!     ********************************************VERSION B00***********
!
!
!     INVERT THE DIAGONAL BLOCKS OF THE MATRIX.
!
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAP&
!!!     &E9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      INCLUDE 'files.cmn'
      COMMON /PARAM/ NBODY,NWING,NTAIL,LBC,THK,MACH,ALPHA,REFA
      COMMON /SEG/ NSEG,NR(20)
      COMMON /VELCOM/ NPOINT,NPART,IMAX,JMAX,NMAX,EM,PRINT,NWTHK,NWBLOK,&
     &NWROW(20),NBBLOK,NBROW(60)
      COMMON /POINT/ ARRAY(6000)
      COMMON /MATCOM/ MATIN
      COMMON /SHELL/ NPTSB,NSEGWT,NPTSWT,NSEGN,NPTSN,NSEGS,NPTSS,       &
     &               NSEGNS,NPTSNS,NNAC,NSHELL,NQ,IPART, iidum
!
      DIMENSION D(60,60)
!
      EQUIVALENCE (ARRAY(1),D(1,1))
!
!!!      INTEGER TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAPE9,TAPE&
!!!     &10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
!
      REWIND TAPE9
      REWIND TAPE10
!
!     SET MATRIX BLOCK SIZE AT 60 BY 60
!
      NDIM=60
!
      IF(MATIN.EQ.0) GO TO 140
      IF (NBODY.EQ.0) GO TO 50
      DO 40 NB=1,NBBLOK
      NROW=NBROW(NB)
      NCOL=NROW
      IF (NBODY.GT.NMAX) GO TO 20
      DO 10 I=1,NBODY
   10 READ (TAPE9) (D(I,J),J=1,NBODY)
      GO TO 30
   20 READ (TAPE7) D
   30 CALL INVERT (D,NCOL,NDIM)
      WRITE (TAPE10) D
   40 END DO
   50 IF (NWING.EQ.0) GO TO 140
      DO 130 NW=1,NWBLOK
      NROW=NWROW(NW)
      NCOL=NROW
      IF (NWING.GT.NMAX) GO TO 110
      IF (NBODY.EQ.0) GO TO 90
!                        SKIP
      NPO = NBODY + NBODY + NWING
      DO 60 I = 1,NPO
   60 READ(TAPE9)
   90 DO 100 I=1,NWING
  100 READ (TAPE9) (D(I,J),J=1,NWING)
      GO TO 120
  110 READ (TAPE7) D
  120 CALL INVERT (D,NCOL,NDIM)
      WRITE (TAPE10) D
  130 END DO
  140 CONTINUE
      REWIND TAPE7
      REWIND TAPE9
      REWIND TAPE10
      IF(IPART.NE.0) MATIN=0
      RETURN
      END
      SUBROUTINE ITRATE
!
!     *******************************************VERSION B00************
!
!     SOLVE THE BOUNDARY CONDITION EQUATIONS BY AN ITERATIVE METHOD AND
!     DETERMINE THE STRENGTHS OF THE BODY SOURCES AND THE WING, FIN
!     (VERTICAL TAIL), AND CANARD (HORIZONTAL TAIL) VORTICES.
!
!     A NEW VERSION OF ITRATE HAS BEEN DEVELOPED AT PURDUE UNIVERSITY
!     THAT PERMITS THE SELECTION OF THE ITERATIVE SOLUTION METHOD.
!
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAP&
!!!     &E9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      INCLUDE 'files.cmn'
      COMMON /PARAM/ NBODY,NWING,NTAIL,LBC,THK,MACH,ALPHA,REFA
      COMMON /SEG/ NSEG,NR(20)
      COMMON /VELCOM/ NPOINT,NPART,IMAX,JMAX,NMAX,EM,PRINT,NWTHK,NWBLOK,&
     &NWROW(20),NBBLOK,NBROW(60)
      COMMON /POINT/ ARRAY(6000)
      COMMON /SCRAT/ BLOCK(7500)
      COMMON /EPS/ EPSPAN,EPSWP1,EPSWP2,EPSTP,EPSVC,EPSSP1,EPSSP2,EPSLV,&
     &EPSSV,EPSVV,EPSWV,EPSVP,EPSIT1,EPSIT2,EPSFM, epssb1,epssb2
      COMMON /ITERAT/ ITMAX,CCTEST,DCTEST,ITMETH,ALF1,ALF2
!
      DIMENSION D(60,60), DNB(600), DNW(600), NW(600), NB(600), NT(600),&
     & A(600), RW(600), RB(600), WW(600), BB(600), GW(600), GB(600), GT(&
     &600)
!
      EQUIVALENCE (ARRAY(1),D(1,1))
      EQUIVALENCE (ARRAY(3601),DNB(1))
      EQUIVALENCE (ARRAY(4201),DNW(1))
      EQUIVALENCE (BLOCK(1),NW(1))
      EQUIVALENCE (BLOCK(601),NB(1))
      EQUIVALENCE (BLOCK(1201),NT(1))
      EQUIVALENCE (BLOCK(1801),A(1))
      EQUIVALENCE (BLOCK(2401),RW(1))
      EQUIVALENCE (BLOCK(3001),RB(1))
      EQUIVALENCE (BLOCK(4501),WW(1))
      EQUIVALENCE (BLOCK(5101),BB(1))
      EQUIVALENCE (BLOCK(5701),GW(1))
      EQUIVALENCE (BLOCK(6301),GB(1))
      EQUIVALENCE (BLOCK(6901),GT(1))
!
      REAL NB,NW,NT
      INTEGER PRINT
!!!,TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAPE&
!!!     &9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15,
  INTEGER:: TAPEI

!
!     SET MAXIMUM NUMBER OF ITERATIONS - IMAX
!
      IMAX=50
      IF (ITMAX.NE.0) IMAX=ITMAX
!
!
!     SET CONVERGENCE CRITERIA - EPSIT1
!
!
      IF (CCTEST.NE.0.) EPSIT1=CCTEST
!
!
!     SET DIVERGENCE CRITERIA - EPSIT2
!
      IF (DCTEST.NE.0.) EPSIT2=DCTEST
!
!
!     SET ITERATIVE SOLUTION METHOD - IMETH
!
      IMETH=1
      IF (ITMETH.NE.0.) IMETH=ITMETH-1
      IF (IMETH.GT.3) IMETH=1
!
!     IN TERMS OF IMETH, THE SOLUTION PROCEDURES ARE
!            IMETH=0     BLOCKED JACOBI
!                  1     BLOCKED GAUSS-SIEDEL
!                  2     BLOCKED CONTROLLED SUCCESSIVE OVERRELAXATION
!                  3     BLOCKED SUCCESSIVE OVERRELAXATION
!
      IF (IMETH.EQ.0) WRITE (TAPE6,600)
      IF (IMETH.EQ.0) GO TO 60
      IF (IMETH.EQ.1) ALF1=1.0
      IF (IMETH.NE.2) GO TO 10
      ALF1=0.9
      ALF2=1.1
   10 ALF=ALF1
      IF (IMETH.EQ.1) WRITE (TAPE6,610) ALF
      IF (IMETH.EQ.2) WRITE (TAPE6,620) ALF1,ALF2
      IF (IMETH.EQ.3) WRITE (TAPE6,630) ALF1,ALF2
      TAPEI=TAPE9
      IF (NWING.EQ.0.OR.NBODY.EQ.0) GO TO 60
      TAPEI=TAPE14
      REWIND TAPE9
      DO 20 I=1,NBODY
   20 READ(TAPE9)
      DO 30 I=1,NBODY
      READ (TAPE9) (A(K),K=1,NWING)
   30 WRITE(TAPE14) (A(K),K=1,NWING)
      IF (NWING.LE.NMAX) GO TO 60
      DO 40 I=1,NWING
   40 READ(TAPE9)
      DO 50 I=1,NWING
      READ (TAPE9) (A(K),K=1,NWING)
   50 WRITE (TAPE14) (A(K),K=1,NWING)
   60 CONTINUE
      IT=0
      REWIND TAPE9
      REWIND TAPE10
      IF (NBODY.EQ.0) GO TO 80
      DO 70 N=1,NBODY
      NT(N)=0.
   70 RB(N)=NB(N)
   80 IF (NWING.EQ.0) GO TO 100
      DO 90 N=1,NWING
      GT(N)=0.
   90 RW(N)=NW(N)
  100 IT=IT+1
      ITEST=0
      JTEST=0
      CALL SECOND (TIME)
      WRITE (TAPE6,710) TIME
      IB=0
      IW=0
      IF (NBODY.EQ.0) GO TO 150
      JS=0
      NBLOK=NBBLOK
      DO 120 NN=1,NBLOK
      NROW=NBROW(NN)
      NCOL=NROW
      READ (TAPE10) D
      DO 110 I=1,NROW
      IB=IB+1
      GB(IB)=0.
      DO 110 J=1,NCOL
      JJ=J+JS
  110 GB(IB)=GB(IB)+D(I,J)*RB(JJ)
      JS=JS+NROW
  120 END DO
      DO 130 N=1,NBODY
      IF (ITEST.EQ.1.OR.IT.EQ.1) GO TO 130
      IF (ABS(GB(N)-NT(N)).GE.EPSIT1) ITEST=1
  130 END DO
      DO 140 N=1,NBODY
      IF (JTEST.EQ.1.OR.IT.EQ.1) GO TO 140
      IF (ABS(GB(N)).GE.EPSIT2) JTEST=1
  140 END DO
  150 IF (NWING.EQ.0) GO TO 200
      JS=0
      NBLOK=NWBLOK
      DO 170 NN=1,NBLOK
      NROW=NWROW(NN)
      NCOL=NROW
      READ (TAPE10) D
      DO 160 I=1,NROW
      IW=IW+1
      GW(IW)=0.
      DO 160 J=1,NCOL
      JJ=J+JS
  160 GW(IW)=GW(IW)+D(I,J)*RW(JJ)
      JS=JS+NROW
  170 END DO
      DO 180 N=1,NWING
      IF (ITEST.EQ.1.OR.IT.EQ.1) GO TO 180
      IF (ABS(GW(N)-GT(N)).GE.EPSIT1) ITEST=1
  180 END DO
      DO 190 N=1,NWING
      IF (JTEST.EQ.1.OR.IT.EQ.1) GO TO 190
      IF (ABS(GW(N)).GE.EPSIT2) JTEST=1
  190 END DO
  200 CONTINUE
      REWIND TAPE10
  210 IF (IABS(PRINT).LT.3) GO TO 220
      WRITE (TAPE6,720) IT
      IF (NBODY.GT.0) WRITE (TAPE6,730) NBODY,(GB(N),N=1,NBODY)
      IF (NWING.GT.0) WRITE (TAPE6,740) NWING,(GW(N),N=1,NWING)
  220 CONTINUE
      IF (IABS(PRINT).LT.4) GO TO 230
      IF (NBODY.GT.0) WRITE (TAPE6,650) NBODY,(RB(N),N=1,NBODY)
      IF (NWING.GT.0) WRITE (TAPE6,660) NWING,(RW(N),N=1,NWING)
  230 CONTINUE
      IF (JTEST.EQ.1.AND.IT.NE.1) GO TO 550
      IF (ITEST.EQ.0.AND.IT.NE.1) GO TO 560
      IF (IT.EQ.IMAX) GO TO 570
      IF (IMETH.NE.0) GO TO 350
      IF (NBODY.EQ.0) GO TO 290
      DO 250 I=1,NBODY
      NT(I)=GB(I)
      DNB(I)=0.
      READ (TAPE9) (A(J),J=1,NBODY)
      IF (NBODY.LE.NMAX) GO TO 250
      DO 240 J=1,NBODY
  240 DNB(I)=DNB(I)+A(J)*GB(J)
  250 RB(I)=NB(I)-DNB(I)
      IF (NWING.EQ.0) GO TO 280
      DO 270 I=1,NBODY
      READ (TAPE9) (A(J),J=1,NWING)
      DO 260 J=1,NWING
  260 DNB(I)=DNB(I)+A(J)*GW(J)
  270 RB(I)=NB(I)-DNB(I)
  280 CONTINUE
  290 IF (NWING.EQ.0) GO TO 340
      DO 310 I=1,NWING
      GT(I)=GW(I)
      DNW(I)=0.
      IF (NBODY.EQ.0) GO TO 310
      READ (TAPE9) (A(J),J=1,NBODY)
      DO 300 J=1,NBODY
  300 DNW(I)=DNW(I)+A(J)*GB(J)
  310 RW(I)=NW(I)-DNW(I)
      IF (NWING.LE.NMAX) GO TO 340
      DO 330 I=1,NWING
      READ (TAPE9) (A(J),J=1,NWING)
      DO 320 J=1,NWING
  320 DNW(I)=DNW(I)+A(J)*GW(J)
  330 RW(I)=NW(I)-DNW(I)
  340 CONTINUE
      REWIND TAPE9
      IF (IT.LT.IMAX) GO TO 100
      IF (IMETH.EQ.0) GO TO 560
  350 JJ=0
      CALL SECOND (TIME)
      WRITE (TAPE6,710) TIME
      IT=IT+1
      II=1
      ITEST=0
      JTEST=0
      REWIND TAPE9
      REWIND TAPE10
      REWIND TAPE14
      IF (NBODY.EQ.0) GO TO 450
      DO 430 I=1,NBBLOK
      NROW=NBROW(I)
      READ (TAPE10) D
      DO 390 J=1,NROW
      DNB(J)=0.
      READ (TAPE9) (A(K),K=1,NBODY)
      IF (NBODY.LE.NMAX) GO TO 370
      DO 360 K=1,NBODY
  360 DNB(J)=DNB(J)+A(K)*GB(K)
  370 CONTINUE
      IF (NWING.EQ.0) GO TO 390
      READ (TAPEI) (A(K),K=1,NWING)
      DO 380 K=1,NWING
  380 DNB(J)=DNB(J)+A(K)*GW(K)
  390 RB(J)=NB(J+JJ)-DNB(J)
      DO 420 M=1,NROW
      NT(II)=GB(II)
      GB(II)=0.
      DO 400 L=1,NROW
  400 GB(II)=GB(II)+D(M,L)*RB(L)
      SAVE=BB(II)
      BB(II)=(GB(II)-NT(II))
      IF (IMETH.NE.2) GO TO 410
      SAVE=BB(II)*SAVE
      ALF=ALF1
      IF (SAVE.GE.0.) ALF=ALF2
  410 CONTINUE
      GB(II)=ALF*GB(II)+(1.-ALF)*NT(II)
      IF (ABS(BB(II)).GT.EPSIT1) ITEST=1
      IF (ABS(BB(II)).GT.EPSIT2) JTEST=1
      II=II+1
  420 END DO
      JJ=JJ+NBROW(I)
  430 END DO
      IF (NWING.EQ.0) GO TO 540
      DO 440 I=1,NBODY
  440 READ (TAPE9) (A(K),K=1,NWING)
  450 CONTINUE
      JJ=0
      II=1
      DO 530 I=1,NWBLOK
      NROW=NWROW(I)
      READ (TAPE10) D
      DO 490 J=1,NROW
      DNW(J)=0.
      IF (NBODY.EQ.0) GO TO 470
      READ (TAPE9) (A(K),K=1,NBODY)
      DO 460 K=1,NBODY
  460 DNW(J)=DNW(J)+A(K)*GB(K)
  470 CONTINUE
      IF (NWING.LE.NMAX) GO TO 490
      READ (TAPEI) (A(K),K=1,NWING)
      DO 480 K=1,NWING
  480 DNW(J)=DNW(J)+A(K)*GW(K)
  490 RW(J)=NW(J+JJ)-DNW(J)
      DO 520 M=1,NROW
      GT(II)=GW(II)
      GW(II)=0.
      DO 500 L=1,NROW
  500 GW(II)=GW(II)+D(M,L)*RW(L)
      SAVE=WW(II)
      WW(II)=GW(II)-GT(II)
      IF (IMETH.NE.2) GO TO 510
      SAVE=WW(II)*SAVE
      ALF=ALF1
      IF (SAVE.GE.0.) ALF=ALF2
  510 CONTINUE
      GW(II)=ALF*GW(II)+(1.-ALF)*GT(II)
      IF (ABS(WW(II)).GT.EPSIT1) ITEST=1
      IF (ABS(WW(II)).GT.EPSIT2) JTEST=1
      II=II+1
  520 END DO
      JJ=JJ+NWROW(I)
  530 END DO
  540 CONTINUE
      GO TO 210
  550 CONTINUE
      WRITE (TAPE6,590)
      WRITE (TAPE6,640) IT,EPSIT1,EPSIT2
      GO TO 580
  560 CONTINUE
      WRITE (TAPE6,670) IT,EPSIT1
      GO TO 580
  570 CONTINUE
      WRITE (TAPE6,590)
      WRITE (TAPE6,680) IMAX,EPSIT1
  580 CONTINUE
      WRITE (TAPE6,690)
      IF (NBODY.GT.0) WRITE (TAPE6,730) NBODY,(NT(N),N=1,NBODY)
      IF (NWING.GT.0) WRITE (TAPE6,740) NWING,(GT(N),N=1,NWING)
      WRITE (TAPE6,700)
      IF (NBODY.GT.0) WRITE (TAPE6,730) NBODY,(GB(N),N=1,NBODY)
      IF (NWING.GT.0) WRITE (TAPE6,740) NWING,(GW(N),N=1,NWING)
      RETURN
!
!
  590 FORMAT (1H0,26H ERROR MESSAGE FROM ITRATE)
  600 FORMAT (//1X,13HJACOBI METHOD//)
  610 FORMAT (//1X,21HGAUSS - SEIDEL METHOD,5X,4HALF=,F5.2//)
  620 FORMAT (//1X,43HCONTROLLED SUCCESSIVE OVERRELAXATION METHOD,5X,5HA&
     &LF1=,F5.2,5X,5HALF2=,F5.2//)
  630 FORMAT (//1X,32HSUCCESSIVE OVERRELAXATION METHOD,5X,5HALF1=,F5.2,5&
     &X,5HALF2=,F5.2//)
  640 FORMAT (1H0,30HTHE ITERATION DIVERGED AFTER  ,I3,2X,35HITERATIONS &
     &WITH A TEST CRITERION OF,F10.7,2X,34HAND A DIVERGENCE TEST CRITERI&
     &ON OF,F10.3)
  650 FORMAT (2X,10HRB(N),N=1,,I3/(1X,10F10.5))
  660 FORMAT (2X,10HRW(N),N=1,,I3/(1X,10F10.5))
  670 FORMAT (1H0,30HTHE ITERATION CONVERGED AFTER ,I3,2X,35HITERATIONS &
     &WITH A TEST CRITERION OF,F10.7)
  680 FORMAT (1H0,37HTHE ITERATION DID NOT CONVERGE AFTER ,I3,2X,35HITER&
     &ATIONS WITH A TEST CRITERION OF,F10.7)
  690 FORMAT (1H0,41HTHE SOLUTION AT THE PREVIOUS ITERATION IS)
  700 FORMAT (1H0,40HTHE SOLUTION AT THE PRESENT ITERATION IS)
  710 FORMAT (1H0,14HITRATE, TIME =,F10.5)
  720 FORMAT (17H0ITERATION NUMBER,I4)
  730 FORMAT (2X,10HGB(N),N=1,,I3/(1X,10F10.5))
  740 FORMAT (2X,10HGW(N),N=1,,I3/(1X,10F10.5))
      END
      SUBROUTINE PressureCoeff(NP,XMACH,ARA,U,V,W,CPP,CPSTAG,CPCRIT,    &
     & CPVAC,COMPT,VFIX,SGN)
!
!     *******************************************VERSION B00************
!
!
!     THE PRESSURE COEFFICIENT IS CALCULATED USING THE EXACT ISENTROPIC
!     FORMULA IN ALL CASES EXCEPT FOR WING, CANARD (HORIZONTAL TAIL),
!     AND FIN (VERTICAL TAIL) COMPONENTS ANALYZED UNDER THE PLANAR
!     BOUNDARY CONDITION OPTION. IN THESE CASES, THE LINEARIZED THEORY
!     PRESSURE COEFFICIENT FORMULA IS USED AND THE PRESSURE COEFFICIENT
!     IS LIMITED BY THE VACUUM PRESSURE COEFFICIENT AND BY THE
!     STAGNATION PRESSURE COEFFICIENT.
!     COMPUTE THE STAGNATION PRESSURE COEFFICIENT, CRITICAL PRESSURE
!     COEFFICIENT, AND VACUUM PRESSURE COEFFICIENT.
!
      COMMON /PARAM/ DUM(3),LBC
!
      DIMENSION U(1), V(1), W(1), CPP(1)
!
      COMMON /SEG/ NSEG,NR(20),NC(20),COSS(20),SINS(20),BTE(20),NWT(20),&
     &             SPNW(20),XLEW(20),BLE(20),ZLEW(20)
      COMMON /NEWCOM/ KDUM(46),LOCPT(20),XCPT(20)
      COMMON /STAR/XSTAR(3600),NSEGW
      DIMENSION ZCSTR(600),ZTSTR(600),CPSTR(600),                       &
     &          DCNSTR(600),DCTSTR(600),DCMSTR(600)
      EQUIVALENCE (XSTAR(1)   , ZCSTR(1)),(XSTAR(601) , ZTSTR(1)),      &
     &            (XSTAR(1201), CPSTR(1)),(XSTAR(1801),DCNSTR(1)),      &
     &            (XSTAR(2401),DCTSTR(1)),(XSTAR(3001),DCMSTR(1))
!
      INTEGER COMPT
      LOGICAL LBC
!
      XM2=XMACH*XMACH
      CPCRIT=0.
      CPSTAG=1.
      CPVAC=-100.
      COSARA=COS(ARA)
      SINARA=SIN(ARA)
      IF (XM2.EQ.0.) GO TO 10
      CON=1.42857/XM2
      CON1=.2*XM2
      CPVAC=-CON
      CPSTAG=CON*((1.+CON1)**3.5-1.)
      CPCRIT=CON*((5./6.+XM2/6.)**3.5-1.)
   10 CONTINUE
      NBLK=NSEG
      IF(COMPT.EQ.1) NBLK=1
      J=0
      DO 40 N=1,NBLK
      NCOL=NC(N)
      NROW=NR(N)+1
      IF(LBC.AND.LOCPT(N).EQ.0) NROW=NR(N)
      IF(COMPT.EQ.1) NCOL=1
      IF(COMPT.EQ.1) NROW=NP
      DO 40 NN=1,NCOL
      DO 40 NNN=1,NROW
      J=J+1
      UWPM=U(J)*COSARA+W(J)*SINARA
      IF(LBC.AND.COMPT.GE.2) GO TO 20
      UWIND=1.+UWPM
      VWIND=V(J)
      WWIND=W(J)*COSARA-U(J)*SINARA
      VW2=VWIND*VWIND+WWIND*WWIND
      Q2=UWIND*UWIND+VW2
      IF (XMACH.EQ.0.) GO TO 30
      ARG=1.+CON1*(1.-Q2)
      IF (ARG.LT.0.) ARG=0.
      CPP(J)=CON*(ARG**3.5-1.)
      GO TO 40
   20 CPP(J)=-2.*U(J)
!
!     CONSTRAIN THE PRESSURE COEFFICIENT SO THAT IT DOES NOT EXCEED THE
!     VACUUM PRESSURE COEFFICIENT OR THE STAGNATION PRESSURE COEFFICIENT
!
      IF (CPP(J).LT.CPVAC) CPP(J)=CPVAC
      IF (CPP(J).GT.CPSTAG) CPP(J)=CPSTAG
      IF(COMPT.EQ.3) GO TO 25
      IF(XMACH .LE. 1.0) GO TO 25
      COST=COSS(N)
      SINT=SINS(N)
      THET=ATAN2(SINT,COST)
      VN=V(J)*COST+W(J)*SINT
      WN=W(J)*COST-V(J)*SINT
      VN=VN-VFIX
!     VN=0.0
      UI=U(J)
      VI=VN*COST-WN*SINT
      WI=WN*COST+VN*SINT
      DC=SGN*ATAN(ZCSTR(J))
      DT=ATAN(ZTSTR(J))
      CALL CARLSN(XMACH,ARA,THET,DC,DT,UI,VI,WI,EPS,DSTR,CPSTRD,4)
      CPSTR(J)=CPSTRD
      GO TO 40
   25 CPSTR(J)=CPP(J)
      GO TO 40
   30 CPP(J)=1.-Q2
   40 CONTINUE
      RETURN
      END Subroutine PressureCoeff

      SUBROUTINE FORMOM (NPAN,NPASS,ALFA,COMPT)
!
!     *******************************************VERSION B00************
!
!
!     CALCULATE THE FORCE AND MOMENT COEFFICIENTS ON THE BODY, WING,
!     FIN (VERTICAL TAIL) AND CANARD (HORIZONTAL TAIL)
!
!
!     IN THE OUTPUT FROM THIS ROUTINE X, Y, Z, AND 2Y/B ARE IN THE
!     REFERENCE AXIS SYSTEM WHILE X/C AND Z/C ARE IN AN AXIS SYSTEM
!     WITH ORIGIN AT THE LOCAL LEADING EDGE OF THE WING.
!
      COMMON /SHELL/ NPTSB,NSEGWT,NPTSWT,NSEGN,NPTSN,NSEGS,NPTSS,       &
     &               NSEGNS,NPTSNS,NNAC,NSHELL,NQ,IPART, iidum
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAP&
!!!     &E9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15,TAPE16,TAPE17
      INCLUDE 'files.cmn'
      COMMON /PARAM/ NBODY,NWING,NTAIL,LBC,THK,MACH,ALPHA,REFA,REFB,REFC&
     &,REFD,REFL,REFX,REFZ

!!!      COMMON /HEAD/ TITLE1(14),TITLE2(14)
      INCLUDE 'head.cmn'

      COMMON /SEG/ NSEG,NROW(20),NCOL(20),COSS(20),SINS(20),BTE(20),NWT(&
     &20),SPNW(20),XLEW(20),BLE(20),ZLEW(20)
      COMMON /POINT/ ARRAY(6000)
      COMMON /SCRAT/ BLOCK(7500)
      COMMON /NEWCOM/KDUM(46),LOCPT(20),XCPT(20)
      COMMON /VELCOM/ NPOINT,NDUM(5),PRINT
      COMMON /FORM/ CNW,CTW,CMW,CNB,CTB,CMB,CNS(20),CTS(20),CMS(20)
      COMMON /PRESS/ KRADX(4),NRADX(4),KFORX(4),NFORX(4),KWAF,NWAF,KWAFO&
     &R,NWAFOR,NFUS
      COMMON /GRAPH/ PLOT
      COMMON /EPS/ EPSPAN,EPSWP1,EPSWP2,EPSTP,EPSVC,EPSSP1,EPSSP2,EPSLV,&
     &EPSSV,EPSVV,EPSWV,EPSVP,EPSIT1,EPSIT2,EPSFM, epssb1,epssb2
      COMMON /PSAVE/ IFREE(600), CTFREE, CNFREE
      COMMON /SAVZLE/  ZLE(600)
      COMMON /NEWCAM/ CAMMID(600)
      COMMON /DESIGN/ CAMBER(600)
      COMMON /POLAR/NCASE,CDBP(10),CLBP(10),CMBP(10),CDWP(10),CLWP(10), &
     &CMWP(10),ALPHAP(10),MACHP(10),NSPAN,XSPAN(20)
!!!      COMMON /OPTION/ NOPT(10),NXSTNS,XSTN(20),XAPEX,YAPEX,XLAMDA
      INCLUDE 'option.cmn'
!
      DIMENSION XPT(600), YPT(600), ZPT(600), THET(600), DELTA(600)
      DIMENSION SGN(600), AREA(600), CHORD(600), CHD(20), XLE(600)
      DIMENSION XC(30,20), DCN(600), DCM(600), DCT(600), II(600)
      DIMENSION SIND(600), COSD(600), CP(600), SINT(600), COST(600)
      DIMENSION GW(600), GB(600), DZTDX(600)
      DIMENSION XINTP(31),CPINTP(31)
      DIMENSION CNSSTR(20),CTSSTR(20),CMSSTR(20)

      EQUIVALENCE (ARRAY(1),XPT(1))
      EQUIVALENCE (ARRAY(601),YPT(1))
      EQUIVALENCE (ARRAY(1201),ZPT(1))
      EQUIVALENCE (ARRAY(1801),THET(1))
      EQUIVALENCE (ARRAY(2401),DELTA(1))
      EQUIVALENCE (ARRAY(3001),XC(1,1))
      EQUIVALENCE (ARRAY(3001),SGN(1))
      EQUIVALENCE (ARRAY(4801),AREA(1))
      EQUIVALENCE (ARRAY(3601),CHORD(1))
      EQUIVALENCE (ARRAY(5401),XLE(1))
      EQUIVALENCE (BLOCK(1),DCN(1))
      EQUIVALENCE (BLOCK(601),DCM(1))
      EQUIVALENCE (BLOCK(1201),DCT(1))
      EQUIVALENCE (BLOCK(1801),II(1))
      EQUIVALENCE (BLOCK(2401),SIND(1))
      EQUIVALENCE (BLOCK(3001),COSD(1))
      EQUIVALENCE (BLOCK(3601),CP(1))
      EQUIVALENCE (BLOCK(4201),CHD(1))
      EQUIVALENCE (BLOCK(4501),SINT(1))
      EQUIVALENCE (BLOCK(5101),COST(1))
      EQUIVALENCE (BLOCK(5701),GW(1))
      EQUIVALENCE (BLOCK(6301),GB(1))
      EQUIVALENCE (BLOCK(6901),DZTDX(1))
!
      COMMON /STAR/XSTAR(3600),NSEGW
      DIMENSION ZCSTR(600),ZTSTR(600),CPSTR(600),                       &
     &          DCNSTR(600),DCTSTR(600),DCMSTR(600)
      EQUIVALENCE (XSTAR(1)   , ZCSTR(1)),(XSTAR(601) , ZTSTR(1)),      &
     &            (XSTAR(1201), CPSTR(1)),(XSTAR(1801),DCNSTR(1)),      &
     &            (XSTAR(2401),DCTSTR(1)),(XSTAR(3001),DCMSTR(1))
!
      REAL MACH
      LOGICAL LBC
      INTEGER PLOT,COMPT,PRINT
!!!,TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7&
!!!     &,TAPE8,TAPE9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15,TAPE16,    &
!!!     &TAPE17
!
!
!     NOTE THAT THE WING, CANARD, AND TAIL ARE ALL SEGMENTS OF THE WING
!     IN THIS SUBROUTINE
!
      NP=NPAN
!
!     COMPT=1 INDICATES BODY FORCE AND MOMENT CALCULATION
!     COMPT=2 INDICATES WING FORCE AND MOMENT CALCULATION
!     COMPT=3 INDICATES NACELLE/SHELL FORCE AND MOMENT CALCULATION
!
!     NPASS=1 FOR THE BODY
!     NPASS=1 FOR THE WING UPPER AND LOWER SURFACES IF THE NON-PLANAR
!             BOUNDARY CONDITION OPTION IS SELECTED
!     NPASS=1 FOR THE WING UPPER SURFACE IF THE PLANAR BOUNDARY
!             CONDITION OPTION IS SELECTED
!     NPASS=2 FOR THE WING LOWER SURFACE IF THE PLANAR BOUNDARY
!             CONDITION OPTION IS SELECTED
!
      KPLOT=0
      IF(NOPT(8).NE.0.AND.NOPT(8).NE.1000) KPLOT=1
      IF(KPLOT.EQ.0) GO TO 720
      IF(COMPT.EQ.1) WRITE(TAPE15) NPAN,NPASS,COMPT
      IF(COMPT.EQ.2.AND.NPASS.EQ.1) WRITE(TAPE16) NPAN,NPASS,COMPT
      IF(COMPT.EQ.2.AND.NPASS.EQ.2) WRITE(TAPE17) NPAN,NPASS,COMPT
  720 CONTINUE
!*
      ATOTAL = 0.0
      AFREE = 0.0
!*
      DO 10 I=1,4
      IF (KRADX(I).NE.0) NRADX(I)=KRADX(I)
      IF (KFORX(I).NE.0) NFORX(I)=KFORX(I)
   10 END DO
      IF (KWAF.NE.0) NWAF=KWAF
      IF (KWAFOR.NE.0) NWAFOR=KWAFOR
      IF (COMPT.EQ.1) XON=XC(1,1)
      IF (COMPT.EQ.2.AND.NBODY.GT.0) GO TO 20
      IF(COMPT.EQ.3.AND.(NPTSB.GT.0.OR.NPTSWT.GT.0)) GO TO 20
      CNB=0.
      CTB=0.
      CMB=0.
   20 CONTINUE
      SIAL=SIN(ALFA)
      COAL=COS(ALFA)
      IF(COMPT.GE.2) GO TO 40
      WRITE(TAPE6,350)
      WRITE(TAPE6,*) TITLE1
      WRITE(TAPE6,*) TITLE2
      WRITE(TAPE6,360)
      WRITE(TAPE6,500)
      WRITE(TAPE6,380) MACH,ALPHA
      WRITE(TAPE6,400)
   40 CONTINUE
      IF (LBC.AND.COMPT.GT.1) GO TO 70
      DO 60 I=1,NPAN
      SGN(I)=1.0
      SIND(I)=SIN(DELTA(I))
      COSD(I)=COS(DELTA(I))
      SINT(I)=SIN(THET(I))
   60 COST(I)=COS(THET(I))
      GO TO 90
   70 CONTINUE
!
!     CALCULATE CHORD LENGTH OF EACH COLUMN OF PANELS ON WING
!
      I=0
      J=0
      DO 100 N=1,NSEG
      NC=NCOL(N)
      NR=NROW(N)
      DO 100 M=1,NC
      J=J+1
      CHD(J)=0.
      KS=2
      IF (LBC) KS=1
      DO 100 K=1,KS
      DO 100 L=1,NR
      I=I+1
      IF (K.EQ.1) CHD(J)=CHD(J)+CHORD(I)
      IF (K.EQ.2) SGN(I)=-1.0
  100 CONTINUE
      IF(COMPT.EQ.3) GO TO 630
!
!          WRITE WING PRESSURES ON TAPE16 AND TAPE17
!
      IF(KPLOT.EQ.0) GO TO 710
      IMAX=0
      JMAX=0
      NTOT=0
      DO 700 N=1,NSEG
      NCC=NCOL(N)
      NRR=NROW(N)
      NR1=NRR+1
      NPTS=NR1
      IF(LOCPT(N).EQ.0) NPTS=NRR
      DO 700 NN=1,NCC
      NTOT=NTOT+1
      IMIN=IMAX+1
      JMIN=JMAX+1
      IMAX=IMIN+NR1-1
      JMAX=JMIN+NPTS-1
      DO 700 NNN=1,NPTS
      IDEX=IMIN+(NNN-1)
      JDEX=JMIN+(NNN-1)
      XQ=(XPT(IDEX)-XLEW(NTOT))/CHD(NTOT)
      IF(NPASS.EQ.1)                                                    &
     & WRITE(TAPE16) XPT(IDEX),YPT(IDEX),XQ,CP(JDEX),ZPT(IDEX)
      IF(NPASS.EQ.2)                                                    &
     & WRITE(TAPE17) XPT(IDEX),YPT(IDEX),XQ,CP(JDEX),ZPT(IDEX)
  700 CONTINUE
  710 CONTINUE
!
!     PUT OUT PRESSURES IN SPANWISE DIRECTION
!
      TANLES=TAN(XLAMDA)
      IF(NXSTNS.EQ.0) GO TO 630
      WRITE(TAPE6,350)
      WRITE(TAPE6,*) TITLE1
      WRITE(TAPE6,*) TITLE2
      WRITE(TAPE6,680)
      IF(NPASS.EQ.1) WRITE(TAPE6,530)
      IF(NPASS.EQ.2) WRITE(TAPE6,540)
      DO 620 JSTN=1,NXSTNS
      WRITE(6,682) JSTN,XSTN(JSTN)
      JQ=0
      IMAX=0
      JMAX=0
      NTOT=0
      DO 615 N=1,NSEG
      NCC=NCOL(N)
      NRR=NROW(N)
      NR1=NRR+1
      NPTS=NR1
      IF(LOCPT(N).EQ.0) NPTS=NRR
      DO 615 NN=1,NCC
      NTOT=NTOT+1
      IMIN=IMAX+1
      JMIN=JMAX+1
      IMAX=IMIN+NR1-1
      JMAX=JMIN+NPTS-1
      IF(XSTN(JSTN).LT.XLEW(NTOT).OR.XSTN(JSTN).GT.                     &
     & (XLEW(NTOT)+CHD(NTOT))) GO TO 615
      IF(N.GT.NSEGW) GO TO 615
      DO 610 NNN=1,NPTS
      IDEX=IMIN+(NNN-1)
      JDEX=JMIN+(NNN-1)
      XINTP(NNN)=XPT(IDEX)
  610 CPINTP(NNN)=CP(JDEX)
      CALL INTERP(XSTN(JSTN),CPPRES,NPTS,XINTP,CPINTP)
      DO 612 NNN=1,NPTS
      JDEX=JMIN+(NNN-1)
  612 CPINTP(NNN)=CPSTR(JDEX)
      CALL INTERP(XSTN(JSTN),CPPSTR,NPTS,XINTP,CPINTP)
      JQ=JQ+1
      ETA=TANLES*(YPT(IMIN)-YAPEX)/(XSTN(JSTN)-XAPEX)
      WRITE(6,684) JQ,YPT(IMIN),ETA,CPPRES,CPPSTR
  615 CONTINUE
  620 END DO
!
  680 FORMAT(/10X,31HSPANWISE PRESSURE DISTRIBUTION  /)
  682 FORMAT(//10X,5HJSTN=,I2,5X,7HX(JST)=,F10.5//13X,2HJQ,             &
     & 8X,1HY,10X,3HETA,9X,2HCP,7X,5HCPSTR,/)
  684 FORMAT(10X,I5,5F12.5)
!
!     FOR THE PLANAR BOUNDARY CONDITION OPTION CALCULATE THE WING CAMBER
!     AND THICKNESS SLOPES AT CENTER OF PANELS, AND (X,Y,Z) COORDINATES
!     OF CENTRAL POINT
!
  630 I=0
      J=0
      JJ = 0
      DO 80 N=1,NSEG
      NC=NCOL(N)
      NR=NROW(N)
      NR1=NR+1
      DO 80 M=1,NC
      NINIT=J+1
      NFINI=J+NR1
      DO 80 L=1,NR1
      J=J+1
      IF (L.EQ.NR1) GO TO 80
      JJ = JJ  +  1
      I=I+1
      SGN(I)=1.0
      IF (NPASS.EQ.2) SGN(I)=-1.0
      DELC=(DELTA(J)+DELTA(J+1))*0.5
      IF(COMPT.EQ.3.OR.NOPT(4).EQ.0) GO TO 78
      DELC=CAMMID(JJ)
   78 CONTINUE
!*
      DELZ=(DZTDX(J)+DZTDX(J+1))*0.5
      IF (NPASS.EQ.1) TAND=DELC+DELZ
      IF (NPASS.EQ.2) TAND=DELC-DELZ
      SIND(I)=TAND/SQRT(1.+TAND*TAND)
      COSD(I)=SQRT(1.-SIND(I)*SIND(I))
      SINT(I)=SINS(N)
      COST(I)=COSS(N)
      XDUM=(XLE(J)+XLE(J+1))*0.5
      IF(LOCPT(N) .EQ. 0) GO TO 76
      NX=NINIT-1
      DO 72 NNX=NINIT,NFINI
      IF(XDUM .GE. XPT(NNX)) GO TO 72
      GO TO 74
   72 NX=NX+1
   74 XF=(XDUM-XPT(NX))/(XPT(NX+1)-XPT(NX))
      CP(I)=(1.-XF)*CP(NX)+XF*CP(NX+1)
      IF(COMPT.EQ.3) GO TO 76
      CPSTR(I)=(1.-XF)*CPSTR(NX)+XF*CPSTR(NX+1)
   76 XPT(I)=XDUM
      YPT(I)=YPT(J)
      ZPT(I)=ZPT(J)
      IF(COMPT.EQ.3.AND.N.LE.NSEGN) NPNAC=I
      IF(COMPT.EQ.3.AND.N.GT.NSEGN) NPSHL=I-NPNAC
   80 CONTINUE
      IF(COMPT.GE.2) NP=I
   90 CONTINUE
      IF(COMPT.EQ.1) GO TO 120
      I=0
      J=0
!
!     ASSOCIATE THE LEADING EDGE COORDINATES AND CHORD LENGTHS OF EACH
!     COLUMN OF PANELS WITH THE INDIVIDUAL PANELS IN THE COLUMN
!
      DO 110 N=1,NSEG
      NC=NCOL(N)
      NR=NROW(N)
      DO 110 M=1,NC
      J=J+1
      KS=2
      IF (LBC) KS=1
      DO 110 K=1,KS
      DO 110 L=1,NR
      I=I+1
      ZLE(I)=ZLEW(J)
      XLE(I)=XLEW(J)
      CHORD(I)=CHD(J)
  110 CONTINUE
  120 CONTINUE
      IF (NPASS.EQ.2) GO TO 130
      CN=0.
      CT=0.
      CM=0.
      CTFREE = 0.0
      CNFREE = 0.0
      CNSTR=0.
      CTSTR=0.
      CMSTR=0.
      CTFRST=0.
      CNFRST=0.
      GO TO 140
  130 CN=CNW
      CT=CTW
      CM=CMW
      CNSTR=CNWSTR
      CTSTR=CTWSTR
      CMSTR=CMWSTR
  140 IP=0
!
!     CALCULATE THE FORCES AND MOMENT ACTING ON EACH PANEL AND SUM OVER
!     THE ENTIRE COMPONENT
!
      IF(COMPT.EQ.1) GO TO 50
      WRITE(TAPE6,350)
      WRITE(TAPE6,*) TITLE1
      WRITE(TAPE6,*) TITLE2
      WRITE(TAPE6,360)
      IF(LBC.AND.NPASS.EQ.1.AND.COMPT.EQ.2) WRITE(TAPE6,530)
      IF(LBC.AND.NPASS.EQ.2.AND.COMPT.EQ.2) WRITE(TAPE6,540)
      IF(LBC.AND.NPASS.EQ.1.AND.COMPT.EQ.3) WRITE(TAPE6,535)
      IF(LBC.AND.NPASS.EQ.2.AND.COMPT.EQ.3) WRITE(TAPE6,545)
      IF(.NOT.LBC) WRITE(TAPE6,510)
      WRITE(TAPE6,380) MACH,ALPHA
      IF(.NOT.LBC) WRITE(TAPE6,390)
      IF(LBC) WRITE(TAPE6,550)
   50 CONTINUE
      WRITE(TAPE12) NP,NPASS,COMPT
      IF(COMPT.EQ.1) GO TO 146
      KUK = 0
      LUK = 0
      I = 0
      DO 144 IK = 1,NSEG
      NWA = NCOL(IK)
      NWR = NROW(IK)
      NWAR = NWR
      IF(.NOT.LBC) NWAR = 2*NWR
      DO 143 J = 1,NWA
      DO 143 K = 1,NWAR
      I = I + 1
      YP = YPT(I)
      XP = XPT(I)
      XQ = XP
      IF(CHORD(I).NE.0) XQ = (XP - XLE(I))/CHORD(I)
      IF(LBC) GO TO 142
      IF(LUK.NE.0) GO TO 141
      KUK = KUK + 1
      IF(KUK.EQ.NWR) LUK = 1
      WRITE(TAPE12) YP,XQ,CP(I)
      IF(LUK.NE.0) KUK = 0
      GO TO 143
  141 KUK = KUK + 1
      IF(KUK.EQ.NWR) LUK = 0
      WRITE(TAPE13) YP,XQ,CP(I)
      IF(LUK.EQ.0) KUK = 0
      GO TO 143
  142 IF(NPASS.EQ.1) WRITE(TAPE12) YP,XQ,CP(I)
      IF(NPASS.EQ.2) WRITE(TAPE13) YP,XQ,CP(I)
  143 CONTINUE
  144 END DO
  146 IF(COMPT.EQ.3.AND.NPASS.EQ.2) GO TO 145
      CNNN=0.0
      CTTN=0.0
      CMMN=0.0
      CNNS=0.0
      CTTS=0.0
      CMMS=0.0
  145 DO 210 I=1,NP
      IP=IP+1
      XP=XPT(I)
      YP=YPT(I)
      ZP=ZPT(I)
      F1=COSD(I)*COST(I)
      F2=SIND(I)
      FAK=AREA(I)*SGN(I)
      IF (LBC.AND.COMPT.GT.1.AND.COSD(I).NE.0.) FAK=FAK/COSD(I)
      IF (ABS(YP).LT.EPSFM) FAK=0.5*FAK
      DCN(I)=-CP(I)*F1*FAK
      DCT(I)=CP(I)*F2*FAK
      DCM(I)=DCN(I)*(REFX-XP)-DCT(I)*(REFZ-ZP)
      IF(.NOT.LBC.OR.COMPT.EQ.1) GO TO 147
      DCNSTR(I)=-CPSTR(I)*F1*FAK
      DCTSTR(I)=CPSTR(I)*F2*FAK
      DCMSTR(I)=DCNSTR(I)*(REFX-XP)-DCTSTR(I)*(REFZ-ZP)
  147 XQ=XP
      YQ=YP
      ZQ=ZP
      IF(COMPT.GE.2) GO TO 150
!
!     NONDIMENSIONALIZE BODY PANEL CONTROL POINT COORDINATES
!     X COORDINATES ARE DIVIDED BY THE BODY REFERENCE LENGTH
!     Y AND Z COORDINATES ARE DIVIDED BY THE BODY REFERENCE DIAMETER
!
      XQ=(XP-XON)/REFL
      YQ=YP/REFD
      ZQ=ZP/REFD
      GO TO 160
!
!     NONDIMENSIONALIZE WING PANEL CONTROL POINT COORDINATES
!     X AND Z COORDINATES ARE DIVIDED BY THE REFERENCE CHORD
!     Y COORDINATES ARE DIVIDED BY THE REFERENCE SEMISPAN
!
  150 IF (CHORD(I).NE.0.) XQ=(XP-XLE(I))/CHORD(I)
      IF (REFB.NE.0.) YQ=YP/REFB
      IF (CHORD(I).NE.0.) ZQ=(ZP-ZLE(I))/CHORD(I)
  160 CONTINUE
      IF(.NOT.LBC.OR.COMPT.EQ.1) GO TO 170
      IF(LBC.AND.COMPT.EQ.2)                                            &
     & WRITE(TAPE6,560) IP,XP,YP,ZP,XQ,YQ,ZQ,CP(I),CPSTR(I),IP
      IF(LBC.AND.COMPT.EQ.3)                                            &
     & WRITE(TAPE6,565) IP,XP,YP,ZP,XQ,YQ,ZQ,CP(I),IP
      CNSTR=CNSTR+DCNSTR(I)
      CTSTR=CTSTR+DCTSTR(I)
      CTFRST=CTFRST+IFREE(I)*DCTSTR(I)
      CNFRST=CNFRST+IFREE(I)*DCNSTR(I)
      CMSTR=CMSTR+DCMSTR(I)
      GO TO 180
  170 CONTINUE
      WRITE (TAPE6,410) IP,XP,YP,ZP,XQ,YQ,ZQ,CP(I),DCN(I),DCT(I),DCM(I),&
     &IP
  180 CONTINUE
      CN=CN+DCN(I)
      CT=CT+DCT(I)
      ATOTAL = ATOTAL + AREA(I)
      AFREE  = AFREE  + IFREE(I) * AREA(I)
      IF( COMPT .EQ. 2 ) CTFREE = CTFREE + IFREE(I) * DCT(I)
      IF( COMPT .EQ. 2 ) CNFREE = CNFREE + IFREE(I) * DCN(I)
      CM=CM+DCM(I)
      IF(COMPT.EQ.2) GO TO 210
      IF(COMPT.EQ.1) GO TO 195
      IF(I.GT.NPNAC) GO TO 190
      CNNN=CNNN+DCN(I)
      CTTN=CTTN+DCT(I)
      CMMN=CMMN+DCM(I)
      GO TO 210
  190 CNNS=CNNS+DCN(I)
      CTTS=CTTS+DCT(I)
      CMMS=CMMS+DCM(I)
      GO TO 210
  195 CONTINUE
!
!          WRITE BODY PRESSURES ON TAPE 15
!
      IF(KPLOT.NE.0)                                                    &
     & WRITE(TAPE15) XP,YP,ZP,CP(I),XQ,YQ,ZQ
      IF(PLOT.NE.0) WRITE(TAPE12) XP,YP,ZP,CP(I)
  210 END DO
      IF (COMPT.GT.1) GO TO 220
!
!     STORE BODY FORCES AND MOMENT
!
      CNB=CN
      CTB=CT
      CMB=CM
      GO TO 230
  220 CONTINUE
!
!     STORE WING FORCES AND MOMENT
!
      CNW=CN
      CTW=CT
      CMW=CM
      CNWSTR=CNSTR
      CTWSTR=CTSTR
      CMWSTR=CMSTR
      IF (LBC.AND.NPASS.EQ.1) GO TO 250
  230 CONTINUE
      WRITE (TAPE6,350)
      WRITE (TAPE6,420)
      IF (COMPT.EQ.1) WRITE (TAPE6,500)
      IF (COMPT.EQ.2) WRITE (TAPE6,510)
      IF(COMPT.EQ.3) WRITE(TAPE6,515)
!
!     COMPUTE NORMAL AND TANGENTIAL (AXIAL) FORCE COEFFICIENTS, PITCHING
!     MOMENT COEFFICIENT, LIFT AND DRAG COEFFICIENT, AND CENTER OF
!     PRESSURE OF COMPONENT
!
      IT=0
  240 CONTINUE
!*
!*          USE HALF-WING REFERENCE AREA
!*
      FACTOR = 1.0
      CN = FACTOR * CN / REFA
      CT = FACTOR * CT / REFA
      CM = FACTOR * CM / ( REFA * REFC )
      CNNAC=FACTOR*CNNN/REFA
      CTNAC=FACTOR*CTTN/REFA
      CMNAC=FACTOR*CMMN/(REFA*REFC)
      CNSHL=FACTOR*CNNS/REFA
      CTSHL=FACTOR*CTTS/REFA
      CMSHL=FACTOR*CMMS/(REFA*REFC)
      IF(COMPT.EQ.2.AND.IT.EQ.0) CTFREE=FACTOR*CTFREE/REFA
      IF(COMPT.EQ.2.AND.IT.EQ.0) CNFREE=FACTOR*CNFREE/REFA
      CL=CN*COAL-CT*SIAL
      CD=CN*SIAL+CT*COAL
!*
      BETA = SQRT( ABS( MACH * MACH - 1.0 ) )
      CDCL = CD / ( BETA * CL * CL )
!*
      CDF = CNFREE * SIAL + CTFREE * COAL
      CNF = CNFREE * COAL - CTFREE * SIAL
      DXN=0.
      IF (CN.NE.0.) DXN=-CM/CN
      IF (COMPT.EQ.1) WRITE (TAPE6,440) REFA,REFD,REFL
      IF(COMPT.GE.2) WRITE(TAPE6,430) REFA,REFB,REFC
      WRITE (TAPE6,450) REFX,REFZ
      WRITE (TAPE6,460) MACH,ALPHA,CN,CT,CM,CL,CD,DXN
      IF(NCASE.EQ.0) GO TO 242
      MACHP(NCASE)=MACH
      ALPHAP(NCASE)=ALPHA
      IF(COMPT.GT.1) GO TO 241
      CLBP(NCASE)=CL
      CDBP(NCASE)=CD
      CMBP(NCASE)=CM
      GO TO 242
  241 CONTINUE
      CLWP(NCASE)=CL
      CDWP(NCASE)=CD
      CMWP(NCASE)=CM
  242 CONTINUE
!*
      WRITE(TAPE6,461) CDCL
      IF(.NOT.LBC.OR.COMPT.EQ.1) GO TO 245
      CNSTR=FACTOR*CNSTR/REFA
      CTSTR=FACTOR*CTSTR/REFA
      CMSTR=FACTOR*CMSTR/(REFA*REFC)
      IF(COMPT.EQ.2.AND.IT.EQ.0) CTFRST=FACTOR*CTFRST/REFA
      IF(COMPT.EQ.2.AND.IT.EQ.0) CNFRST=FACTOR*CNFRST/REFA
      CLSTR=CNSTR*COAL-CTSTR*SIAL
      CDSTR=CNSTR*SIAL+CTSTR*COAL
      CDCLST=0.
      IF(CLSTR.NE.0.) CDCLST=CDSTR/(BETA*CLSTR*CLSTR)
      CDFSTR=CNFRST*SIAL+CTFRST*COAL
      CNFSTR=CNFRST*COAL-CTFRST*SIAL
      DXNSTR=0.
      IF(CNSTR.NE.0.) DXNSTR=-CMSTR/CNSTR
      IF(COMPT.EQ.3.AND.IT.EQ.0) GO TO 245
      WRITE(TAPE6,466)
  466 FORMAT(/,10X,32HBASED ON THE CARLSON CORRECTION )
      WRITE(TAPE6,460) MACH,ALPHA,CNSTR,CTSTR,CMSTR,CLSTR,CDSTR,DXNSTR
      WRITE(TAPE6,461) CDCLST
  245 CONTINUE
  461 FORMAT(/8X,5HCDCL=,F15.5)
      IF(COMPT.EQ.1) GO TO 464
      IF(COMPT.EQ.3.AND.IT.EQ.0) GO TO 464
      IF(NOPT(1).EQ.1.AND.NOPT(2).LE.1) GO TO 464
      IF(NOPT(1).EQ.1.AND.NOPT(2).EQ.3) GO TO 464
      IF(NOPT(1).EQ.2.AND.NOPT(2).GE.0) GO TO 464
      WRITE(TAPE6,458) ATOTAL
      WRITE(TAPE6,459) AFREE
  458 FORMAT(/8X,5HAREA=,F15.5,4X,12H(TOTAL-AREA))
  459 FORMAT(/8X,5HAREA=,F15.5,4X,14H(EXPOSED-AREA))
      EPSCL = 1.0E-10
      CDFCL = 1.0E+20
      IF( ABS(CL) .GT. EPSCL ) CDFCL = CDF / ( BETA * CL * CL )
      WRITE(TAPE6,465) CNF
  465 FORMAT(/9X,4HCNF=,F15.5,4X,14H(EXPOSED-LIFT))
      WRITE(TAPE6,462) CDF
  462 FORMAT(/9X,4HCDF=,F15.5,4X,14H(EXPOSED-DRAG))
      WRITE(TAPE6,463) CDFCL
  463 FORMAT(/7X,6HCDFCL=,F15.5,4X,25H(EXPOSED DRAG-TOTAL LIFT))
      CDFCLS=1.0E+20
      IF(ABS(CLSTR).GT.EPSCL) CDFCLS=CDFSTR/(BETA*CLSTR*CLSTR)
      WRITE(TAPE6,466)
      WRITE(TAPE6,465) CNFSTR
      WRITE(TAPE6,462) CDFSTR
      WRITE(TAPE6,463) CDFCLS
  246 CONTINUE
  464 CONTINUE
!*
  250 CONTINUE
      IF(COMPT.NE.3) GO TO 254
      IF(NPASS.NE.2) GO TO 254
      IF(IT.GT.0) GO TO 254
      DXNNAC=0.0
      CLNAC=CNNAC*COAL-CTNAC*SIAL
      CDNAC=CNNAC*SIAL+CTNAC*COAL
      IF(CNNAC.NE.0) DXNNAC=-CMNAC/CNNAC
      WRITE(TAPE6,800)
      WRITE(TAPE6,460) MACH,ALPHA,CNNAC,CTNAC,CMNAC,CLNAC,CDNAC,DXNNAC
      DXNSHL=0.0
      CLSHL=CNSHL*COAL-CTSHL*SIAL
      CDSHL=CNSHL*SIAL+CTSHL*COAL
      IF(CNSHL.NE.0) DXNSHL=-CMSHL/CNSHL
      WRITE(TAPE6,810)
      WRITE(TAPE6,460) MACH,ALPHA,CNSHL,CTSHL,CMSHL,CLSHL,CDSHL,DXNSHL
  254 CONTINUE
      IF (COMPT.EQ.1) GO TO 340
      IF (LBC.AND.NPASS.EQ.1) GO TO 260
      IF(COMPT.EQ.2.AND.NPTSNS.NE.0) GO TO 256
      IF(COMPT.EQ.2.AND.NPTSNS.EQ.0.AND.NBODY.EQ.0) GO TO 260
      IF(COMPT.EQ.3.AND.NPTSWT.EQ.0.AND.NBODY.EQ.0) GO TO 260
      IF(IT.GT.0) GO TO 260
      IT=IT+1
      IF(COMPT.EQ.3) GO TO 255
      CN=CNB+CNW
      CT=CTB+CTW
      CM=CMB+CMW
      CNSTR=CNB+CNWSTR
      CTSTR=CTB+CTWSTR
      CMSTR=CMB+CMWSTR
  255 WRITE (TAPE6,350)
      WRITE (TAPE6,420)
      WRITE (TAPE6,520)
      IF(COMPT.EQ.2) GO TO 240
      CN=CNW+REGCN
      CT=CTW+REGCT
      CM=CMW+REGCM
      CNSTR=CNWSTR+STRCN
      CTSTR=CTWSTR+STRCT
      CMSTR=CMWSTR+STRCM
      GO TO 240
  256 REGCN=CNB+CNW
      REGCT=CTB+CTW
      REGCM=CMB+CMW
      STRCN=CNB+CNWSTR
      STRCT=CTB+CTWSTR
      STRCM=CMB+CMWSTR
  260 IF (PRINT.EQ.0) GO TO 340
      IF(COMPT.NE.2) GO TO 340
      CAVE=REFA/REFB
      KSEC=1
      IF (LBC) KSEC=2
      DO 330 KSTAR=1,KSEC
      IF(LBC.AND.NPASS.EQ.1) GO TO 270
      WRITE(TAPE6,350)
      WRITE(TAPE6,480)
      IF(COMPT.EQ.2) WRITE(TAPE6,510)
      IF(COMPT.EQ.3) WRITE(TAPE6,515)
      IF(KSTAR.EQ.2) WRITE(TAPE6,570)
      WRITE(TAPE6,580)
  270 CONTINUE
      K=0
      I2=0
!
!     COMPUTE SECTION FORCES AND MOMENTS
!
      DO 330 N=1,NSEG
      NR=NROW(N)
      NR2=NR*2
      NC=NCOL(N)
      DO 330 M=1,NC
      K=K+1
      I1=I2+1
      YSTN=YPT(I1)
      ETA=YSTN/REFB
      COCA=CHD(K)/CAVE
      IF(LBC) I2=I2+NR
      IF(.NOT.LBC) I2=I2+NR2
      IF(LBC.AND.NPASS.EQ.2) GO TO 280
      IF(KSTAR.EQ.2) GO TO 275
      CN=0.
      CT=0.
      CM=0.
      GO TO 290
  275 CNSTR=0.
      CTSTR=0.
      CMSTR=0.
      GO TO 290
  280 IF(KSTAR.EQ.2) GO TO 285
      CN=CNS(K)
      CT=CTS(K)
      CM=CMS(K)
      GO TO 290
  285 CNSTR=CNSSTR(K)
      CTSTR=CTSSTR(K)
      CMSTR=CMSSTR(K)
  290 CONTINUE
!
!     SUM THE FORCES AND MOMENT ACTING ON EACH SECTION
!
      DO 300 I=I1,I2
      IF(KSTAR.EQ.2) GO TO 295
      CN=CN+DCN(I)
      CT=CT+DCT(I)
      CM=CM+DCM(I)
      GO TO 300
  295 CNSTR=CNSTR+DCNSTR(I)
      CTSTR=CTSTR+DCTSTR(I)
      CMSTR=CMSTR+DCMSTR(I)
  300 END DO
      IF(KSTAR.EQ.2) GO TO 305
      CNS(K)=CN
      CTS(K)=CT
      CMS(K)=CM
      GO TO 310
  305 CNSSTR(K)=CNSTR
      CTSSTR(K)=CTSTR
      CMSSTR(K)=CMSTR
  310 IF(LBC.AND.NPASS.EQ.1) GO TO 330
!
!     COMPUTE SECTION CN,CA,CL,CD,CM,XCP
!
      H1=1./(SPNW(K)*CHD(K))
      H2=H1/REFC
      IF(KSTAR.EQ.2) GO TO 315
      CN=CN*H1
      CT=CT*H1
      CM=CM*H2
      GO TO 320
  315 CN=CNSTR*H1
      CT=CTSTR*H1
      CM=CMSTR*H2
  320 CL=CN*COAL-CT*SIAL
      CD=CN*SIAL+CT*COAL
      CCLCA=COCA*CL
      CCDCA=COCA*CD
      DXN=0.
      IF(CN.NE.0.) DXN=-CM/CN
      WRITE(TAPE6,590) K,YSTN,ETA,COCA,CCLCA,CCDCA,CN,CT,CL,CD,CM,DXN,K
  330 CONTINUE
  340 CONTINUE
      RETURN
!
!
  350 FORMAT (1H1)
  360 FORMAT (//,10X,40HINTEGRATION OF THE PRESSURE DISTRIBUTION,/)
  370 FORMAT (10X,13A6,A2/10X,13A6,A2)
  380 FORMAT (/,10X,6HMACH= ,F8.4,4X,6HALPHA=,F8.4/)
  390 FORMAT (3X,5HPOINT,7X,1HX,10X,1HY,10X,1HZ,9X,3HX/C,7X,4H2Y/B,8X,3H&
     &Z/C,8X,2HCP,9X,2HCN,9X,2HCA,9X,2HCM,4X,5HPOINT/)
  400 FORMAT (3X,5HPOINT,7X,1HX,10X,1HY,10X,1HZ,9X,3HX/L,8X,3HY/D,8X,3HZ&
     &/D,8X,2HCP,9X,2HCN,9X,2HCA,9X,2HCM,4X,5HPOINT/)
  410 FORMAT (1X,I6,10F11.5,I6)
  420 FORMAT (///,10X,18HTOTAL COEFFICIENTS,/,10X,18(1H-))
  430 FORMAT (10X,5HREFA=,F14.4,3X,5HREFB=,F14.4,3X,5HREFC=,F14.4)
  440 FORMAT (10X,5HREFA=,F14.4,3X,5HREFD=,F14.4,3X,5HREFL=,F14.4)
  450 FORMAT (/,10X,5HREFX=,F14.4,3X,5HREFZ=,F14.4)
  460 FORMAT (/8X,5HMACH=,F15.5/7X,6HALPHA=,F15.5/10X,3HCN=,F15.5/10X,3H&
     &CA=,F15.5/10X,3HCM=,F15.5/10X,3HCL=,F15.5/10X,3HCD=,F15.5/9X,4HXCP&
     &=,F15.5)
  470 FORMAT (//)
  480 FORMAT (10X,20HSECTION COEFFICIENTS,/,10X,20(1H-))
  490 FORMAT (10X,5HDELY=,F14.4,3X,5HREFL=,F14.4,3X,4HXLE=,F14.4)
  500 FORMAT (10X,11HON THE BODY,//)
  510 FORMAT (10X,11HON THE WING,//)
  515 FORMAT (10X,20HON THE NACELLE/SHELL,//)
  520 FORMAT (10X,29HON THE COMPLETE CONFIGURATION,//)
  530 FORMAT (10X,25HON THE WING UPPER SURFACE,//)
  535 FORMAT (10X,34HON THE NACELLE/SHELL UPPER SURFACE,//)
  540 FORMAT (10X,25HON THE WING LOWER SURFACE,//)
  545 FORMAT (10X,34HON THE NACELLE/SHELL LOWER SURFACE,//)
  550 FORMAT(3X,5HPOINT,4X,1HX,9X,1HY,9X,1HZ,8X,3HX/C,6X,4H2Y/B,7X,     &
     & 3HZ/C,7X,2HCP,7X,5HCPSTR,3X,5HPOINT,/)
  560 FORMAT(1X,I6,8F10.5,I6)
  565 FORMAT(1X,I6,7F10.5,10X,I6)
  570 FORMAT(10X,32HBASED ON THE CARLSON CORRECTION ,/)
  580 FORMAT(5X,1HK,6X,1HY,8X,3HETA,5X,6HC/CAVE,3X,9HC*CL/CAVE,1X,      &
     & 9HC*CD/CAVE,4X,2HCN,8X,2HCA,8X,2HCL,8X,2HCD,8X,2HCM,8X,          &
     & 3HXCP,4X,1HK,/)
  590 FORMAT(1X,I6,11F10.5,I6)
  800 FORMAT (///,30H NACELLE FORCES AND AND MOMENT)
  810 FORMAT (///,28H SHELL FORCES AND AND MOMENT)
      END
      SUBROUTINE CARLSN(XMINF,ALP,THET,DC,DT,U,V,W,EPS,DSTR,CPSTR,MODE)
!
!      HARRY CARLSON@S CORRECTION , NASA TP 1406 , FEB. 1979
!
!      MODE = 1        ORIGINAL
!             2      + IMPROVED M*(NU*)
!             3      + IMPROVED M(U,V)
!             4      + EXACT M(U,V,W)
!
      IF(XMINF .LE. 1.0) GO TO 350
      BETA=SQRT(XMINF**2-1.0)
      DEL=TAN(DC)+TAN(DT)
      XLAM=COS(ALP)*DEL-SIN(ALP)*COS(THET)
      UBO=-XLAM/BETA
      WBO=XLAM
      UO=UBO*COS(ALP)+WBO*SIN(ALP)
      WO=WBO*COS(ALP)-UBO*SIN(ALP)
      UCUT=0.0
      IF(UO .GE. UCUT) GO TO 210
      ULIM=-.9999
      ETAU=(UCUT-UO)/(UCUT-ULIM)
      ETAU=2./(1./(1.+ETAU)**2+1.)-1.
      UO=(1.-ETAU)*UCUT+ETAU*ULIM
  210 CONTINUE
      IF(MODE .LT. 4) WO=0.
      UI=U*COS(ALP)+W*SIN(ALP)
      VI=V
      WI=W*COS(ALP)-U*SIN(ALP)
      UCUT=0.0
      IF(UI .GE. UCUT) GO TO 220
      ULIM=-.9999
      ETAU=(UCUT-UI)/(UCUT-ULIM)
      ETAU=2./(1./(1.+ETAU)**2+1.)-1.
      UI=(1.-ETAU)*UCUT+ETAU*ULIM
  220 CONTINUE
      IF(MODE .LT. 4) WI=0.
      NMACH=0
      IF(MODE .GE. 3) NMACH=1
      CALL MACH(UO,0.,WO,XMINF,XMO,EPSO,NMACH)
      CALL MACH(UI,VI,WI,XMINF,XMI,EPS,NMACH)
      IF(XMO .LT. 1.0) GO TO 310
      CALL PMA(XMO,XNUO,0)
      GO TO 320
  310 CALL PMA(XMINF,XNUINF,0)
      XNUO=(XNUINF-1.570796)*(1.0-XMO)**2
  320 IF(XMI .LT. 1.0) GO TO 330
      CALL PMA(XMI,XNUI,0)
      GO TO 340
  330 CALL PMA(XMINF,XNUINF,0)
      XNUI=(XNUINF-1.570796)*(1.0-XMI)**2
  340 CONTINUE
      DEL=DC+DT
      XLAM=ASIN(COS(ALP)*SIN(DEL)-SIN(ALP)*COS(THET))
      DSTR=XLAM+XNUO-XNUI
      CALL SHEXRE(XMINF,DSTR,CPSTR,MODE)
      RETURN
  350 CPSTR=0.0
      RETURN
!
  800 FORMAT(/,1X,I2,5X,F16.7,/)
!
      END
!
      SUBROUTINE SHEXRE(XM,DSTR,CPSTR,MODE)
!
!      SHOCK-EXPANSION RELATIONS
!
!      MODE = 1     ORIGINAL M*(NU*)
!           # 1     REFINED  M*(NU*)
!
!
      PPT(XMD,XNUD)=((1.0+COS(2.0*(XNUD+ACOS(1.0/XMD))/SQRT(6.0))       &
     &              )/2.4)**3.5
      PI=3.141592654
      IF(XM .LE. 1.0) GO TO 210
      IF(DSTR.GE.0.0) GO TO 100
!
      CALL PMA(XM,XNUI,0)
      XNUSTR=XNUI-DSTR
      CALL PMA(XMSTR,XNUSTR,1)
      CPSTR=(PPT(XMSTR,XNUSTR)/PPT(XM,XNUI)-1.0)/0.7/XM**2
      RETURN
!
  100 SINTD2=(3.*XM**2-2.+SQRT(3.*(3.*XM**4-4.*XM**2+13.)))             &
     & /7./XM**2
      THETD=ASIN(SQRT(SINTD2))
      TANDD=5.*(XM**2*SIN(2.*THETD)-2./TAN(THETD))/                     &
     & (10.+XM**2*(7.+5.*COS(2.*THETD)))
      DD=ATAN(TANDD)
      IF(DSTR.GE.DD) GO TO 200
      B=-(XM**2+2.0)/XM**2-1.4*SIN(DSTR)**2
      C=(2.0*XM**2+1.0)/XM**4+(1.44+.4/XM**2)*SIN(DSTR)**2
      D=-COS(DSTR)**2/XM**4
      TERM=B*B-3.*C
      TERM2=(4.5*B*C-B**3-13.5*D)/TERM**1.5
      IF(ABS(TERM2) .GT. 1.) TERM2=TERM2/ABS(TERM2)
      PHI=ACOS(TERM2)
      ARG=-B/3.+2./3.*SQRT(TERM)*COS((PHI+4.*PI)/3.)
      THET=ASIN(SQRT(ARG))
      CPSTR=5./3.*(XM**2*SIN(THET)**2-1.)/XM**2
      RETURN
!
  200 CONTINUE
      CPDD=5./3.*(XM**2*SIN(THETD)**2-1.0)/XM**2
      CP90=((1.2*XM**2)**3.5*(6./(7.*XM**2-1.))**2.5-1.0)/.7/XM**2
      CPSTR=CPDD+(CP90-CPDD)*(DSTR-DD)/(PI/2.-DD)
      RETURN
  210 CPSTR=0.
      RETURN
!
  250 FORMAT(/10X,25HANGLE GT DETACHMENT ANGLE  ,6X,                    &
     & 5HDSTR=,F10.5,10X,3HDD=,F10.5/)
!
  800 FORMAT(/,1X,I2,5X,F16.7,/)
!
      END
      SUBROUTINE MACH(U,V,W,XMINF,XM,EPS,MODE)
!
!          MACH NUMBER AS A FUNCTION OF
!          PERTUBATION VELOCITIES
!
!          MODE = 0 ... CARLSON@S FORMULATION
!                 1 ... EXACT ISENTROPIC RELATIONSHIP
!
      ARG=(V*V+W*W)/(1.+U)**2
      EPS=ATAN(SQRT(ARG))
      VHAT=SQRT((1.+U)**2+V**2+W**2)
      VLIM=SQRT((1.0+0.2*XMINF**2)/(0.2*XMINF**2))
      VLIM=VLIM-.0001
      VCUT=1.+(VLIM-1.)/2.
      IF(VHAT .LE. VCUT) GO TO 300
      ETAV=(VCUT-VHAT)/(VCUT-VLIM)
      ETAV=2./(1./(1.+ETAV)**2+1.)-1.
      VHAT=(1.-ETAV)*VCUT+ETAV*VLIM
  300 CONTINUE
      XNUM=XMINF*VHAT
      ARG=1.+0.2*XMINF**2*(1.-VHAT**2)
      DEN=SQRT(ARG)
      XM=XNUM
      IF(MODE .EQ. 1) XM=XNUM/DEN
      RETURN
!
  800 FORMAT(/,1X,I2,5X,F16.7,/)
!
      END
!
      SUBROUTINE PMA(XM,XNU,MODE)
!
!      PRANDTL-MEYER ANGLE
!
!      MODE=0  GIVE MACH  - GET ANGLE
!           1  GIVE ANGLE - GET MACH
!
      EPS=0.1E-5
      G=1.4
      PIO2=1.570796327
      ARG=(G+1.0)/(G-1.0)
      A=SQRT(ARG)
      XNUMAX=PIO2*(A-1.0)
      IF(MODE .EQ. 1) GO TO 100
!
      IF(XM .LE. 1.0) GO TO 90
      Q=SQRT(XM**2-1.0)
      XNU=A*ATAN(Q/A)-ATAN(Q)
      RETURN
   90 XNU=0.0
      RETURN
!
  100 CONTINUE
      IF(XNU.GE.XNUMAX) GO TO 120
      IF(XNU .LE. 0.0) GO TO 130
      I=0
      C1=XNU/XNUMAX
      XN=0.56+0.313*C1-0.42*SQRT(C1)
      XM=1.0/(1.0-C1**XN)
      Q=SQRT(XM**2-1.0)
!
!
   50 I=I+1
      IF(I.GT.20) GO TO 60
      FQ=XNU-A*ATAN(Q/A)+ATAN(Q)
      FPQ=-A*A/(A*A+Q*Q)+1.0/(1.0+Q*Q)
      QP=Q
      Q=Q-FQ/FPQ
      DELTA=ABS(Q-QP)
      IF(DELTA.GT.EPS) GO TO 50
   60 ARG=Q*Q+1.0
      XM=SQRT(ARG)
      RETURN
  120 XM=100.
      RETURN
  130 XM=1.0
      RETURN
!
  800 FORMAT(/,1X,I2,5X,F16.7,/)
!
      END
      SUBROUTINE INTERP(XIN,YOUT,NMAX,X,Y)
!
!      LINEAR INTERPOLATION/EXTRAPLOLATION
!
      DIMENSION X(1),Y(1)
      N=2
      IF(XIN.LT.X(1)) GO TO 12
      N=NMAX
      IF(XIN.GT.X(NMAX)) GO TO 12
      DO 10 I=2,NMAX
      N=I
      IF(X(N-1).LE.XIN.AND.X(N).GE.XIN) GO TO 12
   10 END DO
   12 YOUT=Y(N-1)+(Y(N)-Y(N-1))*(XIN-X(N-1))/(X(N)-X(N-1))
      RETURN
      END

      SUBROUTINE MIXED
      COMMON /PI/ PI, RADEG, DEGRAD
      COMMON /DESIGN/ CAMBER(600)
      COMMON /PARAM/ NBODY,NWING,NTAIL,LBC,THK,MACH,ALPHA,REFA
      COMMON /POINT/ ARRAY(6000)
      COMMON /TAPES/ ITAPES(20)
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAP&
!!!     &E9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15,TAPE16
      INCLUDE 'files.cmn'
      INTEGER COMPT,PRINT
!!!,TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE&
!!!     &8,TAPE9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15,TAPE16
  INTEGER:: FFILE
!!!      COMMON /OPTION/ NOPT(10),NXSTNS,XSTN(20),XAPEX,YAPEX,XLAMDA
      INCLUDE 'option.cmn'
      COMMON /SCRAT/ BLOCK(7500)
      COMMON /NEWCOM/KDUM(9),KFUS,KRADX(4),KFORX(4),KDUMX(27),           &
     &              NCPT,LOCPT(20),XCPT(20)
      COMMON /SEG/NSEG,NROW(20),NCOL(20),COSS(20),SINS(20),              &
     &           BT(20),DUM(20),BL(20)
      DIMENSION XP(600),YP(600),ZP(600),ETAC(80),DCPC(80),ATITL(20)
      DIMENSION P1(600),P2(600),A(600),B(600),GW(600),GB(600)
      DIMENSION BCW(600),BCB(600),DEL(600),COSTH(600)
      DIMENSION SLOPES(600),IPERM(600),IFREE(600),PRESS(600)
      EQUIVALENCE (BLOCK(1),BCW(1)),(BLOCK(601),BCB(1))                 &
     &,(BLOCK(5701),GW(1)),(BLOCK(6301),GB(1)),(BLOCK(1201),P1(1))      &
     &,(BLOCK(1801),P2(1)),(BLOCK(2401),A(1)),(BLOCK(3001),B(1))        &
     &,(BLOCK(1201),DEL(1)),(BLOCK(1801),COSTH(1)),(BLOCK(4801),SLOPES(1&
     &)),(BLOCK(3601),IPERM(1)),(BLOCK(4201),IFREE(1))                  &
     &,(BLOCK(6901),PRESS(1)),(ARRAY(1),XP(1)),(ARRAY(601),YP(1)),      &
     & (ARRAY(1201),ZP(1))
!
      COMMON /STAR/XSTAR(3600),NSEGW
      DIMENSION ZCSTR(600),ZTSTR(600),CPSTR(600),                       &
     &          DCNSTR(600),DCTSTR(600),DCMSTR(600)
      EQUIVALENCE (XSTAR(1)   , ZCSTR(1)),(XSTAR(601) , ZTSTR(1)),      &
     &            (XSTAR(1201), CPSTR(1)),(XSTAR(1801),DCNSTR(1)),      &
     &            (XSTAR(2401),DCTSTR(1)),(XSTAR(3001),DCMSTR(1))
      COMMON /SHELL/ NPTSB,NSEGWT,NPTSWT,NSEGN,NPTSN,NSEGS,NPTSS,       &
     &               NSEGNS,NPTSNS,NNAC,NSHELL,NQ, ipart,iidum
      LOGICAL THK
!
!     ASSIGN SCRATCH UNITS
!
      N1 = ITAPES(1)
      N2 = ITAPES(2)
      N3 = ITAPES(3)
      N4 = ITAPES(4)
      N5 = ITAPES(5)
      N6 = ITAPES(6)
      N7 = ITAPES(7)
      N8 = ITAPES(8)
      N9 = ITAPES(9)
      N10 = ITAPES(10)
!
!     MIXED ANALYSIS OPTION, NOPT(2) = 2
!     FULL INVERSE ANALYSIS, NOPT(2) = 3
!
!     NFIX = NUMBER OF PRESSURE CONSTRAINTS
!     IFREE(I) = 1, ITH BOX PRESSURE IS FIXED
!     IFREE(I) = 0, ITH BOX SLOPE IS FIXED
!
      ALP = ALPHA * DEGRAD
      SINAL = SIN( ALP )
      COSAL = COS( ALP )
      DO 1 I = 1, NWING
    1 IPERM(I) = I
      NFIX = NWING
      NSLOPE = NWING
      IF( NOPT(2) .EQ. 3 ) GO TO 9
      IF(NOPT(2).EQ.4) GO TO 180
      READ(TAPE5,5) NFIX
      NSLOPE = NWING - NFIX
      READ(TAPE5,5) ( IPERM(I), I = 1, NFIX )
    5 FORMAT(10I5)
    9 CONTINUE
      READ(TAPE5,10) ( PRESS(I), I = 1, NFIX )
      IF(NOPT(2).EQ.2) READ(TAPE5,10)(SLOPES(I),I=1,NSLOPE)
   10 FORMAT(7E10.2)
      GO TO 210
  180 READ(TAPE5,224) ATITL
      WRITE(TAPE6,226) ATITL
      READ(TAPE5,228) FCCC,XLAMLE,ETADR,XORIGC,YORIGC
      WRITE(TAPE6,230)FCCC,XLAMLE,ETADR,XORIGC,YORIGC
      IFCCC=FCCC
      DO 170 I=1,IFCCC
      READ(TAPE5,228) ETAC(I),DCPC(I)
  170 WRITE(TAPE6,232)ETAC(I),DCPC(I)
      WRITE(TAPE6,222)
      REWIND 7
      READ(TAPE7) ARRAY
      XLAMLE=0.0174533*XLAMLE
      XLAMLE=TAN(XLAMLE)
      NSLOPE=0
      NFIX=0
      NNWG=0
      NNCPT=0
      WRITE(TAPE6,231)
      DO 200 N=1,NSEG
      NC=NCOL(N)
      DO 200 NN=1,NC
      NR=NROW(N)
      NR1=NR+1
      DO 200 NNN=1,NR1
      NNWG=NNWG+1
      IF(NNN.EQ.NR1.AND.LOCPT(N).EQ.0) GO TO 195
      NNCPT=NNCPT+1
      ETA=1.E10
      IF(N.GT.NSEGW) GO TO 190
      ETA=XLAMLE*(YP(NNWG)-YORIGC)/(XP(NNWG)-XORIGC)
      IF(ETA.LT.0.0.OR.ETA.GT.1.0) ETA=1.0
      IF(ETA.LE.ETADR) GO TO 190
      NFIX=NFIX+1
      IPERM(NFIX)=NNCPT
      CALL INTERP(ETA,CPPRES,IFCCC,ETAC,DCPC)
      PRESS(NFIX)=CPPRES
      WRITE(TAPE6,234) NNCPT,XP(NNWG),YP(NNWG),ETA,CPPRES
      GO TO 195
  190 NSLOPE=NSLOPE+1
      SLOPES(NSLOPE)=CAMBER(NNCPT)
      WRITE(TAPE6,236) NNCPT,XP(NNWG),YP(NNWG),ETA,SLOPES(NSLOPE)
  195 CONTINUE
      IF(NNN.EQ.NR1) WRITE(6,238)
  200 CONTINUE
  208 CONTINUE
      IF(NSLOPE.EQ.0) NOPT(2)=3
      IF(NSLOPE.GT.0) NOPT(2)=2
      IF(NSLOPE.EQ.0) NSLOPE=NWING
  220 FORMAT(2X,2I5,2E14.5)
  222 FORMAT(///)
  224 FORMAT(20A4)
  226 FORMAT(//10X,29HCONICAL CAMBER PANEL ANALYSIS //10X,              &
     & 5HINPUT ,2X,20A4/)
  228 FORMAT(8F10.5)
  230 FORMAT(/9X,5HFCCCC,9X,6HXLAMLE,11X,5HETADR,9X,                    &
     & 6HXORIGC,9X,6HYORIGC/5F15.5//16X,3HETA,17X,3HDCP)
  231 FORMAT(//2X,5HPANEL,9X,1HX,15X,1HY,13X,3HETA,9X,8HPRESSURE,       &
     &       9X,5HSLOPE)
  232 FORMAT(10X,F10.5,10X,F10.5)
  234 FORMAT(I5,5(2X,F13.5))
  236 FORMAT(I5,3(2X,F13.5),15X,1(2X,F13.5))
  238 FORMAT(/)
!
  210 CONTINUE
!
!
!     SET UP IFREE AND IPERM VECTORS
!
      DO 15 I = 1, NWING
   15 IFREE(I) = 0
      DO 20 I = 1, NFIX
      J = IPERM(I)
   20 IFREE(J) = 1
      IF( NOPT(2) .EQ. 3 ) GO TO 49
!
!     MIXED ANALYSIS FOR WING ALONE OR WING/BODY CASES
!     --  LINEAR BOUNDARY CONDITIONS --
!
      IF( NWING .EQ. 0 ) GO TO 999
      REWIND TAPE11
      READ(TAPE11) DEL, COSTH
!
!     SAVE FIXED BOUNDARY CONDITIONS
!     IN VECTOR  BCW
!
      ICT = 0
      DO 16 I = 1, NWING
      IF( IFREE(I) .EQ. 1 ) GO TO 16
      ICT = ICT + 1
      BCW(I) = COSAL * SLOPES( ICT ) - SINAL * COSTH(I)
   16 END DO
!
!         ADD EFFECTS OF WING AND NACELLE THICKNESS
!         AND EFFECTS OF BODY (WHEN NACELLE OR SHELL IS PRESENT)
!
      REWIND TAPE16
      NCPTS=NWING
      DO 410 JP=1,2
      JPART=JP
      CALL WINGS(JPART)
      IF(.NOT.THK) GO TO 410
      IF(NWING.EQ.0) GO TO 410
      READ(TAPE16) A
      DO 400 I=1,NCPTS
      IF(IFREE(I).EQ.1) GO TO 400
      BCW(I)=BCW(I)-A(I)
  400 END DO
  410 END DO
      CALL WINGS(1)
      IF(NBODY.EQ.0.OR.NPTSNS.EQ.0) GO TO 430
      READ(TAPE16) A
      DO 420 I=1,NCPTS
      IF(IFREE(I).EQ.1) GO TO 420
      BCW(I)=BCW(I)-A(I)
  420 END DO
  430 IF(NPTSNS.NE.0) CALL WINGS(3)
      I1 = 0
      I2 = NFIX
      DO 30 I = 1, NWING
      IF( IFREE(I) .EQ. 0 ) GO TO 25
      I1 = I1 + 1
      IPERM(I1) = I
      GO TO 30
   25 I2 = I2 + 1
      IPERM(I2) = I
   30 END DO
!
!     IPERM VECTOR ORDERS REDUCED MATRIX, CONSTRAINED BOXES FIRST
!
      NF1 = NFIX + 1
   49 CONTINUE
      REWIND N5
      REWIND N3
      REWIND N4
      DO 60 I = 1, NWING
      READ(N5) ( A(J), J = 1, NWING )
      IF(NOPT(2).EQ.2 .AND. IFREE(I).EQ.1) GO TO 60
      DO 50 J = 1, NWING
      IP = IPERM(J)
   50 B(J)=A(IP)/2.0
      WRITE(N3) ( B(J), J = 1, NFIX )
      IF(NOPT(2).EQ.2) WRITE(N4)(B(J),J=NF1,NWING)
   60 END DO
      IF( NOPT(2) .EQ. 3 ) GO TO 64
!
!     CALCULATE UNCONSTRAINED PRESSURES FOR FREE WING BOXES
!
      REWIND N4
      CALL INV(NSLOPE,N4,N1,N2,N6,N7,N8,N9,N10,600,A,B,1)
!
!     INV( A22) IN ROW SORT ON UNIT N1
!
   64 CONTINUE
      DO 65 I = 1, NWING
   65 GW(I) = 0.0
!
!     FORM AWB * INV(ABB) * ( BOUNDARY CONDITIONS )
!
!     TRANSFER AWB * INV(ABB) TO UNIT N2 IN COLUMN SORT
!
      IF( NBODY .EQ. 0 ) GO TO 66
      CALL XFER(N5,N2,NBODY, NWING , A )
      REWIND N6
      WRITE(N6) ( BCB(J), J = 1, NBODY )
      REWIND N6
      REWIND N4
      CALL TRIXY(N2,N6,N4,NWING,NBODY,1,A,B,P1,600,IRET)
      REWIND N4
      READ(N4) ( GW(J), J = 1, NWING )
   66 CONTINUE
!
!     FOR MIXED ANALYSIS FORM COLUMN VECTOR (B) = - ( AR21 ) * (PRESS)
!     FOR FULL INVERSE ANALYSIS FORM  AR * PRESS + GW = GW
!
      REWIND N3
      DO 85 I = 1, NSLOPE
      READ(N3) (A(J) , J = 1, NFIX )
      SUM = 0.00
   85 B(I)=-SOSCAP(A,PRESS,SUM,NFIX,1,1,1,1)
      IF( NOPT(2) .EQ. 2 ) GO TO 84
      DO 67 I = 1, NWING
   67 BCW(I) = - B(I) + GW(I)
      GO TO 94
   84 CONTINUE
      ICT = NFIX
      DO 87 I = 1, NSLOPE
      ICT = ICT + 1
      J = IPERM(ICT)
   87 B(I) = B(I) + BCW(J) - GW(J)
   88 CONTINUE
!
!     FORM INV( A22) * ( B )
!
      REWIND N1
      DO 90 I = 1,  NSLOPE
      READ(N1) ( A(J) , J = 1,  NSLOPE )
      SUM = 0.00
   90 P2(I) = SOSCAP(A,B,SUM, NSLOPE,1,1,1,1)
!
!     CALCULATE BOUNDARY CONDITIONS ON WING BOXES WHERE
!     PRESSURES ARE PRESCRIBED
!
!     DETERMINE AR11 AND AR12 PARTITIONS OF REDUCED MATRIX
!
      REWIND N5
      REWIND N3
      REWIND N4
      DO 92 I = 1, NWING
      READ(N5) ( A(J) , J = 1, NWING )
      IF( IFREE(I) .EQ. 0 ) GO TO 92
      DO 91 J = 1, NWING
      IP = IPERM(J)
   91 B(J)=A(IP)/2.0
      WRITE(N3) ( B(J), J = 1, NFIX )
      WRITE(N4) ( B(J), J = NF1, NWING )
   92 END DO
      REWIND N3
      REWIND N4
      DO 93 I = 1, NFIX
      K = IPERM(I)
      READ(N3) ( A(J), J = 1, NFIX )
      SUM = 0.00
      X = SOSCAP(A,PRESS,SUM,NFIX,1,1,1,1)
      BCW(K)=X
      READ(N4) ( A(J) , J = 1, NSLOPE )
      SUM = 0.00
      X = SOSCAP(A,P2,SUM,NSLOPE,1,1,1,1)
      BCW(K) = BCW(K) + X + GW(K)
   93 END DO
   94 CONTINUE
!
!     REORDER WING PRESSURES
!
      I1 = 0
      I2 = 0
      DO 105 I = 1, NWING
      IF( IFREE(I) .EQ. 1 ) GO TO 95
      I1 = I1 + 1
      GW(I)=P2(I1)/2.0
      GO TO 105
   95 CONTINUE
      I2 = I2 + 1
      GW(I)=PRESS(I2)/2.0
  105 END DO
!
!          ADD EFFECTS OF WING AND NACELLE THICKNESS
!          AND EFFECTS OF BODY (WHEN NACELLE OR SHELL IS PRESENT)
!          TO WING CAMBERS
!
      REWIND TAPE16
      NCPTS=NWING
      DO 310 JP=1,2
      JPART=JP
      CALL WINGS(JPART)
      IF(.NOT.THK) GO TO 310
      IF(NWING.EQ.0) GO TO 310
      READ(TAPE16) A
      DO 300 I=1,NCPTS
  300 BCW(I)=BCW(I)+A(I)
  310 END DO
      CALL WINGS(1)
      IF(NBODY.EQ.0.OR.NPTSNS.EQ.0) GO TO 330
      READ(TAPE16) A
      DO 320 I=1,NCPTS
  320 BCW(I)=BCW(I)+A(I)
  330 IF(NPTSNS.NE.0) CALL WINGS(3)
!
!      DETERMINE CAMBER SLOPES FOR ALL WING BOXES
!
  110 CONTINUE
      REWIND TAPE11
      READ(TAPE11) DEL, COSTH
      DO 120 I = 1, NWING
  120 CAMBER(I) = ( BCW(I) + SINAL * COSTH(I) ) / COSAL
!
!     PRINT CAMBER SLOPES FOR ALL WING BOXES
!
      IF( NOPT(2) .EQ. 2 ) WRITE(TAPE6,130)
      IF( NOPT(2) .EQ. 3 ) WRITE(TAPE6,135)
  130 FORMAT(1H1,/2X,24H***  MIXED ANALYSIS  ***)
  135 FORMAT(1H1,/2X,31H***  FULL INVERSE ANALYSIS  ***)
  140 FORMAT(1H ,/2X,37HCAMBER SLOPES AT PANEL CONTROL POINTS,//)
      WRITE(TAPE6,140)
      CALL OUTW(TAPE6,CAMBER)
!
!     DETERMINE BODY PANEL SINGULARITY STRENGTHS
!
      IF( NBODY .EQ. 0 ) GO TO 999
      REWIND N5
!
!     DUMMY READS TO LOCATE ABW
!
      CALL XFER(N5,N1,NWING,NWING,A)
      CALL XFER(N5,N1,NBODY,NWING,A)
      CALL XFER(N5,N1,NWING,NBODY,A)
!
!     TRANSFER WING SINGULARITY STRENGTH VECTOR TO N2
!
      REWIND N2
      WRITE(N2) ( GW(J), J = 1, NWING )
      REWIND N2
      REWIND N3
!
!     FORM VECTOR BCB - ABW * GW
!
      CALL TRIXY(N1,N2,N3,NBODY,NWING,1,A,B,P1,600,IRET)
      REWIND N3
      READ(N3) ( A(J), J = 1, NBODY )
      DO 145 I = 1, NBODY
  145 B(I) = BCB(I) - A(I)
      REWIND N2
      WRITE(N2) ( B(J), J = 1, NBODY )
!
!     CALCULATE BODY PANEL SINGULARITY STRENGTHS
!     AND STORE IN VECTOR,  GB
!
      CALL XFER(N5,N1,NBODY,NBODY,A)
      REWIND N2
      REWIND N3
      CALL TRIXY(N1,N2,N3,NBODY,NBODY,1,A,B,P1,600,IRET)
      REWIND N3
      READ(N3) ( GB(J), J = 1, NBODY )
  999 CONTINUE
      RETURN
      END

      SUBROUTINE DRAG
      COMMON /PI/ PI,RADEG,DEGRAD
      COMMON /DESIGN/ CAMBER(600)
      COMMON /PARAM/ NBODY,NWING,NTAIL,LBC,THK,MACH,ALPHA,REFA
      COMMON /POINT/ ARRAY(6000)
      COMMON /SCRAT/ BLOCK(7500)
      COMMON /ROWS/ NWCOL, NWRPC(20)
      COMMON /SEG/ NSEG,NROW(20),NCOL(20),COSS(20),SINS(20),BTE(20),NWT(&
     &20),SPNW(20),XLEW(20),BLE(20),ZLEW(20)
      DIMENSION AREA(600), XBAR(600), BCB(600)
      DIMENSION A(1202), B(1202), C(1202)
      DIMENSION CHORD(600), DELTA(600), DZTDX(600)
      EQUIVALENCE ( ARRAY(4801),AREA(1)), ( ARRAY(1), XBAR(1) )         &
     &, ( BLOCK(1201), A(1)), ( BLOCK(2403),B(1)), ( BLOCK(3605),C(1))  &
     &, ( ARRAY(2401),DELTA(1)), ( ARRAY(3601), CHORD(1))               &
     &, ( BLOCK(6901), DZTDX(1)), (BLOCK(601),BCB(1))
      COMMON /TAPES/ ITAPES(20)
      COMMON /DES/ CLBAR, CMBAR
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAP&
!!!     &E9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15,TAPE16
      INCLUDE 'files.cmn'
      INTEGER COMPT,PRINT
!!!,TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE&
!!!     &8,TAPE9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15,TAPE16,
  INTEGER:: FFILE
!!!      COMMON /OPTION/ NOPT(10),NXSTNS,XSTN(20),XAPEX,YAPEX,XLAMDA
      INCLUDE 'option.cmn'
      COMMON /CYCLE/ NEWAIC, IPREP, NDRAG1, NDRAG2, IDIR
      COMMON /PSAVE/ IFREE(600), CTFREE, CNFREE
      COMMON /SHELL/ NPTSB,NSEGWT,NPTSWT,NSEGN,NPTSN,NSEGS,NPTSS,       &
     &               NSEGNS,NPTSNS,NNAC,NSHELL,NQ, ipart,iidum
      DIMENSION DEL(600),COSTH(600),DPART(600),DTOT(600)
      EQUIVALENCE (ARRAY(1801),DEL(1)),(ARRAY(3001),COSTH(1)),          &
     &            (ARRAY(4201),DPART(1)),(BLOCK(1),DTOT(1))
      DIMENSION GW(600),GB(600)
      EQUIVALENCE (BLOCK(5701),GW(1)),(BLOCK(6301),GB(1))
!
      LOGICAL THK
      DO 1 I = 1, NWING
    1 IFREE(I) = 0
!
!     ASSIGN SCRATCH UNITS
!
      N1 = ITAPES(1)
      N2 = ITAPES(2)
      N3 = ITAPES(3)
      N4 = ITAPES(4)
      N5 = ITAPES(5)
      N6 = ITAPES(6)
      N7 = ITAPES(7)
      N8 = ITAPES(8)
      N9 = ITAPES(9)
      N10 = ITAPES(10)
      IN = ITAPES(11)
      IUNIT = N5
      REWIND IUNIT
      REWIND IN
      READ(IN) ARRAY
      READ(IN) XBAR
      REWIND TAPE11
      READ(TAPE11) DEL,COSTH
      REWIND TAPE16
      NCPTS=NWING
      DO 300 I=1,NCPTS
  300 DTOT(I)=0.0
      DO 320 JP=1,2
      JPART=JP
      CALL WINGS(JPART)
      IF(.NOT.THK) GO TO 320
      IF(NWING.EQ.0) GO TO 320
      READ(TAPE16) DPART
      DO 310 I=1,NCPTS
  310 DTOT(I)=DTOT(I)+DPART(I)
  320 END DO
      CALL WINGS(1)
      IF(NBODY.EQ.0.OR.NPTSNS.EQ.0) GO TO 340
      READ(TAPE16) DPART
      DO 330 I=1,NCPTS
  330 DTOT(I)=DTOT(I)+DPART(I)
  340 IF(NPTSNS.NE.0) CALL WINGS(3)
      NN = NWING + 1
      NNN = NWING + 2
!
!
!     SET STORAGE FOR SUBROUTINE TRIXY
!
      NMAX = 1202
      IGEN = 1
!
!     IF MIXED DESIGN CASE, MUST READ REDUCED ARRAY FROM N5
!
      IF( NOPT(2) .LT. 0 ) GO TO 55
!
!     IF FULL DESIGN CASE, DETERMINE IF DRAG MATRIX FOR CURRENT
!     MACH NUMBER IS ON ITAPES(11)
!
      NCASE = NOPT(2)
      NSUM = NDRAG1 + NDRAG2
      IF( NSUM .EQ. 0 ) GO TO 55
      IF( NCASE .EQ. 1 .AND. NDRAG1 .EQ. 0 ) GO TO 55
      IF( NCASE .EQ. 2 .AND. NDRAG2 .EQ. 0 ) GO TO 55
      IGEN = 0
      READ(IN) ICASE, ISIZE
      CALL XFER(IN, N1, ISIZE, ISIZE, A )
      IF( ICASE .EQ. NCASE ) GO TO 129
      READ(IN) ICASE, ISIZE
      CALL XFER(IN, N1, ISIZE, ISIZE, A )
      GO TO 129
   55 CONTINUE
!
!     FOR WING ALONE, (AWW) IS FIRST FILE ON N5
!     FOR WING/BODY, REDUCED MATRIX,(AR) IS FIRST FILE ON N5
!
!     FORM DRAG MINIMIZATION MATRIX
!
      IUNIT = N5
      REWIND N1
      REWIND IUNIT
      DO 70 I = 1, NWING
      READ(IUNIT) ( A(J), J = 1, NWING )
      DO 60 J = 1, NWING
   60 A(J)=A(J)*AREA(I)/2.0
      WRITE(N1) ( A(J), J = 1, NWING )
   70 END DO
!
!     TAKE TRANSPOSE OF MATRIX ON N1
!
      REWIND N1
      REWIND N8
      REWIND N7
      CALL TRPOSE(N1,A,B,NWING,NWING,N8,N7,NMAX)
!
!     PARTIAL PRESSURE OPTION
!
      IF( NOPT(1) .EQ. 2 .AND. NOPT(2) .LT. 0 ) GO TO 129
!
!     ADD ELEMENTS OF MATRICES ON N1 AND N8
!
      REWIND N1
      REWIND N2
      DO 90 I = 1, NWING
      READ(N1) ( A(J), J = 1, NWING )
      READ(N8) ( B(J) , J = 1, NWING )
      DO 80 J = 1, NWING
   80 C(J) = A(J) + B(J)
!
!     NOPT(2) .EQ. 1 , CL CONSTRAINT ONLY
!     NOPT(2) .EQ. 2 , CL AND CM CONSTRAINT
!
      C(NN) = AREA(I) * COSTH(I)
      NUM = NN
      IF( NOPT(2) .EQ. 1 ) GO TO 90
      C(NNN) = AREA(I) * COSTH(I) * XBAR(I)
      NUM = NNN
   90 WRITE(N2) (C(J), J = 1, NUM )
      DO 110 I = 1, NWING
  110 A(I) = AREA(I) * COSTH(I)
      A(NN) = 0.0
      A(NNN)  = 0.0
      WRITE(N2) ( A(J), J = 1, NUM )
      DO 120 I = 1, NWING
  120 A(I) = AREA(I) * COSTH(I) * XBAR(I)
      A(NN) = 0.0
      A(NNN) = 0.0
      WRITE(N2) ( A(J) , J = 1, NUM )
      REWIND N2
!
!     INVERT DRAG MINIMIZATION MATRIX. PLACE ON N1 IN ROW SORT
!
      CALL INV(NUM,N2,N1,N3,N4,N6,N7,N8,N9,NMAX,A,B,1)
!
!     WRITE DRAG MINIMIZATION MATRIX ON UNIT IN
!
      IF( NSUM .EQ. 0 ) GO TO 123
      READ(IN) ICASE, ISIZE
      DO 122 I = 1, ISIZE
  122 READ(IN)
  123 WRITE(IN) NCASE, NUM
      REWIND N1
      DO 124 I = 1, NUM
      READ(N1) ( A(J), J = 1, NUM )
  124 WRITE(IN) ( A(J), J = 1, NUM )
      IF( NCASE .EQ. 1 ) NDRAG1 = 1
      IF( NCASE .EQ. 2 ) NDRAG2 = 1
      REWIND N1
  129 CONTINUE
!
!     FORM RIGHT HAND SIDE VECTOR FOR OPTIMIZATION
!
      DO 130 I = 1, NWING
      CAMBER(I) = 0.0
  130 B(I) = 0.0
      IF( NBODY .EQ. 0 ) GO TO 134
!
!      FORM AWB * INV(ABB) * C
!
      REWIND N2
      WRITE(N2) ( BCB(J), J = 1, NBODY )
      REWIND N2
      REWIND N3
      IF( IGEN .EQ. 1 ) GO TO 131
      CALL XFER( IUNIT, N10, NWING, NWING, A )
!
!     TRANSFER AWB*INV(ABB) IN COLUMN SORT TO UNIT N10
!
  131 CALL XFER(IUNIT,N10,NBODY,NWING,A)
      CALL TRIXY(N10,N2,N3,NWING,NBODY,1,A,B,C,NMAX,IRET )
      REWIND N10
      REWIND N3
      READ(N3) ( CAMBER(J), J = 1, NWING )
      DO 133 I = 1, NWING
  133 B(I) = - AREA(I) * ( CAMBER(I) + DTOT(I) )
  134 CONTINUE
      B(NN) = CLBAR * REFA
      B(NNN) = CLBAR * CMBAR * REFA
      IF( NOPT(2) .GT. 0 ) GO TO 135
      REWIND N10
      WRITE(N10) ( B(J), J = 1, NWING )
      CALL PCONST
!
!     SINGULARITY STRENGTHS ARE STORED IN ARRAY  C
!     BY SUBROUTINE PCONST
!
      GO TO 150
  135 CONTINUE
!
!     DOUBLE PRECISION WING PRESSURES FOR OPTIMUM DRAG
!     ARE STORED IN ARRAY @ C @
!
      REWIND N1
      DO 140 I = 1, NUM
      SUM = 0.00
      READ(N1) ( A(J), J = 1, NUM )
  140 C(I) = SOSCAP(A,B,SUM,NUM,1,1,1,1)
  150 CONTINUE
!
!     CALCULATE UPPER SURFACE PRESSURE
!
      DO 155 I = 1, NWING
  155 C(I) = C(I) * .5
!
!     CALCULATE WING CAMBER FOR OPTIMUM DRAG
!
      IUNIT = N5
      REWIND IUNIT
      DO 160 I = 1, NWING
      READ(N5) ( A(J), J = 1, NWING )
      SUM = 0.00
  160 B(I) = SOSCAP(A,C,SUM,NWING,1,1,1,1)
      IF( NBODY .EQ. 0 ) GO TO 170
      DO 165 I = 1, NWING
  165 B(I) = B(I) + CAMBER(I)
  170 CONTINUE
!
!          ADD EFFECTS OF WING AND NACELLE THICKNESS
!          AND EFFECTS OF BODY (WHEN NACELLE OR SHELL IS PRESENT)
!          TO WING CAMBERS
!
      ALP=ALPHA*DEGRAD
      SINAL=SIN(ALP)
      COSAL=COS(ALP)
      DO 175 I=1,NWING
      B(I)=B(I)+DTOT(I)
  175 B(I)=(B(I)+SINAL*COSTH(I))/COSAL
!
!      PRINT CAMBER SLOPES
!
      WRITE(TAPE6,200)
      IF(IABS(NOPT(2)).EQ.1) WRITE(TAPE6,400) CLBAR
      IF(IABS(NOPT(2)).EQ.2) WRITE(TAPE6,401) CLBAR,CMBAR
  400 FORMAT(1H ,23HWING DESIGNED FOR CL = ,1F10.5,/)
  401 FORMAT(1H ,23HWING DESIGNED FOR CL = ,1F10.5,5X,6HXCP = ,1F10.5,/)
      WRITE(TAPE6,422)
      CALL OUTW(TAPE6,B)
  422 FORMAT(1H ,/2X,37HCAMBER SLOPES AT PANEL CONTROL POINTS,//)
!
!     TRANSFER CAMBER SLOPES TO COMMON /DESIGN/
!
      DO 195 I = 1, NWING
  195 CAMBER(I) = B(I)
  199 CONTINUE
  200 FORMAT(1H1)
      DO 600 I = 1, NWING
      NSAVE = IFREE(I)
      IF( NSAVE .EQ. 0 ) IFREE(I) = 1
      IF( NSAVE .EQ. 1 ) IFREE(I) = 0
  600 END DO
!
!          STORE WING SINGULARITY STRENGTHS IN ARRAY GW
!
      DO 350 I=1,NWING
  350 GW(I)=C(I)
!
!          CALCULATE BODY SINGULARITY STRENGTHS
!          AND STORE IN ARRAY GB
!
      IF(NBODY.EQ.0) GO TO 370
      REWIND N5
      CALL XFER(N5,N1,NWING,NWING,DEL)
      CALL XFER(N5,N1,NBODY,NWING,DEL)
      CALL XFER(N5,N1,NWING,NBODY,DEL)
      REWIND N2
      WRITE(N2) (GW(I),I=1,NWING)
      REWIND N2
      REWIND N3
      CALL TRIXY(N1,N2,N3,NBODY,NWING,1,A,B,C,600,IRET)
      REWIND N3
      READ(N3) (A(I),I=1,NBODY)
      DO 360 I=1,NBODY
  360 B(I)=BCB(I)-A(I)
      REWIND N2
      WRITE(N2) (B(I),I=1,NBODY)
      CALL XFER(N5,N1,NBODY,NBODY,DEL)
      REWIND N2
      REWIND N3
      CALL TRIXY(N1,N2,N3,NBODY,NBODY,1,A,B,C,600,IRET)
      REWIND N3
      READ(N3) (GB(I),I=1,NBODY)
  370 CONTINUE
      RETURN
      END
      SUBROUTINE DIRECT
      COMMON /PI/ PI, RADEG, DEGRAD
      COMMON /DESIGN/ CAMBER(600)
      COMMON /PARAM/ NBODY,NWING,NTAIL,LBC,THK,MACH,ALPHA,REFA
      COMMON /POINT/ ARRAY(6000)
      COMMON /TAPES/ ITAPES(20)
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAP&
!!!     &E9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      INCLUDE 'files.cmn'
      INTEGER COMPT,PRINT
!!!,TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE&
!!!     &8,TAPE9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15,
  INTEGER:: FFILE
!!!      COMMON /OPTION/ NOPT(10),NXSTNS,XSTN(20),XAPEX,YAPEX,XLAMDA
      INCLUDE 'option.cmn'
      COMMON /SCRAT/ BLOCK(7500)
      COMMON /CYCLE/ NEWAIC,IPREP,NDRAG1,NDRAG2,IDIR
      DIMENSION P1(600),P2(600),A(600),B(600),GW(600),GB(600)
      DIMENSION BCW(600),BCB(600),DEL(600),COSTH(600)
      DIMENSION SLOPES(600),IPERM(600),IFREE(600),PRESS(600)
      EQUIVALENCE (BLOCK(1),BCW(1)),(BLOCK(601),BCB(1))                 &
     &,(BLOCK(5701),GW(1)),(BLOCK(6301),GB(1)),(BLOCK(1201),P1(1))      &
     &,(BLOCK(1801),P2(1)),(BLOCK(2401),A(1)),(BLOCK(3001),B(1))        &
     &,(BLOCK(1201),DEL(1)),(BLOCK(1801),COSTH(1)),(BLOCK(4801),SLOPES(1&
     &)),(BLOCK(3601),IPERM(1)),(BLOCK(4201),IFREE(1))                  &
     &,(BLOCK(6901),PRESS(1))
      COMMON /SHELL/ NPTSB,NSEGWT,NPTSWT,NSEGN,NPTSN,NSEGS,NPTSS,       &
     &               NSEGNS,NPTSNS,NNAC,NSHELL,NQ,IPART, iidum
!
!     ASSIGN SCRATCH UNITS
!
      N1 = ITAPES(1)
      N2 = ITAPES(2)
      N3 = ITAPES(3)
      N4 = ITAPES(4)
      N5 = ITAPES(5)
      N6 = ITAPES(6)
      N7 = ITAPES(7)
      N8 = ITAPES(8)
      N9 = ITAPES(9)
      N10 = ITAPES(10)
!
!     ****    BODY ALONE CASE    ****
!
      IF( NWING .NE. 0 ) GO TO 20
      REWIND N5
      IF( IDIR .EQ. 0 ) GO TO 5
      CALL XFER(N5,N10,NBODY,NBODY,A)
      CALL XFER(N5,N10,NBODY,NBODY,A)
      GO TO 10
    5 CONTINUE
!
!     INVERT ABB MATRIX , PUT ON N10 IN COLUMN SORT
!
      CALL INV(NBODY,N5,N1,N10,N2,N3,N4,N6,N7,600,A,B,0)
!
!     CALCULATE BODY PANEL SINGULARITY STRENGTHS
!
   10 CONTINUE
      REWIND N1
      WRITE(N1) ( BCB(J), J = 1, NBODY )
      REWIND N1
      REWIND N2
      REWIND N10
      CALL TRIXY(N10,N1,N2,NBODY,NBODY,1,A,B,P1,600,IRET)
!
!     BODY PANEL STRENGTHS ON UNIT N2
!
      REWIND N2
      READ(N2) ( GB(J), J = 1, NBODY )
      IF( IDIR .EQ. 1 ) GO TO 160
      REWIND N10
      DO 15 I = 1, NBODY
      READ(N10) ( A(J), J = 1, NBODY )
      WRITE(N5) ( A(J), J = 1, NBODY )
   15 END DO
      IF(IPART.NE.0) IDIR=1
      GO TO 160
   20 CONTINUE
!
!     ****    WING ALONE OR WING/BODY CASES    ****
!
      IF( IDIR .EQ. 0 ) GO TO 50
      REWIND N5
!
!     DUMMY READS TO LOCATE INV(AWW) OR INV(AR)
!
      CALL XFER(N5,N10,NWING,NWING,A)
      IF( NBODY .EQ. 0 ) GO TO 40
      CALL XFER(N5,N10,NBODY,NWING,A)
      CALL XFER(N5,N10,NWING,NBODY,A)
      CALL XFER(N5,N10,NBODY,NBODY,A)
   40 CONTINUE
      CALL XFER(N5,N10,NWING,NWING,A)
      GO TO 64
   50 CONTINUE
      REWIND N5
      REWIND N3
      REWIND N4
      DO 60 I = 1, NWING
      READ(N5) ( A(J), J = 1, NWING )
      WRITE(N3) ( A(J), J = 1, NWING )
   60 END DO
!
!     INVERT AWW(WING ALONE) OR AR(WING/BODY) MATRIX IF IDIR = 0
!
      REWIND N3
      CALL INV(NWING,N3,N2,N10,N6,N7,N8,N9,N4,600,A,B,0)
!
!     INV( AR) IN COLUMN SORT ON UNIT N10
!
   64 CONTINUE
      DO 65 I = 1, NWING
   65 GW(I) = 0.0
!
!     FORM AWB * INV(ABB) * ( BOUNDARY CONDITIONS )
!
!     TRANSFER AWB * INV(ABB) TO UNIT N2 IN COLUMN SORT
!
      IF( NBODY .EQ. 0 ) GO TO 66
      REWIND N5
      CALL XFER(N5,N2,NWING,NWING,A)
      CALL XFER(N5,N2,NBODY, NWING , A )
      REWIND N6
      WRITE(N6) ( BCB(J), J = 1, NBODY )
      REWIND N6
      REWIND N4
      CALL TRIXY(N2,N6,N4,NWING,NBODY,1,A,B,P1,600,IRET)
      REWIND N4
      READ(N4) ( GW(J), J = 1, NWING )
   66 CONTINUE
!
!     FORM RIGHT HAND SIDE, BCW - AWB * INV(ABB) * BCB
!
      DO 85 I = 1, NWING
   85 GW(I) = BCW(I) - GW(I)
!
!     CALCULATE WING PRESSURES
!
      REWIND N10
      REWIND N2
      REWIND N3
      WRITE(N2) ( GW(J), J = 1, NWING )
      REWIND N2
      CALL TRIXY(N10,N2,N3,NWING,NWING,1,A,B,P1,600,IRET)
!
!     WING SINGULARITY STRENGTHS N UNIT N3
!
      REWIND N3
      READ(N3) ( GW(J), J = 1, NWING )
!
!     DETERMINE BODY PANEL SINGULARITY STRENGTHS
!
      IF( NBODY .EQ. 0 ) GO TO 999
      REWIND N5
!
!     DUMMY READS TO LOCATE ABW
!
      CALL XFER(N5,N1,NWING,NWING,A)
      CALL XFER(N5,N1,NBODY,NWING,A)
      CALL XFER(N5,N1,NWING,NBODY,A)
!
!     TRANSFER WING SINGULARITY STRENGTH VECTOR TO N2
!
      REWIND N2
      WRITE(N2) ( GW(J), J = 1, NWING )
      REWIND N2
      REWIND N3
!
!     FORM VECTOR BCB - ABW * GW
!
      CALL TRIXY(N1,N2,N3,NBODY,NWING,1,A,B,P1,600,IRET)
      REWIND N3
      READ(N3) ( A(J), J = 1, NBODY )
      DO 145 I = 1, NBODY
  145 B(I) = BCB(I) - A(I)
      REWIND N2
      WRITE(N2) ( B(J), J = 1, NBODY )
!
!     CALCULATE BODY PANEL SINGULARITY STRENGTHS
!     AND STORE IN VECTOR,  GB
!
      CALL XFER(N5,N1,NBODY,NBODY,A)
      REWIND N2
      REWIND N3
      CALL TRIXY(N1,N2,N3,NBODY,NBODY,1,A,B,P1,600,IRET)
      REWIND N3
      READ(N3) ( GB(J), J = 1, NBODY )
  999 CONTINUE
!
!     WRITE INV(AWW) OR INV(AR) ON UNIT N5 ( IN COLUMN SORT ) IF
!     FIRST TIME THROUGH SUBROUTINE DIRECT FOR CURRENT MACH NUMBER
!
      IF( IDIR .EQ. 1 ) GO TO 160
      REWIND N10
      DO 150 I = 1, NWING
      READ(N10) ( A(J), J = 1, NWING )
  150 WRITE(N5) ( A(J), J = 1, NWING )
      IDIR = 1
  160 CONTINUE
      RETURN
      END
      SUBROUTINE PCONST
      COMMON /PARAM/ NBODY,NWING,NTAIL,LBC,THK,MACH,ALPHA,REFA
      COMMON /POINT/ ARRAY(6000)
      DIMENSION AREA(600), XBAR(600)
      DIMENSION RHS(600)
      EQUIVALENCE ( ARRAY(4801),AREA(1)), ( ARRAY(1), XBAR(1) )
      COMMON /TAPES/ ITAPES(20)
      COMMON /DES/ CLBAR, CMBAR
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAP&
!!!     &E9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      INCLUDE 'files.cmn'
      INTEGER COMPT,PRINT
!!!,TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE&
!!!     &8,TAPE9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15,
  INTEGER:: FFILE
!!!      COMMON /OPTION/ NOPT(10),NXSTNS,XSTN(20),XAPEX,YAPEX,XLAMDA
      INCLUDE 'option.cmn'
      COMMON /SCRAT/ BLOCK(7500)
      DIMENSION A(1202), B(1202), C(1202)
      DIMENSION XP(600),YP(600),ZP(600),ETAC(80),DCPC(80),ATITL(20)
      COMMON /NEWCOM/KDUM(9),KFUS,KRADX(4),KFORX(4),KDUMX(27),           &
     &              NCPT,LOCPT(20),XCPT(20)
      COMMON /SEG/NSEG,NROW(20),NCOL(20),COSS(20),SINS(20),              &
     &           BT(20),DUM(20),BL(20)
      COMMON /PSAVE/ IFREE(600), CTFREE, CNFREE
      DIMENSION IPERM(600), PRESS(602)
      EQUIVALENCE (BLOCK(1201),A(1)),(BLOCK(2403),B(1))                 &
     &,(BLOCK(3605),C(1)),(BLOCK(4807),PRESS(1))                        &
     &,(ARRAY(1),XP(1)),(ARRAY(601),YP(1)),(ARRAY(1201),ZP(1))
      DIMENSION DEL(600),COSTH(600)
      EQUIVALENCE (ARRAY(1801),DEL(1)),(ARRAY(3001),COSTH(1))
!
      COMMON /STAR/XSTAR(3600),NSEGW
!     ASSIGN SCRATCH UNITS
!
      N1 = ITAPES(1)
      N2 = ITAPES(2)
      N3 = ITAPES(3)
      N4 = ITAPES(4)
      N5 = ITAPES(5)
      N6 = ITAPES(6)
      N7 = ITAPES(7)
      N8 = ITAPES(8)
      N9 = ITAPES(9)
      N10 = ITAPES(10)
      IN=ITAPES(11)
!
!     READ IN RIGHT HAND SIDE VECTOR
!
      REWIND N10
      READ(N10) ( RHS(J), J = 1, NWING )
!
!     PARTIAL PRESSURE OPTION
!
!     NFIX = NUMBER OF PRESSURE CONSTRAINTS
!     IFREE(I) = 1, ITH BOX PRESSURE IS FIXED
!     IFREE(I) = 0, ITH BOX PRESSURE IS VARIABLE
!
      IF(NOPT(2).LE.-3) GO TO 180
      READ(TAPE5,5) NFIX
      READ(TAPE5,5) ( IPERM(I), I = 1, NFIX )
    5 FORMAT(10I5)
      READ(TAPE5,10) ( PRESS(I), I = 1, NFIX )
   10 FORMAT(7E10.2)
      GO TO 210
  180 READ(TAPE5,224) ATITL
      WRITE(TAPE6,226) ATITL
      READ(TAPE5,228) FCCC,XLAMLE,ETADR,XORIGC,YORIGC
      WRITE(TAPE6,230)FCCC,XLAMLE,ETADR,XORIGC,YORIGC
      IFCCC=FCCC
      DO 170 I=1,IFCCC
      READ(TAPE5,228) ETAC(I),DCPC(I)
  170 WRITE(TAPE6,232)ETAC(I),DCPC(I)
      WRITE(TAPE6,222)
      REWIND TAPE7
      READ(TAPE7) ARRAY
      XLAMLE=0.0174533*XLAMLE
      XLAMLE=TAN(XLAMLE)
      NSLOPE=0
      NFIX=0
      NNWG=0
      NNCPT=0
      WRITE(TAPE6,231)
      DO 200 N=1,NSEG
      NC=NCOL(N)
      DO 200 NN=1,NC
      NR=NROW(N)
      NR1=NR+1
      DO 200 NNN=1,NR1
      NNWG=NNWG+1
      IF(NNN.EQ.NR1.AND.LOCPT(N).EQ.0) GO TO 195
      NNCPT=NNCPT+1
      ETA=1.E10
      IF(N.GT.NSEGW) GO TO 193
      ETA=XLAMLE*(YP(NNWG)-YORIGC)/(XP(NNWG)-XORIGC)
      IF(ETA.LT.0.0.OR.ETA.GT.1.0) ETA=1.0
      IF(ETA.LE.ETADR) GO TO 193
      NFIX=NFIX+1
      IPERM(NFIX)=NNCPT
      CALL INTERP(ETA,CPPRES,IFCCC,ETAC,DCPC)
      PRESS(NFIX)=CPPRES
      WRITE(TAPE6,234) NNCPT,XP(NNWG),YP(NNWG),ETA,CPPRES
      GO TO 195
  193 CONTINUE
      WRITE(TAPE6,236) NNCPT,XP(NNWG),YP(NNWG),ETA
  195 CONTINUE
      IF(NNN.EQ.NR1) WRITE(6,238)
  200 CONTINUE
  220 FORMAT(I5,2F10.5)
  222 FORMAT(///)
  224 FORMAT(20A4)
  226 FORMAT(//10X,29HCONICAL CAMBER PANEL DESIGN   //10X,              &
     & 5HINPUT ,2X,20A4/)
  228 FORMAT(8F10.5)
  230 FORMAT(/9X,5HFCCCC,9X,6HXLAMLE,11X,5HETADR,9X,                    &
     & 6HXORIGC,9X,6HYORIGC/5F15.5//16X,3HETA,17X,3HDCP)
  231 FORMAT(//2X,5HPANEL,9X,1HX,15X,1HY,13X,3HETA,9X,8HPRESSURE)
  232 FORMAT(10X,F10.5,10X,F10.5)
  234 FORMAT(I5,5(2X,F13.5))
  236 FORMAT(I5,3(2X,F13.5),15X,1(2X,F13.5))
  238 FORMAT(/)
!
      REWIND IN
      READ(IN) ARRAY
      READ(IN) XBAR
      REWIND TAPE11
      READ(TAPE11) DEL,COSTH
  210 CONTINUE
!
!
!     SET UP IFREE AND IPERM VECTORS
!
      DO 15 I = 1, NWING
   15 IFREE(I) = 0
      DO 20 I = 1, NFIX
      J = IPERM(I)
   20 IFREE(J) = 1
      I1 = 0
      I2 = NFIX
      DO 30 I = 1, NWING
      IF( IFREE(I) .EQ. 0 ) GO TO 25
      I1 = I1 + 1
      IPERM(I1) = I
      GO TO 30
   25 I2 = I2 + 1
      IPERM(I2) = I
   30 END DO
!
!     IPERM VECTOR ORDERS DRAG MATRIX, CONSTRAINED BOXES FIRST
!
      IF(NOPT(2).EQ.-3) NOPT(2)=-1
      IF(NOPT(2).EQ.-4) NOPT(2)=-2
      NUM = NWING + IABS( NOPT(2))
      NF1 = NFIX + 1
      REWIND N1
      REWIND N8
      REWIND N3
      REWIND N4
      DO 60 I = 1, NWING
      READ(N1) ( A(J), J = 1, NWING )
      READ(N8) ( B(J), J = 1, NWING )
      IF( IFREE(I) .NE. 0 ) GO TO 60
      DO 40 J = 1, NWING
      IF( IFREE(J) .NE. 0 ) GO TO 35
      C(J) = A(J) + B(J)
      GO TO 40
   35 C(J) = A(J)
      IF( NOPT(5) .EQ. 0 ) GO TO 40
      C(J) = C(J) + B(J)
   40 END DO
      B(NWING+1) = AREA(I) * COSTH(I)
      B(NWING+2) = AREA(I) * COSTH(I) * XBAR(I)
      DO 50 J = 1, NWING
      IP = IPERM(J)
   50 B(J) = C(IP)
      WRITE(N3) ( B(J), J = 1, NFIX )
      WRITE(N4) ( B(J), J = NF1, NUM )
   60 END DO
      DO 70 I = 1, NWING
      IP = IPERM(I)
      A(I) = AREA(IP) * COSTH(IP)
   70 B(I) = AREA(IP) * COSTH(IP) * XBAR(IP)
      A(NWING + 1) = 0.0
      A(NWING + 2) = 0.0
      B(NWING + 1) = 0.0
      B(NWING + 2) = 0.0
      WRITE(N3) ( A(J), J = 1, NFIX )
      WRITE(N4) ( A(J), J = NF1, NUM )
      WRITE(N3) ( B(J), J = 1, NFIX )
      WRITE(N4) ( B(J), J = NF1, NUM )
!
!     CALCULATE UNCONSTRAINED PRESSURES FOR OPTIMUM DRAG
!     OF FREE WING BOXES
!
      NFREE = NWING - NFIX
      NX = NFREE + IABS( NOPT(2))
      REWIND N4
      CALL INV(NX,N4,N1,N2,N6,N7,N8,N9,N10,1202,A,B,1)
!
!     INV( A22) IN ROW SORT ON UNIT N1
!
      NF = 0
      DO 80 I = 1, NWING
      IF( IFREE(I) .EQ. 1 ) GO TO 80
      NF = NF + 1
      B(NF) = RHS(I)
   80 END DO
      B( NFREE + 1 ) = CLBAR * REFA
      B( NFREE + 2 ) = CMBAR * CLBAR * REFA
!
!     FORM COLUMN VECTOR (B) = ( (B) - ( A21) * (PRESS) )
!
      REWIND N3
      DO 85 I = 1, NX
      READ(N3) (A(J) , J = 1, NFIX )
      SUM = 0.0
      C(I) = SOSCAP(A,PRESS,SUM,NFIX,1,1,1,1)
   85 B(I) = B(I) - C(I)
!
!     FORM INV( A22) * ( B )
!
      REWIND N1
      DO 90 I = 1, NX
      READ(N1) ( C(J) , J = 1, NX )
      SUM = 0.0
   90 A(I) = SOSCAP(C,B,SUM,NX,1,1,1,1)
!
!     REORDER WING PRESSURES
!
      I1 = 0
      I2 = 0
      DO 105 I = 1, NWING
      IF( IFREE(I) .EQ. 1 ) GO TO 95
      I1 = I1 + 1
      C(I) = A(I1)
      GO TO 105
   95 CONTINUE
      I2 = I2 + 1
  100 C(I) = PRESS(I2)
  105 END DO
      RETURN
      END
      SUBROUTINE INTPOL(RATIOX,RATIOD)
!!!      COMMON /FILES/TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,     &
!!!     &TAPE9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      INCLUDE 'files.cmn'
      INTEGER COMPT
!!!,TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,    &
!!!     &TAPE9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      COMMON /POINT/ ARRAY(6000)
      COMMON /ROWS/ NWCOL, NWRPC(20)
      COMMON /DESIGN/ CAMBER(600)
      COMMON /NEWCAM/ CAMMID(600)
      DIMENSION CHORD(600)
      EQUIVALENCE ( ARRAY(3601),CHORD(1))
      R = RATIOD - RATIOX
      RR = - R
      J = 0
      DO 400 M = 1, NWCOL
      NR = NWRPC(M)
      J = J + 1
      CAMMID(J) = CAMBER(J)+R*(CAMBER(J+1)-CAMBER(J))/(1.+RATIOX        &
     &*(CHORD(J+1)/CHORD(J)-1.))
      DO 100 I = 2, NR
      J = J + 1
      CR = CHORD(J)/CHORD(J-1)
      CAMMID(J) = CAMBER(J)+CR*R*(CAMBER(J)-CAMBER(J-1))/(1.+           &
     &RATIOX*(CR-1.))
  100 END DO
  400 END DO
      WRITE(TAPE6,500)
  500 FORMAT(1H1,/,45H WING CAMBER SLOPES AT 75 PERCENT PANEL CHORD,/)
      CALL OUTW(TAPE6,CAMMID)
      RETURN
      END
      SUBROUTINE SHAPE
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAP&
!!!     &E9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      INCLUDE 'files.cmn'
      INTEGER COMPT,PRINT
!!!,TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE&
!!!     &8,TAPE9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15,
  INTEGER:: FFILE
      COMMON /PARAM/ NBODY,NWING,NTAIL,LBC,THK,MACH,ALPHA,REFA
      COMMON /DESIGN/ CAMBER(600)
      COMMON /ROWS/ NWCOL, NWRPC(20)
      COMMON /POINT/ ARRAY(6000)
      DIMENSION ALPHAC(600), CHORD(600), CSHAPE(600)
      DIMENSION TWIST(20), CHORDL(20)
      EQUIVALENCE ( ARRAY(1),ALPHAC(1)),(ARRAY(601),CSHAPE(1))          &
     &,(ARRAY(3601),CHORD(1)),(ARRAY(1201),TWIST(1))                    &
     &,(ARRAY(1221),CHORDL(1))
!
!     SUBROUTINE DETERMINES CAMBER SHAPE,TWIST,AND CHORD LENGTHS -
!     CORRESPONDES TO - CAMBER - IN A17 ( WOODWARD 1 PROGRAM )
!
      REWIND TAPE7
      READ(TAPE7) ARRAY, CHORD
      KKK = 0
      KK = 0
      K = 0
      DO 400 NCT = 1, NWCOL
      NR = NWRPC(NCT)
      CHORDL(NCT) = 0.0
      DO 200 I = 1, NR
      K = K + 1
  200 CHORDL(NCT) = CHORDL(NCT) + CHORD(K)
      SUM = 0.0
      DO 310 I = 1, NR
      KK = KK + 1
  310 SUM = SUM + CAMBER(KK) * CHORD(KK) / CHORDL(NCT)
      TWIST(NCT) = - SUM
      SUM = 0.0
      DO 320 I = 1, NR
      KKK = KKK + 1
      ALPHAC(KKK) = CAMBER(KKK) + TWIST(NCT)
      SUM = SUM + ALPHAC(KKK) * CHORD(KKK)/ CHORDL(NCT)
  320 CSHAPE(KKK) = SUM
  400 END DO
      WRITE(TAPE6,6055)
      WRITE(TAPE6,6000)(CHORDL(I),I=1,NWCOL)
      WRITE(TAPE6,6082)
      WRITE(TAPE6,6000)(TWIST(I),I=1,NWCOL)
      WRITE(TAPE6,6083)
      CALL OUTW(TAPE6,ALPHAC)
      WRITE(TAPE6,6084)
      CALL OUTW(TAPE6,CSHAPE)
 6000 FORMAT(1H ,F29.5,9F10.5)
 6055 FORMAT(1H0/23H  WING CHORD LENGTHS(C),/)
 6082 FORMAT(1H0/T5,36HTANGENT OF THE LOCAL INCIDENCE ANGLE,/)
 6083 FORMAT(1H0/T5,12HCAMBER SLOPE)
 6084 FORMAT(1H0/T5,12HCAMBER SHAPE)
      RETURN
      END



!!!     OVERLAY(LWB,4,0)
      SUBROUTINE ControlPlot()
!
!     *******************************************VERSION B00************
!
!
!     PROGRAM TO SELECT PROPER PLOT CONTROL PROGRAM
!
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAP&
!!!     &E9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      INCLUDE 'files.cmn'
      COMMON /BLANK2/ HORZ,VERT,TEST1,PHI,THETA,PSI,XF,YF,ZF,DIST,FMAG,P&
     &LOTSZ,TYPE,KODE
      COMMON /LWB/ LWB
      COMMON /GRAPH/ PLOT
      COMMON /CONPLT/ KONPLT
      COMMON /PTYPE/ ORT,VU3,STE,PER
!
      INTEGER PLOT
!!!,TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAPE9&
!!!     &,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15,
  INTEGER:: FFILE
!
      IF (KONPLT.EQ.0) GO TO 10
      IF (KONPLT.EQ.1) GO TO 60
   10 IF (PLOT) 20,70,40
!
!     COMPUTE AND SAVE GEOMETRY PANEL PLOTTING INFORMATION
!
!!!   20 CALL OVERLAY (LWB,4,1)
   20 CALL GpltSv()
   30 READ (TAPE13,80) HORZ,VERT,TEST1,PHI,THETA,PSI,XF,YF,ZF,DIST,FMAG,&
     &PLOTSZ,TYPE,KODE
!
!     GENERATE ORTHOGRAPHIC AND THREE VIEW PLOTS
!
!!!      IF (TYPE.EQ.ORT.OR.TYPE.EQ.VU3) CALL OVERLAY (LWB,4,2)
      IF (TYPE.EQ.ORT .OR. TYPE.EQ.VU3) CALL PltOrt()
!
!     GENERATE PERSPECTIVE AND STEREOSCOPIC PLOTS
!
!!!      IF (TYPE.EQ.STE.OR.TYPE.EQ.PER) CALL OVERLAY (LWB,4,3)
      IF (TYPE.EQ.STE .OR. TYPE.EQ.PER) CALL PltSte()
      IF (KODE.EQ.0) GO TO 30
   40 IF (PLOT.LT.0) PLOT=-PLOT
      REWIND TAPE12
!
!     COMPUTE AND SAVE SINGULARITY PANEL PLOTTING INFORMATION
!
!!!      CALL OVERLAY (LWB,4,4)
      CALL SpltSv()
   50 READ (TAPE13,80) HORZ,VERT,TEST1,PHI,THETA,PSI,XF,YF,ZF,DIST,FMAG,&
     &PLOTSZ,TYPE,KODE
      REWIND TAPE11
      READ (TAPE11) J0,J1,J2,J3,J4,J5,J6
!
!     GENERATE ORTHOGRAPHIC AND THREE VIEW PLOTS
!
!!!      IF (TYPE.EQ.ORT.OR.TYPE.EQ.VU3) CALL OVERLAY (LWB,4,2)
      IF (TYPE.EQ.ORT .OR. TYPE.EQ.VU3) CALL PltOrt()
!
!     GENERATE PERSPECTIVE AND STEREOSCOPIC PLOTS
!
!!!      IF (TYPE.EQ.STE.OR.TYPE.EQ.PER) CALL OVERLAY (LWB,4,3)
      IF (TYPE.EQ.STE .OR. TYPE.EQ.PER) CALL PltSte()
      IF (KODE.EQ.0) GO TO 50
      REWIND TAPE11
      GO TO 70
   60 CONTINUE
!
!     GENERATE PLOTS OF PRESSURE DISTRIBUTION
!
!!!      CALL OVERLAY (LWB,4,5)
      CALL Plpres()
   70 CONTINUE
!
!
!
   80 FORMAT (2(A1,1X),A3,9F5.0,A3,16X,I1)
      END
      SUBROUTINE AXES(XP,YP,ANG,AXLEN,FIRST,DELTAV,XDUM1,               &
     & XDUM2,IBCD,SIZE,NCHAR)
      DIMENSION IBCD(1)
      CALL AXIS(XP,YP,IBCD,NCHAR,AXLEN,ANG,FIRST,DELTAV)
      RETURN
      END
      SUBROUTINE GRID(X1,Y1,X2,Y2,NX,NY)
      RETURN
      END
      SUBROUTINE LINPLT(X,Y,N,INC,IDUM,ISYM,ICON,JDUM)
      DIMENSION X(1),Y(1)
      CALL LINE(X,Y,N,INC,0,0)
      RETURN
      END
      SUBROUTINE PNTPLT(X,Y,ISYM,ISIZE)
      HEIGHT=0.1*ISIZE
      CALL SYMBOL(X,Y,HEIGHT,ISYM+2,0.,1)
      RETURN
      END
      SUBROUTINE XYLINE(XVEC,YVEC,NPT,IA,IB,IC,ID)
      DIMENSION XVEC(1),YVEC(1)
      CALL LINE(XVEC,YVEC,NPT,IA,0,0)
      RETURN
      END



!!!     OVERLAY(LWB,4,1)
      SUBROUTINE GPLTSV
!
!     *******************************************VERSION B00************
!
!
!     SAVE GEOMETRY PANELING PLOT INFORMATION
!
!     BASED ON PROGRAM START OF NASA TMX-2074.
!
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAP&
!!!     &E9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      INCLUDE 'files.cmn'
      COMMON /BLANK/ J0,J1,J2,J3,J4,J5,J6,NWAF,NWAFOR,NFUS,NRADX(4),NFOR&
     &X(4),NP,NPODOR,NF,NFINOR,NCAN,NCANOR,J2TEST,NW,NC
      COMMON /BLANK2/ HORZ,VERT,TEST1,PHI,THETA,PSI,XF,YF,ZF,DIST,FMAG,P&
     &LOTSZ,TYPE,KODE,XMIN,XMAX,YMIN,YMAX,ZMIN,ZMAX,XMID,YMID,ZMID,BIGD,&
     &ISP
      COMMON /ONE/ NAN2
      COMMON /SCRAT/ BLOCK(7500)
      COMMON /PI/ PI,RADEG,DEGRAD
!
      DIMENSION XAF(30), WAFORG(20,4), WAFORD(20,3,30), TZORD(20,30)
      DIMENSION WAFOR(20,30), XFUS(30,4), ZFUS(30,4), FUSARD(30,4)
      DIMENSION FUSRAD(30,4), SFUS(30,30,8)
      DIMENSION PODORG(9,3), XPOD(9,30), PODORD(9,30), XPOD1(9,30)
      DIMENSION FINORG(6,2,4), XFIN(6,10), FINORD(6,2,10), FINX2(6,2,10)
      DIMENSION FINX3(6,2,10), FINOR(6,10), FINCR(6,10)
      DIMENSION CANORG(6,2,4), XCAN(6,10), CANORD(6,2,10), CANOR1(6,2,10)
      DIMENSION CANORX(6,2,10), CANOR(6,10), CANCR(6,10)
      DIMENSION ALRT(31,3,2), VECRT(30,3), ANSIN(30), ANCOS(30)

      EQUIVALENCE (BLOCK(1),XFUS(1,1))
      EQUIVALENCE (BLOCK(121),ZFUS(1,1))
      EQUIVALENCE (BLOCK(241),FUSARD(1,1))
      EQUIVALENCE (BLOCK(361),FUSRAD(1,1))
      EQUIVALENCE (BLOCK(241),SFUS(1,1,1))
      EQUIVALENCE (BLOCK(1),PODORG(1,1))
      EQUIVALENCE (BLOCK(28),XPOD(1,1))
      EQUIVALENCE (BLOCK(298),PODORD(1,1))
      EQUIVALENCE (BLOCK(568),XPOD1(1,1))
      EQUIVALENCE (BLOCK(1),XAF(1))
      EQUIVALENCE (BLOCK(31),WAFORG(1,1))
      EQUIVALENCE (BLOCK(111),WAFORD(1,1,1))
      EQUIVALENCE (BLOCK(1911),TZORD(1,1))
      EQUIVALENCE (BLOCK(2511),WAFOR(1,1))
      EQUIVALENCE (BLOCK(1),FINORG(1,1,1))
      EQUIVALENCE (BLOCK(49),XFIN(1,1))
      EQUIVALENCE (BLOCK(109),FINORD(1,1,1))
      EQUIVALENCE (BLOCK(229),FINX2(1,1,1))
      EQUIVALENCE (BLOCK(349),FINX3(1,1,1))
      EQUIVALENCE (BLOCK(469),FINOR(1,1))
      EQUIVALENCE (BLOCK(529),FINCR(1,1))
      EQUIVALENCE (BLOCK(1),CANORG(1,1,1))
      EQUIVALENCE (BLOCK(49),XCAN(1,1))
      EQUIVALENCE (BLOCK(109),CANORD(1,1,1))
      EQUIVALENCE (BLOCK(229),CANOR1(1,1,1))
      EQUIVALENCE (BLOCK(349),CANORX(1,1,1))
      EQUIVALENCE (BLOCK(469),CANOR(1,1))
      EQUIVALENCE (BLOCK(529),CANCR(1,1))
!
!!!      INTEGER TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAPE9,TAPE&
!!!     &10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
!
      PI2=PI*2.
      PI32=PI*(3./2.)
!
!     WING
!
      READ (TAPE11) J0,J1,J2,J3,J4,J5,J6
      IF (J1.EQ.0) GO TO 180
      READ (TAPE3) BLOCK
      XMIN=WAFORG(1,1)
      XMAX=WAFORG(1,1)
      YMAX=WAFORG(1,2)
      ZMIN=WAFORD(1,1,1)
      ZMAX=WAFORD(1,1,1)
      DO 20 N=1,NWAF
      XMAX=AMAX1(XMAX,WAFORD(N,3,NW))
      XMIN=AMIN1(XMIN,WAFORD(N,3,1))
      YMAX=AMAX1(YMAX,WAFORG(N,2))
      DO 10 NN=1,NW
      ZMAX=AMAX1(ZMAX,WAFORD(N,1,NN))
      ZMIN=AMIN1(ZMIN,WAFORD(N,2,NN))
   10 END DO
   20 END DO
!
!     WRITE PLOT TAPE
!
      NL1=NWAF-1
      NE1=NW-1
!
!     SETUP FIRST LINE IN STREAMWISE DIRECTION
!
      DO 170 I=1,2
      DO 30 N=1,NW
      ALRT(N,1,2)=WAFORD(1,3,N)
      ALRT(N,2,2)=WAFORG(1,2)
      ALRT(N,3,2)=WAFORD(1,I,N)
   30 END DO
      WRITE (TAPE12) ((ALRT(N,N3,2),N=1,NW),N3=1,3)
      DO 90 NN=2,NWAF
      DO 40 N=1,NW
      DO 40 N3=1,3
      ALRT(N,N3,1)=ALRT(N,N3,2)
   40 CONTINUE
      DO 50 N=1,NW
      ALRT(N,1,2)=WAFORD(NN,3,N)
      ALRT(N,2,2)=WAFORG(NN,2)
      ALRT(N,3,2)=WAFORD(NN,I,N)
   50 END DO
      IF(I.EQ.2) GO TO 70
   60 CALL SURCC (NW,ALRT,VECRT)
      GO TO 80
   70 CALL SURCL (NW,ALRT,VECRT)
   80 CONTINUE
      WRITE (TAPE12) ((VECRT(N,N3),N=1,NE1),N3=1,3)
      WRITE (TAPE12) ((ALRT(N,N3,2),N=1,NW),N3=1,3)
   90 END DO
      DO 100 N=1,NWAF
!
!     SETUP FIRST LINE IN SPANWISE DIRECTION
!
      ALRT(N,1,2)=WAFORD(N,3,1)
      ALRT(N,2,2)=WAFORG(N,2)
      ALRT(N,3,2)=WAFORD(N,I,1)
  100 END DO
      WRITE(TAPE12)((ALRT(N,N3,2),N=1,NWAF),N3=1,3)
      DO 160 NN=2,NW
      DO 110 N=1,NWAF
      DO 110 N3=1,3
      ALRT(N,N3,1)=ALRT(N,N3,2)
  110 CONTINUE
      DO 120 N=1,NWAF
      ALRT(N,1,2)=WAFORD(N,3,NN)
      ALRT(N,2,2)=WAFORG(N,2)
      ALRT(N,3,2)=WAFORD(N,I,NN)
  120 END DO
      IF(I.EQ.2) GO TO 140
  130 CALL SURCL (NWAF,ALRT,VECRT)
      GO TO 150
  140 CALL SURCC (NWAF,ALRT,VECRT)
  150 CONTINUE
      WRITE (TAPE12) ((VECRT(N,N3),N=1,NL1),N3=1,3)
      WRITE (TAPE12) ((ALRT(N,N3,2),N=1,NWAF),N3=1,3)
  160 END DO
  170 END DO
!
!     FUSELAGE (BODY)
!
  180 IF (J2.EQ.0) GO TO 460
      READ (TAPE3) BLOCK
!
!     MAXIMA AND MINIMA
!
      IF (J1.NE.0) GO TO 200
      XMIN=XFUS(1,1)
      XMAX=XFUS(1,1)
      IF (J2TEST.EQ.3) GO TO 190
      YMAX=FUSRAD(1,1)
      ZMIN=-FUSRAD(1,1)+ZFUS(1,1)
      ZMAX=FUSRAD(1,1)+ZFUS(1,1)
      GO TO 200
  190 YMAX=SFUS(1,1,1)
      ZMIN=SFUS(1,1,2)
      ZMAX=SFUS(1,1,2)
  200 DO 240 N=1,NFUS
      NRAD=NRADX(N)
      NFUSOR=NFORX(N)
      XMIN=AMIN1(XMIN,XFUS(1,N))
      XMAX=AMAX1(XMAX,XFUS(NFUSOR,N))
      DO 230 NN=1,NFUSOR
      IF (J2TEST.EQ.3) GO TO 210
      YMAX=AMAX1(YMAX,FUSRAD(NN,N))
      ZMAX=AMAX1(ZMAX,FUSRAD(NN,N)+ZFUS(NN,N))
      ZMIN=AMIN1(ZMIN,-FUSRAD(NN,N)+ZFUS(NN,N))
      GO TO 230
  210 KK=1+(N-1)*2
      DO 220 NR=1,NRAD
      YMAX=AMAX1(YMAX,SFUS(NR,NN,KK))
      ZMIN=AMIN1(ZMIN,SFUS(NR,NN,KK+1))
  220 ZMAX=AMAX1(ZMAX,SFUS(NR,NN,KK+1))
  230 END DO
  240 END DO
!
!     WRITE PLOT TAPE
!
!
!     SETUP FIRST LINE IN STREAMWISE DIRECTION
!
      DO 450 NFU=1,NFUS
      NRAD=NRADX(NFU)
      NFUSOR=NFORX(NFU)
      NL1=NFUSOR-1
      NAN=NRAD
      IF (J2TEST.EQ.3) GO TO 260
      FANG=(NRAD-1)*2
      DELE=PI2/FANG
      DO 250 N=1,NAN
      E=N-1
      ANSIN(N)=SIN(E*DELE+PI32)
  250 ANCOS(N)=COS(E*DELE+PI32)
  260 CONTINUE
      KK=1+(NFU-1)*2
      DO 290 N=1,NFUSOR
      ALRT(N,1,2)=XFUS(N,NFU)
      IF (J2TEST.EQ.3) GO TO 270
      ALRT(N,2,2)=FUSRAD(N,NFU)*ANCOS(1)
      ALRT(N,3,2)=FUSRAD(N,NFU)*ANSIN(1)+ZFUS(N,NFU)
      GO TO 280
  270 ALRT(N,2,2)=SFUS(1,N,KK)
      ALRT(N,3,2)=SFUS(1,N,KK+1)
  280 CONTINUE
  290 END DO
      WRITE (TAPE12) ((ALRT(N,N3,2),N=1,NFUSOR),N3=1,3)
      DO 340 NN=2,NAN
      DO 300 N=1,NFUSOR
      DO 300 N3=1,3
      ALRT(N,N3,1)=ALRT(N,N3,2)
  300 CONTINUE
      DO 330 N=1,NFUSOR
      IF (J2TEST.EQ.3) GO TO 310
      ALRT(N,2,2)=FUSRAD(N,NFU)*ANCOS(NN)
      ALRT(N,3,2)=FUSRAD(N,NFU)*ANSIN(NN)+ZFUS(N,NFU)
      GO TO 320
  310 ALRT(N,2,2)=SFUS(NN,N,KK)
      ALRT(N,3,2)=SFUS(NN,N,KK+1)
  320 CONTINUE
  330 END DO
      CALL SURCL (NFUSOR,ALRT,VECRT)
      WRITE (TAPE12) ((VECRT(N,N3),N=1,NL1),N3=1,3)
      WRITE (TAPE12) ((ALRT(N,N3,2),N=1,NFUSOR),N3=1,3)
  340 END DO
!
!     SETUP FIRST LINE AROUND BODY (FUSELAGE)
!
      NRAD=NRADX(NFU)
      NFUSOR=NFORX(NFU)
      NAN=NRAD
      NL1=NAN-1
      IF (J2TEST.EQ.3) GO TO 360
      FANG=(NRAD-1)*2
      DELE = PI2/FANG
      DO 350 N=1,NAN
      E=N-1
      ANSIN(N)=SIN(E*DELE+PI32)
  350 ANCOS(N)=COS(E*DELE+PI32)
  360 CONTINUE
      KK=1+(NFU-1)*2
      DO 390 N=1,NAN
      ALRT(N,1,2)=XFUS(1,NFU)
      IF (J2TEST.EQ.3) GO TO 370
      ALRT(N,2,2)=FUSRAD(1,NFU)*ANCOS(N)
      ALRT(N,3,2)=FUSRAD(1,NFU)*ANSIN(N)+ZFUS(1,NFU)
      GO TO 380
  370 ALRT(N,2,2)=SFUS(N,1,KK)
      ALRT(N,3,2)=SFUS(N,1,KK+1)
  380 CONTINUE
  390 END DO
      WRITE (TAPE12) ((ALRT(N,N3,2),N=1,NAN),N3=1,3)
      DO 440 NN=2,NFUSOR
      DO 400 N=1,NAN
      DO 400 N3=1,3
      ALRT(N,N3,1)=ALRT(N,N3,2)
  400 CONTINUE
      DO 430 N=1,NAN
      ALRT(N,1,2)=XFUS(NN,NFU)
      IF (J2TEST.EQ.3) GO TO 410
      ALRT(N,2,2)=FUSRAD(NN,NFU)*ANCOS(N)
      ALRT(N,3,2)=FUSRAD(NN,NFU)*ANSIN(N)+ZFUS(NN,NFU)
      GO TO 420
  410 ALRT(N,2,2)=SFUS(N,NN,KK)
      ALRT(N,3,2)=SFUS(N,NN,KK+1)
  420 CONTINUE
  430 END DO
      CALL SURCC (NAN,ALRT,VECRT)
      WRITE (TAPE12) ((VECRT(N,N3),N=1,NL1),N3=1,3)
      WRITE (TAPE12) ((ALRT(N,N3,2),N=1,NAN),N3=1,3)
  440 END DO
  450 END DO
!
!     POD (NACELLE)
!
  460 IF (J3.EQ.0) GO TO 620
      READ (TAPE3) BLOCK
!
!     COMPUTE ACTUAL X, MINIMUM, MAXIMUM
!
      DO 470 N=1,NP
      DO 470 NN=1,NPODOR
  470 XPOD1(N,NN)=XPOD(N,NN)+PODORG(N,1)
      IF (J1.NE.0.OR.J2.NE.0) GO TO 480
      XMIN=XPOD1(1,1)
      XMAX=XPOD1(1,NPODOR)
      YMAX=PODORG(1,2)+PODORD(1,1)
      ZMIN=PODORG(1,3)-PODORD(1,1)
      ZMAX=PODORG(1,3)+PODORD(1,1)
  480 DO 500 N=1,NP
      XMIN=AMIN1(XMIN,XPOD1(N,1))
      XMAX=AMAX1(XMAX,XPOD1(N,NPODOR))
      DO 490 NN=1,NPODOR
      YMAX=AMAX1(YMAX,PODORD(N,NN)+PODORG(N,2))
      ZMIN=AMIN1(ZMIN,PODORG(N,3)-PODORD(N,NN))
  490 ZMAX=AMAX1(ZMAX,PODORG(N,3)+PODORD(N,NN))
  500 END DO
      NANG1=NAN2+1
      FANG=NAN2
      DELE=PI2/FANG
      DO 510 N=1,NANG1
      E=N-1
      ANSIN(N)=SIN(E*DELE)
  510 ANCOS(N)=COS(E*DELE)
!
!     WRITE PLOT TAPE
!
      NL1=NPODOR-1
!
!     SETUP FIRST LINE IN STREAMWISE DIRECTION
!
      DO 560 NP1=1,NP
      DO 520 N=1,NPODOR
      ALRT(N,1,2)=XPOD(NP1,N)+PODORG(NP1,1)
      ALRT(N,2,2)=PODORD(NP1,N)*ANCOS(1)+PODORG(NP1,2)
      ALRT(N,3,2)=PODORD(NP1,N)*ANSIN(1)+PODORG(NP1,3)
  520 END DO
      WRITE (TAPE12) ((ALRT(N,N3,2),N=1,NPODOR),N3=1,3)
      DO 550 NN=2,NANG1
      DO 530 N=1,NPODOR
      DO 530 N3=1,3
      ALRT(N,N3,1)=ALRT(N,N3,2)
  530 CONTINUE
      DO 540 N=1,NPODOR
      ALRT(N,2,2)=PODORD(NP1,N)*ANCOS(NN)+PODORG(NP1,2)
      ALRT(N,3,2)=PODORD(NP1,N)*ANSIN(NN)+PODORG(NP1,3)
  540 END DO
      CALL SURCL (NPODOR,ALRT,VECRT)
      WRITE (TAPE12) ((VECRT(N,N3),N=1,NL1),N3=1,3)
      WRITE (TAPE12) ((ALRT(N,N3,2),N=1,NPODOR),N3=1,3)
  550 END DO
  560 END DO
!
!     SETUP FIRST LINE AROUND POD (NACELLE)
!
      NL1=NANG1-1
      DO 610 NP1=1,NP
      DO 570 N=1,NANG1
      M=N
      ALRT(M,1,2)=XPOD(NP1,1)+PODORG(NP1,1)
      ALRT(M,2,2)=PODORD(NP1,1)*ANCOS(N)+PODORG(NP1,2)
      ALRT(M,3,2)=PODORD(NP1,1)*ANSIN(N)+PODORG(NP1,3)
  570 END DO
      WRITE (TAPE12) ((ALRT(N,N3,2),N=1,NANG1),N3=1,3)
!
      DO 600 NN=2,NPODOR
      DO 580 N=1,NANG1
      DO 580 N3=1,3
      ALRT(N,N3,1)=ALRT(N,N3,2)
  580 CONTINUE
      DO 590 N=1,NANG1
      M=N
      ALRT(M,1,2)=XPOD(NP1,NN)+PODORG(NP1,1)
      ALRT(M,2,2)=PODORD(NP1,NN)*ANCOS(N)+PODORG(NP1,2)
      ALRT(M,3,2)=PODORD(NP1,NN)*ANSIN(N)+PODORG(NP1,3)
  590 END DO
      CALL SURCC (NANG1,ALRT,VECRT)
      WRITE (TAPE12) ((VECRT(N,N3),N=1,NL1),N3=1,3)
      WRITE (TAPE12) ((ALRT(N,N3,2),N=1,NANG1),N3=1,3)
  600 END DO
  610 END DO
!
!     FIN (VERTICAL TAIL)
!
  620 IF (J4.EQ.0) GO TO 890
      READ (TAPE3) BLOCK
      IF (J1.NE.0.OR.J2.NE.0.OR.J3.NE.0) GO TO 630
      XMIN=FINORG(1,1,1)
      XMAX=FINORG(1,1,1)
      YMAX=FINORG(1,1,2)
      ZMIN=FINORG(1,1,3)
      ZMAX=FINORG(1,1,3)
  630 DO 640 N=1,NF
      ZMIN=AMIN1(ZMIN,FINORG(N,1,3))
      ZMAX=AMAX1(ZMAX,FINORG(N,2,3))
      DO 640 N2=1,2
      XMIN=AMIN1(XMIN,FINORG(N,N2,1))
      XMAX=AMAX1(XMAX,FINX3(N,N2,NFINOR))
      DO 640 NN=1,NFINOR
      YMAX=AMAX1(YMAX,FINORD(N,N2,NN))
  640 CONTINUE
!
!     WRITE PLOT TAPE
!
      NL1=NFINOR-1
      NOR=2
      DO 880 NF1=1,NF
      DO 870 I=1,2
      DO 670 N=1,NFINOR
      ALRT(N,1,2)=FINX3(NF1,1,N)
      IF (I.EQ.2) GO TO 650
      ALRT(N,2,2)=FINORD(NF1,1,N)
      GO TO 660
  650 ALRT(N,2,2)=FINX2(NF1,1,N)
  660 ALRT(N,3,2)=FINORG(NF1,1,3)
  670 END DO
      WRITE (TAPE12) ((ALRT(N,N3,2),N=1,NFINOR),N3=1,3)
!
!     SETUP LOWER AND UPPER LINES IN STREAMWISE DIRECTION
!
      DO 750 NR=2,NOR
      DO 680 N=1,NFINOR
      DO 680 N3=1,3
      ALRT(N,N3,1)=ALRT(N,N3,2)
  680 CONTINUE
      DO 710 N=1,NFINOR
      ALRT(N,1,2)=FINX3(NF1,NR,N)
      IF (I.EQ.2) GO TO 690
      ALRT(N,2,2)=FINORD(NF1,NR,N)
      GO TO 700
  690 ALRT(N,2,2)=FINX2(NF1,NR,N)
  700 ALRT(N,3,2)=FINORG(NF1,NR,3)
  710 END DO
      IF(I.EQ.2) GO TO 730
  720 CALL SURCL(NFINOR,ALRT,VECRT)
      GO TO 740
  730 CALL SURCC(NFINOR,ALRT,VECRT)
  740 CONTINUE
      WRITE (TAPE12) ((VECRT(N,N3),N=1,NL1),N3=1,3)
      WRITE (TAPE12) ((ALRT(N,N3,2),N=1,NFINOR),N3=1,3)
  750 END DO
!
!     SETUP LINES IN VERTICAL DIRECTION
!
      DO 780 N=1,NOR
      ALRT(N,1,2)=FINX3(NF1,N,1)
      IF (I.EQ.2) GO TO 760
      ALRT(N,2,2)=FINORD(NF1,N,1)
      GO TO 770
  760 ALRT(N,2,2)=FINX2(NF1,N,1)
  770 ALRT(N,3,2)=FINORG(NF1,N,3)
  780 END DO
      WRITE (TAPE12) ((ALRT(N,N3,2),N=1,NOR),N3=1,3)
      DO 860 NN=2,NFINOR
      DO 790 N3=1,3
      DO 790 N=1,NOR
      ALRT(N,N3,1)=ALRT(N,N3,2)
  790 CONTINUE
      NE1=NOR-1
      DO 820 N=1,NOR
      ALRT(N,1,2)=FINX3(NF1,N,NN)
      IF (I.EQ.2) GO TO 800
      ALRT(N,2,2)=FINORD(NF1,N,NN)
      GO TO 810
  800 ALRT(N,2,2)=FINX2(NF1,N,NN)
  810 ALRT(N,3,2)=FINORG(NF1,N,3)
  820 END DO
      IF(I.EQ.2) GO TO 840
  830 CALL SURCC (NOR,ALRT,VECRT)
      GO TO 850
  840 CALL SURCL (NOR,ALRT,VECRT)
  850 CONTINUE
      WRITE (TAPE12) ((VECRT(N,N3),N=1,NE1),N3=1,3)
      WRITE (TAPE12) ((ALRT(N,N3,2),N=1,NOR),N3=1,3)
  860 END DO
  870 END DO
  880 END DO
!
!     CANARD (HORIZONTAL TAIL)
!
  890 IF (J5.EQ.0) GO TO 1100
      READ (TAPE3) BLOCK
      IF (J1.NE.0.OR.J2.NE.0.OR.J3.NE.0.OR.J4.NE.0) GO TO 900
      XMIN=CANORX(1,1,1)
      XMAX=CANORX(1,1,NCANOR)
      YMAX=CANORG(1,2,2)
      ZMIN=CANOR1(1,1,1)
      ZMAX=CANORD(1,1,1)
  900 DO 930 NCA=1,NCAN
      YMAX=AMAX1(YMAX,CANORG(NCA,2,2))
      DO 920 N2=1,2
      XMIN=AMIN1(XMIN,CANORX(NCA,N2,1))
      XMAX=AMAX1(XMAX,CANORX(NCA,N2,NCANOR))
      DO 910 NN=1,NCANOR
      ZMIN=AMIN1(ZMIN,CANOR1(NCA,N2,NN))
  910 ZMAX=AMAX1(ZMAX,CANORD(NCA,N2,NN))
  920 END DO
  930 END DO
!
!     WRITE PLOT TAPE
!
      NL1=NC-1
!
!     SETUP LOWER AND UPPER LINES IN STREAMWISE DIRECTION
!
      DO 1090 NCA=1,NCAN
      DO 1080 I=1,2
      DO 950 N2=1,2
      DO 950 N=1,NC
      ALRT(N,1,N2)=CANORX(NCA,N2,N)
      ALRT(N,2,N2)=CANORG(NCA,N2,2)
      IF (I.EQ.2) GO TO 940
      ALRT(N,3,N2)=CANORD(NCA,N2,N)
      GO TO 950
  940 ALRT(N,3,N2)=CANOR1(NCA,N2,N)
  950 CONTINUE
      IF(I.EQ.2) GO TO 970
  960 CALL SURCC (NC,ALRT,VECRT)
      GO TO 980
  970 CALL SURCL (NC,ALRT,VECRT)
  980 CONTINUE
      WRITE (TAPE12) ((ALRT(N,N3,1),N=1,NC),N3=1,3)
      WRITE (TAPE12) ((VECRT(N,N3),N=1,NL1),N3=1,3)
      WRITE (TAPE12) ((ALRT(N,N3,2),N=1,NC),N3=1,3)
!
!     SETUP LINES IN SPANWISE DIRECTION
!
      DO 1000 N2=1,2
      ALRT(N2,1,2)=CANORX(NCA,N2,1)
      ALRT(N2,2,2)=CANORG(NCA,N2,2)
      IF (I.EQ.2) GO TO 990
      ALRT(N2,3,2)=CANORD(NCA,N2,1)
      GO TO 1000
  990 ALRT(N2,3,2)=CANOR1(NCA,N2,1)
 1000 END DO
      WRITE (TAPE12) ((ALRT(N2,N3,2),N2=1,2),N3=1,3)
      DO 1070 NN=2,NC
      DO 1010 N3=1,3
      DO 1010 N2=1,2
      ALRT(N2,N3,1)=ALRT(N2,N3,2)
 1010 CONTINUE
      DO 1030 N2=1,2
      ALRT(N2,1,2)=CANORX(NCA,N2,NN)
      ALRT(N2,2,2)=CANORG(NCA,N2,2)
      IF (I.EQ.2) GO TO 1020
      ALRT(N2,3,2)=CANORD(NCA,N2,NN)
      GO TO 1030
 1020 ALRT(N2,3,2)=CANOR1(NCA,N2,NN)
 1030 END DO
      IF(I.EQ.2) GO TO 1050
 1040 CALL SURCL (2,ALRT,VECRT)
      GO TO 1060
 1050 CALL SURCC (2,ALRT,VECRT)
 1060 CONTINUE
      WRITE (TAPE12) (VECRT(1,N3),N3=1,3)
      WRITE (TAPE12) ((ALRT(N2,N3,2),N2=1,2),N3=1,3)
 1070 END DO
 1080 END DO
 1090 END DO
!
!     FIND MAXIMUM DISTANCE AND MIDPOINT
!
 1100 CONTINUE
      YMIN=-YMAX
      XDIS=XMAX-XMIN
      YDIS=YMAX-YMIN
      ZDIS=ZMAX-ZMIN
      BIGD=AMAX1(XDIS,YDIS,ZDIS)
      XMID=.5*(XMAX-XMIN)+XMIN
      YMID=0.
      ZMID=.5*(ZMAX-ZMIN)+ZMIN
!
!
      END
      SUBROUTINE SURCL (NPT,FLINE,FVEC)
!
!     *******************************************VERSION B00************
!
!
!     COMPUTE OUTWARD NORMAL VECTORS WITH FOUR ADJOINING INPUT POINTS
!     USED IN A CLOCKWISE DIRECTION
!
!     BASED ON SUBROUTINE SURCL OF NASA TMX-2074.
!
      DIMENSION FLINE(31,3,2), FVEC(30,3)
!
!
      DO 20 N=2,NPT
      T1X=FLINE(N,1,2)-FLINE(N-1,1,1)
      T2X=FLINE(N-1,1,2)-FLINE(N,1,1)
      T1Y=FLINE(N,2,2)-FLINE(N-1,2,1)
      T2Y=FLINE(N-1,2,2)-FLINE(N,2,1)
      T1Z=FLINE(N,3,2)-FLINE(N-1,3,1)
      T2Z=FLINE(N-1,3,2)-FLINE(N,3,1)
      XNX=T2Y*T1Z-T1Y*T2Z
      YNY=T1X*T2Z-T2X*T1Z
      ZNZ=T2X*T1Y-T1X*T2Y
      FN=SQRT(XNX**2+YNY**2+ZNZ**2)
      IF (FN.EQ.0.) GO TO 10
      FVEC(N-1,1)=XNX/FN
      FVEC(N-1,2)=YNY/FN
      FVEC(N-1,3)=ZNZ/FN
      GO TO 20
   10 FVEC(N-1,1)=0.
      FVEC(N-1,2)=0.
      FVEC(N-1,3)=0.
   20 END DO
      RETURN
      END
      SUBROUTINE SURCC (NPT,FLINE,FVEC)
!
!     *******************************************VERSION B00************
!
!
!     COMPUTE OUTWARD NORMAL VECTORS WITH FOUR ADJOINING INPUT POINTS
!     USED IN A COUNTERCLOCKWISE DIRECTION
!
!     BASED ON SUBROUTINE SURCC OF NASA TMX-2074.
!
      DIMENSION FLINE(31,3,2), FVEC(30,3)
!
!
      DO 20 N=2,NPT
      T1X=FLINE(N,1,2)-FLINE(N-1,1,1)
      T2X=FLINE(N,1,1)-FLINE(N-1,1,2)
      T1Y=FLINE(N,2,2)-FLINE(N-1,2,1)
      T2Y=FLINE(N,2,1)-FLINE(N-1,2,2)
      T1Z=FLINE(N,3,2)-FLINE(N-1,3,1)
      T2Z=FLINE(N,3,1)-FLINE(N-1,3,2)
      XNX=T2Y*T1Z-T1Y*T2Z
      YNY=T1X*T2Z-T2X*T1Z
      ZNZ=T2X*T1Y-T1X*T2Y
      FN=SQRT(XNX**2+YNY**2+ZNZ**2)
      IF (FN.EQ.0.) GO TO 10
      FVEC(N-1,1)=XNX/FN
      FVEC(N-1,2)=YNY/FN
      FVEC(N-1,3)=ZNZ/FN
      GO TO 20
   10 FVEC(N-1,1)=0.
      FVEC(N-1,2)=0.
      FVEC(N-1,3)=0.
   20 END DO
      RETURN
      END



!!!     OVERLAY(LWB,4,2)
      SUBROUTINE PLTORT()
!
!     *******************************************VERSION B00************
!
!
!     CONTROL ROUTINE FOR ORTHOGRAPHIC AND THREE-VIEW PLOTS
!
!     BASED ON PROGRAM PLTCON OF NASA TMX-2074.
!
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAP&
!!!     &E9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      INCLUDE 'files.cmn'
      COMMON /BLANK/ J0,J1,J2,J3,J4,J5,J6,NWAF,NWAFOR,NFUS,NRADX(4),NFOR&
     &X(4),NP,NPODOR,NF,NFINOR,NCAN,NCANOR,J2TEST,NW,NC
      COMMON /BLANK2/ HORZ,VERT,TEST1,PHI,THETA,PSI,XF,YF,ZF,DIST,FMAG,P&
     &LOTSZ,TYPE,KODE,XMIN,XMAX,YMIN,YMAX,ZMIN,ZMAX,XMID,YMID,ZMID,BIGD,&
     &ISP
      COMMON /NEWCOM/K0,K1,K2,K3,K4,K5,K6

!!!      COMMON /HEAD/ TITLE1(14),TITLE2(14)
      INCLUDE 'head.cmn'

      COMMON /GRAPH/ PLOT
      COMMON /PTYPE/ ORT,VU3,STE,PER
!
      DIMENSION ORG(3), ABC(8), ABCDE(8)
!
      INTEGER PLOT
!!!,TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAPE9&
!!!     &,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
!
      HX=1HX
      HY=1HY
      HZ=1HZ
      REWIND TAPE12
      REWIND TAPE14
      IF (PLOT.LT.0) GO TO 10
      IF (PLOT.GE.0) GO TO 20
   10 CONTINUE
      WRITE (TAPE14,*) TITLE1
      REWIND TAPE14
      READ (TAPE14,130) ABC
      GO TO 30
   20 CONTINUE
      WRITE (TAPE14,*) TITLE2
      REWIND TAPE14
      READ (TAPE14,130) ABC
   30 REWIND TAPE14
      WRITE (TAPE14,110) HORZ,VERT,TEST1,PHI,THETA,PSI,XF,YF,ZF,DIST,FMA&
     &G,PLOTSZ,TYPE,KODE
      REWIND TAPE14
      READ (TAPE14,130) ABCDE
      REWIND TAPE14
!
!     SAVE MIN AND MAX
!
      XSAV=XMIN
      YSAV=YMIN
      ZSAV=ZMIN
      XMSAV=XMAX
      YMSAV=YMAX
      ZMSAV=ZMAX
      IF (TYPE.NE.VU3) GO TO 80
      SCALE=BIGD/PLOTSZ
      ORG(1)=PHI
      ORG(2)=THETA
      ORG(3)=PSI
      PHI=0.
      THETA=0.
      PSI=0.
      YBIG=ORG(1)
      YORG=FLOAT(IFIX(YMAX/SCALE))+ORG(1)
      IF (YBIG.GT.ORG(2)) GO TO 40
      YBIG=ORG(2)
      YORG=FLOAT(IFIX(ZMAX/SCALE))+ORG(2)
   40 IF (YBIG.GT.ORG(3)) GO TO 50
      YBIG=ORG(3)
      YORG=FLOAT(IFIX(ZMAX/SCALE))+ORG(3)
   50 CALL CALPLT (1.0,YORG,-3)
!
!     NOTATE ON THREE VIEW PLOTS
!
      NCHAR=IFIX(6.*PLOTSZ)
      IF (NCHAR.GT.80) GO TO 60
      X=0.
      GO TO 70
   60 CONTINUE
      NDIF=(NCHAR-80)/2
      X=FLOAT(NDIF)/6.
      NCHAR=80
   70 CALL NOTATE (X,0.,.2,ABC,0.,NCHAR)
      XMIN=0.
      YMIN=0.
      ZMIN=0.
      HORZ=HX
      VERT=HY
      YORG=ORG(1)-YORG-1.
      CALL CALPLT (0.,YORG,-3)
      CALL OTHPLT
      REWIND TAPE12
      VERT=HZ
      YORG=ORG(2)-ORG(1)
      CALL CALPLT (0.,YORG,-3)
      CALL OTHPLT
      REWIND TAPE12
      HORZ=HY
      YORG=ORG(3)-ORG(2)
      YMIN=FLOAT(IFIX(YSAV/SCALE))*SCALE
      CALL CALPLT (0.,YORG,-3)
      CALL OTHPLT
      X=FLOAT(IFIX(PLOTSZ+6.))
      Y=1.-ORG(3)
      GO TO 100
   80 CONTINUE
!
!     NOTATE IDENTIFICATION ON PLOTS
!
      X=0.
      NCHAR=IFIX(11.*PLOTSZ)+3
      IF (NCHAR.LE.80) GO TO 90
      NDIF=(NCHAR-80)/2
      X=FLOAT(NDIF)/11.
      NCHAR=80
   90 CALL NOTATE (X,1.5,0.1,ABC,0.0,NCHAR)
      CALL NOTATE (X,0.5,0.1,ABCDE,0.0,NCHAR)
      CALL CALPLT (0.,2.,-3)
!
!     ORTHOGRAPHIC
!
      CALL OTHPLT
      X=FLOAT(IFIX(PLOTSZ+2.))
      Y=-2.
  100 CONTINUE
!
!     RESTORE MIN AND MAX
!
      IF (PLOT.NE.0) CALL NFRAME
      XMIN=XSAV
      YMIN=YSAV
      ZMIN=ZSAV
      XMAX=XMSAV
      YMAX=YMSAV
      ZMAX=ZMSAV
!
!
!
  110 FORMAT (2(A1,1X),A3,9F5.0,A3,16X,I1)
  120 FORMAT (13A6,A2)
  130 FORMAT (8A10)
      END
      SUBROUTINE OTHPLT
!
!     *******************************************VERSION B00************
!
!
!     CONTROL ROUTINE FOR ORTHOGRAPHIC PROJECTIONS
!
!
!     DETERMINE THE SPECIFIED AXIS SYSTEM AND PAPER PLANE, SET UP THE
!     ROTATION MATRIX AND THE EQUATION OF TRANSFORMATION OF THE OUTWARD
!     NORMAL VECTORS, AND ESTABLISH THE NECESSARY OFFSETS FOR PLACEMENT
!     OF THE PLOT
!
!     BASED ON PROGRAM OTHPLT OF NASA TMX-2074.
!
      COMMON /BLANK/ J0,J1,J2,J3,J4,J5,J6,NWAF,NWAFOR,NFUS,NRADX(4),NFOR&
     &X(4),NP,NPODOR,NF,NFINOR,NCAN,NCANOR,J2TEST,NW,NC
      COMMON /BLANK2/ HORZ,VERT,TEST1,PHI,THETA,PSI,XF,YF,ZF,DIST,FMAG,P&
     &LOTSZ,TYPE,KODE,XMIN,XMAX,YMIN,YMAX,ZMIN,ZMAX,XMID,YMID,ZMID,BIGD,&
     &ISP
      COMMON /NEWCOM/K0,K1,K2,K3,K4,K5,K6,KWAF,KWAFOR,KFUS,KRADX(4),     &
     &              KFORX(4),KF(6),KFINOR(6),KAN(6),KANOR(6)
      COMMON /GRAPH/ PLOT
      COMMON /PTYPE/ ORT,VU3,STE,PER
      COMMON /ONE/ NAN2
      COMMON /PI/ PI,RADEG,DEGRAD
!
      DIMENSION A(2,3), C(3)
!
      INTEGER PLOT
!
      XSEE=2HX
      YSEE=2HY
      ZSEE=2HZ
      XINTST=3HOUT
      CONV=DEGRAD
      NUM2=2
!
!     INITIALIZE
!
      DMAX=BIGD
      ITEST1=1
      ITEST2=1
      IF (XINTST.NE.TEST1) ITEST1=0
      IF (PSI.EQ.0..AND.THETA.EQ.0..AND.PHI.EQ.0.) ITEST2=0
      SCALE=DMAX/PLOTSZ
      PHI=CONV*PHI
      THETA=CONV*THETA
      PSI=CONV*PSI
      IF (TYPE.EQ.VU3) GO TO 10
      XDIS=XMAX-XMIN
      YDIS=YMAX-YMIN
      ZDIS=ZMAX-ZMIN
      XFIX=.5*(DMAX-XDIS)
      XMIN=XMIN-XFIX
      XMAX=XMAX+XFIX
      YFIX=.5*(DMAX-YDIS)
      YMIN=YMIN-YFIX
      YMAX=YMAX+YFIX
      ZFIX=.5*(DMAX-ZDIS)
      ZMIN=ZMIN-ZFIX
      ZMAX=ZMAX+ZFIX
!
!     ADJUST MINIMUMS FOR GRID LINES
!
      XMIN=FLOAT(IFIX(XMIN/SCALE))*SCALE
      YMIN=FLOAT(IFIX(YMIN/SCALE))*SCALE
      ZMIN=FLOAT(IFIX(ZMIN/SCALE))*SCALE
   10 CONTINUE
!
!     SETUP AXIS
!
      SINPSI=SIN(PSI)
      SINTHE=SIN(THETA)
      SINPHI=SIN(PHI)
      COSPSI=COS(PSI)
      COSTHE=COS(THETA)
      COSPHI=COS(PHI)
      IF (XSEE.NE.HORZ) GO TO 30
!
!     USE X FOR HORIZONTAL VARIABLE
!
      IF (ITEST2.EQ.0) GO TO 20
      A(1,1)=COSTHE*COSPSI
      A(1,2)=-SINPSI*COSPHI+SINTHE*COSPSI*SINPHI
      A(1,3)=SINPSI*SINPHI+SINTHE*COSPSI*COSPHI
   20 HMIN=XMIN
      HMAX=XMAX
      HMID=XMID
      IHORZ=1
      GO TO 70
   30 IF (YSEE.NE.HORZ) GO TO 50
!
!     USE Y FOR HORIZONTAL VARIABLE
!
      IF (ITEST2.EQ.0) GO TO 40
      A(1,1)=COSTHE*SINPSI
      A(1,2)=COSPSI*COSPHI+SINTHE*SINPSI*SINPHI
      A(1,3)=-COSPSI*SINPHI+SINTHE*SINPSI*COSPHI
   40 HMIN=YMIN
      HMAX=YMAX
      HMID=YMID
      IHORZ=2
      GO TO 70
!
!     USE Z FOR HORIZONTAL VARIABLE
!
   50 CONTINUE
      IF (ITEST2.EQ.0) GO TO 60
      A(1,1)=-SINTHE
      A(1,2)=COSTHE*SINPHI
      A(1,3)=COSTHE*COSPHI
   60 HMIN=ZMIN
      HMAX=ZMAX
      HMID=ZMID
      IHORZ=3
   70 IF (XSEE.NE.VERT) GO TO 90
!
!     USE X FOR VERTICAL VARIABLE
!
      IF (ITEST2.EQ.0) GO TO 80
      A(2,1)=COSTHE*COSPSI
      A(2,2)=-SINPSI*COSPHI+SINTHE*COSPSI*SINPHI
      A(2,3)=SINPSI*SINPHI+SINTHE*COSPSI*COSPHI
   80 VMIN=XMIN
      VMAX=XMAX
      VMID=XMID
      IVERT=1
      GO TO 130
   90 IF (YSEE.NE.VERT) GO TO 110
!
!     USE Y FOR VERTICAL VARIABLE
!
      IF (ITEST2.EQ.0) GO TO 100
      A(2,1)=COSTHE*SINPSI
      A(2,2)=COSPSI*COSPHI+SINTHE*SINPSI*SINPHI
      A(2,3)=-COSPSI*SINPHI+SINTHE*SINPSI*COSPHI
  100 VMIN=YMIN
      VMAX=YMAX
      VMID=YMID
      IVERT=2
      GO TO 130
!
!     USE Z FOR VERTICAL VARIABLE
!
  110 CONTINUE
      IF (ITEST2.EQ.0) GO TO 120
      A(2,1)=-SINTHE
      A(2,2)=COSTHE*SINPHI
      A(2,3)=COSTHE*COSPHI
  120 VMIN=ZMIN
      VMAX=ZMAX
      VMID=ZMID
      IVERT=3
!
!     CHECK PAPER PLANE
!
  130 IF (.NOT.((IHORZ.EQ.1.AND.IVERT.EQ.2).OR.(IVERT.EQ.1.AND.IHORZ.EQ.&
     &2))) GO TO 140
      ITEST=3
      C(1)=-SINTHE
      C(2)=COSTHE*SINPHI
      C(3)=COSTHE*COSPHI
      GO TO 160
  140 IF (.NOT.((IHORZ.EQ.1.AND.IVERT.EQ.3).OR.(IVERT.EQ.1.AND.IHORZ.EQ.&
     &3))) GO TO 150
      ITEST=2
      C(1)=COSTHE*SINPSI
      C(2)=COSPSI*COSPHI+SINTHE*SINPSI*SINPHI
      C(3)=-COSPSI*SINPHI+SINTHE*SINPSI*COSPHI
      GO TO 160
  150 ITEST=1
      C(1)=COSTHE*COSPSI
      C(2)=-SINPSI*COSPHI+SINTHE*COSPSI*SINPHI
      C(3)=SINPSI*SINPHI+SINTHE*COSPSI*COSPHI
  160 CONTINUE
!
!     CENTER WITHIN PAGE SIZE IF SIZE GREATER THAN 28 INCHES
!
      IF (PLOTSZ.GT.28..AND.TYPE.NE.VU3) VMIN=-13.*SCALE+FLOAT(IFIX(VMID&
     &/SCALE))*SCALE
!
!     ROTATE MIDPOINT TO PLACE ROTATED VIEW CORRECTLY
!
      IF (ITEST2.EQ.0) GO TO 170
      AMID1=A(1,1)*XMID+A(1,2)*YMID+A(1,3)*ZMID
      AMID2=A(2,1)*XMID+A(2,2)*YMID+A(2,3)*ZMID
      HMIN=HMIN-HMID+AMID1
      VMIN=VMIN-VMID+AMID2
  170 CONTINUE
      IF (PLOT.LT.0) GO TO 190
      J1=K1
      NW=NWAFOR
      IF (KWAF.NE.0) NWAF=KWAF
      IF (KWAFOR.NE.0) NW=KWAFOR
      J2=K2
      DO 180 I=1,NFUS
      IF (KRADX(I).NE.0) NRADX(I)=KRADX(I)
      IF (KFORX(I).NE.0) NFORX(I)=KFORX(I)
  180 END DO
      J4=K4
      J5=K5
!
!     BEGIN PLOTTING LINES
!
!
!     WING
!
  190 IF (J1.EQ.0) GO TO 210
      DO 200 I=1,2
      CALL PLOTIT (NWAF,NW,ITEST,ITEST1,ITEST2,IHORZ,IVERT,HMIN,VMIN,SCA&
     &LE,A,C)
      CALL PLOTIT (NW,NWAF,ITEST,ITEST1,ITEST2,IHORZ,IVERT,HMIN,VMIN,SCA&
     &LE,A,C)
  200 END DO
!
!     FUSELAGE (BODY)
!
  210 IF (J2.EQ.0) GO TO 230
      DO 220 NFU=1,NFUS
      NANG1=NRADX(NFU)
      NFUSOR=NFORX(NFU)
      CALL PLOTIT (NANG1,NFUSOR,ITEST,ITEST1,ITEST2,IHORZ,IVERT,HMIN,VMI&
     &N,SCALE,A,C)
      CALL PLOTIT (NFUSOR,NANG1,ITEST,ITEST1,ITEST2,IHORZ,IVERT,HMIN,VMI&
     &N,SCALE,A,C)
  220 END DO
!
!     POD (NACELLE)
!
  230 IF (J3.EQ.0.OR.PLOT.GT.0) GO TO 260
      NANG1=NAN2+1
      DO 240 NP1=1,NP
      CALL PLOTIT (NANG1,NPODOR,ITEST,ITEST1,ITEST2,IHORZ,IVERT,HMIN,VMI&
     &N,SCALE,A,C)
  240 END DO
      DO 250 NP1=1,NP
      CALL PLOTIT (NPODOR,NANG1,ITEST,ITEST1,ITEST2,IHORZ,IVERT,HMIN,VMI&
     &N,SCALE,A,C)
  250 END DO
!
!     FIN (VERTICAL TAIL)
!
  260 IF (J4.EQ.0) GO TO 300
      DO 290 NF1=1,NF
      DO 290 I=1,2
      NFOR=NFINOR
      IF (PLOT.LT.0) GO TO 270
      IA=KF(NF1)
      IF (IA.EQ.0) IA=2
      IF (KFINOR(NF1).NE.0) NFOR=KFINOR(NF1)
      GO TO 280
  270 IA=2
  280 CONTINUE
      NUM2=IA
      CALL PLOTIT (NUM2,NFOR,ITEST,ITEST1,ITEST2,IHORZ,IVERT,HMIN,VMIN,S&
     &CALE,A,C)
      CALL PLOTIT (NFOR,NUM2,ITEST,ITEST1,ITEST2,IHORZ,IVERT,HMIN,VMIN,S&
     &CALE,A,C)
  290 CONTINUE
!
!     CANARD (HORIZONTAL TAIL)
!
  300 IF (J5.EQ.0) GO TO 350
      DO 340 NCA=1,NCAN
      DO 330 I=1,2
      NCOR=NCANOR
      IF (PLOT.LT.0) GO TO 310
      IA=KAN(NCA)
      IF (IA.EQ.0) IA=2
      IF (KANOR(NCA).NE.0) NCOR=KANOR(NCA)
      GO TO 320
  310 IA=2
  320 CONTINUE
      NUM2=IA
      CALL PLOTIT (NUM2,NCOR,ITEST,ITEST1,ITEST2,IHORZ,IVERT,HMIN,VMIN,S&
     &CALE,A,C)
      CALL PLOTIT (NCOR,NUM2,ITEST,ITEST1,ITEST2,IHORZ,IVERT,HMIN,VMIN,S&
     &CALE,A,C)
  330 END DO
  340 END DO
  350 CONTINUE
      RETURN
      END
      SUBROUTINE PLOTIT (NL,NPT,ITEST,ITEST1,ITEST2,IHORZ,IVERT,HMIN,VMI&
     &N,SCALE,A,C)
!
!     *******************************************VERSION B00************
!
!
!     READS LINES OF POINTS DEFINING A SURFACE FROM A FILE, MANIPULATES
!     IN THE MANNER SPECIFIED, AND PLOTS IT
!
!     BASED ON SUBROUTINE PLOTIT OF NASA TMX-2074.
!
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAP&
!!!     &E9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      INCLUDE 'files.cmn'
!
      DIMENSION VECRT(30,3,2), VECLF(30,3,2), ALINE(31,3), RLINE(31,2), &
     &RVEC(30,2), XLINE(33,2), NNUM(4), PLINE(31,2), A(2,3), C(3)
!
!!!      INTEGER TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAPE9,TAPE&
!!!     &10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
!
      NVEC=NPT-1
      DO 310 N=1,NL
      IF (N.GT.1) GO TO 10
      KODE=3
      K1=2
      K2=2
      GO TO 40
   10 KODE=1
      K1=1
      K2=2
      DO 30 NV=1,NVEC
      DO 20 N3=1,3
      VECRT(NV,N3,1)=VECRT(NV,N3,2)
      VECLF(NV,N3,1)=VECLF(NV,N3,2)
   20 END DO
   30 END DO
   40 CONTINUE
      READ (TAPE12) ((ALINE(NN,N3),NN=1,NPT),N3=1,3)
      IF (N.NE.NL) GO TO 50
      KODE=2
      K1=1
      K2=1
      GO TO 70
   50 CONTINUE
      READ (TAPE12) ((VECRT(NN,N3,2),NN=1,NVEC),N3=1,3)
      DO 60 NN=1,NVEC
      VECLF(NN,1,2)=VECRT(NN,1,2)
      VECLF(NN,2,2)=-VECRT(NN,2,2)
   60 VECLF(NN,3,2)=VECRT(NN,3,2)
!
!     LOOP FOR RIGHT AND LEFT SIDE OF CONFIGURATION
!
   70 DO 300 NN2=1,2
      IF (NN2.EQ.1) GO TO 90
      DO 80 NN=1,NPT
   80 ALINE(NN,2)=-ALINE(NN,2)
   90 IF (ITEST1.EQ.1) GO TO 140
      IF (ITEST2.EQ.1) GO TO 110
!
!     NO ROTATION OR VISIBILITY TEST
!
      DO 100 NN=1,NPT
      XLINE(NN,1)=ALINE(NN,IHORZ)
  100 XLINE(NN,2)=ALINE(NN,IVERT)
      GO TO 130
!
!     ROTATE BUT NO VISIBILITY TEST
!
  110 CALL PTROT (NPT,A,ALINE,RLINE)
      DO 120 NN=1,NPT
      DO 120 N2=1,2
      XLINE(NN,N2)=RLINE(NN,N2)
  120 CONTINUE
!
!     SCALE AND PLOT
!
  130 XLINE(NPT+1,1)=HMIN
      XLINE(NPT+1,2)=VMIN
      XLINE(NPT+2,1)=SCALE
      XLINE(NPT+2,2)=SCALE
      CALL XYLINE(XLINE(1,1),XLINE(1,2),NPT,1,0,0)
      GO TO 300
  140 IF (ITEST2.EQ.1) GO TO 190
!
!     CHECK VISIBILITY BUT NO ROTATION
!
      DO 150 NN=1,NPT
      RLINE(NN,1)=ALINE(NN,IHORZ)
      RLINE(NN,2)=ALINE(NN,IVERT)
  150 END DO
      DO 180 NN=1,NVEC
      DO 170 N2=1,2
      IF (NN2.EQ.2) GO TO 160
      RVEC(NN,N2)=VECRT(NN,ITEST,N2)
      GO TO 170
  160 RVEC(NN,N2)=VECLF(NN,ITEST,N2)
  170 END DO
  180 END DO
      GO TO 230
!
!     ROTATE AND CHECK VISIBILITY
!
  190 CALL PTROT (NPT,A,ALINE,RLINE)
      IF (NN2.EQ.2) GO TO 210
      DO 200 N2=K1,K2
      CALL VECROT (NVEC,C,VECRT(1,1,N2),RVEC(1,N2))
  200 END DO
      GO TO 230
  210 DO 220 N2=K1,K2
      CALL VECROT (NVEC,C,VECLF(1,1,N2),RVEC(1,N2))
  220 END DO
!
!     FIND VISIBLE LINES
!
  230 IF (ITEST.NE.2) GO TO 250
      DO 240 N2=K1,K2
      DO 240 M=1,NVEC
  240 RVEC(M,N2)=-RVEC(M,N2)
  250 CALL VISTST (KODE,NPT,NSET,NNUM,RLINE,RVEC,PLINE)
      IF (ITEST.NE.2) GO TO 270
      DO 260 N2=K1,K2
      DO 260 M=1,NVEC
  260 RVEC(M,N2)=-RVEC(M,N2)
  270 CONTINUE
      IF (NSET.EQ.0) GO TO 300
!
!     SCALE AND PLOT
!
      NIT=0
      DO 290 N1=1,NSET
      NN=NNUM(N1)
      DO 280 NN1=1,NN
      NIT=NIT+1
      XLINE(NN1,1)=PLINE(NIT,1)
      XLINE(NN1,2)=PLINE(NIT,2)
  280 END DO
      XLINE(NN+1,1)=HMIN
      XLINE(NN+1,2)=VMIN
      XLINE(NN+2,1)=SCALE
      XLINE(NN+2,2)=SCALE
      CALL XYLINE(XLINE(1,1),XLINE(1,2),NN,1,0,0)
  290 END DO
  300 END DO
  310 END DO
      RETURN
      END
      SUBROUTINE VISTST (KODE,NPT,NSET,NNUM,RLINE,RVEC,PLINE)
!
!     *******************************************VERSION B00************
!
!
!     TESTS A LINE OF POINTS FOR VISIBILITY
!
!     BASED ON SUBROUTINE VISTST OF NASA TMX-2074.
!
      DIMENSION NNUM(4), RLINE(31,2), RVEC(30,2), PLINE(31,2)
!
      NVEC=NPT-1
      NPLT=0
      NSET=0
      ICOUNT=0
      IF(KODE.EQ.2) GO TO 20
      IF(KODE.EQ.3) GO TO 30
   10 N1=1
      N2=2
      GO TO 40
   20 N1=1
      N2=1
      GO TO 40
   30 N1=2
      N2=2
   40 DO 130 N=1,NPT
      IF (N.EQ.1) GO TO 60
      IF (N.EQ.NPT) GO TO 80
      DO 50 NN=N1,N2
      IF ((RVEC(N-1,NN).GT.0.).OR.(RVEC(N,NN).GT.0.)) GO TO 120
   50 END DO
      GO TO 100
   60 DO 70 NN=N1,N2
      IF (RVEC(1,NN).GT.0.) GO TO 120
   70 END DO
      GO TO 100
   80 DO 90 NN=N1,N2
      IF (RVEC(NVEC,NN).GT.0.) GO TO 120
   90 END DO
!
!     POINT IS NOT VISIBLE
!
  100 IF (ICOUNT.LE.1) GO TO 110
      NSET=NSET+1
      NNUM(NSET)=ICOUNT
  110 ICOUNT=0
      GO TO 130
!
!     POINT IS VISIBLE
!
  120 NPLT=NPLT+1
      ICOUNT=ICOUNT+1
      PLINE(NPLT,1)=RLINE(N,1)
      PLINE(NPLT,2)=RLINE(N,2)
  130 END DO
      IF (ICOUNT.LE.1) GO TO 140
      NSET=NSET+1
      NNUM(NSET)=ICOUNT
  140 RETURN
      END
      SUBROUTINE PTROT (NPT,A,ALINE,RLINE)
!
!     *******************************************VERSION B00************
!
!
!     ROTATES AND PROJECTS A SET OF THREE DIMENSIONAL POINTS
!
!     BASED ON SUBROUTINE PTROT OF NASA TMX-2074.
!
      DIMENSION A(2,3), ALINE(31,3), RLINE(31,2)
!
      DO 20 N=1,NPT
      RLINE(N,1)=0.
      RLINE(N,2)=0.
      DO 10 I=1,2
      DO 10 J=1,3
   10 RLINE(N,I)=RLINE(N,I)+A(I,J)*ALINE(N,J)
   20 END DO
      RETURN
      END
      SUBROUTINE VECROT (NVEC,C,FVEC,RVEC)
!
!     *******************************************VERSION B00************
!
!
!     TRANSFORMS VECTORS
!
!     BASED ON SUBROUTINE VECROT OF NASA TMX-2074.
!
      DIMENSION C(3), FVEC(30,3), RVEC(30)
!
      DO 20 N=1,NVEC
      SUM=0.
      DO 10 NN=1,3
   10 SUM=SUM+C(NN)*FVEC(N,NN)
   20 RVEC(N)=SUM
      RETURN
      END



!!!     OVERLAY(LWB,4,3)
      SUBROUTINE PLTSTE()
!
!     *******************************************VERSION B00************
!
!     CONTROL ROUTINE FOR STEREOSCOPIC AND PERSPECTIVE PLOTS
!
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAP&
!!!     &E9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      INCLUDE 'files.cmn'
      COMMON /BLANK/ J0,J1,J2,J3,J4,J5,J6,NWAF,NWAFOR,NFUS,NRADX(4),NFOR&
     &X(4),NP,NPODOR,NF,NFINOR,NCAN,NCANOR,J2TEST,NW,NC
      COMMON /BLANK2/ HORZ,VERT,TEST1,PHI,THETA,PSI,XF,YF,ZF,DIST,FMAG,P&
     &LOTSZ,TYPE,KODE,XMIN,XMAX,YMIN,YMAX,ZMIN,ZMAX,XMID,YMID,ZMID,BIGD,&
     &ISP

!!!      COMMON /HEAD/ TITLE1(14),TITLE2(14)
      INCLUDE 'head.cmn'

      COMMON /GRAPH/ PLOT
      COMMON /PTYPE/ ORT,VU3,STE,PER
!
      DIMENSION ABC(8), ABCDE(8)
!
      INTEGER PLOT
!!!,TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAPE9&
!!!     &,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
!
      REWIND TAPE12
      XSAV=XMIN
      YSAV=YMIN
      ZSAV=ZMIN
      XMSAV=XMAX
      YMSAV=YMAX
      ZMSAV=ZMAX
      REWIND TAPE14
      IF (PLOT.LT.0) GO TO 10
      IF (PLOT.GE.0) GO TO 20
   10 CONTINUE
      WRITE (TAPE14,*) TITLE1
      REWIND TAPE14
      READ (TAPE14,80) ABC
      GO TO 30
   20 CONTINUE
      WRITE (TAPE14,*) TITLE2
      REWIND TAPE14
      READ (TAPE14,80) ABC
   30 REWIND TAPE14
      WRITE (TAPE14,60) HORZ,VERT,TEST1,PHI,THETA,PSI,XF,YF,ZF,DIST,FMAG&
     &,PLOTSZ,TYPE,KODE
      REWIND TAPE14
      READ (TAPE14,80) ABCDE
      REWIND TAPE14
      IF (TYPE.EQ.STE) GO TO 50
      X=0.
      NCHAR=IFIX(11.*PLOTSZ)+3
      IF (NCHAR.LE.80) GO TO 40
      NDIF=(NCHAR-80)/2
      X=FLOAT(NDIF)/11.
      NCHAR=80
   40 CALL NOTATE (X,1.5,0.1,ABC,0.0,NCHAR)
      CALL NOTATE (X,0.5,0.1,ABCDE,0.0,NCHAR)
      CALL CALPLT (0.,2.,-3)
   50 CONTINUE
      ISP=1
      IF (TYPE.EQ.STE) ISP=2
!
!     PERSPECTIVE OR STEREO
!
      CALL SPPLT
      X=PLOTSZ+2.
      IF (TYPE.EQ.STE) X=X+PLOTSZ
      Y=-2.
!
!     END OF COMPLETE PLOT
!
      IF (PLOT.NE.0) CALL NFRAME (X,Y)
      XMIN=XSAV
      YMIN=YSAV
      ZMIN=ZSAV
      XMAX=XMSAV
      YMAX=YMSAV
      ZMAX=ZMSAV
!
!
!
   60 FORMAT (2(A1,1X),A3,9F5.0,A3,16X,I1)
   70 FORMAT (13A6,A2)
   80 FORMAT (8A10)
      END
      SUBROUTINE SPPLT
!
!     *******************************************VERSION B00************
!
!     CONTROL ROUTINE FOR PERSPECTIVE OR STEREO PLOTS
!
!     BASED ON PROGRAM SPPLT OF NASA TMX-2074.
!
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAP&
!!!     &E9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      INCLUDE 'files.cmn'
      COMMON /BLANK/ J0,J1,J2,J3,J4,J5,J6,NWAF,NWAFOR,NFUS,NRADX(4),NFOR&
     &X(4),NP,NPODOR,NF,NFINOR,NCAN,NCANOR,J2TEST,NW,NC
      COMMON /BLANK2/ HORZ,VERT,TEST1,PHI,THETA,PSI,XF,YF,ZF,DIST,FMAG,P&
     &LOTSZ,TYPE,KODE,XMIN,XMAX,YMIN,YMAX,ZMIN,ZMAX,XMID,YMID,ZMID,BIGD,&
     &ISP
      COMMON /NEWCOM/K0,K1,K2,K3,K4,K5,K6,KWAF,KWAFOR,KFUS,KRADX(4),     &
     &              KFORX(4),KF(6),KFINOR(6),KAN(6),KANOR(6)
      COMMON /GRAPH/ PLOT
      COMMON /ONE/ NAN2
!
      DIMENSION XINIT(2), YINIT(2), ZINIT(2)
!
      INTEGER PLOT
!!!,TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAPE9&
!!!     &,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
!
      XINIT(1)=PHI
      XINIT(2)=XF
      YINIT(1)=THETA
      YINIT(2)=YF
      ZINIT(1)=PSI
      ZINIT(2)=ZF
      CALL STERPT (XINIT,YINIT,ZINIT,0,1,0,3,PLOTSZ,DIST,FMAG)
!
!     LOOP FOR RIGHT AND LEFT FRAMES
!
      DO 180 IC=1,ISP
      REWIND TAPE12
      NCI=-IC
      IF (PLOT.LT.0) GO TO 20
      J1=K1
      NWAF=KWAF
      NW=KWAFOR
      J2=K2
      DO 10 I=1,NFUS
      NRADX(I)=KRADX(I)
   10 NFORX(I)=KFORX(I)
      J4=K4
      J5=K5
!
!     BEGIN PLOTTING LINES
!
!
!     WING
!
   20 IF (J1.EQ.0) GO TO 40
      DO 30 I=1,2
      CALL PLTIT3 (NWAF,NW,PHI,THETA,PSI,XF,YF,ZF,PLOTSZ,DIST,FMAG,NCI)
   30 CALL PLTIT3 (NW,NWAF,PHI,THETA,PSI,XF,YF,ZF,PLOTSZ,DIST,FMAG,NCI)
!
!     FUSELAGE (BODY)
!
   40 IF (J2.EQ.0) GO TO 60
      DO 50 NFU=1,NFUS
      NANG1=NRADX(NFU)
      NFUSOR=NFORX(NFU)
      CALL PLTIT3 (NANG1,NFUSOR,PHI,THETA,PSI,XF,YF,ZF,PLOTSZ,DIST,FMAG,&
     &NCI)
      CALL PLTIT3 (NFUSOR,NANG1,PHI,THETA,PSI,XF,YF,ZF,PLOTSZ,DIST,FMAG,&
     &NCI)
   50 END DO
!
!     POD (NACELLE)
!
   60 IF (J3.EQ.0.OR.PLOT.GT.0) GO TO 80
      NANG1=NAN2+1
      DO 70 NP1=1,NP
      CALL PLTIT3 (NANG1,NPODOR,PHI,THETA,PSI,XF,YF,ZF,PLOTSZ,DIST,FMAG,&
     &NCI)
      CALL PLTIT3 (NPODOR,NANG1,PHI,THETA,PSI,XF,YF,ZF,PLOTSZ,DIST,FMAG,&
     &NCI)
   70 END DO
!
!     FIN (VERTICAL TAIL)
!
   80 IF (J4.EQ.0) GO TO 120
      DO 110 NF1=1,NF
      NFOR=NFINOR
      IF (PLOT.LT.0) GO TO 90
      IA=KF(NF1)
      IF (IA.EQ.0) IA=2
      IF (KFINOR(NF1).NE.0) NFOR=KFINOR(NF1)
      GO TO 100
   90 IA=2
  100 CONTINUE
      DO 110 I=1,2
      CALL PLTIT3 (IA,NFOR,PHI,THETA,PSI,XF,YF,ZF,PLOTSZ,DIST,FMAG,NCI)
      CALL PLTIT3 (NFOR,IA,PHI,THETA,PSI,XF,YF,ZF,PLOTSZ,DIST,FMAG,NCI)
  110 CONTINUE
!
!     CANARD (HORIZONTAL TAIL)
!
  120 IF (J5.EQ.0) GO TO 170
      DO 160 NCA=1,NCAN
      NCOR=NCANOR
      IF (PLOT.LT.0) GO TO 130
      IA=KAN(NCA)
      IF (IA.EQ.0) IA=2
      IKANOR=KANOR(NCA)
      IF (IKANOR.NE.0) NCOR=KANOR(NCA)
      GO TO 140
  130 IA=2
  140 CONTINUE
      DO 150 I=1,2
      CALL PLTIT3 (IA,NCOR,PHI,THETA,PSI,XF,YF,ZF,PLOTSZ,DIST,FMAG,NCI)
      CALL PLTIT3 (NCOR,IA,PHI,THETA,PSI,XF,YF,ZF,PLOTSZ,DIST,FMAG,NCI)
  150 END DO
  160 END DO
  170 CONTINUE
  180 END DO
      RETURN
      END
      SUBROUTINE PLTIT3 (NL,NPT,PHI,THETA,PSI,XF,YF,ZF,PLOTSZ,DIST,FMAG,&
     &NCI)
!
!     *******************************************VERSION B00************
!
!
!     READS LINES OF POINTS DEFINING A SURFACE FROM A FILE AND PLOTS
!     PERSPECTIVE VIEWS OR STEREO FRAMES
!
!     BASED ON SUBROUTINE PLTIT3 OF NASA TMX-2074.
!
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAP&
!!!     &E9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      INCLUDE 'files.cmn'
!
      DIMENSION ALINE(33,3)
!
!!!      INTEGER TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAPE9,TAPE&
!!!     &10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
!
      ALINE(NPT+1,1)=PHI
      ALINE(NPT+2,1)=XF
      ALINE(NPT+1,2)=THETA
      ALINE(NPT+2,2)=YF
      ALINE(NPT+1,3)=PSI
      ALINE(NPT+2,3)=ZF
      DO 50 N=1,NL
      READ (TAPE12) ((ALINE(NN,N3),NN=1,NPT),N3=1,3)
      IF (N.EQ.NL) GO TO 10
!
!     SKIP VECTORS
!
      READ (TAPE12) VEC
!
!     LOOP FOR RIGHT AND LEFT SIDE OF CONFIGURATION
!
   10 DO 40 NN2=1,2
      IF (NN2.EQ.1) GO TO 30
      DO 20 NN=1,NPT
   20 ALINE(NN,2)=-ALINE(NN,2)
   30 CALL STERPT (ALINE(1,1),ALINE(1,2),ALINE(1,3),NPT,1,NCI,3,PLOTSZ,D&
     &IST,FMAG)
   40 END DO
   50 END DO
      RETURN
      END
      SUBROUTINE STERPT (X,Y,Z,N,K,NC,IP,PAG,PLA,XPR)
!
!     *******************************************VERSION B00************
!
!
!     PLOTS THE PERSPECTIVE PROJECTION OF DATA FOR A GIVEN THREE
!     DIMENSIONAL ARRAY. TWO PASSES THROUGH THIS ROUTINE WILL GENERATE A
!     PAIR OF STEREO FRAMES.
!
!     BASED ON SUBROUTINE STERPT OF NASA TMX-2074.
!
      COMMON /PI/ PI,RADEG,DEGRAD
!
      DIMENSION VP(3), TRAN(3), SANG(3), CANG(3), ADJ(3), PT(4), XLP(2),&
     & ZLP(2), X(1), Y(1), Z(1), PLX(4), PLY(4), PLZ(2), PIX(4), PIY(4),&
     & PIZ(2), ILP(4), IPL(4)
!
      PI2=PI/2.
      PI32=PI*(3./2.)
      PI42=2.*PI
      PAR=1.125
      NPG=0
      NPT=1
      FRAME=9.80
      TURN=11.01
      NO=1
      KK=K
      II=IP
      IF (NC) 160,10,220
   10 NP=N*K+1
      NR=NP+K
      PLIM=PAG/2.
      SF=XPR
      VPL=PLA
      DO 20 I=1,4
      PLX(I)=0.
      PLY(I)=0.
      PIX(I)=0.
      PIY(I)=0.
      IPL(I)=0
   20 ILP(I)=0
      DO 30 I=1,2
      PLZ(I)=0.
   30 PIZ(I)=0.
      VPX=X(NP)
      VPY=Y(NP)
      VPZ=Z(NP)
      FPX=X(NR)
      FPY=Y(NR)
      FPZ=Z(NR)
      VX=VPX-FPX
      VY=VPY-FPY
      VZ=VPZ-FPZ
      VP(2)=SQRT((VX**2)+(VY**2))
      VP(3)=SQRT((VZ**2)+(VP(2)**2))
      TRAN(1)=VPX-(VPL*(VX/VP(2)))
      TRAN(2)=VPY-(VPL*(VY/VP(2)))
      TRAN(3)=VPZ-(VPL*(VZ/VP(3)))
      VANG=ATAN((PAR/VP(3)))
      IF (VX) 110,40,70
   40 IF (VY) 60,600,50
   50 PANG=PI2
      GO TO 150
   60 PANG=PI32
      GO TO 150
   70 IF (VY) 100,80,90
   80 PANG=0.
      GO TO 150
   90 PANG=ATAN((VY/VX))
      GO TO 150
  100 PANG=PI42-ATAN(((ABS(VY))/VX))
      GO TO 150
  110 IF (VY) 140,120,130
  120 PANG=PI
      GO TO 150
  130 PANG=PI-ATAN((VY/(ABS(VX))))
      GO TO 150
  140 PANG=PI+ATAN(((ABS(VY))/(ABS(VX))))
  150 PANG=PI32-PANG
      UANG=PANG-VANG
      RANG=UANG+(2.*VANG)
      SANG(1)=SIN(UANG)
      SANG(2)=SIN(RANG)
      CANG(1)=COS(UANG)
      CANG(2)=COS(RANG)
      SANG(3)=VZ/VP(3)
      CANG(3)=VP(2)/VP(3)
      VP(3)=VPL
      XLP(1)=0.
      ZLP(1)=0.
      XLP(2)=FRAME
      ZLP(2)=0.
      ADJ(1)=PLIM
      ADJ(2)=PLIM
      ADJ(3)=ADJ(2)+FRAME
      IF (N) 600,600,220
  160 M=IABS(NC)
      L=M
      IF (NPG+NC) 230,170,230
  170 IF (2+NC) 600,190,180
  180 NPG=2
      GO TO 200
  190 NPG=1
  200 CONTINUE
      DO 210 I=1,L
      CALL CALPLT (TURN,0.,-3)
  210 END DO
      CALL CALPLT (XLP(M),ZLP(M),3)
  220 M=1
      L=2
  230 DO 590 I=M,L
      IF (NPG) 600,240,300
  240 IF (NC) 250,290,290
  250 IF (NPT+NC) 300,260,300
  260 IF (2+NC) 600,280,270
  270 NPT=2
      GO TO 290
  280 NPT=1
  290 CONTINUE
      CALL CALPLT (XLP(I),ZLP(I),3)
  300 DO 580 J=1,N
      PT(1)=((X(NO)-TRAN(1))*CANG(I))-((Y(NO)-TRAN(2))*SANG(I))
      PT(4)=((X(NO)-TRAN(1))*SANG(I))+((Y(NO)-TRAN(2))*CANG(I))
      PT(2)=((PT(4)*CANG(3))-((Z(NO)-TRAN(3))*SANG(3)))
      PT(3)=((PT(4)*SANG(3))+((Z(NO)-TRAN(3))*CANG(3)))
      IF (PT(2)) 310,360,360
  310 IF (ILP(I)) 600,320,350
  320 IF (II-3) 330,340,600
  330 VX=PLX(I)-PT(1)
      VY=PLY(I)-PT(2)
      VZ=PLZ(I)-PT(3)
      VP(1)=SQRT((VX**2)+(VY**2))
      VP(2)=SQRT((VZ**2)+(VP(1)**2))
      VPL=PLY(I)/(VY/VP(1))
      PT(4)=PLX(I)-((VX/VP(1))*VPL)
      PLX(I)=PT(1)
      PT(1)=PT(4)
      PLY(I)=PT(2)
      PT(2)=0.
      PT(4)=PLZ(I)-((VZ/VP(2))*VPL)
      PLZ(I)=PT(3)
      PT(3)=PT(4)
      ILP(I)=1
      GO TO 410
  340 ILP(I)=1
  350 PLX(I)=PT(1)
      PLY(I)=PT(2)
      PLZ(I)=PT(3)
      GO TO 540
  360 IF (ILP(I)) 600,400,370
  370 IF (II-3) 380,390,600
  380 II=3
      IPL(I)=1
      PIX(I)=PT(1)
      PIY(I)=PT(2)
      PIZ(I)=PT(3)
      VX=PT(1)-PLX(I)
      VX=PT(1)-PLX(I)
      VY=PT(2)-PLY(I)
      VZ=PT(3)-PLZ(I)
      VP(1)=SQRT((VX**2)+(VY**2))
      VP(2)=SQRT((VZ**2)+(VP(1)**2))
      VPL=PT(2)/(VY/VP(1))
      PT(2)=0.
      PT(1)=PT(1)-((VX/VP(1))*VPL)
      PT(3)=PT(3)-((VZ/VP(2))*VPL)
  390 ILP(I)=0
  400 PLX(I)=PT(1)
      PLY(I)=PT(2)
      PLZ(I)=PT(3)
  410 XP=(PT(1)+(PT(2)*(-PT(1))/(PT(2)+VP(3))))*SF
      ZP=(PT(3)+(PT(2)*(-PT(3))/(PT(2)+VP(3))))*SF
      VPL=SQRT((XP**2)+(ZP**2))
      IF (VPL-PLIM) 470,470,420
  420 IF (ILP(I+2)) 600,430,460
  430 IF (II-3) 440,450,600
  440 PLX(I+2)=XP
      PLY(I+2)=ZP
      XP=PLIM*(XP/VPL)
      ZP=PLIM*(ZP/VPL)
      ILP(I+2)=1
      GO TO 530
  450 ILP(I+2)=1
  460 PLX(I+2)=XP
      PLY(I+2)=ZP
      GO TO 540
  470 IF (ILP(I+2)) 600,510,480
  480 IF (II-3) 490,500,600
  490 II=3
      IPL(I+2)=1
      PIX(I+2)=XP
      PIY(I+2)=ZP
      VPL=SQRT((PLX(I+2)**2)+(PLY(I+2)**2))
      XP=PLIM*(PLX(I+2)/VPL)
      ZP=PLIM*(PLY(I+2)/VPL)
  500 ILP(I+2)=0
      GO TO 520
  510 IF ((SQRT(((PLX(I+2))**2)+((PLY(I+2))**2)))-PLIM) 520,520,480
  520 PLX(I+2)=XP
      PLY(I+2)=ZP
  530 XPT=XP+ADJ(I+1)
      YPT=ZP+ADJ(1)
      CALL CALPLT (XPT,YPT,II)
  540 II=2
      IF (IPL(I+2)) 600,560,550
  550 IPL(I+2)=0
      XP=PIX(I+2)
      ZP=PIY(I+2)
      GO TO 510
  560 IF (IPL(I)) 600,580,570
  570 IPL(I)=0
      PT(1)=PIX(I)
      PT(2)=PIY(I)
      PT(3)=PIZ(I)
      GO TO 400
  580 NO=NO+KK
      XLP(I)=XPT
      ZLP(I)=YPT
      NO=1
  590 II=IP
  600 CONTINUE
      RETURN
      END



!!!     OVERLAY(LWB,4,4)
      SUBROUTINE SPLTSV()
!
!     *******************************************VERSION B00************
!
!
!     SAVE SINGULARITY PANELING PLOT INFORMATION
!
!     BASED ON PROGRAM START OF NASA TMX-2074.
!
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAP&
!!!     &E9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      INCLUDE 'files.cmn'
      COMMON /BLANK/ J0,J1,J2,J3,J4,J5,J6,NWAF,NWAFOR,NFUS,NRADX(4),NFOR&
     &X(4),NP,NPODOR,NF,NFINOR,NCAN,NCANOR,J2TEST,NW,NC
      COMMON /BLANK2/ HORZ,VERT,TEST1,PHI,THETA,PSI,XF,YF,ZF,DIST,FMAG,P&
     &LOTSZ,TYPE,KODE,XMIN,XMAX,YMIN,YMAX,ZMIN,ZMAX,XMID,YMID,ZMID,BIGD,&
     &ISP
      COMMON /NEWCOM/K0,K1,K2,K3,K4,K5,K6,KWAF,KWAFOR,KFUS,KRADX(4),     &
     &              KFORX(4),KF(6),KFINOR(6),KAN(6),KANOR(6)
!
      DIMENSION XC(30,20), YC(30,20), ZC(30,20), ALRT(31,3,2), VECRT(30,&
     &3), KWFR(6), KWF(6), NWF(6), NCANR(6)
!
!!!      INTEGER TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAPE9,TAPE&
!!!     &10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
!
      READ (TAPE11) K0,K1,K2,K3,K4,K5,K6
      IF (K1.EQ.0) GO TO 190
!
!     WING
!
      PRINT 5
    5 FORMAT(10X,'  SPLTSV   - COMMON /NEWCOM/  ',//)
!!!      CALL PDUMP(K0,KANOR(6),2)
      WRITE(2,*) K0,K1,K2,K3,K4,K5,K6,KWAF,KWAFOR,KFUS,KRADX,           &
     &              KFORX,KF,KFINOR,KAN,KANOR

      KKF = KWAF
      IF (KKF.EQ.0) KKF=2
      KWAF=KKF
      IF (KWAFOR.EQ.0) KWAFOR=NWAFOR
      DO 180 I=1,2
      READ (TAPE3) ((XC(J,K),YC(J,K),ZC(J,K),J=1,KWAFOR),K=1,KKF)
!
!     MAXIMA AND MINIMA
!
      IF (I.EQ.2) GO TO 10
      XMIN=XC(1,1)
      XMAX=XC(1,1)
      YMIN=YC(1,1)
      YMAX=YC(1,1)
      ZMIN=ZC(1,1)
      ZMAX=ZC(1,1)
   10 DO 30 N=1,KKF
      DO 20 NN=1,KWAFOR
      XMIN=AMIN1(XMIN,XC(NN,N))
      XMAX=AMAX1(XMAX,XC(NN,N))
      YMIN=AMIN1(YMIN,YC(NN,N))
      YMAX=AMAX1(YMAX,YC(NN,N))
      ZMIN=AMIN1(ZMIN,ZC(NN,N))
      ZMAX=AMAX1(ZMAX,ZC(NN,N))
   20 END DO
   30 END DO
!
!     WRITE PLOT TAPE
!
      NL1=KWAFOR-1
!
!     SETUP FIRST LINE IN STREAMWISE DIRECTION
!
      DO 40 N=1,KWAFOR
      ALRT(N,1,2)=XC(N,1)
      ALRT(N,2,2)=YC(N,1)
      ALRT(N,3,2)=ZC(N,1)
   40 END DO
      WRITE (TAPE12) ((ALRT(N,N3,2),N=1,KWAFOR),N3=1,3)
      DO 100 NN=2,KKF
      DO 50 N=1,KWAFOR
      DO 50 N3=1,3
      ALRT(N,N3,1)=ALRT(N,N3,2)
   50 CONTINUE
      DO 60 N=1,KWAFOR
      ALRT(N,1,2)=XC(N,NN)
      ALRT(N,2,2)=YC(N,NN)
      ALRT(N,3,2)=ZC(N,NN)
   60 END DO
      IF(I.EQ.2) GO TO 80
   70 CALL SURCC (KWAFOR,ALRT,VECRT)
      GO TO 90
   80 CALL SURCL (KWAFOR,ALRT,VECRT)
   90 CONTINUE
      WRITE (TAPE12) ((VECRT(N,N3),N=1,NL1),N3=1,3)
      WRITE (TAPE12) ((ALRT(N,N3,2),N=1,KWAFOR),N3=1,3)
  100 END DO
!
!     SETUP FIRST LINE IN SPANWISE DIRECTION
!
      NL1=KKF-1
      DO 110 N=1,KKF
      ALRT(N,1,2)=XC(1,N)
      ALRT(N,2,2)=YC(1,N)
      ALRT(N,3,2)=ZC(1,N)
  110 END DO
      WRITE (TAPE12) ((ALRT(N,N3,2),N=1,KKF),N3=1,3)
      DO 170 NN=2,KWAFOR
      DO 120 N=1,KKF
      DO 120 N3=1,3
      ALRT(N,N3,1)=ALRT(N,N3,2)
  120 CONTINUE
      DO 130 N=1,KKF
      ALRT(N,1,2)=XC(NN,N)
      ALRT(N,2,2)=YC(NN,N)
      ALRT(N,3,2)=ZC(NN,N)
  130 END DO
      IF(I.EQ.2) GO TO 150
  140 CALL SURCL (KKF,ALRT,VECRT)
      GO TO 160
  150 CALL SURCC (KKF,ALRT,VECRT)
  160 CONTINUE
      WRITE (TAPE12) ((VECRT(N,N3),N=1,NL1),N3=1,3)
      WRITE (TAPE12) ((ALRT(N,N3,2),N=1,KKF),N3=1,3)
  170 END DO
  180 END DO
  190 IF (K2.EQ.0) GO TO 310
!
!     FUSELAGE (BODY)
!
      NFUS = KFUS
      DO 300 NFU=1,NFUS
      NRAD=KRADX(NFU)
      NFUSOR=KFORX(NFU)
      IF (NRAD.EQ.0) NRAD=NRADX(NFU)
      IF (NFUSOR.EQ.0) NFUSOR=NFORX(NFU)
      NL1=NFUSOR-1
      READ (TAPE3) ((XC(J,K),YC(J,K),ZC(J,K),J=1,NFUSOR),K=1,NRAD)
      IF (NFU.EQ.1.AND.K1.NE.0) GO TO 200
      IF (NFU.NE.1) GO TO 200
!
!     MAXIMA AND MINIMA
!
      XMIN=XC(1,1)
      XMAX=XC(1,1)
      YMIN=YC(1,1)
      YMAX=YC(1,1)
      ZMIN=ZC(1,1)
      ZMAX=ZC(1,1)
  200 DO 210 NN=1,NFUSOR
      XMIN=AMIN1(XMIN,XC(NN,1))
      XMAX=AMAX1(XMAX,XC(NN,1))
      DO 210 K=1,NRAD
      YMIN=AMIN1(YMIN,YC(NN,K))
      YMAX=AMAX1(YMAX,YC(NN,K))
      ZMIN=AMIN1(ZMIN,ZC(NN,K))
      ZMAX=AMAX1(ZMAX,ZC(NN,K))
  210 CONTINUE
!
!     SETUP FIRST LINE IN STREAMWISE DIRECTION
!
      DO 220 N=1,NFUSOR
      ALRT(N,1,2)=XC(N,1)
      ALRT(N,2,2)=YC(N,1)
      ALRT(N,3,2)=ZC(N,1)
  220 END DO
      WRITE (TAPE12) ((ALRT(N,N3,2),N=1,NFUSOR),N3=1,3)
      DO 250 NN=2,NRAD
      DO 230 N=1,NFUSOR
      DO 230 N3=1,3
      ALRT(N,N3,1)=ALRT(N,N3,2)
  230 CONTINUE
      DO 240 N=1,NFUSOR
      ALRT(N,1,2)=XC(N,NN)
      ALRT(N,2,2)=YC(N,NN)
      ALRT(N,3,2)=ZC(N,NN)
  240 END DO
      CALL SURCL (NFUSOR,ALRT,VECRT)
      WRITE (TAPE12) ((VECRT(N,N3),N=1,NL1),N3=1,3)
      WRITE (TAPE12) ((ALRT(N,N3,2),N=1,NFUSOR),N3=1,3)
  250 END DO
!
!     SETUP FIRST LINE AROUND BODY (FUSELAGE)
!
      NL1=NRAD-1
      DO 260 N=1,NRAD
      ALRT(N,1,2)=XC(1,N)
      ALRT(N,2,2)=YC(1,N)
      ALRT(N,3,2)=ZC(1,N)
  260 END DO
      WRITE (TAPE12) ((ALRT(N,N3,2),N=1,NRAD),N3=1,3)
      DO 290 NN=2,NFUSOR
      DO 270 N=1,NRAD
      DO 270 N3=1,3
      ALRT(N,N3,1)=ALRT(N,N3,2)
  270 CONTINUE
      DO 280 N=1,NRAD
      ALRT(N,1,2)=XC(NN,N)
      ALRT(N,2,2)=YC(NN,N)
      ALRT(N,3,2)=ZC(NN,N)
  280 END DO
      CALL SURCC (NRAD,ALRT,VECRT)
      WRITE (TAPE12) ((VECRT(N,N3),N=1,NL1),N3=1,3)
      WRITE (TAPE12) ((ALRT(N,N3,2),N=1,NRAD),N3=1,3)
  290 END DO
  300 END DO
  310 CONTINUE
!
!     FIN (VERTICAL TAIL)
!
      IF (K4.EQ.0) GO TO 490
      DO 312 I = 1,NF
      KWF(I) = KF(I)
  312 KWFR(I) = KFINOR(I)
      DO 480 NT=1,NF
      KW=KWFR(NT)
      KK=KWF(NT)
      IF (KK.EQ.0) KK=2
      IF (KW.EQ.0) KW=NFINOR
      DO 470 I=1,2
      READ (TAPE3) ((XC(J,L),YC(J,L),ZC(J,L),J=1,KW),L=1,KK)
      IF (NT.EQ.1.AND.(K1.NE.0.OR.K2.NE.0)) GO TO 320
      IF (NT.NE.1) GO TO 320
!
!     MAXIMA AND MINIMA
!
      XMIN=XC(1,1)
      XMAX=XC(1,1)
      YMIN=YC(1,1)
      YMAX=YC(1,1)
      ZMIN=ZC(1,1)
      ZMAX=ZC(1,1)
  320 DO 330 IK=1,KK
      DO 330 K=1,KW
      XMIN=AMIN1(XMIN,XC(K,IK))
      XMAX=AMAX1(XMAX,XC(K,IK))
      YMAX=AMAX1(YMAX,YC(K,IK))
      ZMIN=AMIN1(ZMIN,ZC(K,IK))
      ZMAX=AMAX1(ZMAX,ZC(K,IK))
  330 CONTINUE
!
!     WRITE PLOT TAPE
!
      NN1=KW-1
!
!     SETUP LINES IN STREAMWISE DIRECTION
!
      DO 340 J=1,KW
      ALRT(J,1,2)=XC(J,1)
      ALRT(J,2,2)=YC(J,1)
      ALRT(J,3,2)=ZC(J,1)
  340 END DO
      WRITE (TAPE12) ((ALRT(N,N3,2),N=1,KW),N3=1,3)
      DO 400 K=2,KK
      DO 350 J=1,KW
      ALRT(J,1,1)=ALRT(J,1,2)
      ALRT(J,2,1)=ALRT(J,2,2)
      ALRT(J,3,1)=ALRT(J,3,2)
  350 END DO
      DO 360 J=1,KW
      ALRT(J,1,2)=XC(J,K)
      ALRT(J,2,2)=YC(J,K)
      ALRT(J,3,2)=ZC(J,K)
  360 END DO
      IF(I.EQ.2) GO TO 380
  370 CALL SURCL (KW,ALRT,VECRT)
      GO TO 390
  380 CALL SURCC (KW,ALRT,VECRT)
  390 CONTINUE
      WRITE (TAPE12) ((VECRT(N,N3),N=1,NN1),N3=1,3)
      WRITE (TAPE12) ((ALRT(N,N3,2),N=1,KW),N3=1,3)
  400 END DO
!
!     SETUP LINES IN SPANWISE DIRECTION
!
      KK1=KK-1
      DO 410 II=1,KK
      ALRT(II,1,2)=XC(1,II)
      ALRT(II,2,2)=YC(1,II)
      ALRT(II,3,2)=ZC(1,II)
  410 END DO
      WRITE (TAPE12) ((ALRT(N,N3,2),N=1,KK),N3=1,3)
      DO 470 K=2,KW
      DO 420 J=1,KK
      ALRT(J,1,1)=ALRT(J,1,2)
      ALRT(J,2,1)=ALRT(J,2,2)
      ALRT(J,3,1)=ALRT(J,3,2)
  420 END DO
      DO 430 J=1,KK
      ALRT(J,1,2)=XC(K,J)
      ALRT(J,2,2)=YC(K,J)
      ALRT(J,3,2)=ZC(K,J)
  430 END DO
      IF(I.EQ.2) GO TO 450
  440 CALL SURCC (KK,ALRT,VECRT)
      GO TO 460
  450 CALL SURCL (KK,ALRT,VECRT)
  460 CONTINUE
      WRITE (TAPE12) ((VECRT(N,N3),N=1,KK1),N3=1,3)
      WRITE (TAPE12) ((ALRT(N,N3,2),N=1,KK),N3=1,3)
  470 CONTINUE
  480 END DO
!
!     CANARD (HORIZONTAL TAIL)
!
  490 IF (K5.EQ.0) GO TO 680
      DO 495 I = 1,NCAN
      NWF(I) = KAN(I)
  495 NCANR(I) = KANOR(I)
      DO 670 KAC=1,NCAN
      NCN=NWF(KAC)
      NC=NCANR(KAC)
      IF (NCN.EQ.0) NCN=2
      IF (NC.EQ.0) NC=NCANOR
      DO 660 I=1,2
      READ (TAPE3) ((XC(J,L),YC(J,L),ZC(J,L),J=1,NC),L=1,NCN)
      IF (KAC.EQ.1.AND.(K1.NE.0.OR.K2.NE.0.OR.K4.NE.0)) GO TO 500
      IF (KAC.NE.1) GO TO 500
!
!     MAXIMA AND MINIMA
!
      XMIN=XC(1,1)
      XMAX=XC(1,1)
      YMAX=YC(1,NCN)
      ZMIN=ZC(1,1)
      ZMAX=ZC(1,1)
  500 DO 510 IK=1,NCN
      DO 510 K=1,NC
      XMIN=AMIN1(XMIN,XC(K,IK))
      XMAX=AMAX1(XMAX,XC(K,IK))
      YMAX=AMAX1(YMAX,YC(K,IK))
      ZMIN=AMIN1(ZMIN,ZC(K,IK))
      ZMAX=AMAX1(ZMAX,ZC(K,IK))
  510 CONTINUE
!
!     WRITE PLOT TAPE
!
      NN1=NC-1
!
!     SETUP LINES IN STREAMWISE DIRECTION
!
      DO 520 J=1,NC
      ALRT(J,1,2)=XC(J,1)
      ALRT(J,2,2)=YC(J,1)
      ALRT(J,3,2)=ZC(J,1)
  520 END DO
      WRITE (TAPE12) ((ALRT(N,N3,2),N=1,NC),N3=1,3)
      DO 580 K=2,NCN
      DO 530 J=1,NC
      ALRT(J,1,1)=ALRT(J,1,2)
      ALRT(J,2,1)=ALRT(J,2,2)
      ALRT(J,3,1)=ALRT(J,3,2)
  530 END DO
      DO 540 J=1,NC
      ALRT(J,1,2)=XC(J,K)
      ALRT(J,2,2)=YC(J,K)
      ALRT(J,3,2)=ZC(J,K)
  540 END DO
      GO TO (550,560), I
  550 CALL SURCC (NC,ALRT,VECRT)
      GO TO 570
  560 CALL SURCL (NC,ALRT,VECRT)
  570 CONTINUE
      WRITE (TAPE12) ((VECRT(N,N3),N=1,NN1),N3=1,3)
      WRITE (TAPE12) ((ALRT(N,N3,2),N=1,NC),N3=1,3)
  580 END DO
!
!     SETUP LINES IN SPANWISE DIRECTION
!
      KK1=NCN-1
      DO 590 II=1,NCN
      ALRT(II,1,2)=XC(1,II)
      ALRT(II,2,2)=YC(1,II)
      ALRT(II,3,2)=ZC(1,II)
  590 END DO
      WRITE (TAPE12) ((ALRT(N,N3,2),N=1,NCN),N3=1,3)
      DO 650 K=2,NC
      DO 600 J=1,NCN
      ALRT(J,1,1)=ALRT(J,1,2)
      ALRT(J,2,1)=ALRT(J,2,2)
      ALRT(J,3,1)=ALRT(J,3,2)
  600 END DO
      DO 610 J=1,NCN
      ALRT(J,1,2)=XC(K,J)
      ALRT(J,2,2)=YC(K,J)
      ALRT(J,3,2)=ZC(K,J)
  610 END DO
      IF(I.EQ.2) GO TO 630
  620 CALL SURCL (NCN,ALRT,VECRT)
      GO TO 640
  630 CALL SURCC (NCN,ALRT,VECRT)
  640 CONTINUE
      WRITE (TAPE12) ((VECRT(N,N3),N=1,KK1),N3=1,3)
      WRITE (TAPE12) ((ALRT(N,N3,2),N=1,NCN),N3=1,3)
  650 END DO
  660 END DO
  670 END DO
  680 CONTINUE
!
!     FIND MAXIMUM DISTANCE AND MIDPOINT
!
      YMIN=-YMAX
      XDIS=XMAX-XMIN
      YDIS=YMAX-YMIN
      ZDIS=ZMAX-ZMIN
      BIGD=AMAX1(XDIS,YDIS,ZDIS)
      XMID=(XMIN+XMAX)*0.5
      YMID=0.0
      ZMID=(ZMIN+ZMAX)*0.5
!
!
      END

!!!     OVERLAY(LWB,4,5)
      SUBROUTINE PLPRES()
!
!     *******************************************VERSION B00************
!
!
!     CONTROL ROUTINE FOR PRESSURE DISTRIBUTION PLOTS
!
!
!     THIS PROGRAM PLOTS THE COMPUTED PRESSURE COEFFICIENTS FOR THE WING
!     AND FOR THE FUSELAGE
!
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAP&
!!!     &E9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      INCLUDE 'files.cmn'
      COMMON /PARAM/ DUM1(3),LBC,THK,MACH,ALPHA
      COMMON /SEG/ NSEG,NROW(20),NCOL(20),DUM(60),NWT(20)
      COMMON /PRESS/ KRADX(4),NRADX(4),KFORX(4),NFORX(4),KWAF,NWAF,KWAFO&
     &R,NWAFOR,NFUS
      COMMON /GRAPH/ PLOT
      COMMON /SCALE/ CSCALE,CPMIN,KK,AMA(4),IS
      COMMON /PI/ PI,RADEG,DEGRAD
      COMMON /NEWCOM/DUMW(2),K2
      COMMON /CLINE/CCL(30),XL(30)
!
      DIMENSION PHI(23),CP(33),XQ(33),XX(63),YY(23),ZC(4,30)
!
      LOGICAL LBC
      INTEGER COMPT,PLOT
!!!,TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8&
!!!     &,TAPE9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      INTEGER:: errCode
!
!     IN LINE FUNCTION DEFINITION
!
      XIN(X1,Y1,X2,Y2,Y)=X1 + (X2-X1)*(Y-Y1)/(Y2-Y1)
!
!
!
!
      HMACH=10HMACH NO. =
      HALPHA=10H  ALPHA =
      PLOT=IABS(PLOT)
      PIO2=90.
      REWIND TAPE12
      REWIND TAPE13
      REWIND TAPE14
      WRITE (TAPE14,230) HMACH,MACH,HALPHA,ALPHA
      REWIND TAPE14
      READ (TAPE14,230) AMA
      REWIND TAPE14
!
!     START INPUT
!
   10 CONTINUE
      READ (TAPE12,IOSTAT=errCode) NP,NPASS,COMPT
!
!     END OF FILE TEST ON TAPE12
      IF (errCode < 0) GO TO 220
!!!      IF(EOF(TAPE12)) 220,20
!
!!!   20 CONTINUE
      IF (COMPT.EQ.2) GO TO 120
!
!     COMPUTE MAXIMUM AND MINIMUM VALUES OF CP-ARRAY FOR FUSELAGE AND
!     COMPUTE SCALE FACTORS
!
      NP1=NP
      CPMAX=0.0
      CPMIN=0.0
      KL=0
      DO 50 I=1,NFUS
      NR=NRADX(I)-1
      NF=NFORX(I)-1
      DO 40 J=1,NF
      KL=KL+1
      DO 30 K=1,NR
      READ (TAPE12) X,Y,Z,CPP
      CPMIN=AMAX1(CPMIN,CPP)
      CPMAX=AMIN1(CPMAX,CPP)
      IF(K.NE.NR) GO TO 30
      IF(X.GT.XL(KL)) ZC(I,J)=XIN(CCL(KL),XL(KL),CCL(KL+1),XL(KL+1),X)
   30 END DO
      XX(KL)=X
   40 END DO
   50 END DO
      REWIND TAPE12
      READ (TAPE12)
      CSCALE=(CPMAX-CPMIN)/10.
      ICS=CSCALE*130.
      CSC=ICS
      CSCALE=CSC/100.
      ICPMIN=CPMIN*100.
      CMIN=ICPMIN
      CPMIN=CMIN/100.
      YSCALE=CSCALE
      AA=0.00
      DO 60 I=1,10
      AA=AA-YSCALE
      IF (CPMIN.LE.AA) GO TO 70
   60 END DO
   70 CPMIN=AA
!
!     READ IN FUSELAGE PRESSURE COEFFICIENTS
!
      LL=0
      L=0
      DO 110 I=1,NFUS
      NR=NRADX(I)-1
      NF=NFORX(I)-1
      DO 100 J=1,NF
      L=L+1
      LL=LL+1
      DO 80 K=1,NR
      READ (TAPE12) X,Y,Z,CP(K)
      DZ = Z - ZC(I,J)
      PHI(K) = ATAN2(DZ,Y)*RADEG + PIO2
   80 END DO
      XX(L)=X
      IF (L.NE.1) GO TO 90
      CALL AXLES(COMPT,MACH,ALPHA)
   90 KK=L
      CALL PRESBO (X,NR,PHI,CP)
      IF (L.EQ.10.OR.LL.EQ.KL) CALL LABEL (LL,XX,KL,L,COMPT)
  100 END DO
  110 END DO
      GO TO 10
!
!     WING
!
!
!     COMPUTE MAXIMUM AND MINIMUM VALUES OF CP-ARRAY AND COMPUTE SCALE
!     FACTORS
!
  120 CONTINUE
      CPMAX=0.0
      CPMIN=0.0
      IF (.NOT.LBC) NP=NP/2
      DO 130 I=1,NP
      READ (TAPE12) Y,XQ1,CP1
      READ (TAPE13) Y,XQ2,CP2
      CPMAX=AMIN1(CPMAX,CP1,CP2)
      CPMIN=AMAX1(CPMIN,CP1,CP2)
  130 END DO
      REWIND TAPE12
      REWIND TAPE13
      CSCALE=(CPMAX-CPMIN)/10.
      ISKIP = 1
      IF( ISKIP .EQ. 1 ) GO TO 150
      ICS=CSCALE*120.
      IC = IABS(ICS)
      ITEST = 0
  300 IF(ITEST.GE.IC) GO TO 310
      ITEST = ITEST + 10
      GO TO 300
  310 IF(ICS) 320,340,340
  320 ICS = -ITEST
      GO TO 350
  340 ICS = ITEST
  350 CONTINUE
      CSC=ICS
      CSCALE=CSC/100.
      AB=0.00
      DO 140 I=1,10
      AB=AB-CSCALE
      IF (CPMIN.LE.AB) GO TO 150
  140 END DO
  150 CPMIN=AB
      NNN=NP1+2
      IF (K2.EQ.0) NNN=1
      DO 160 I=1,NNN
      READ (TAPE12)
  160 END DO
!
!     PLOT WING PRESSURE DISTRIBUTION
!
      L=0
      DO 210 II=1,2
      IS=II
      DO 210 NS=1,NSEG
      NC=NCOL(NS)
      NR=NROW(NS)
      K=0
      DO 200 I=1,NC
      K=K+1
      DO 180 J=1,NR
      IF (IS.EQ.2) GO TO 170
      READ (TAPE12) Y,XQ(J),CP(J)
      GO TO 180
  170 CONTINUE
      READ (TAPE13) Y,XQ(J),CP(J)
  180 END DO
      YY(I)=Y
      L=L+1
      IF (L.NE.1) GO TO 190
      CALL AXLES(COMPT,MACH,ALPHA)
  190 CALL PRSWNG (NR,L,XQ,CP)
      IF (L.EQ.10.OR.K.EQ.NC) CALL LABEL (K,YY,NC,L,COMPT)
  200 END DO
  210 CONTINUE
  220 CONTINUE
      REWIND TAPE12
      REWIND TAPE13
!
!
!
  230 FORMAT (A10,F10.5,A10,F10.5)
      END
      SUBROUTINE PRSWNG (NR,L,XQ,CP)
!
!     *******************************************VERSION B00************
!
!
!     PLOT WING PRESSURES
!
      COMMON /SCALE/ CSCALE,CPMIN,KK,AMA(4),IS
!
      DIMENSION XQ(52), CP(52)
!
!
      YORG=CPMIN
      YSCALE=CSCALE
      XORG=0.0
      XSCALE=0.1
      XQ(NR+1)=XORG
      XQ(NR+2)=XSCALE
      CP(NR+1)=YORG
      CP(NR+2)=YSCALE
!
!     PLOT PRESSURE COEFFICIENTS VS XQ
!
      CALL PLOT((XQ(1)-XORG)/XSCALE,(CP(1)-CPMIN)/CSCALE,3)
      DO 10 I = 1, NR
      CALL PLOT((XQ(I)-XORG)/XSCALE,(CP(I)-CPMIN)/CSCALE,2)
   10 END DO
      RETURN
      END
      SUBROUTINE AXLES(COMPT,MACH,ALPHA)
!
!     *******************************************VERSION B00************
!
!
!     COMPUTE AXES FOR PLOTTING FUSELAGE PRESSURE COEFFICIENTS
!
      COMMON /GRAPH/ PLOT
      COMMON /SCALE/ CSCALE,CPMIN,KK,AMA(4),IS
!
      INTEGER PLOT,COMPT
!
      CALL CALPLT(1.0,1.0,-3)
      XORG=0.0
      XSCALE=18.
      TMIN = 9.0
      IF(COMPT.EQ.2) TMIN = 10.0
      IF (COMPT.EQ.2) XSCALE=0.10
      YORG=CPMIN
      YSCALE=CSCALE
      IF (PLOT.EQ.1) GO TO 10
      CALL AXES(0.0,11.0,0.0,10.0,XORG,XSCALE,1.0,TMIN,' ',0.0,1)
      CALL AXES(10.0,0.0,90.0,11.0,YORG,YSCALE,1.0,10.0,' ',0.0,-1)
      CALL GRID(0.0,0.0,1.0,1.0,10,11)
   10 IF (COMPT.EQ.2) GO TO 20
      CALL AXES(0.0,0.0,0.0,10.0,XORG,XSCALE,1.0,9.0,'PHI',0.125,-3)
      GO TO 30
   20 CALL AXES (0.0,0.0,0.0,10.0,XORG,XSCALE,1.0,10.0,'X/C',0.125,-3)
   30 CALL AXES(0.0,0.0,90.0,11.0,YORG,YSCALE,1.0,10.0,'CP',0.125,2)
      CALL NOTATE(3.8,10.7,.12,'USSAERO',0.0,7)
      IF (COMPT.EQ.2) GO TO 40
      CALL NOTATE(2.9,10.45,.12,'FUSELAGE PRESSURE COEFFICIENTS',0.,30)
      GO TO 50
   40 CALL NOTATE(2.9,10.45,.12,'WING PRESSURE COEFFICIENTS',0.0,26)
      IF(IS.EQ.1) CALL NOTATE(2.9,10.2,.12,'UPPER SURFACE',0.0,13)
      IF(IS.EQ.2) CALL NOTATE(2.9,10.2,.12,'LOWER SURFACE',0.0,13)
   50 X = 2.9
      Y = 9.95
      CALL NOTATE(X,Y,.12,'MACH NO. =',0.0,10)
      Z = X + 1.13
      CALL NUMBER(Z,Y,.12,MACH,0.0,5)
      Z = Z + 1.13
      CALL NOTATE(Z,Y,.12,'   ALPHA =',0.0,10)
      Z = Z + 1.13
      CALL NUMBER(Z,Y,.12,ALPHA,0.0,5)
      RETURN
      END
      SUBROUTINE PRESBO (X,KF,PHI,CP)
!
!     *******************************************VERSION B00************
!
!
!     PLOT BODY PRESSURES
!
!!!      COMMON /SCALE/ CSCALE,CPMIN,KK,DUB(5)
      COMMON /SCALE/ CSCALE,CPMIN,KK,ama(4),is
!
      DIMENSION PHI(23), CP(33)
!
!
!     SCALE FACTORS
!
      YORG=CPMIN
      YSCALE=CSCALE
      XORG=0.0
      XSCALE=18.0
      CP(KF+1)=YORG
      CP(KF+2)=YSCALE
      PHI(KF+1)=XORG
      PHI(KF+2)=XSCALE
      CALL LINPLT (PHI,CP,KF,1,1,KK,1,0)
   10 CONTINUE
      RETURN
      END

      SUBROUTINE LABEL (LL,XX,KL,L,COMPT)
!
!     *******************************************VERSION B00************
!
!
!     THIS SUBROUTINE PLOTS LEGENDS TO THE GRAPHS OF THE FUSELAGE
!     PRESSURE DISTRIBUTION OR THE WING PRESSURE DISTRIBUTION
      REAL,INTENT(IN),DIMENSION(*):: xx
      INTEGER,INTENT(IN):: compt
!
!!!      COMMON /FILES/ TAPE1,TAPE2,TAPE3,TAPE4,TAPE5,TAPE6,TAPE7,TAPE8,TAP&
!!!     &E9,TAPE10,TAPE11,TAPE12,TAPE13,TAPE14,TAPE15
      INCLUDE 'files.cmn'
      COMMON /GRAPH/ PLOT
      COMMON /SCALE/ CSCALE,CPMIN,KK,AMA(4),IS
      COMMON /PARAM/DUMMY(5),MACH,ALPHA
!
      DIMENSION AZ(2)   !!!, XX(1)
!
      INTEGER PLOT   !!!,COMPT
      CHARACTER(LEN=4):: hx = 'X = '
      CHARACTER(LEN=4):: hy = 'Y = '


!!!      HX=4HX =
!!!      HY=4HY =
      IF (PLOT.NE.0) CALL NFRAME()
      CALL NOTATE (2.8,9.80,.12,'LEGEND',0.0,6)
      IF (COMPT.EQ.2) GO TO 10
      CALL NOTATE (2.8,9.55,.12,'FUSELAGE PRESSURE PLOTS',0.0,23)
      GO TO 20
   10 CALL NOTATE (2.8,9.55,.12,'WING PRESSURE PLOTS',0.0,19)
      IF (IS.EQ.1) CALL NOTATE (2.8,9.30,.12,'UPPER SURFACE',0.0,13)
      IF (IS.EQ.2) CALL NOTATE (2.8,9.30,.12,'LOWER SURFACE',0.0,13)
   20 CONTINUE
      X = 2.8
      YY = 9.00
      CALL NOTATE(X,YY,.12,'MACH NO. =',0.0,10)
      Z = X + 1.13
      CALL NUMBER(Z,YY,.12,MACH,0.0,5)
      Z = Z + 1.13
      CALL NOTATE(Z,YY,.12,'   ALPHA =',0.0,10)
      Z = Z + 1.13
      CALL NUMBER(Z,YY,.12,ALPHA,0.0,5)
      I1=0
      IF (LL.EQ.KL) I1=KL-L
      X0=2.8
      Y=8.1
      DO 50 J=1,10
      I1=I1+1
      IF (I1.GT.KL) GO TO 60
      XA=XX(I1)
      CALL PNTPLT(X0,Y,J,1)
      IF (COMPT.EQ.2) GO TO 30
      Z = X0 + 0.2
      CALL NOTATE(Z,Y,.12,'X = ',0.0,4)
      GO TO 40
   30 Z = X0 + 0.2
      CALL NOTATE(Z,Y,.12,'Y = ',0.0,4)
   40 Z = Z + 0.411
      CALL NUMBER(Z,Y,.12,XA,0.0,5)
      Y=Y-0.2
   50 END DO
   60 CONTINUE
      IF (PLOT.NE.0) CALL NFRAME
      L=0
      RETURN
!
!
   70 FORMAT (A4,F10.5)
      END

PURE FUNCTION XinFunction(x1,y1,x2,y2,y) RESULT(f)
  REAL,INTENT(IN):: x1,y1,x2,y2,y
  REAL:: f
!----------------------------------------------------------------------------
  f=x1+(x2-x1)*(y-y1)/(y2-y1)
  RETURN
END Function XinFunction   ! ------------------------------------------------
!
 